/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package actions;

import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import outils.Outil;
import mathgraph32.MtgFrame;
import mathgraph32.MtgImageIcon;


/**
 * Classe anctre de toutes les actions utilises dans MathGraph32.
 * Pour qu'une telle action soit utilise au moins une figure doit tre ouverte
 * @author Yves
 */
@SuppressWarnings("serial")
public abstract class MtgAction extends AbstractAction {
  public MtgFrame frame;
  public ImageIcon imicon = null; // Ajout version 4.3
  public ImageIcon imiconMenu = null; // Ajout version 5.0.0.5 pour rduire le remps de rduction des icnes pour les menus
  public String tip; // Ajout version 5.0
  // Si l'action est associe  un outil, outil  pointera sur celui-ci, sinon outil sera null;
  public Outil outilAssocie;
  public boolean dansBarreHorizontale; // Rajout version 5.0
 
  /**
   * 
   * @param frameInit le frame de l'application
   * @param outilAssocieInit l'outim associ
   * @param menu la chane pour le menu
   * @param tip le Tooltip asoci
   * @param icon L'icne associ (nul si pas de bouton associ)
   * @param mnemonic Le raccouric clavier associ (nul si aucun)
   */
  public MtgAction(MtgFrame frameInit, Outil outilAssocieInit, String menu,
          String tip, String URL, Object acceleratorKey, boolean dansBarreHorizontaleInit) {
    super(menu);
    if (URL != null) {
	    	imicon = new ImageIcon(MtgFrame.class.getResource(URL + ".png"));
	    	imiconMenu = new MtgImageIcon(0.35*MtgFrame.coefMultForDlgBoxIcons,  imicon);
    }
    outilAssocie = outilAssocieInit;
    frame = frameInit;
    int size = (int) (MtgFrame.coefMultForDlgBoxIcons*MtgFrame.sizeFontTooltip);
    // Pour grer les hautes rsolutions on passe en html
    if (tip != null) {
    	this.tip = "<html><p style=\"color: blue; background-color: #FFF9D5; font-family:verdana; font-size:" + size + "\">" + tip + "</p></html>";
    	putValue(SHORT_DESCRIPTION, this.tip);
    }
    else this.tip = null;
    putValue(Action.ACCELERATOR_KEY, acceleratorKey);
    dansBarreHorizontale = dansBarreHorizontaleInit;
  }
  
  /**
   * Dfinition des constantes C++ associes aux items de menus
   */
  public int idMenu() {
    return 0;
  }
  
  /**
   * Fonction qui renverra true seulement pour les outils figurant par dfaut dans la barre
   * d'outils horizontale
   * Devenu final version 5.0 car pass comme dernier paramtre du constructeur
   */
  public boolean estDansBarreHorizontale() {
  	return dansBarreHorizontale;
  }
  
  public void actionPerformed(ActionEvent e) {
  	if (frame.cadreActif() == null) {
  		frame.dialogueAvertissement(getString("messagePasDeFigureOuverte"));
  		return;
  	}
  	//
    if (outilAssocie != null) {
      if (outilAssocie.activationValide()) {
        frame.outilActif.deselect();
        // frame.activeIconeAssocieeAction(this);
        // Modification version 4.9.9
        // frame.outilActif = outilAssocie;
        // On informe les barres d'outil de l'activation si cela a t fait via un menu
        // frame.activeIconeAssocieeAOutil(frame.outilActif);
      	frame.activeActionDansBarreGauche(this); // Ajout version 5.0
        outilAssocie.select();
        // frame.outilActif = outilAssocie; //Supprim version 5.0 car inutile puisque fait lors de select() de chaque outil
        // Provoquait un bug. par exemple, l'outil de cration de variable : Si on crait une variable et changeait ensuite de fentre, l'outil tait
        // ractiv et la bote de dialogue se rouvrait
        frame.actionEnCours = this;
       // requestFocus();
      }
      else {
      	String ch = getString("messageOutilPasDisponible");
      	if (outilAssocie.messageActivationNonValide() != null) ch = ch + "\n" + outilAssocie.messageActivationNonValide();
        frame.dialogueAvertissement(ch);
        outilAssocie.deselect();
        // On ractive l'outil de capture si l'outil ne peut pas tre activ
        frame.activeOutilCapture();
      }         
    }
  	frame.fermeMenusHorizontaux();
    frame.cadreActif().requestFocus();
  }
  
  public static final String getString(String key) {
  	return MtgFrame.getString(key);
  }
  
}