package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionListener;
import java.text.NumberFormat;
import java.util.Vector;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.DefaultListModel;
import javax.swing.JFormattedTextField;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import objets.CListeObjets;
import objets.CPrototype;
import outils.Outil;
import types.FonteNom;
import mathgraph32.MtgFrame;

/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
@SuppressWarnings("serial")
public class ConstructionRecursiveDlg extends MtgDlg {
	public int pasRec, nbImp, profondMax, nbSources, choixLigne;
	public boolean adapterStyle;
	public boolean creationNiveauMax;
	public Vector<CPrototype> tableProto;
	private MtgLabel label3, label4, label5, label5bis, label6, label9;
	private ListePrototypesPourRecurOuIter listePrototypes;
	private JScrollPane scrollPaneListePrototypes;
	private MtgButton btnOK, btnCancel;
	private MtgRadioButton[] br = new MtgRadioButton[13]; // Les six boutons radios associs aux profondeurs de rcursion disponibles
  private javax.swing.JFormattedTextField editeurPasRec, editeurNbImp, editeurNbSources;
  private MtgCheckBox checkAdapterCouleurEtStyle,  checkConstructionNiveauMax;
  private MtgRadioButton radioSansLigne, radioAvecPolygone, radioAvecLigneBrisee;
	private int[] indconst = new int[13]; // L'indice de la construction associe  chaque niveau de rcursion
  private DefaultListModel<String> listModel;
	private JList<String> listeProf; // La liste contenant les niveaus de profondeur

  public ConstructionRecursiveDlg(MtgFrame frameInit) {
    super (getString("recursionConstructionDlg1"), true, frameInit);
    initComponents();
    getRootPane().setDefaultButton(btnOK);    
  }

  private void initComponents() {
  	label3 = new MtgLabel(frame, getString("macroConstructionIterativeDlg2"));  	
  	label4 = new MtgLabel(frame, getString("recursionConstructionDlg2"));  	
  	label5 = new MtgLabel(frame, getString("recursionConstructionDlg3"));
  	label5bis = new MtgLabel(frame, getString("recursionConstructionDlg4"));
  	label6 = new MtgLabel(frame, getString("iterationConstructionDlg4"));
  	label9 = new MtgLabel(frame, getString("recursionConstructionDlg8")); 
	  btnOK = new MtgButton(frame, getString("btnOK"));
	  btnOK.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnOKActionPerformed();
	    }
	  });    
	  btnCancel = new MtgButton(frame, getString("btnCancel"));
	  btnCancel.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnCancelActionPerformed();
	    }
	  });
	  btnOK.setMaximumSize(btnCancel.getPreferredSize());	
		listeProf = new JList<String>(); // La liste contenant les profondeurs de rcursion
		listeProf.setFont(FonteNom.getFonteEdition(MtgFrame.sizeFontList, FonteNom.Normal,
				(double)frame.pref_addSystemFont/MtgFrame.sizeFontList + MtgFrame.coefMultForDlgBoxIcons));		

    listModel = new DefaultListModel<String>();
    listeProf.setModel(listModel);
		listeProf.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		listeProf.setLayoutOrientation(JList.VERTICAL);
		listeProf.setVisibleRowCount(4);
		listeProf.addListSelectionListener(new ListSelectionListener() {
			
			public void valueChanged(ListSelectionEvent e) {
				listeProfSelectionChanged();
			}
		});
		JScrollPane scrollpaneList = new JScrollPane(listeProf, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

		// Cration de 7 boutons radios pour la pprofondeur de rcursion maximale
		ButtonGroup group1 = new ButtonGroup();
		for (int i = 0; i < 13; i++) {
			br[i] = new MtgRadioButton(frame, Integer.toString(i));
			group1.add(br[i]);
		  br[i].addActionListener(new ActionListener() {
	      public void actionPerformed(java.awt.event.ActionEvent evt) {
	      	radioProfActionPerformed();
	      }
	 		});
		}
	  for (int i = 0; i <= 12; i++) indconst[i]=0; // par dfaut le protoype associ  chaque niveau est le premier prototype de la liste
	  listePrototypes = new ListePrototypesPourRecurOuIter(frame, frame.cadreActif().document);
	  listePrototypes.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				listePrototypesListSelectionChanged();
			}
		});
  	profondMax = 3; // Par dfaut prodfonduer de rcursion de 3}
  	br[3].setSelected(true);
  	prepareListeProf();
	  scrollPaneListePrototypes = new JScrollPane(listePrototypes,
    		JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    NumberFormat format;
    format = NumberFormat.getInstance();
    format.setParseIntegerOnly(true);
    format.setGroupingUsed(false);
    editeurNbImp = new JFormattedTextField(format);
    editeurNbImp.setColumns(2);
    editeurPasRec = new JFormattedTextField(format);
    editeurPasRec.setColumns(2);
    editeurNbSources = new JFormattedTextField(format);
    editeurNbSources.setColumns(2);
    editeurNbSources.setText("0");
	  radioSansLigne = new MtgRadioButton(frame, getString("recursionConstructionDlg5"));
	  radioAvecPolygone = new MtgRadioButton(frame, getString("recursionConstructionDlg6"));
	  radioAvecLigneBrisee = new MtgRadioButton(frame, getString("recursionConstructionDlg7"));
	  radioSansLigne.setSelected(true);
	  checkAdapterCouleurEtStyle = new MtgCheckBox(frame, getString("iterationConstructionDlg5"));
	  checkAdapterCouleurEtStyle.setSelected(false);
	  checkConstructionNiveauMax = new MtgCheckBox(frame, getString("recursionConstructionDlg9"));
	  checkConstructionNiveauMax.setSelected(false);
	  ButtonGroup group = new ButtonGroup();
	  group.add(radioSansLigne);
	  group.add(radioAvecPolygone);
	  group.add(radioAvecLigneBrisee);

		editeurNbSources.setText("0");
		editeurPasRec.setText("1");
		editeurNbImp.setText("2");
		br[3].setSelected(true);
  	radioSansLigne.setSelected(true);

    JPanel panePrincipal = (JPanel) getContentPane();
    JPanel paneRight = new JPanel();
    panePrincipal.setLayout(new GridBagLayout());
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    JPanel paneChoixProf = new JPanel();
    paneChoixProf.setLayout(new BoxLayout(paneChoixProf, BoxLayout.X_AXIS));
    paneChoixProf.add(label5bis);
    for (int i = 0; i < 13; i++) paneChoixProf.add(br[i]);
    JPanel paneChoixNiv = new JPanel();
    paneChoixNiv.setLayout(new BoxLayout(paneChoixNiv, BoxLayout.X_AXIS));
    paneChoixNiv.add(label9);
    paneChoixNiv.add(scrollpaneList);
    paneChoixNiv.add(label3);
    paneChoixNiv.add(scrollPaneListePrototypes);

		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets.bottom = 0;
		constraints.insets.top = 2;
		constraints.insets.left = 5;
		constraints.insets.right = 5;

		setConstraints(constraints, 0, 0, 2, 1, 0, 30, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(paneChoixProf, constraints);
		
		setConstraints(constraints, 0, 1, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(paneChoixNiv, constraints);
    setConstraints(constraints, 0, 2, 1, 1, 50, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label4, constraints);	
    setConstraints(constraints, 1, 2, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(editeurNbImp, constraints);
    setConstraints(constraints, 0, 3, 1, 1, 50, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label5, constraints);	
    setConstraints(constraints, 1, 3, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(editeurPasRec, constraints);
    setConstraints(constraints, 0, 4, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label6, constraints);	
    setConstraints(constraints, 1, 4, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(editeurNbSources, constraints);
	  setConstraints(constraints, 0, 5, 2, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  panePrincipal.add(checkConstructionNiveauMax, constraints);
	  setConstraints(constraints, 0, 6, 2, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  panePrincipal.add(radioSansLigne, constraints);
	  setConstraints(constraints, 0, 7, 2, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  panePrincipal.add(radioAvecPolygone, constraints);
	  setConstraints(constraints, 0, 8, 2, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  panePrincipal.add(radioAvecLigneBrisee, constraints);
    setConstraints(constraints, 0, 9, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(checkAdapterCouleurEtStyle, constraints);
		
 		setConstraints(constraints, 2, 1, 1, 2, 20, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(paneRight, constraints);

		pack();	
 
  }
  
  private void radioProfActionPerformed() {
  	for (int i = 0; i < 13; i++) {
  		if (br[i].isSelected()) {
  			profondMax = i;
  			break;
  		}
  	}
  	prepareListeProf();
  }
  
  private void prepareListeProf() {
  	listModel.removeAllElements();
		for (int i = 0; i <= profondMax; i++) listModel.addElement(Integer.toString(i));
		listeProf.setSelectedIndex(0);
		listeProf.ensureIndexIsVisible(0);
		listePrototypes.setSelectedIndex(0);
		listePrototypes.ensureIndexIsVisible(0);
  }

  private void listeProfSelectionChanged() {
  	int ind = listeProf.getSelectedIndex();
  	if (ind != -1) {
	  	listePrototypes.setSelectedIndex(indconst[ind]);
	  	listePrototypes.ensureIndexIsVisible(indconst[ind]);
  	}
  }
  
  private void listePrototypesListSelectionChanged() {
  	int ind1 = listeProf.getSelectedIndex();
  	int ind2 = listePrototypes.getSelectedIndex();
  	indconst[ind1] = ind2;
  }

  private void btnOKActionPerformed() {
  	CListeObjets liste = frame.cadreActif().listePr;
  	try {
  		pasRec = Integer.parseInt(editeurPasRec.getText());
  	}
		catch (NumberFormatException e) {
			dialogueAvertissement(getString("ch71"));
			editeurPasRec.requestFocus();
			return;
		}
 	
    if (pasRec <= 0 ) {
    	dialogueAvertissement(getString("ch71"));
    	editeurPasRec.requestFocus();
    	return;
    }
  	try {
  		nbImp = Integer.parseInt(editeurNbImp.getText());
  	}
		catch (NumberFormatException e) {
			dialogueAvertissement(getString("ch71"));
			editeurNbImp.requestFocus();
			return;
		}
    if (nbImp <= 0 ) {
    	dialogueAvertissement(getString("ch71"));
    	editeurNbImp.requestFocus();
    	return;
    }
  	try {
  		nbSources = Integer.parseInt(editeurNbSources.getText());
  	}
		catch (NumberFormatException e) {
			dialogueAvertissement(getString("ch71"));
			editeurNbSources.requestFocus();
			return;
		}
 	
    if (nbSources < 0) {
    	dialogueAvertissement(getString("ch71"));
    	editeurNbSources.requestFocus();
    	return;
    }
    for (int i = 0; i <= profondMax; i++) {
	    CPrototype proto = liste.documentProprietaire.tablePrototypes.get(indconst[i]);
	    String chErr = proto.implementationPossible(liste);
	    if (chErr.length() != 0) {
	    	frame.dialogueAvertissement(getString("ch142") + "\n" + chErr);
	    	listeProf.setSelectedIndex(i);
	    	return;
	    }
	    if ((liste.pointeurLongueurUnite == null) && proto.utiliseLongueurUnite()) {
     		frame.dialogueAvertissement(getString("ch138"));
     		listeProf.setSelectedIndex(i);
	    	return;
	    }
			if (!proto.estRecursible(nbSources, nbImp, pasRec)) {
				frame.dialogueAvertissement(getString("ch199"));
				listeProf.setSelectedIndex(i);
				return;
			}
    }
    // On fait une estimation du nombre d'objets  crer avce le prototype de niveau de rcursion 0
    CPrototype proto0 = liste.documentProprietaire.tablePrototypes.get(indconst[0]);
    double n;
    if (creationNiveauMax) {
    	double p = proto0.nombreObjetsJusqueFinal(proto0.nbObjSources-nbSources); // Nombre objets crs sauf dernire tape
    	n = (proto0.size()-proto0.nbObjSources-nbSources)*Math.exp((double)(profondMax)*Math.log(nbImp))
    			+ p*(Math.exp((double)(profondMax)*Math.log(nbImp))-1)/(nbImp-1);
    }
    else n = (double)(proto0.size()-proto0.nbObjSources-nbSources)*(Math.exp((double)(profondMax+1)*Math.log(nbImp))-1)/(nbImp-1);
		if (n > (double) Outil.nombreMaxObjets) {
			frame.dialogueAvertissement(getString("ch198"));
			return;
		}
  	for (int i = 0; i < 7; i++) {
  		if (br[i].isSelected()) {
  			profondMax = i;
  			break;
  		}
  	}
    tableProto = new Vector<CPrototype>();
    for (int i = 0; i <= profondMax; i++) tableProto.add(liste.documentProprietaire.tablePrototypes.get(indconst[i]));

	  choixLigne = (radioSansLigne.isSelected() ? Outil.sansLigne 
	  		: (radioAvecPolygone.isSelected() ? Outil.avecPolygone : Outil.avecLigneBrisee));
	  adapterStyle = checkAdapterCouleurEtStyle.isSelected();
	  creationNiveauMax = checkConstructionNiveauMax.isSelected();
    setRetourOK(true);
    dispose();
  }
  
  private void btnCancelActionPerformed() {                                          
    dispose();
  }       
}
