/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionListener;
import java.util.Vector;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JPanel;

import glob.Kernel;
import mathgraph32.MtgFrame;
import objets.CListeObjets;
import objets.CValeurAssocieeAVariable;
import types.DimensionFenetre;
import types.NatureCarSpe;

// Modifi version 5.4
@SuppressWarnings("serial")
public class CurseurDlg extends MtgDlg {

	public MtgRadioButton radioHorizontal;
	public MtgRadioButton radioVertical;
	private ButtonGroup btnGroup;
	// Ajout version 4.7.2
	public MtgRadioButton radioPteTaille;
	public MtgRadioButton radioGdeTaille;
	private ButtonGroup btnGroup2;
	private MtgLabel label1, label2, label3, label4, label5,label6,label7;
	private EditeurValeurDynamique editeurValeurMini, editeurValeurMaxi,editeurValeurPas,editeurValeurPasGrad;
	public MtgTextField editeurNom;
	public TextFieldAvecCarSpeciaux editeurPreChaine;
	public TextFieldAvecCarSpeciaux editeurPostChaine;
	private MtgButton btnOK, btnCancel;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsMin, paneListeValeurEtFonctionsMax, paneListeValeurEtFonctionsPas, paneListeValeurEtFonctionsPasGrad;
	public PaneListeDecimales paneDecimales;
	public MtgCheckBox checkValeursDiscretes, checkExtremitesPunaisees, checkAvecGrad;
  private MtgFocusTraversalPolicy mtgFocusTraversalPolicy;
	
	public CValeurAssocieeAVariable valMin, valMax,valPas, valPasGrad;
	
  public CurseurDlg(MtgFrame frameInit) {
    super(getString("curseurDlg1"), true, frameInit);
    CListeObjets liste = frame.cadreActif().listePr;
    valMin = new CValeurAssocieeAVariable(liste);
    valMax = new CValeurAssocieeAVariable(liste);
    valPas =  new CValeurAssocieeAVariable(liste);
    valPasGrad =  new CValeurAssocieeAVariable(liste);
    initComponents();
    getRootPane().setDefaultButton(btnOK);
  }
  
  private void initComponents() {
  	radioHorizontal = new MtgRadioButton(frame, getString("curseurDlg2"));
  	radioHorizontal.setSelected(true);
  	radioVertical =new MtgRadioButton(frame, getString("curseurDlg3"));
  	btnGroup = new ButtonGroup();
  	btnGroup.add(radioHorizontal);
  	btnGroup.add(radioVertical);
  	radioPteTaille = new MtgRadioButton(frame, getString("curseurDlg11"));
  	radioGdeTaille = new MtgRadioButton(frame, getString("curseurDlg12"));
  	btnGroup2 = new ButtonGroup();
  	btnGroup2.add(radioPteTaille);
  	btnGroup2.add(radioGdeTaille);
  	radioPteTaille.setSelected(true);

  	// Pas de valeurs entires par dfaut
  	checkValeursDiscretes = new MtgCheckBox(frame, getString("curseurDlg4"), false);
  	checkValeursDiscretes.addActionListener(new ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        checkValeursEntieresActionPerformed();
      }
  	});
  	checkAvecGrad = new MtgCheckBox(frame, getString("curseurDlg13"), false);
  	checkAvecGrad.addActionListener(new ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        checkAvecGradActionPerformed();
      }
  	});
  	
		btnOK = new MtgButton(frame, getString("btnOK"));
		btnOK.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				btnOKActionPerformed();
			}
		});
		btnCancel = new MtgButton(frame, getString("btnCancel"));
		btnCancel.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				btnCancelActionPerformed();
			}
		});
		btnOK.setMaximumSize(btnCancel.getPreferredSize());
		label1 = new MtgLabel(frame, getString("curseurDlg5"));
		label2 = new MtgLabel(frame, getString("curseurDlg6"));
		label3 = new MtgLabel(frame, getString("curseurDlg7"));
		label4 = new MtgLabel(frame, getString("curseurDlg8"));
		label5 = new MtgLabel(frame, getString("curseurDlg9"));
		label6 = new MtgLabel(frame, getString("curseurDlg14"));
		label6.setVisible(false);
		label7 = new MtgLabel(frame, getString("curseurDlg15"));
		label7.setVisible( false);
		checkExtremitesPunaisees = new MtgCheckBox(frame, getString("curseurDlg10"));
		checkExtremitesPunaisees.setSelected(true);
		editeurValeurMini = new EditeurValeurDynamique(frame, valMin, null);
		editeurValeurMaxi = new EditeurValeurDynamique(frame, valMax, null);
		editeurValeurPas = new EditeurValeurDynamique(frame, valPas, null);
		editeurValeurPas.setVisible(false);
		editeurValeurPasGrad = new EditeurValeurDynamique(frame, valPasGrad, null);
		editeurValeurPasGrad.setVisible(false);
  	int indmax = frame.cadreActif().listePr.size() - 1;
    paneListeValeurEtFonctionsMin = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurValeurMini, true, indmax, false);
    paneListeValeurEtFonctionsMax = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurValeurMaxi, true, indmax, false);
    paneListeValeurEtFonctionsPas = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurValeurPas, true, indmax, false);
    paneListeValeurEtFonctionsPas.setVisible(false);
    paneListeValeurEtFonctionsPasGrad = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurValeurPasGrad, true, indmax, false);
    paneListeValeurEtFonctionsPasGrad.setVisible(false);
    editeurNom = new MtgTextField(frame, 8);
  	editeurPreChaine = new TextFieldAvecCarSpeciaux(this, frame, NatureCarSpe.Tout);
  	editeurPostChaine = new TextFieldAvecCarSpeciaux(this, frame, NatureCarSpe.Tout);
  	paneDecimales = new PaneListeDecimales(frame, 2);
  	
  	JPanel panePrincipal = (JPanel) getContentPane();
		JPanel paneRight = new JPanel();
		JPanel paneCenter = new JPanel();
		panePrincipal.setLayout(new GridBagLayout());
		paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
		paneCenter.setLayout(new GridBagLayout());
		
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets.bottom = 2;
		constraints.insets.top = 2;
		constraints.insets.left = 5;
		constraints.insets.right = 5;
		
		setConstraints(constraints, 0, 0, 1, 1, 30, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneCenter.add(label1, constraints);
		setConstraints(constraints, 1, 0, 1, 1, 50, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		paneCenter.add(editeurValeurMini, constraints);
		setConstraints(constraints, 1, 1, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneCenter.add(paneListeValeurEtFonctionsMin, constraints);
		setConstraints(constraints, 0, 2, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneCenter.add(label2, constraints);
		setConstraints(constraints, 1, 2, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		paneCenter.add(editeurValeurMaxi, constraints);
		setConstraints(constraints, 1, 3, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneCenter.add(paneListeValeurEtFonctionsMax, constraints);
		setConstraints(constraints, 0, 4, 1, 1, 30, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneCenter.add(label6, constraints);
		setConstraints(constraints, 1, 4, 1, 1, 50, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		paneCenter.add(editeurValeurPas, constraints);
		setConstraints(constraints, 1, 5, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneCenter.add(paneListeValeurEtFonctionsPas, constraints);
		setConstraints(constraints, 0, 6, 1, 1, 30, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneCenter.add(label7, constraints);
		setConstraints(constraints, 1, 6, 1, 1, 50, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		paneCenter.add(editeurValeurPasGrad, constraints);
		setConstraints(constraints, 1, 7, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneCenter.add(paneListeValeurEtFonctionsPasGrad, constraints);


	
		
		setConstraints(constraints, 2, 0, 1, 4, 20, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		panePrincipal.add(paneDecimales, constraints);
		
		
		setConstraints(constraints, 0, 0, 1, 1, 40, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(radioHorizontal, constraints);
		setConstraints(constraints, 1, 0, 1, 1, 40, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(radioVertical, constraints);
		setConstraints(constraints, 0, 1, 1, 1, 40, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(radioPteTaille, constraints);
		setConstraints(constraints, 1, 1, 1, 1, 40, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(radioGdeTaille, constraints);
		setConstraints(constraints, 0, 2, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(checkValeursDiscretes, constraints);
		setConstraints(constraints, 1, 2, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(checkAvecGrad, constraints);
		
		setConstraints(constraints, 0, 3, 2, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(paneCenter, constraints);
	
		setConstraints(constraints, 0, 4, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label3, constraints);
		setConstraints(constraints, 1, 4, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		panePrincipal.add(editeurNom, constraints);
		setConstraints(constraints, 0, 5, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label4, constraints);
		setConstraints(constraints, 1, 5, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		panePrincipal.add(editeurPreChaine, constraints);
		setConstraints(constraints, 0, 6, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label5, constraints);
		setConstraints(constraints, 1, 6, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		panePrincipal.add(editeurPostChaine, constraints);
		setConstraints(constraints, 0, 7, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(checkExtremitesPunaisees, constraints);
		
		paneRight.add(btnOK);
		paneRight.add(Box.createRigidArea(new Dimension(0, 5)));
		paneRight.add(btnCancel);

		setConstraints(constraints, 2, 0, 1, 2, 20, 0, GridBagConstraints.NONE, GridBagConstraints.NORTH);
		panePrincipal.add(paneRight, constraints);
    // On change l'ordre par dfaut des composants
    setFocusOrder();
		pack();
  }
  
  private void setFocusOrder() {
  	int nbEditors = 7;
  	if(checkValeursDiscretes.isSelected()) nbEditors++;
  	if (checkAvecGrad.isSelected()) nbEditors++;
    Vector<Component> order = new Vector<Component>(nbEditors);
    order.add(editeurValeurMini);
    order.add(editeurValeurMaxi);
    if (checkValeursDiscretes.isSelected()) order.add(editeurValeurPas);
    if( checkAvecGrad.isSelected()) order.add(editeurValeurPasGrad);
    order.add(editeurNom);
    order.add(editeurPreChaine);
    order.add(editeurPostChaine);
    order.add(btnOK);
    order.add(btnCancel);
    mtgFocusTraversalPolicy = new MtgFocusTraversalPolicy(order);
    setFocusTraversalPolicy(mtgFocusTraversalPolicy);

  }
  
  private void btnOKActionPerformed() {
  	// On vrifie si le nom choisi pour la calcul final du curseur est valide
    if (!frame.cadreActif().listePr.validationNomCalcul(frame, editeurNom.getText())) {
      editeurNom.requestFocus();
      return;
    }
    if (!editeurValeurMini.verifieValidite()) {
    	editeurValeurMini.requestFocus();
    	return;
    }
    if (!editeurValeurMaxi.verifieValidite()) {
    	editeurValeurMaxi.requestFocus();
    	return;
    }
    // Amlioration version 3.5.1
    // Un curseur  valeurs entires n'accepte pas que la valeur maxi soit plus petite que la valeur mini
    // et un curseur  valeurs non entires n'accepte pas que la valeur mini soit gale  la valeur maxi
    DimensionFenetre dimfen = frame.cadreActif().dimf;
	  valMin.positionne(false, dimfen);
	  double min = valMin.valeur;
	  valMax.positionne(false, dimfen);
	  double max = valMax.valeur;
	  if (min == max) {
    	MtgDlg.dialogueAvertissementCentreEcran(getString("ch71"));
    	editeurValeurMini.requestFocus();
    	return;
	  }
    if (checkValeursDiscretes.isSelected()) {
    	if (!editeurValeurPas.verifieValidite()) {
      	editeurValeurPas.requestFocus();
      	return;
    	}
    	valPas.positionne(false,  dimfen);
    	double pas = valPas.valeur;
	    if (max < min) {
	    	MtgDlg.dialogueAvertissementCentreEcran(getString("ch71"));
	    	editeurValeurMini.requestFocus();
	    	return;
	    }
	    // Il faut que le pas soit un diviseur de maxi-mini
	    double qpas = (max - min)/pas;
	    if (!Kernel.zero(qpas-Math.round(qpas))) {
	    	MtgDlg.dialogueAvertissementCentreEcran(getString("curseurDlg16"));
	    	editeurValeurPas.requestFocus();
	    	return;
	    }
    }
    if (checkAvecGrad.isSelected()) {
    	if (!editeurValeurPasGrad.verifieValidite()) {
	    	editeurValeurPasGrad.requestFocus();
	    	return;
    	}
    	valPasGrad.positionne(false,  dimfen);
    	double pasGrad = valPasGrad.valeur;
    	double qgrad = (max - min)/pasGrad;
	    if (!Kernel.zero(qgrad-Math.round(qgrad))) {
	    	MtgDlg.dialogueAvertissementCentreEcran(getString("curseurDlg17"));
	    	editeurValeurPasGrad.requestFocus();
	    	return;
	    }
    }
  	setRetourOK(true);
  	dispose();
  }
  
  private void btnCancelActionPerformed() {
  	dispose();
  }
     
  private void checkValeursEntieresActionPerformed() {
  	if (checkValeursDiscretes.isSelected()) {
  		paneDecimales.setVisible(false);
  		label6.setVisible(true);
  		editeurValeurPas.setVisible(true);
  		paneListeValeurEtFonctionsPas.setVisible(true);
  	}
  	else {
  		paneDecimales.setVisible(true);
  		label6.setVisible(false);
  		editeurValeurPas.setVisible(false);
  		paneListeValeurEtFonctionsPas.setVisible(false);
  	}
  	setFocusOrder();
  	pack();
  }
	
  private void checkAvecGradActionPerformed() {
  	if (checkAvecGrad.isSelected()) {
  		label7.setVisible(true);
  		editeurValeurPasGrad.setVisible(true);
  		paneListeValeurEtFonctionsPasGrad.setVisible(true);
  	}
  	else {
  		label7.setVisible(false);
  		editeurValeurPasGrad.setVisible(false);
  		paneListeValeurEtFonctionsPasGrad.setVisible(false);
  	}
  	setFocusOrder();
  	pack();
  }
}
