package dialogs;

import objets.CCalculBase;
import objets.CListeObjets;
import types.ErreurCalculException;
import types.Int;


import mathgraph32.MtgFrame;

/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/* Classe ancetre de tous les diteurs insrs dans une boite de dialogue et chargs
* de modifier la valeur relle d'une valeur constante comme par exemple les valeurs caractrisant une variable
*/
@SuppressWarnings("serial")
public class EditeurValeurConstante extends MtgTextField {
  protected MtgFrame frame;
 	public double resultat;
  
  /** Constructeur */
  public EditeurValeurConstante(MtgFrame frameInit) {
  	super(frameInit);
  	frame = frameInit;
  	setColumns(25);
  	setAutoscrolls(true);
  }
  /**
   * Fonction charge de vrifier si la rponse, quand elle est correcte, est valide
   * pour le type de valeur attendu. Par exemple, le rayon d'un cercle doit tre non nul.
   * Renvoie true par dfaut et doit tre redfinie pour tous les descendants qui doivent imposer une condition.
   *
  */
  public boolean valeurAcceptee() {
    return true;
  }
  
  /**
   * Vrifie si le contenu est correct sur le plan de la syntaxe et si ui met dans resultat le rsultat
   * @return
   */
  public final boolean verifieValidite() {
  	CListeObjets liste = frame.cadreActif().listePr;
    Int inderr = new Int();
    // On vrifie d'abord la syntaxe de la rponse
    // La seule valeur qu'on a le droit d'utiliser est pi qui est toujours
    // le premier objet de la liste
    String rep = getText();
    if (CCalculBase.verifieSyntaxe(liste, rep, inderr, 0, null)) {
    
      CCalculBase calc = CCalculBase.creeCalculBase(rep ,liste, 0, rep.length()-1, null);
      try {
        calc.listeProprietaire.initialiseNombreIterations();
        resultat = calc.resultat(false);
      }
      catch (ErreurCalculException e) {
      	if (e.depassementCapacite)
      		frame.dialogueAvertissement(MtgFrame.getString("ch179"));
      	else frame.dialogueAvertissement(MtgFrame.getString("ch44"));
        requestFocus();
        return false;
      }
      return valeurAcceptee();
    }
    else {
      // On place le curseur  l'endroit de l'erreur de syntaxe
      this.select(inderr.getValue(),  inderr.getValue());
      frame.dialogueAvertissement(MtgFrame.getString("ch5"));
      requestFocus();
      return false;
    }
  }
}
