/*
 * EntreeNomDlg.java
 *
 * Created on 9 aot 2008, 17:59
 */

package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import objets.CElementGraphique;
import objets.CListeObjets;
import types.NatureCarSpe;
import types.NatureObjet;

import mathgraph32.MtgFrame;

/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
@SuppressWarnings("serial")
public class EntreeNomDlg extends MtgDlg {
  CElementGraphique elementAssocie; // L'lment que l'on cherche  renommer
  private MtgButton btnCancel;
  private MtgButton btnOK;
  private TextFieldAvecCarSpeciaux editeurNom;
  private MtgLabel label1;
  private MtgCheckBox checkNomMasque;
  private PaneTaillePolice paneTaillePolice;
  
  /** Creates new form EntreeNomDlg */
  public EntreeNomDlg(MtgFrame frameInit, CElementGraphique elementAssocieInit) {
    super (getString("EntreeNomDlg1"), true, frameInit);
    elementAssocie = elementAssocieInit;
    initComponents();
    getRootPane().setDefaultButton(btnOK);
    editeurNom.setText(elementAssocie.rendNom());
    editeurNom.setSelectionStart(0);
    editeurNom.setSelectionEnd(editeurNom.getText().length());
    editeurNom.requestFocus();
    checkNomMasque.setSelected(elementAssocie.nomMasque);
  }
  
  private void initComponents() {
    label1 = new MtgLabel(frame, getString("entreeNomDlg1"), JLabel.RIGHT);
    label1.setRequestFocusEnabled(false);
    editeurNom = new TextFieldAvecCarSpeciaux(this, frame, NatureCarSpe.Grec);
    editeurNom.setColumns(8);
    btnOK = new MtgButton(frame, getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed(evt);
      }
    });    
    btnCancel = new MtgButton(frame, getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed(evt);
      }
    });
    btnOK.setMaximumSize(btnCancel.getPreferredSize());
    checkNomMasque = new MtgCheckBox(frame, getString("entreeNomDlg2"));
    paneTaillePolice = new PaneTaillePolice(elementAssocie.indiceTailleNom);
    JPanel panePrincipal = new JPanel();
    JPanel paneRight = new JPanel();
    panePrincipal.setLayout(new GridBagLayout());
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 20, 50, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label1, constraints);
    setConstraints(constraints, 1, 0, 1, 1, 40, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(editeurNom, constraints);
    setConstraints(constraints, 2, 0, 1, 2, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(paneTaillePolice, constraints);
    setConstraints(constraints, 0, 1, 2, 1, 0, 50, GridBagConstraints.NONE, GridBagConstraints.NORTHWEST);
    panePrincipal.add(checkNomMasque, constraints);
    setConstraints(constraints, 3, 0, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(paneRight, constraints);
    setContentPane(panePrincipal);
    pack();
  }
  private void btnOKActionPerformed(java.awt.event.ActionEvent evt) {
    CListeObjets liste = frame.cadreActif().listePr;
    String ch = editeurNom.getText();
    // On retire tous les ventuels espaces
    StringBuffer buffer = new StringBuffer(ch);
    int ind;
    while((ind = buffer.indexOf("\\Val{ ", 0)) != - 1) {
    	buffer.deleteCharAt(ind+5);
    }    
    if (elementAssocie.estDeNature(NatureObjet.NToutPoint | NatureObjet.NDroite) && 
            elementAssocie.rendNom().length() != 0) {
      CElementGraphique el = (CElementGraphique) elementAssocie;
      if ((ch.length() == 0) && liste.nomIndispensable(el)) {
        dialogueAvertissementCentreEcran(getString("ch33"));
        editeurNom.setText(elementAssocie.rendNom());
        editeurNom.requestFocus();
        return;
      }
    }
    boolean nomValide = elementAssocie.estDeNature(NatureObjet.NToutPoint) ? liste.validationNomPoint(ch) : liste.validationNomDroite(ch);
    if (nomValide) {
      if (liste.existeMemeNomNonIntermediaire(elementAssocie,ch)) {
        dialogueAvertissementCentreEcran(getString("ch20"));
        editeurNom.requestFocus();
        return;
      }
      else {
    	boolean modifie = (!elementAssocie.rendNom().equals(ch) || elementAssocie.nomMasque!= checkNomMasque.isSelected()
    			|| elementAssocie.indiceTailleNom != paneTaillePolice.indiceTaille());
    	if (modifie) {
	        elementAssocie.donneNom(ch);
	        // On reconstruit les chanes des calculs ou fonctions dpendant de l'objet renomm
	        liste.reconstruitChainesCalculDependantsDe(liste.indexOf(elementAssocie) +1, elementAssocie);
	        elementAssocie.nomMasque = checkNomMasque.isSelected();
	        elementAssocie.indiceTailleNom = paneTaillePolice.indiceTaille();
	        frame.pref_indiceTaillePoliceNom = elementAssocie.indiceTailleNom;
	        frame.outilActif.annuleClignotement();
	        // frame.cadreActif().redessineSansCalcul();
	        frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil67"));
    	}
        dispose();
      }
    }
    else {
      dialogueAvertissementCentreEcran(getString("ch19"));
      return;
    }
  }                                     

  private void btnCancelActionPerformed(java.awt.event.ActionEvent evt) {                                          
    dispose();
  }
  
	public boolean estDePetiteHauteur() {
		return true;
	}
  
}
