/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.Vector;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import mathgraph32.MtgFrame;
import objets.CGrapheSuiteRecComplexe;
import objets.CListeObjets;
import objets.CSuiteRecComplexe;
import types.NatureCalcul;

@SuppressWarnings("serial")
public class GrapheSuiteRecurrenteComplexeDlg extends MtgDlg {
	private CGrapheSuiteRecComplexe grapheAssocie;
	private boolean modification;
	// private ListeSuitesRecurrentesComplexes listeSuites; // Modifi version 4.7.5
	private ListeObjetsParNatureCalcul listeSuites;
	private ListeReperes listeReperes;
	@SuppressWarnings("unused")
	private JScrollPane scrollpaneListeSuites;
	@SuppressWarnings("unused")
	private JScrollPane scrollPaneListeReperes;
	private MtgLabel label1, label2;
  private MtgButton btnOK, btnCancel;
  private MtgCheckBox checkTraitsRappel;
  private MtgFocusTraversalPolicy mtgFocusTraversalPolicy;
	
  public GrapheSuiteRecurrenteComplexeDlg(MtgFrame frameInit, CGrapheSuiteRecComplexe grapheAssocieInit, boolean modificationInit) {
  	super(getString("GrapheSuiteRecurrenteComplexeDlg1"), true, frameInit);
  	grapheAssocie = grapheAssocieInit;
  	modification = modificationInit;
  	initComponents();
  	getRootPane().setDefaultButton(btnOK);      	
  }
  
  private void initComponents() {
    label1 = new MtgLabel(frame, getString("GrapheSuiteRecurrenteReelleDlg3"), JLabel.RIGHT);
    label1.setRequestFocusEnabled(false);
    label2 = new MtgLabel(frame, getString("choixRepere"), JLabel.RIGHT);
    label2.setRequestFocusEnabled(false);
    CListeObjets liste = frame.cadreActif().listePr;
    int indmax;
    if (modification) {
    	indmax = liste.indexOf(grapheAssocie) - 1;
    	// Modifi version 4.7.5
    	listeSuites = new ListeObjetsParNatureCalcul(frame, liste, indmax, NatureCalcul.NSuiteRecurrenteComplexe
    			| NatureCalcul.NSuiteRecurrenteComplexe2 | NatureCalcul.NSuiteRecurrenteComplexe3, false, 10);
    	listeSuites.setSel(grapheAssocie.suiteAssociee);
    	listeReperes = new ListeReperes(frame, liste, indmax);
    	listeReperes.setSel(grapheAssocie.repereAssocie);
    }
    else {
    	indmax = liste.size() - 1;
    	// Modifi version 4.7.5
    	listeSuites = new ListeObjetsParNatureCalcul(frame, liste, indmax,NatureCalcul.NSuiteRecurrenteComplexe
    			| NatureCalcul.NSuiteRecurrenteComplexe2 | NatureCalcul.NSuiteRecurrenteComplexe3, false, 10);
    	listeReperes = new ListeReperes(frame, liste, indmax);
    }
    JScrollPane scrollPaneSuites = new JScrollPane(listeSuites, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    JScrollPane scrollPaneReperes = new JScrollPane(listeReperes, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    checkTraitsRappel = new MtgCheckBox(frame, getString("GrapheSuiteRecurrenteComplexeDlg4"));
    if (modification) checkTraitsRappel.setSelected(grapheAssocie.pointsRelies);
    else checkTraitsRappel.setSelected(true);
    
    btnOK = new MtgButton(frame, getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed();
      }
    });    
    btnCancel = new MtgButton(frame, getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed();
      }
    });
    btnOK.setPreferredSize(btnCancel.getPreferredSize());    
    
    JPanel panePrincipal = new JPanel();
    JPanel paneRight = new JPanel();
    panePrincipal.setLayout(new GridBagLayout());
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 40, 40, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    panePrincipal.add(label1, constraints);
    setConstraints(constraints, 1, 0, 1, 1, 40, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST);
    panePrincipal.add(scrollPaneSuites, constraints);
    setConstraints(constraints, 0, 1, 1, 1, 0, 40, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    panePrincipal.add(label2, constraints);
    setConstraints(constraints, 1, 1, 1, 1, 40, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST);
    panePrincipal.add(scrollPaneReperes, constraints);
    setConstraints(constraints, 0, 2, 2, 1, 40, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    panePrincipal.add(checkTraitsRappel, constraints);
    
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
    setConstraints(constraints, 2, 0, 1, 2, 20, 0, GridBagConstraints.NONE, GridBagConstraints.NORTH);
    panePrincipal.add(paneRight, constraints);
    
    setContentPane(panePrincipal);
    // On change l'ordre par dfaut des composants
    Vector<Component> order = new Vector<Component>(5);
    order.add(scrollPaneSuites);
    order.add(scrollPaneReperes);
    order.add(checkTraitsRappel);
    order.add(btnOK);
    order.add(btnCancel);
    mtgFocusTraversalPolicy = new MtgFocusTraversalPolicy(order);
    setFocusTraversalPolicy(mtgFocusTraversalPolicy);
    pack();  
  }

  private void btnCancelActionPerformed() {                                          
    dispose();
  }       

  private void btnOKActionPerformed() {
  	grapheAssocie.associeARepere(listeReperes.repereChoisi());
  	// grapheAssocie.suiteAssociee = listeSuites.suiteSelectionnee(); // Modifi version 4.7.5
  	grapheAssocie.suiteAssociee = (CSuiteRecComplexe)listeSuites.elementSelectionne();
  	grapheAssocie.pointsRelies = checkTraitsRappel.isSelected();
  	if (grapheAssocie.pointsRelies) grapheAssocie.donneStyle(frame.styleTraitActif()); // Ajout version 4.7.5.2
  	setRetourOK(true);
  	dispose();
  }
}
