/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Toolkit;
import java.util.ListIterator;
import java.util.Vector;

import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.ListSelectionModel;

import mathgraph32.MtgFrame;
import objets.CElementBase;
import objets.CListeObjets;
import objets.CMacro;
import objets.CMacroSuiteMacros;
import objets.CSousListeObjets;
import types.FonteNom;
import types.NatureObjet;

@SuppressWarnings("serial")
public class ListeMacros extends JList<String> {
  private CListeObjets liste; // La liste contenant les valeurs  afficher
  private int indiceMaxi; // l'indice maxi permis pour un objet  insrer dans la liste
  public Vector<CMacro> pointeurs;
  private DefaultListModel<String> listModel;

  /**
   * Constructeur pour une liste vide
   * @param listeInit
   */
  public ListeMacros(MtgFrame frame, CListeObjets listeInit) {
    super();
		setFont(FonteNom.getFonteEdition(MtgFrame.sizeFontList, FonteNom.Normal,
				(double)frame.pref_addSystemFont/MtgFrame.sizeFontList + MtgFrame.coefMultForSystemFonts));		
    listModel = new DefaultListModel<String>();
    setModel(listModel);
    pointeurs = new Vector<CMacro>();
    setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    liste = listeInit;
    setVisibleRowCount(5);
    setSelectedIndex(0);  	
  }
  
  // Modifi version 5.0.2
  public ListeMacros(MtgFrame frame, CListeObjets listeInit, int indiceMaxiInit) {
    super();
		setFont(FonteNom.getFonteEdition(MtgFrame.sizeFontList, FonteNom.Normal,
				(double)frame.pref_addSystemFont/MtgFrame.sizeFontList + MtgFrame.coefMultForSystemFonts));		
    listModel = new DefaultListModel<String>();
    setModel(listModel);
    pointeurs = new Vector<CMacro>();
    setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    liste = listeInit;
    indiceMaxi = indiceMaxiInit;
    int i = 0;
    
    ListIterator<CElementBase> li = liste.listIterator();
    while (i <= indiceMaxi)
    {
      CElementBase elb = li.next();
      if (!elb.estElementIntermediaire() && (elb.getNature() == NatureObjet.NMacro)) {
        listModel.addElement(((CMacro)elb).intitule);
        pointeurs.add((CMacro) elb);
      }
      i++;
    }
    setVisibleRowCount(5);
    setSelectedIndex(0);
  }
  
  public CMacro macroChoisie() {
  	return (CMacro) pointeurs.get(getSelectedIndex());
  }
  
  /**
   * Fonction ajoutant les macros de macro qui est une CMacroSuiteMacro
   * @param macro
   */
  public void ajouteMacrosDe(CMacroSuiteMacros macro) {
  	ListIterator<CElementBase> li = macro.listeAssociee.listIterator();
  	while (li.hasNext()) {
  		CMacro mac = (CMacro) li.next();
      listModel.addElement(mac.intitule);
      pointeurs.add(mac);
  	}
  }
  
  public void insereMacroDebut(CMacro macro) {
  	listModel.add(0,macro.intitule);
  	pointeurs.insertElementAt(macro, 0);
  	setSelectedIndex(0);
  }
  
  public void insereMacroFin(CMacro macro) {
  	listModel.addElement(macro.intitule);
  	pointeurs.add(macro);
  	setSelectedIndex(listModel.size() - 1);
  }
  
  /**
   * Insre la macro macro au point de slection de la liste.
   * @param macro
   */
  public void insereMacro(CMacro macro) {
  	int ind = getSelectedIndex();
  	if (ind == -1) Toolkit.getDefaultToolkit().beep();
  	else {
	  	listModel.add(ind, macro.intitule);
	  	pointeurs.add(ind, macro);
  	}
  }
  
  public CSousListeObjets creeSousListe() {
  	CSousListeObjets sousListe = new CSousListeObjets();
  	for (int i = 0; i < pointeurs.size(); i++) {
  		sousListe.add(pointeurs.elementAt(i));
  	}
  	return sousListe;
  }
  
  public void retireMacroSelectionnee() {
  	int ind = getSelectedIndex();
  	if (ind != -1) {
  		listModel.removeElementAt(ind);
  		pointeurs.removeElementAt(ind);
  	}
  	if (ind == listModel.size()) ind --;
  	if (ind != -1) setSelectedIndex(ind);
  }
  
  public void retireTout() {
  	listModel.removeAllElements();
  	pointeurs.removeAllElements();
  }
  
  public void decaleBasMacroSelectionnee() {
  	int ind =  getSelectedIndex();
  	if (ind >= 1) {
  		CMacro mac = pointeurs.elementAt(ind);
  		String ch = (String) listModel.elementAt(ind);
  		pointeurs.remove(ind);
  		listModel.removeElementAt(ind);
  		pointeurs.add(ind - 1, mac);
  		listModel.add(ind - 1, ch);
  		setSelectedIndex(ind - 1);
  	}
  }
  
  public void decaleHautMacroSelectionnee() {
  	int ind = getSelectedIndex();
  	if ((ind != -1) && (ind < listModel.size() - 1)) {
  		CMacro mac = pointeurs.elementAt(ind);
  		String ch = (String) listModel.elementAt(ind);
  		pointeurs.remove(ind);
  		listModel.removeElementAt(ind);
  		pointeurs.add(ind + 1, mac);
  		listModel.add(ind + 1, ch);
  		setSelectedIndex(ind + 1);
  	}
  }
  
  public void metAJourIndiceMaxi(int indMax) {
  	indiceMaxi = indMax;
  }
  
  public void metAJourListe() {
  	listModel.removeAllElements();
  	pointeurs.removeAllElements();
    int i = 0;
    ListIterator<CElementBase> li = liste.listIterator();
    while (i <= indiceMaxi)
    {
      CElementBase elb = li.next();
      if (!elb.estElementIntermediaire() && (elb.getNature() == NatureObjet.NMacro)) {
        listModel.addElement(((CMacro)elb).intitule);
        pointeurs.add((CMacro) elb);
      }
      i++;
    }  	
    setSelectedIndex(0);
  }
  
}