/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.NumberFormat;
import java.util.Vector;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import mathgraph32.MtgFrame;
import objets.CListeObjets;
import objets.CMacroAnimationPointLie;
import types.NatureCarSpe;

@SuppressWarnings("serial")
public class MacroAnimationDlg extends MtgDlg {
	private boolean modification;
	private CMacroAnimationPointLie macro;
	private MtgLabel label1, label2, label3;
	private TextFieldAvecCarSpeciaux	editeurIntitule;
	private MtgTextArea editeurCommentaire;
	private PaneTransition paneTransition;
	private PaneFrequenceAnimation paneFrequenceAnimation;
	private MtgRadioButton radioArretParClic;
	private MtgRadioButton radioDureeFixee;
	private MtgFormattedTextField editeurDuree;
	private MtgCheckBox cbInverserSens;
	private MtgCheckBox cbUnCycle;
	private MtgCheckBox cbRetour;
	private PaneTaillePolice paneTaillePolice;
	private PaneModeAffichagePourMacro paneModeAffichage;
	private MtgButton btnOK, btnCancel;
	private JPanel paneDuree;
	private MtgFocusTraversalPolicy mtgFocusTraversalPolicy;
	
  public MacroAnimationDlg(MtgFrame frameInit, CMacroAnimationPointLie macroInit, boolean modificationInit) {
    super (getString("macroAnimationDlg1"), true, frameInit);
    macro = macroInit;
    modification = modificationInit;
    initComponents();
    getRootPane().setDefaultButton(btnOK);    
  }
  
  private void initComponents() {
		label1 = new MtgLabel(frame, getString("macroApparitionDlg2"));
		label2 = new MtgLabel(frame, getString("macroApparitionDlg3"));
  	label3 = new MtgLabel(frame, getString("macroAnimationDlg4"));
		editeurIntitule = new TextFieldAvecCarSpeciaux(this, frame, NatureCarSpe.Tout);
		editeurIntitule.setColumns(30); // Ajout version 4.9.2 pour viter que la fentre dgnre quand on agit sur le PaneModeAffichagePourMacro
		editeurCommentaire = new MtgTextArea(frame);
		editeurCommentaire.setColumns(20);
		editeurCommentaire.setLineWrap(false);
		editeurCommentaire.setRows(4);
    JScrollPane scrollPaneCommentaire = new JScrollPane(editeurCommentaire,
    		JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    // Modification version 5.0 pour tenri compte des crans haute dfinition
    double k = MtgFrame.coefMultForDlgBoxIcons;
    scrollPaneCommentaire.setPreferredSize(new Dimension((int)(k*300), (int)(k*90)));
    paneTransition = new PaneTransition(frame, macro.animationCyclique, macro.nombrePointsPourAnimation);
    paneFrequenceAnimation = new PaneFrequenceAnimation(frame, macro.frequenceAnimationMacro,false); // false acr pas de frquence max permise
    radioArretParClic = new MtgRadioButton(frame, getString("macroAnimationDlg2"));
    radioArretParClic.setSelected(macro.arretParClic());
    radioArretParClic.addActionListener(new ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        radioArretParClicActionPerformed();
      }
 		});
    radioDureeFixee = new MtgRadioButton(frame, getString("macroAnimationDlg3")+".  ");
    radioDureeFixee.setSelected(!macro.arretParClic());
    radioDureeFixee.addActionListener(new ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        radioDureeFixeeActionPerformed();
      }
 		});
    ButtonGroup btnGroup = new ButtonGroup();
    btnGroup.add(radioArretParClic);
    btnGroup.add(radioDureeFixee);
    cbInverserSens = new MtgCheckBox(frame, getString("macroAnimationDlg5")+"  ");
    cbInverserSens.setSelected(macro.inverserSens);
    cbUnCycle = new MtgCheckBox(frame, getString("macroAnimationAvecTraceDlg5")+"  ");
    cbUnCycle.setSelected(macro.unCycle);
    cbUnCycle.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent evt) {
      	paneDuree.setVisible(!cbUnCycle.isSelected());
      	radioArretParClic.setVisible(!cbUnCycle.isSelected());
      	pack();
      }
    });
    cbRetour = new MtgCheckBox(frame, getString("macroAnimationDlg7"));
    cbRetour.setSelected(macro.retourDepart);
    NumberFormat format = NumberFormat.getInstance();
    format.setParseIntegerOnly(true);
    format.setGroupingUsed(false);
    editeurDuree = new MtgFormattedTextField(frame, format);
    editeurDuree.setColumns(5);
		if (modification) {
			if (macro.arretParClic()){
				editeurDuree.setText("50");
				editeurDuree.setEditable(false);
			}
			else editeurDuree.setText(Integer.toString(macro.dureeAnimation));
			editeurIntitule.setText(macro.intitule);
			editeurCommentaire.setText(macro.commentaireMacro);
			editeurCommentaire.setCaretPosition(0);
		}
		else {
			editeurDuree.setText("50");
			radioArretParClic.setSelected(true);
			editeurDuree.setEditable(false);
		}
	  btnOK = new MtgButton(frame, getString("btnOK"));
	  btnOK.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnOKActionPerformed();
	    }
	  });    
	  btnCancel = new MtgButton(frame, getString("btnCancel"));
	  btnCancel.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnCancelActionPerformed();
	    }
	  });
	  btnOK.setMaximumSize(btnCancel.getPreferredSize());	
	  paneTaillePolice = new PaneTaillePolice(macro.indiceTaillePolice);
	  paneModeAffichage = new PaneModeAffichagePourMacro(frame, macro);

    JPanel panePrincipal = new JPanel();
    JPanel paneRight = new JPanel();
    JPanel paneMilieu = new JPanel();
    JPanel paneCentre = new JPanel();
    paneCentre.setLayout(new BoxLayout(paneCentre, BoxLayout.X_AXIS));
    paneDuree = new JPanel();
    paneDuree.setLayout(new BoxLayout(paneDuree, BoxLayout.X_AXIS));
    paneDuree.setVisible(!cbUnCycle.isSelected());
    radioArretParClic.setVisible(!cbUnCycle.isSelected());;
    JPanel paneBas = new JPanel();
    panePrincipal.setLayout(new GridBagLayout());
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets.bottom = 0;
		constraints.insets.top = 2;
		constraints.insets.left = 5;
		constraints.insets.right = 5;

		setConstraints(constraints, 0, 0, 1, 1, 20, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label1, constraints);
		setConstraints(constraints, 1, 0, 1, 1, 60, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		panePrincipal.add(editeurIntitule, constraints);
		setConstraints(constraints, 0, 1, 3, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label2, constraints);
		setConstraints(constraints, 0, 2, 2, 1, 0, 30, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(scrollPaneCommentaire, constraints);
		paneMilieu.add(paneTransition);
		paneMilieu.add(paneFrequenceAnimation);
		setConstraints(constraints, 0, 3, 3, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(paneMilieu, constraints);
		paneCentre.add(cbUnCycle);
		paneCentre.add(cbInverserSens);		
		paneCentre.add(cbRetour);
		setConstraints(constraints, 0, 4, 3, 1, 0, 100, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(paneCentre, constraints);	
		setConstraints(constraints, 0, 5, 3, 1, 0, 10, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(radioArretParClic, constraints);
		// setConstraints(constraints, 0, 5, 1, 1, 0, 10, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		// panePrincipal.add(radioDureeFixee, constraints);
		paneDuree.add(radioDureeFixee);
		paneDuree.add(label3);
		paneDuree.add(editeurDuree);
		setConstraints(constraints, 0, 6, 3, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(paneDuree, constraints);
		paneBas.add(paneTaillePolice);
		paneBas.add(paneModeAffichage);
		setConstraints(constraints, 0, 7, 2, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(paneBas, constraints);
 		setConstraints(constraints, 3, 0, 1, 2, 20, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(paneRight, constraints);
		
		setContentPane(panePrincipal);
		// On change l'ordre par dfaut des composants
		Vector<Component> order = new Vector<Component>(14);
		order.add(editeurIntitule);
		order.add(editeurCommentaire);
		order.add(paneTransition.editeurNombrePoints);
		order.add(paneTransition.checkAnimationCyclique);
		order.add(paneFrequenceAnimation.listeFreq);
		// Ajout version 4.9.8
		order.add(cbUnCycle);
		order.add(cbInverserSens);
		order.add(cbRetour);
		//
		order.add(radioArretParClic);
		order.add(radioDureeFixee);
		order.add(paneTaillePolice.listeTailles);
		order.add(paneModeAffichage.effacementFond);
		order.add(btnOK);
		order.add(btnCancel);
		mtgFocusTraversalPolicy = new MtgFocusTraversalPolicy(order);
		setFocusTraversalPolicy(mtgFocusTraversalPolicy);
		pack();	
  }
  
  private void radioArretParClicActionPerformed() {
  	editeurDuree.setEditable(false);
  }

  private void radioDureeFixeeActionPerformed() {
  	editeurDuree.setEditable(true);
  }
  
  private void btnOKActionPerformed() {
  	String chintitule = editeurIntitule.getText();
  	if (chintitule.length() == 0) {
  		dialogueAvertissement(getString("ch115"));
  		editeurIntitule.requestFocus();
  		return;
  	}
  	CListeObjets liste = frame.cadreActif().listePr;
  	if (!liste.validationIntituleMacro(chintitule, macro)) {
  		dialogueAvertissement(getString("ch116"));
  		editeurIntitule.requestFocus();
  		return;
  	}
  	if (!paneTransition.validationNombrePoints()) return; // validationNombrePoints() se charge ddu message d'erreur
  	if (radioArretParClic.isSelected()) macro.dureeAnimation = 0;
  	else {
  		int duree;
  		try {
  			duree = Integer.parseInt(editeurDuree.getText());
  		}
  		catch (NumberFormatException e) {
  			dialogueAvertissement(getString("ch71"));
  			editeurDuree.requestFocus();
  			return;
  		}
  		if ((duree <= 0) || (duree>3000)) {
  			dialogueAvertissement(getString("ch71"));
  			editeurDuree.requestFocus();
  			return;
  		}
  		else macro.dureeAnimation = duree;
  	}
  	macro.intitule = chintitule;
  	macro.commentaireMacro = editeurCommentaire.getText();
  	macro.animationCyclique = paneTransition.animationCyclique();
  	macro.nombrePointsPourAnimation = paneTransition.nombrePoints;
  	macro.frequenceAnimationMacro = paneFrequenceAnimation.frequenceChoisie();
  	macro.indiceTaillePolice = paneTaillePolice.indiceTaille();
  	macro.effacementFond = paneModeAffichage.effacementFond();
  	macro.couleurFondAffichage = paneModeAffichage.couleurFond();
		// Ajouts version 4.5
		macro.alignementHorizontal = paneModeAffichage.alignementHorizontal();
		macro.alignementVertical = paneModeAffichage.alignementVertical();
		//
		// Ajouts version 4.9.8
		macro.unCycle = cbUnCycle.isSelected();
		macro.retourDepart = cbRetour.isSelected();
		macro.inverserSens = cbInverserSens.isSelected();
		if (macro.unCycle) macro.dureeAnimation = 0;
		//
		frame.pref_IndiceTaillePoliceAffichages = macro.indiceTaillePolice;
		frame.cadreActif().couleurFondAffichage = macro.couleurFondAffichage;
  	setRetourOK(true);
  	dispose();
  }
  
  private void btnCancelActionPerformed() {
  	dispose();
  }  
}
