/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.Vector;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import mathgraph32.MtgFrame;
import objets.CListeObjets;
import objets.CMacroBoucleAvecAnimation;
import types.NatureCarSpe;

@SuppressWarnings("serial")
public class MacroBoucleAvecAnimationDlg extends MtgDlg {
	private boolean modification;
	private CMacroBoucleAvecAnimation macro;
	private MtgLabel label1, label2, label3, label4;
	private TextFieldAvecCarSpeciaux	editeurIntitule;
	private MtgTextArea editeurCommentaire;
	private PaneFrequenceAnimation paneFrequenceAnimation;
	private ListeVariables listeVariables;
	private MtgTextArea info; // Info sur la variable slectionne
	private JScrollPane scrollPaneInfo;
	private JScrollPane scrollPaneListeVariables;
	private MtgCheckBox checkAnimationVisible;
	private ListeMacrosPourBoucle listeMacros1;
	private JScrollPane scrollPaneListeMacros1;
	private ListeMacrosPourBoucle listeMacros2;
	private JScrollPane scrollPaneListeMacros2;
	private PaneTaillePolice paneTaillePolice;
	private PaneModeAffichagePourMacro paneModeAffichage;
	private MtgButton btnOK, btnCancel;
	private MtgFocusTraversalPolicy mtgFocusTraversalPolicy;

  public MacroBoucleAvecAnimationDlg(MtgFrame frameInit, CMacroBoucleAvecAnimation macroInit, boolean modificationInit) {
    super (getString("macroBoucleAvecAnimationDlg1"), true, frameInit);
    macro = macroInit;
    modification = modificationInit;
    initComponents();
    getRootPane().setDefaultButton(btnOK);    
    listeVariablesSelectionChanged();    
  }
	
  private void initComponents() {
    int indmax;
    
		label1 = new MtgLabel(frame, getString("macroApparitionDlg2"));
		label2 = new MtgLabel(frame, getString("macroApparitionDlg3"));
		label3 = new MtgLabel(frame, getString("macroIncrementationVariableDlg2"));
		label4 = new MtgLabel(frame, getString("macroIncrementationVariableDlg3"));
		editeurIntitule = new TextFieldAvecCarSpeciaux(this, frame, NatureCarSpe.Tout);
		editeurIntitule.setColumns(30); // Ajout version 4.9.2 pour viter que la fentre dgnre quand on agit sur le PaneModeAffichagePourMacro
		editeurCommentaire = new MtgTextArea(frame);
		editeurCommentaire.setColumns(20);
		editeurCommentaire.setLineWrap(false);
		editeurCommentaire.setRows(4);
    JScrollPane scrollPaneCommentaire = new JScrollPane(editeurCommentaire,
    		JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    // Modification version 5.0 pour tenir compte des crans haute dfinition
    double k = MtgFrame.coefMultForDlgBoxIcons;
    scrollPaneCommentaire.setPreferredSize(new Dimension((int)(k*300), (int)(k*90)));
    checkAnimationVisible=new MtgCheckBox(frame, getString("animationVisible"));
    checkAnimationVisible.setSelected(macro.animationVisible);
    paneFrequenceAnimation = new PaneFrequenceAnimation(frame, macro.frequenceAnimation, true);
    CListeObjets liste = frame.cadreActif().listePr;
    listeVariables = new ListeVariables(frame, liste, liste.size()-1, false);
    listeVariables.setVisibleRowCount(5);
    listeVariables.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent evt) {
      	listeVariablesSelectionChanged();
      }
    });
    scrollPaneListeVariables = new JScrollPane(listeVariables, 
    		JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    info = new MtgTextArea(frame);
    info.setEditable(false);
    info.setColumns(20);
    info.setLineWrap(false);
    info.setRows(4);
    scrollPaneInfo = new JScrollPane(info, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    scrollPaneInfo.setPreferredSize(new Dimension((int)(k*300), (int)(k*90)));
    if (modification)  {
    	indmax = liste.indexOf(macro);
    	editeurIntitule.setText(macro.intitule);
    	editeurCommentaire.setText(macro.commentaireMacro);
			editeurCommentaire.setCaretPosition(0);    	
    	listeVariables.setSel(macro.variableAssociee);
    	listeVariables.ensureIndexIsVisible(listeVariables.getSelectedIndex());
    }
    else indmax = liste.size() - 1;
    listeMacros1 = new ListeMacrosPourBoucle(frame, liste, indmax);
    listeMacros1.setSel(macro.macroAvantBoucles);
    scrollPaneListeMacros1 = new JScrollPane(listeMacros1, 
    		JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    scrollPaneListeMacros1.setBorder(BorderFactory.createTitledBorder(getString("macroBoucleAvecAnimationDlg2")));
    scrollPaneListeMacros1.setPreferredSize(new Dimension((int)(k*250),(int)(k*100)));
    listeMacros1.ensureIndexIsVisible(listeMacros1.getSelectedIndex());
    listeMacros2 = new ListeMacrosPourBoucle(frame, liste, indmax);
    listeMacros2.setSel(macro.macroFinBoucle);
    scrollPaneListeMacros2 = new JScrollPane(listeMacros2, 
    		JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    scrollPaneListeMacros2.setBorder(BorderFactory.createTitledBorder(getString("macroBoucleAvecAnimationDlg3")));
    scrollPaneListeMacros2.setPreferredSize(new Dimension((int)(k*250),(int)(k*100)));
    listeMacros2.ensureIndexIsVisible(listeMacros2.getSelectedIndex());
    
		if (modification) {
			editeurIntitule.setText(macro.intitule);
			editeurCommentaire.setText(macro.commentaireMacro);
		}
	  btnOK = new MtgButton(frame, getString("btnOK"));
	  btnOK.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnOKActionPerformed();
	    }
	  });    
	  btnCancel = new MtgButton(frame, getString("btnCancel"));
	  btnCancel.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnCancelActionPerformed();
	    }
	  });
	  btnOK.setMaximumSize(btnCancel.getPreferredSize());	
	  paneTaillePolice = new PaneTaillePolice(macro.indiceTaillePolice);
	  paneModeAffichage = new PaneModeAffichagePourMacro(frame, macro);
	  
    JPanel panePrincipal = new JPanel();
    JPanel paneRight = new JPanel();
    JPanel paneMilieu = new JPanel();  // Pour la liste des variables et l'info sur la variable slectionne
    JPanel paneListeMacros = new JPanel();
    JPanel paneBas = new JPanel();
    panePrincipal.setLayout(new GridBagLayout());
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets.bottom = 0;
		constraints.insets.top = 2;
		constraints.insets.left = 5;
		constraints.insets.right = 5;
		
		// paneMilieu est le JPanel qui contient la liste des variables
		// avec  sa droite un JTextArea d'info sur la variable slectionne
		paneMilieu.setLayout(new GridBagLayout());
		setConstraints(constraints, 0, 0, 1, 1, 30, 20, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		paneMilieu.add(label4, constraints);
		setConstraints(constraints, 0, 1, 1, 1, 0, 50, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		paneMilieu.add(scrollPaneListeVariables, constraints);
		setConstraints(constraints, 1, 0, 1, 1, 40, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneMilieu.add(label3, constraints);
		setConstraints(constraints, 1, 1, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneMilieu.add(scrollPaneInfo, constraints);
		setConstraints(constraints, 2, 0, 1, 2, 30, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneMilieu.add(paneFrequenceAnimation, constraints);
		
		//paneListeMacros est le JPanel qui contient les deux listes de macros
		paneListeMacros.setLayout(new GridBagLayout());
		setConstraints(constraints, 0, 0, 1, 1, 50, 50, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		paneListeMacros.add(scrollPaneListeMacros1, constraints);
		setConstraints(constraints, 1, 0, 1, 1, 50, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.EAST);
		paneListeMacros.add(scrollPaneListeMacros2, constraints);
		
		paneBas.add(paneTaillePolice);
		paneBas.add(paneModeAffichage);		

		setConstraints(constraints, 0, 0, 1, 1, 20, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label1, constraints);
		setConstraints(constraints, 1, 0, 1, 1, 60, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		panePrincipal.add(editeurIntitule, constraints);
		setConstraints(constraints, 0, 1, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label2, constraints);
		setConstraints(constraints, 0, 2, 2, 1, 0, 30, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(scrollPaneCommentaire, constraints);
		setConstraints(constraints, 0, 3, 2, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(checkAnimationVisible, constraints);
		setConstraints(constraints, 0, 4, 2, 1, 0, 30, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(paneMilieu, constraints);
		setConstraints(constraints, 0, 5, 2, 1, 0, 30, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(paneListeMacros, constraints);
		setConstraints(constraints, 0, 6, 2, 1, 0, 30, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(paneBas, constraints);
 		setConstraints(constraints, 2, 0, 1, 2, 20, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(paneRight, constraints);

		setContentPane(panePrincipal);
		// On change l'ordre par dfaut des composants
		Vector<Component> order = new Vector<Component>(10);
		order.add(editeurIntitule);
		order.add(editeurCommentaire);
		order.add(checkAnimationVisible);
		order.add(listeVariables);
		order.add(listeMacros1);
		order.add(listeMacros2);
		order.add(paneTaillePolice.listeTailles);
		order.add(paneModeAffichage.effacementFond);
		order.add(btnOK);
		order.add(btnCancel);
		mtgFocusTraversalPolicy = new MtgFocusTraversalPolicy(order);
		setFocusTraversalPolicy(mtgFocusTraversalPolicy);
		pack();				
  }
  
  public void btnOKActionPerformed() {
  	String chintitule = editeurIntitule.getText();
  	if (chintitule.length() == 0) {
  		dialogueAvertissement(getString("ch115"));
  		editeurIntitule.requestFocus();
  		return;
  	}
  	CListeObjets liste = frame.cadreActif().listePr;
  	if (!liste.validationIntituleMacro(chintitule, macro)) {
  		dialogueAvertissement(getString("ch116"));
  		editeurIntitule.requestFocus();
  		return;
  	}
  	macro.intitule = chintitule;
  	macro.commentaireMacro = editeurCommentaire.getText();
  	macro.animationVisible = checkAnimationVisible.isSelected();
  	macro.frequenceAnimation = paneFrequenceAnimation.frequenceChoisie();
  	macro.variableAssociee = listeVariables.variableSelectionnee();
  	macro.macroAvantBoucles = listeMacros1.macroSelectionnee();
  	macro.macroFinBoucle = listeMacros2.macroSelectionnee();
  	macro.indiceTaillePolice = paneTaillePolice.indiceTaille();
  	macro.effacementFond = paneModeAffichage.effacementFond();
  	macro.couleurFondAffichage = paneModeAffichage.couleurFond();
		// Ajouts version 4.5
		macro.alignementHorizontal = paneModeAffichage.alignementHorizontal();
		macro.alignementVertical = paneModeAffichage.alignementVertical();
		//
		frame.pref_IndiceTaillePoliceAffichages = macro.indiceTaillePolice;
		frame.cadreActif().couleurFondAffichage = macro.couleurFondAffichage;
  	setRetourOK(true);
  	dispose();  	
  }
  
  public void btnCancelActionPerformed() {
  	dispose();
  }

  public void listeVariablesSelectionChanged() {
  	info.setText(listeVariables.variableSelectionnee().infoPourHistorique());
  	pack();
  }  
}
