/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
// Ajout version 4.8
package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionListener;
import java.text.NumberFormat;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JFormattedTextField;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import outils.Outil;
import mathgraph32.MtgColorButton;
import mathgraph32.MtgFrame;
// import mathgraph32.MtgListeStyleTrait; Supprim version 5.0
import objets.CListeObjets;
import objets.CMacroConstructionIterative;
import objets.CPrototype;
import types.NatureCarSpe;

@SuppressWarnings("serial")
public class MacroConstructionIterativeDlg extends MtgDlg {
	private CMacroConstructionIterative macro;
	private boolean modification;
	private MtgLabel label1, label2, label3, label4, label6;
	private TextFieldAvecCarSpeciaux	editeurIntitule;
	private MtgTextArea editeurCommentaire;
	private ListePrototypesPourRecurOuIter listePrototypes;
	private JScrollPane scrollPaneListePrototypes;
	private PaneTaillePolice paneTaillePolice;
	private PaneModeAffichagePourMacro paneModeAffichage;
	private MtgButton btnOK, btnCancel;
  private javax.swing.JFormattedTextField editeurNombreIterations, editeurNbSources;
	private MtgCheckBox checkImposerCouleur, checkPolygoneRempli;
	private MtgColorButton colorButtonImp;
  private JRadioButton radioSansLigne, radioAvecPolygone, radioAvecLigneBrisee;
  // private MtgListeStyleTrait listeStyleTrait; // Supprim version 5.0
  private PaneStyleTrait paneStyleTrait;
	private MtgColorButton colorButtonLigne;

  public MacroConstructionIterativeDlg(MtgFrame frameInit, CMacroConstructionIterative macroInit, boolean modificationInit) {
    super (getString("macroConstructionIterativeDlg1"), true, frameInit);
    macro = macroInit;
    modification = modificationInit;
    initComponents();
    getRootPane().setDefaultButton(btnOK);    
  }

  private void initComponents() {
		label1 = new MtgLabel(frame, getString("macroApparitionDlg2"));
		label2 = new MtgLabel(frame, getString("macroApparitionDlg3"));
  	label3 = new MtgLabel(frame, getString("macroConstructionIterativeDlg2"));  	
  	label4 = new MtgLabel(frame, getString("iterationConstructionDlg2"));  	
  	label6 = new MtgLabel(frame, getString("iterationConstructionDlg4"));
  	
		editeurIntitule = new TextFieldAvecCarSpeciaux(this, frame, NatureCarSpe.Tout);
		editeurIntitule.setColumns(30);
		editeurCommentaire = new MtgTextArea(frame);
		editeurCommentaire.setColumns(20);
		editeurCommentaire.setLineWrap(false);
		editeurCommentaire.setRows(4);
    JScrollPane scrollPaneCommentaire = new JScrollPane(editeurCommentaire,
    		JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    // Modification version 5.0 pour tenir compte des crans haute dfinition
    double k = MtgFrame.coefMultForDlgBoxIcons;
    scrollPaneCommentaire.setPreferredSize(new Dimension((int)(k*300), (int)(k*90)));
    
	  btnOK = new MtgButton(frame, getString("btnOK"));
	  btnOK.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnOKActionPerformed();
	    }
	  });    
	  btnCancel = new MtgButton(frame, getString("btnCancel"));
	  btnCancel.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnCancelActionPerformed();
	    }
	  });
	  btnOK.setMaximumSize(btnCancel.getPreferredSize());	
	  paneTaillePolice = new PaneTaillePolice(macro.indiceTaillePolice);
	  paneModeAffichage = new PaneModeAffichagePourMacro(frame, macro);
	  listePrototypes = new ListePrototypesPourRecurOuIter(frame, frame.cadreActif().document);
	  if (modification) {
	  	int ind = frame.cadreActif().document.tablePrototypes.indexOf(macro.proto);
	  	listePrototypes.setSelectedIndex(ind);
	  	listePrototypes.ensureIndexIsVisible(ind);
	  }
    listePrototypes.setVisibleRowCount(3);
	  scrollPaneListePrototypes = new JScrollPane(listePrototypes,
    		JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		checkImposerCouleur = new MtgCheckBox(frame, getString("macroConstructionIterativeDlg3"));  	
		checkImposerCouleur.setSelected(modification ? macro.imposerCouleur : false);
		checkImposerCouleur.addActionListener(new ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
      	checkImposerCouleurActionPerformed();
      }
 		});
		colorButtonImp = new MtgColorButton(frame);
		colorButtonImp.couleur = modification ?(macro.imposerCouleur ? macro.couleurImp : frame.couleurActive()) : frame.couleurActive();
		if (!checkImposerCouleur.isSelected()) {
			// label4.setVisible(false);
			colorButtonImp.setVisible(false);
		}
    NumberFormat format;
    format = NumberFormat.getInstance();
    format.setParseIntegerOnly(true);
    format.setGroupingUsed(false);
    editeurNombreIterations = new JFormattedTextField(format);
    editeurNombreIterations.setColumns(4);
    editeurNbSources = new JFormattedTextField(format);
    editeurNbSources.setEditable(!modification); // Ajout version 5.1.1 car on pourrait choisir une autre macro d'une autre type
    editeurNbSources.setColumns(2);
    editeurNbSources.setText("0");
	  radioSansLigne = new JRadioButton(getString("recursionConstructionDlg5"));
	  radioSansLigne.addActionListener(new ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
      	radioSansLigneActionPerformed();
      }
 		});
	  radioAvecPolygone = new JRadioButton(getString("iterationConstructionDlg6"));
	  radioAvecPolygone.addActionListener(new ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
      	radioSansLigneActionPerformed();
      }
 		});
	  radioAvecLigneBrisee = new JRadioButton(getString("iterationConstructionDlg7"));
	  radioAvecLigneBrisee.addActionListener(new ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
      	radioSansLigneActionPerformed();
      }
 		});
	  radioSansLigne.setSelected(true);
	  ButtonGroup group = new ButtonGroup();
	  group.add(radioSansLigne);
	  group.add(radioAvecPolygone);
	  group.add(radioAvecLigneBrisee);
		checkPolygoneRempli = new MtgCheckBox(frame, getString("recursionConstructionDlg10"));
		checkPolygoneRempli.setSelected(modification ? macro.polygoneRempli : false);
		colorButtonLigne= new MtgColorButton(frame);
		colorButtonLigne.couleur = modification ? (macro.choixLigne == 0 ? frame.couleurActive() : macro.couleurLigne) : frame.couleurActive();
    // listeStyleTrait = new MtgListeStyleTrait(frame); Modifi version 5.0
		paneStyleTrait = new PaneStyleTrait(frame, this);
    // scrollPaneStyleTrait = new JScrollPane(listeStyleTrait, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
	  if (modification) {
    	editeurIntitule.setText(macro.intitule);
    	editeurCommentaire.setText(macro.commentaireMacro);
			editeurCommentaire.setCaretPosition(0);
			editeurNbSources.setText(Integer.toString(macro.nbSourcesPourIter));
			editeurNombreIterations.setText(Integer.toString(macro.nbIter));
			checkImposerCouleur.setSelected(macro.imposerCouleur);
			// StyleTrait.setSelectedIndex(macro.styleLigne.style); Modifi version 5.0
			// listeStyleTrait.ensureIndexIsVisible(macro.styleLigne.style);
			if (macro.choixLigne != 0) paneStyleTrait.setSelection(macro.styleLigne);
	  	switch (macro.choixLigne) {
			case 0: 
				radioSansLigne.setSelected(true);			
				break;
			case 1:
				radioAvecPolygone.setSelected(true);
				checkPolygoneRempli.setSelected(macro.polygoneRempli);
				break;
			case 2:
				radioAvecLigneBrisee.setSelected(true);
			default:
				break;
			}
	  }
	  else radioSansLigne.setSelected(true);

    JPanel panePrincipal = (JPanel) getContentPane();
    JPanel paneRight = new JPanel();
    JPanel paneBas = new JPanel();
    panePrincipal.setLayout(new GridBagLayout());
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);

		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets.bottom = 0;
		constraints.insets.top = 2;
		constraints.insets.left = 5;
		constraints.insets.right = 5;
		
    JPanel paneLigne = new JPanel();
    paneLigne.setLayout(new GridBagLayout());
	  setConstraints(constraints, 0, 0, 2, 1, 40, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  paneLigne.add(radioSansLigne, constraints);
	  setConstraints(constraints, 0, 1, 1, 1, 40, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  paneLigne.add(radioAvecPolygone, constraints);
	  setConstraints(constraints, 1, 1, 1, 1, 20, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  paneLigne.add(checkPolygoneRempli, constraints);
	  setConstraints(constraints, 0, 2, 2, 1, 40, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  paneLigne.add(radioAvecLigneBrisee, constraints);
	  setConstraints(constraints, 2, 0, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  paneLigne.add(colorButtonLigne, constraints);
	  setConstraints(constraints, 2, 1, 1, 2, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  paneLigne.add(paneStyleTrait, constraints);


		JPanel paneIntitule = new JPanel();
		paneIntitule.setLayout(new BoxLayout(paneIntitule , BoxLayout.X_AXIS));
		paneIntitule.add(label1);
		paneIntitule.add(editeurIntitule);
		setConstraints(constraints, 0, 0, 2, 1, 80, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(paneIntitule, constraints);
		setConstraints(constraints, 0, 1, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label2, constraints);
		setConstraints(constraints, 0, 2, 2, 1, 0, 30, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(scrollPaneCommentaire, constraints);
		setConstraints(constraints, 0, 3, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label3, constraints);
		setConstraints(constraints, 1, 3, 1, 1, 0, 10, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(scrollPaneListePrototypes, constraints);
    setConstraints(constraints, 0, 4, 1, 1, 50, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label4, constraints);	
    setConstraints(constraints, 1, 4, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(editeurNombreIterations, constraints);
    setConstraints(constraints, 0, 5, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label6, constraints);	
    setConstraints(constraints, 1, 5, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(editeurNbSources, constraints);
    setConstraints(constraints, 0, 6, 1, 1, 50, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(checkImposerCouleur, constraints);
    setConstraints(constraints, 1, 6, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(colorButtonImp, constraints);
    /*
	  setConstraints(constraints, 0, 7, 2, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  panePrincipal.add(radioSansLigne, constraints);
	  setConstraints(constraints, 0, 8, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  panePrincipal.add(radioAvecPolygone, constraints);
	  setConstraints(constraints, 1, 8, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  panePrincipal.add(checkPolygoneRempli, constraints);
	  setConstraints(constraints, 0, 9, 2, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  panePrincipal.add(radioAvecLigneBrisee, constraints);
    setConstraints(constraints, 0, 10, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label7, constraints);
    setConstraints(constraints, 1, 10, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(colorButtonLigne, constraints);
    setConstraints(constraints, 0, 11, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label8, constraints);
    setConstraints(constraints, 1, 11, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(scrollPaneStyleTrait, constraints);
 		*/
    setConstraints(constraints, 0, 7, 3, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(paneLigne, constraints);
		
		paneBas.add(paneTaillePolice);
		paneBas.add(paneModeAffichage);
		setConstraints(constraints, 0, 12, 2, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(paneBas, constraints);
 		setConstraints(constraints, 2, 0, 1, 2, 20, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(paneRight, constraints);

		pack();	
		if (radioSansLigne.isSelected()) {
			colorButtonLigne.setVisible(false);
			paneStyleTrait.setVisible(false);
			checkPolygoneRempli.setVisible(false);
		}
		else if (!radioAvecPolygone.isSelected()) checkPolygoneRempli.setVisible(false);

  }

  private void checkImposerCouleurActionPerformed() {
  	colorButtonImp.setVisible(checkImposerCouleur.isSelected());
  	pack();
  }
  
  private void radioSansLigneActionPerformed() {
  	colorButtonLigne.setVisible(!radioSansLigne.isSelected());
  	// scrollPaneStyleTrait.setVisible(!radioSansLigne.isSelected()); Modifi version 5.0
  	paneStyleTrait.setVisible(!radioSansLigne.isSelected());
  	checkPolygoneRempli.setVisible(radioAvecPolygone.isSelected());
  	pack();
  }


  private void btnOKActionPerformed() {
  	int nombreIt, nbSources;
  	String chintitule = editeurIntitule.getText();
  	if (chintitule.length() == 0) {
  		dialogueAvertissement(getString("ch115"));
  		editeurIntitule.requestFocus();
  		return;
  	}
  	CListeObjets liste = frame.cadreActif().listePr;
  	if (!liste.validationIntituleMacro(chintitule, macro)) {
  		dialogueAvertissement(getString("ch116"));
  		editeurIntitule.requestFocus();
  		return;
  	}
  	try {
  		nombreIt = Integer.parseInt(editeurNombreIterations.getText());
  	}
		catch (NumberFormatException e) {
			dialogueAvertissement(getString("ch71"));
			editeurNombreIterations.requestFocus();
			return;
		}
 	
    if (nombreIt < 2 ) {
    	dialogueAvertissement(getString("ch71"));
    	editeurNombreIterations.requestFocus();
    	return;
    }
  	try {
  		nbSources = Integer.parseInt(editeurNbSources.getText());
  	}
		catch (NumberFormatException e) {
			dialogueAvertissement(getString("ch71"));
			editeurNbSources.requestFocus();
			return;
		}
 	
    if (nbSources < 0) {
    	dialogueAvertissement(getString("ch71"));
    	editeurNbSources.requestFocus();
    	return;
    }
    CPrototype proto = listePrototypes.prototypeChoisi();
    String chErr = proto.implementationPossible(liste);
    if (chErr.length() != 0) {
    	frame.dialogueAvertissement(getString("ch142") + "\n" + chErr);
    	frame.activeOutilCapture();
    	return;
    }
    if ((liste.pointeurLongueurUnite == null) && proto.utiliseLongueurUnite()) {
     	if (proto.premiereLongueur() == null) {
     		frame.dialogueAvertissement(getString("ch138"));
	    	frame.activeOutilCapture();
	    	return;
    	}
    }
		if (!proto.estIterable(nbSources)) {
			frame.dialogueAvertissement(getString("ch197"));
			return;
		}
    double n = (double)(proto.size()-proto.nbObjSources-nbSources)*(nombreIt);
		if (n > (double) Outil.nombreMaxObjets) {
			frame.dialogueAvertissement(getString("ch198"));
			return;
		}

  	macro.intitule = chintitule;
  	macro.commentaireMacro = editeurCommentaire.getText();
  	macro.indiceTaillePolice = paneTaillePolice.indiceTaille();
  	macro.effacementFond = paneModeAffichage.effacementFond();
  	macro.couleurFondAffichage = paneModeAffichage.couleurFond();
		macro.alignementHorizontal = paneModeAffichage.alignementHorizontal();
		macro.alignementVertical = paneModeAffichage.alignementVertical();
		
		frame.pref_IndiceTaillePoliceAffichages = macro.indiceTaillePolice;
		frame.cadreActif().couleurFondAffichage = macro.couleurFondAffichage;
    macro.proto = proto;
    macro.nbIter = nombreIt;
    macro.nbSourcesPourIter = nbSources;
    macro.imposerCouleur = checkImposerCouleur.isSelected();
    macro.couleurImp = colorButtonImp.couleur;
    macro.choixLigne = radioSansLigne.isSelected() ? Outil.sansLigne
    		: (radioAvecPolygone.isSelected() ? Outil.avecPolygone : Outil.avecLigneBrisee);
    macro.polygoneRempli = (macro.choixLigne != Outil.avecPolygone) ? false
    		: radioAvecPolygone.isSelected() ? (checkPolygoneRempli.isSelected() ? true : false) : false;
    // macro.styleLigne = listeStyleTrait.styleTrait(); Modifi version 5.0
    		macro.styleLigne = paneStyleTrait.getStyle();
    macro.couleurLigne = colorButtonLigne.couleur;
    setRetourOK(true);
    dispose();
  }
  
  private void btnCancelActionPerformed() {                                          
    dispose();
  }       


}