
/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Component;
import java.awt.event.MouseEvent;
import java.util.EventObject;

import javax.swing.JLabel;
import javax.swing.JTree;
import javax.swing.event.CellEditorListener;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeCellEditor;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import mathgraph32.MtgCheckBoxMenuItemForNode;
import mathgraph32.MtgFrame;
import types.FonteNom;

@SuppressWarnings("serial")
public class MtgMenuTree extends JTree implements TreeExpansionListener {

	public MtgMenuTree(DefaultTreeModel model) {
		super(model);
		setRowHeight(0);
		addTreeExpansionListener(this);
	}

	public void treeCollapsed(TreeExpansionEvent arg0) {
		// TODO Auto-generated method stub
	}

	// Nouveau version 4.3 : Quand un noeud est ouvert, on ouvre tous les sous-noeuds
	public void treeExpanded(TreeExpansionEvent arg0) {
		// TODO Auto-generated method stub
		TreePath path = arg0.getPath();
		expandPath(path);
    TreeNode node = (TreeNode)path.getLastPathComponent() ;
    for(int i=0 ; i<node.getChildCount() ; i++) {
    	 TreePath tp = path.pathByAddingChild(node.getChildAt(i)) ;
    	 expandPath(tp);
    }		
	}
}

class RenduComposant implements TreeCellRenderer{
	private DefaultTreeCellRenderer defaultRenderer;
	private JLabel lab;
	public RenduComposant() {
		defaultRenderer = new DefaultTreeCellRenderer();
		defaultRenderer.setFont(FonteNom.getFonteNom(MtgFrame.sizeFontMenuItem, FonteNom.Normal, MtgFrame.coefMultForSystemFonts));
	  	lab = new JLabel();
		lab.setFont(FonteNom.getFonteNom(MtgFrame.sizeFontMenuItem, FonteNom.Normal, MtgFrame.coefMultForSystemFonts));			
	}
	public Component getTreeCellRendererComponent(JTree tree, Object obj,
			boolean selected, boolean expanded, boolean leaf, 
		int row, boolean hasFocus){		
		if ((obj != null) && (obj instanceof DefaultMutableTreeNode)) {
      Object userObject = ((DefaultMutableTreeNode) obj).getUserObject();
			if (userObject != null) {
				if (userObject.getClass() == MtgCheckBoxMenuItemForNode.class) return (MtgCheckBoxMenuItemForNode)userObject;
				else {
					lab.setText(userObject.toString());
					return lab;
				}
			}
		}
		return defaultRenderer.getTreeCellRendererComponent(tree, obj, selected, expanded, leaf, row, hasFocus);
	}
}

class EditComposant implements TreeCellEditor{
	MtgMenuTree menuTree;
	public EditComposant(MtgMenuTree menuTreeInit) {
		super();
		menuTree = menuTreeInit;
	}
	public void addCellEditorListener(CellEditorListener l){ }
	public void cancelCellEditing() { }
	public Object getCellEditorValue(){
		return this;
	}
	/**
	 * Version 4.3 : Pour viter des nullpointerException quand on clique sur une branche
	 * il faut regarder si on clique bien sur un objet ditable
	 */
	public boolean isCellEditable(EventObject evt){
		boolean returnvalue = false;
		if(evt instanceof MouseEvent){
			MouseEvent mevt = (MouseEvent) evt;
			if (mevt.getClickCount() == 1){
	      TreePath path = menuTree.getPathForLocation(mevt.getX(), mevt.getY());
	      if (path != null) {
	        Object node = path.getLastPathComponent();
	        if ((node != null) && (node instanceof DefaultMutableTreeNode)) {
	          DefaultMutableTreeNode treeNode = (DefaultMutableTreeNode) node;
	          Object userObject = treeNode.getUserObject();
	          returnvalue =  ((treeNode.isLeaf()) && (userObject instanceof MtgCheckBoxMenuItemForNode));
	        }
	      }
			}
		}
		return returnvalue;
	}
	
	public void removeCellEditorListener(CellEditorListener l){}
	
	public boolean shouldSelectCell(EventObject anEvent){
		return true;
	}
	public boolean stopCellEditing(){
		return false;
	}
	public Component getTreeCellEditorComponent(JTree tree, Object obj, boolean isSelected, boolean expanded, boolean leaf, int row){
		if ((obj != null) && (obj instanceof DefaultMutableTreeNode)) {
      Object userObject = ((DefaultMutableTreeNode) obj).getUserObject();
			if (userObject != null) {
				if (userObject.getClass() == MtgCheckBoxMenuItemForNode.class) {
					MtgCheckBoxMenuItemForNode item = (MtgCheckBoxMenuItemForNode) userObject;
					boolean b = item.itemAssocie.isSelected();
					item.itemAssocie.setSelected(!b);
					return item;
				}
			}
		}
		return null;
	}	
}
