/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import mathgraph32.MtgFrame;
import mathgraph32.MtgListeStyleTrait;
import types.StyleTrait;

@SuppressWarnings("serial")
public class PaneStyleTrait extends JPanel {
	private MtgListeStyleTrait listeStyleTraits;
	private MtgDlg parentDlg;
	private JSlider sliderEpaisseur;
	private JLabel labelEpTrait;
	
	// Modifi version 5.0.2 car si l'paisseur dpassait 10 la bote de dialogue parent n'tait pas correctement raffiche quand on modifiant le curseur
	public PaneStyleTrait(MtgFrame frameInit, MtgDlg parentDdgInit) {
		listeStyleTraits = new MtgListeStyleTrait(frameInit);
		parentDlg = parentDdgInit;
		// setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		labelEpTrait = new JLabel(" 1 ");
		labelEpTrait.setToolTipText(MtgDlg.getString("tipEpaisseur"));
		sliderEpaisseur = new JSlider(JSlider.HORIZONTAL, 1, 15, 1);
		sliderEpaisseur.setMajorTickSpacing(10);
		sliderEpaisseur.setMinorTickSpacing(1);
		sliderEpaisseur.setPaintTicks(true);
		sliderEpaisseur.setPaintLabels(true);
		sliderEpaisseur.setToolTipText(MtgDlg.getString("tipEpaisseur"));
		double k = MtgFrame.coefMultForDlgBoxIcons;
		Font font = new Font("Serif", Font.ROMAN_BASELINE, (int)(k*11)); 
		labelEpTrait.setFont(font); // Ajout version 5.0.2
		sliderEpaisseur.setFont(font);
		sliderEpaisseur.setPreferredSize(new Dimension((int)(k*90), (int)(k*40)));
		sliderEpaisseur.addChangeListener(new ChangeListener() {
			
			public void stateChanged(ChangeEvent e) {
				int a = sliderEpaisseur.getValue();
				String s = " " + Integer.toString(a) + " ";
				s = s.substring(0, 3);
				labelEpTrait.setText(s);
				if (a>=10) parentDlg.pack();
				
			}
		});
		JPanel paneSlider = new JPanel();
		paneSlider.setLayout(new BoxLayout(paneSlider, BoxLayout.LINE_AXIS));
		paneSlider.add(sliderEpaisseur);
		paneSlider.add(labelEpTrait);
		add(listeStyleTraits);
		add(paneSlider);
		setBorder(BorderFactory.createLineBorder(Color.black));
	}
	
	public StyleTrait getStyle() {
		return new StyleTrait(listeStyleTraits.getSelectedIndex(), sliderEpaisseur.getValue());
	}
	
	public void setSelection(StyleTrait style) {
		listeStyleTraits.setSelectedIndex(style.style);
		listeStyleTraits.ensureIndexIsVisible(style.style);
		sliderEpaisseur.setValue(style.epaisseur);
	}
	
	public void setSelection(int style, int epaisseur) {
		listeStyleTraits.setSelectedIndex(StyleTrait.styleTraitPointille);
		listeStyleTraits.ensureIndexIsVisible(StyleTrait.styleTraitPointille);
		sliderEpaisseur.setValue(epaisseur);
		
	}

}
