/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CListeObjets;
import objets.CPartieImaginaire;

@SuppressWarnings("serial")
public class PartieImaginaireDlg extends MtgDlg {
	private boolean modification;
	private CPartieImaginaire partieImaginaire;
	private MtgLabel label1, label2, label3;
	private MtgTextField editeurNom;
	private MtgTextArea info;
  private MtgButton btnOK, btnCancel;  
  private ListeValeursComplexes listeValeurs;
	
  public PartieImaginaireDlg(MtgFrame frameInit, CPartieImaginaire partieImaginaireInit, boolean modificationInit) {
  	super(getString("partieImaginaireDlg1"), true, frameInit);
  	modification = modificationInit;
  	partieImaginaire = partieImaginaireInit;
    initComponents();
    getRootPane().setDefaultButton(btnOK);      	
  }
  
  private void initComponents() {
    label1 = new MtgLabel(frame, getString("creerCalculDlg1"), JLabel.RIGHT);
    label1.setRequestFocusEnabled(false);
    label2 = new MtgLabel(frame, getString("partieImaginaireDlg3"), JLabel.RIGHT);
    label2.setRequestFocusEnabled(false);
    label3 = new MtgLabel(frame, getString("partieReelleDlg4"), JLabel.RIGHT);
    label3.setRequestFocusEnabled(false);
    editeurNom = new MtgTextField(frame,8);
    CListeObjets liste = frame.cadreActif().listePr;
    int indmax;
    if (modification) {
    	indmax = liste.indexOf(partieImaginaire) - 1;
    	listeValeurs = new ListeValeursComplexes(frame, liste, indmax, false);
    }
    else {
    	indmax = liste.size() - 1;
    	listeValeurs = new ListeValeursComplexes(frame, liste, indmax, false);
    }
    listeValeurs.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent evt) {
      	listeValeursSelectionChanged();
      }
    });
    
    JScrollPane scrollPaneListe = new JScrollPane(listeValeurs, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    
    info = new MtgTextArea(frame);
    info.setEditable(false);
    info.setColumns(20);
    info.setLineWrap(false);
    info.setRows(4);
    // Modification version 5.0 pour tenir compte des crans haute dfinition
    double k = MtgFrame.coefMultForDlgBoxIcons;
    JScrollPane scrollPaneInfo = new JScrollPane(info, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    scrollPaneInfo.setPreferredSize(new Dimension((int)(k*300), (int)(k*90)));
    //
    btnOK = new MtgButton(frame, getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed();
      }
    });    
    btnCancel = new MtgButton(frame, getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed();
      }
    });
    btnOK.setMaximumSize(btnCancel.getPreferredSize());
    if (modification) {
      editeurNom.setText(partieImaginaire.nom());
      editeurNom.setEditable(false);
    	listeValeurs.setSel(partieImaginaire.complexeAssocie);
    }        
    JPanel panePrincipal = new JPanel();
    JPanel paneRight = new JPanel();
    panePrincipal.setLayout(new GridBagLayout());
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 40, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label1, constraints);
    
    setConstraints(constraints, 1, 0, 1, 1, 40, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(editeurNom, constraints);    
    
    setConstraints(constraints, 0, 1, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label2, constraints);

    setConstraints(constraints, 0, 2, 2, 1, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST);
    panePrincipal.add(scrollPaneListe, constraints);
    
    setConstraints(constraints, 0, 3, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label3, constraints);

    setConstraints(constraints, 0, 4, 3, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(scrollPaneInfo, constraints);
    
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
    setConstraints(constraints, 2, 0, 1, 2, 20, 0, GridBagConstraints.NONE, GridBagConstraints.NORTH);
    panePrincipal.add(paneRight, constraints);
    
    setContentPane(panePrincipal);
    pack(); 
    listeValeursSelectionChanged();
  }
  
  public void listeValeursSelectionChanged() {
  	info.setText(listeValeurs.valeurSelectionnee().infoPourHistorique());
  	info.setCaretPosition(0);
  	pack();
  }
  
  private void btnOKActionPerformed() {                                      
    MtgInternalFrame cadre = frame.cadreActif();
    CListeObjets listePr = cadre.listePr;
    String chNom = editeurNom.getText();
    if (!modification) {
	    // On vrifie d'abord la validit du nom de la fonction
	    if (!listePr.validationNomCalcul(frame, chNom)) {
	      editeurNom.requestFocus();
	      return;
	    }
    }
    if (!modification) partieImaginaire.nomCalcul = chNom;
    partieImaginaire.complexeAssocie = listeValeurs.valeurSelectionnee();
  	setRetourOK(true);
  	dispose();
  }
  
  public void btnCancelActionPerformed() {
  	dispose();
  }
}
