/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import javax.swing.ButtonGroup;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

import objets.CMathGrafDoc;

import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import mathgraph32.MtgMenu;

/**
 * Dialogue servant  personnaliser le niveau niv
 * @author Yves
 * Modifi version 5.0 car la sauvegarde des figures dfinissant le niveaux utilisateurs se fait dans le dossier utilisateur
 */
@SuppressWarnings("serial")
public class PersonnalisationNiveauDlg extends MtgDlg {
	private int niv; // Le niveau  personnaliser (1  4)
	// private String path; // Chemin d'accs au jar excutable PLus utilis vesion 5.0
	private MtgMenu menuFichier, menuEdition, menuCreer, menuCalculs, menuModifier, menuMouvoir;
	private MtgMenu menuMacros, menuConstructions;
	private MtgLabel label1, label2;
	private MtgRadioButton radioItemCochesInterdits;
	private MtgRadioButton radioItemCochesPermis;
	private MtgMenuTree tree;
	private DefaultTreeModel model;
	private DefaultMutableTreeNode principalNode;
	private JScrollPane scrollPaneTree;
	private ButtonGroup btnGroup;
	private MtgButton btnCocherTout;
	private MtgButton btnDecocherTout;
	private DefaultMutableTreeNode nodeFichier;
	private DefaultMutableTreeNode nodeEdition;
	private DefaultMutableTreeNode nodeCreer;
	private DefaultMutableTreeNode nodeCalculs;
	private DefaultMutableTreeNode nodeModifier;
	private DefaultMutableTreeNode nodeMouvoir;
	private DefaultMutableTreeNode nodeMacros;
	private DefaultMutableTreeNode nodeConstructions;
	private MtgButton btnSauvegarder, btnCancel;
	private MtgTextField editeurDescripteur;
	private CMathGrafDoc docNiveau;
	// Modification version 5.0.0.5 : On recre les menus initiaux au lieu de les crer  l'initialisation de MtgFrame
	private MtgMenu menuFichierInitial;
	private MtgMenu menuEditionInitial;
	private MtgMenu menuCreerInitial;
	private MtgMenu menuModifierInitial;
	private MtgMenu menuMouvoirInitial;
	private MtgMenu menuCalculsInitial;
	private MtgMenu menuMacrosInitial;
	private MtgMenu menuConstructionsInitial;

  public PersonnalisationNiveauDlg(MtgFrame frameInit, int nivInit) {
    super(getString("personnalisationNiveau") + " " + nivInit, true, frameInit);
    niv = nivInit;
		menuFichierInitial =frame.creeMenuFichier();
		menuEditionInitial = frame.creeMenuEdition();
		menuCreerInitial = frame.creeMenuCreer();
		menuModifierInitial = frame.creeMenuModifier();
		menuMouvoirInitial = frame.creeMenuMouvoir();
		menuCalculsInitial = frame.creeMenuCalculs();
		menuMacrosInitial = frame.creeMenuMacros();
		menuConstructionsInitial = frame.creeMenuConstructions();
    initComponents();
    getRootPane().setDefaultButton(btnCancel);
  }
  
  private void initComponents() {
  	editeurDescripteur = new MtgTextField(frame, 30);
    btnCancel = new MtgButton(frame, getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed();
      }
    });
    btnSauvegarder = new MtgButton(frame, getString("btnSauvegarder"));
    btnSauvegarder.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnSauvegarderActionPerformed();
      }
    });
    label1 = new MtgLabel(frame, getString("nomDuNiveau") + " " + niv + ":");
		label2 = new MtgLabel(frame, getString("preferencesMenu1"));
		btnGroup = new ButtonGroup();
		// Ajout version 4.3. Si on clique sur le bouton on dcochera tout.
		radioItemCochesInterdits = new MtgRadioButton(frame, getString("preferencesMenu2"));
		radioItemCochesInterdits.addActionListener(new ActionListener() {	
			public void actionPerformed(ActionEvent arg0) {
				btnDecocherToutPerformed(true);			
			}
		});
		btnGroup.add(radioItemCochesInterdits);
		// Ajout version 4.3. Si on clique sur le bouton on cochera tout. 
		radioItemCochesPermis = new MtgRadioButton(frame, getString("preferencesMenu3"));
		radioItemCochesPermis.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				btnCocherToutPerformed(true);
			}
		});
		btnGroup.add(radioItemCochesPermis);
		btnCocherTout = new MtgButton(frame, getString("btnCocherTout"));
		btnCocherTout.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnCocherToutPerformed(false);
	    }
	  });    
		btnDecocherTout = new MtgButton(frame, getString("btnDecocherTout"));
		btnDecocherTout.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnDecocherToutPerformed(false);
	    }
	  });
		prepareItems();
		if (docNiveau != null) {
			String ch = docNiveau.TexteFigure;
			int len = ch.length();
			if (len > 80) ch = ch.substring(0, 79);
			// On retire les ventuels retours  la ligne
			int lf = ch.indexOf("\n");
			if (lf != -1) ch = ch.substring(0, lf);
			if (len != 0) editeurDescripteur.setText(ch);
		}
		tree.setCellRenderer(new RenduComposant());
		tree.setCellEditor(new EditComposant(tree));
		tree.setEditable(true);

		scrollPaneTree = new JScrollPane(tree, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    // Modification version 5.0 pour tenir compte des crans haute dfinition
    double k = MtgFrame.coefMultForDlgBoxIcons;
		scrollPaneTree.setPreferredSize(new Dimension((int)(k*400), (int)(k*200)));
		radioItemCochesInterdits.setSelected(docNiveau != null ? docNiveau.itemsCochesInterdits : true);
		radioItemCochesPermis.setSelected(!radioItemCochesInterdits.isSelected());
    setLayout(new GridBagLayout());
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    MtgDlg.setConstraints(constraints, 0, 0, 1, 1, 30, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    add(label1, constraints);
    MtgDlg.setConstraints(constraints, 1, 0, 1, 1, 70, 10, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    add(editeurDescripteur, constraints);
    MtgDlg.setConstraints(constraints, 0, 1, 2, 1, 100, 60, GridBagConstraints.NONE, GridBagConstraints.NORTHWEST);
    add(scrollPaneTree, constraints);
    MtgDlg.setConstraints(constraints, 0, 2, 2, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    add(label2, constraints);
    MtgDlg.setConstraints(constraints, 0, 3, 2, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    add(radioItemCochesInterdits, constraints);
    MtgDlg.setConstraints(constraints, 0, 4, 2, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    add(radioItemCochesPermis, constraints);		
    MtgDlg.setConstraints(constraints, 0, 5, 2, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.CENTER);
    add(btnCocherTout, constraints);		
    MtgDlg.setConstraints(constraints, 0, 6, 2, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.CENTER);
    add(btnDecocherTout, constraints);
    JPanel paneBas = new JPanel();
    // MtgDlg.setConstraints(constraints, 0, 7, 1, 1, 50, 10, GridBagConstraints.NONE, GridBagConstraints.EAST);
    paneBas.add(btnSauvegarder);
    // MtgDlg.setConstraints(constraints, 1, 7, 1, 1, 50, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneBas.add(btnCancel);
    // add(btnCancel, constraints);
    MtgDlg.setConstraints(constraints, 0, 7, 2, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.CENTER);
    add(paneBas, constraints);
    pack();
  }
  
	private void prepareItems() {
		// On charge s'il existe le fichier nomm level suivi du chiffre du niveau
		// S'il n'est pas prsent tous les outils sont initialement permis
		// Le texte de la figure de ce fichier contient le descripteur du fichier
		// Modifi version 5.0 / Le fichier est charg depuis le dossier utilisatteur

		
		
		String defaut = "level" + Integer.toString(niv) + ".mgj";
		String fs = System.getProperty("file.separator");
		String path;
		// Obligation d'intercepter exceptions du type SecuriteException sur certains systme
		try {
			String se = System.getProperty("os.name");
			if (se.startsWith("Linux")) path = System.getProperty("user.home") + fs +".config" + fs + "MathGraph32";
			else if (se.startsWith("Mac")) path = System.getProperty("user.home") + fs + ".MathGraph32";
			else path = System.getProperty("user.home") + fs + "MathGraph32";
		}
		catch (Exception e) {
			path = System.getProperty("user.home") + fs + "MathGraph32";
		}
				
		String ch2 = path + fs + defaut;
		File file2 = new File(ch2);
		try {
			FileInputStream fis = new FileInputStream(file2);
			MathGraphDataInputStream inps = new MathGraphDataInputStream(fis);
			if (file2.exists()) {
				try {
					docNiveau = new CMathGrafDoc(frame);
					// docPreferencesOutils.chargeDepuisFlux(inps);
					docNiveau.chargeDepuisFlux(inps);
					String ch = docNiveau.TexteFigure;
					int len = ch.length();
					if (len > 80) ch = ch.substring(0, 79);
					// On retire les ventuels retours  la ligne
					int lf = ch.indexOf("\n");
					if (lf != -1) ch = ch.substring(0, lf);
					if (len != 0) editeurDescripteur.setText(ch);
				} catch (IOException e) {
					docNiveau = new CMathGrafDoc(frame);
				}
			}
		} catch (FileNotFoundException e) {
			docNiveau = new CMathGrafDoc(frame);
		}
		
		menuFichier = new MtgMenu(getString("menuFichier"), frame);
		menuEdition = new MtgMenu(getString("menuEdition"), frame);
		menuCreer = new MtgMenu(getString("menuCreer"), frame);
		menuModifier = new MtgMenu(getString("menuModifier"), frame);
		menuMouvoir = new MtgMenu(getString("menuMouvoir"), frame);
		menuCalculs = new MtgMenu(getString("menuCalculs"), frame);
		menuMacros = new MtgMenu(getString("menuMacros"), frame);
		menuConstructions = new MtgMenu(getString("menuConstructions"), frame);
		
		// On cre un document vide pour que tous les items de menu soient accessibles
		CMathGrafDoc docRef = new CMathGrafDoc(frame);
		menuFichierInitial.addCloneAvecCheck(menuFichier, docRef, docNiveau);
		menuEditionInitial.addCloneAvecCheck(menuEdition, docRef, docNiveau);
		menuCreerInitial.addCloneAvecCheck(menuCreer, docRef, docNiveau);
		menuCalculsInitial.addCloneAvecCheck(menuCalculs, docRef, docNiveau);
		menuModifierInitial.addCloneAvecCheck(menuModifier, docRef, docNiveau);
		menuMouvoirInitial.addCloneAvecCheck(menuMouvoir, docRef, docNiveau);
		menuMacrosInitial.addCloneAvecCheck(menuMacros, docRef, docNiveau);
		menuConstructionsInitial.addCloneAvecCheck(menuConstructions, docRef, docNiveau);
		// principalNode = new DefaultMutableTreeNode(); Corrig version 5.0 pour Linux : Indispensable que le noeu principal
		// ait un texte non vide
		principalNode = new DefaultMutableTreeNode(MtgFrame.getString("outils"));
		creeNoeuds();
		model = new DefaultTreeModel(principalNode);
		tree = new MtgMenuTree(model);
	}
	
	private void creeNoeuds() {
		nodeFichier = new DefaultMutableTreeNode(menuFichier.getText());
		menuFichier.ajouteNoeud(nodeFichier);
		nodeEdition = new DefaultMutableTreeNode(menuEdition.getText());
		menuEdition.ajouteNoeud(nodeEdition);
		nodeCreer = new DefaultMutableTreeNode(menuCreer.getText());
		menuCreer.ajouteNoeud(nodeCreer);
		nodeCalculs = new DefaultMutableTreeNode(menuCalculs.getText());
		menuCalculs.ajouteNoeud(nodeCalculs);
		nodeModifier = new DefaultMutableTreeNode(menuModifier.getText());
		menuModifier.ajouteNoeud(nodeModifier);
		nodeMouvoir = new DefaultMutableTreeNode(menuMouvoir.getText());
		menuMouvoir.ajouteNoeud(nodeMouvoir);
		nodeMacros = new DefaultMutableTreeNode(menuMacros.getText());
		menuMacros.ajouteNoeud(nodeMacros);
		nodeConstructions = new DefaultMutableTreeNode(menuConstructions.getText());
		menuConstructions.ajouteNoeud(nodeConstructions);
		principalNode.add(nodeFichier);
		principalNode.add(nodeEdition);
		principalNode.add(nodeCreer);
		principalNode.add(nodeCalculs);
		principalNode.add(nodeMouvoir);
		principalNode.add(nodeModifier);
		principalNode.add(nodeMacros);
		principalNode.add(nodeConstructions);
	}
	
	private void btnCancelActionPerformed() {
		dispose();
	}
	
	private void btnCocherToutPerformed(boolean verbose) {
		boolean b = radioItemCochesInterdits.isSelected();
		menuFichier.selectAll(b);
		menuEdition.selectAll(b);
		menuCreer.selectAll(b);
		menuCalculs.selectAll(b);
		menuModifier.selectAll(b);
		menuMouvoir.selectAll(b);
		menuMacros.selectAll(b);
		menuConstructions.selectAll(b);
		principalNode.removeAllChildren();
		creeNoeuds();
		tree.updateUI();
		if (verbose) dialogueAvertissement(getString("outilsReinitialises"));
	}
	

	
	private void btnDecocherToutPerformed(boolean verbose) {
		boolean b = !radioItemCochesInterdits.isSelected();
		menuFichier.deselectAll(b);
		menuEdition.deselectAll(b);
		menuCreer.deselectAll(b);
		menuCalculs.deselectAll(b);
		menuModifier.deselectAll(b);
		menuMouvoir.deselectAll(b);
		menuMacros.deselectAll(b);
		menuConstructions.deselectAll(b);
		principalNode.removeAllChildren();
		creeNoeuds();
		tree.updateUI();
		if (verbose) dialogueAvertissement(getString("outilsReinitialises"));
	}
	
	private void btnSauvegarderActionPerformed() {
	try {
		if (editeurDescripteur.getText().length() == 0) {
			dialogueAvertissement(getString("ch97"));
			editeurDescripteur.requestFocus();
			return;
		}
		menuFichier.adapteDocument(docNiveau);
		menuEdition.adapteDocument(docNiveau);
		menuCreer.adapteDocument(docNiveau);
		menuCalculs.adapteDocument(docNiveau);
		menuModifier.adapteDocument(docNiveau);
		menuMouvoir.adapteDocument(docNiveau);
		menuMacros.adapteDocument(docNiveau);
		menuConstructions.adapteDocument(docNiveau);
		docNiveau.itemsCochesInterdits = radioItemCochesInterdits.isSelected();
		
		// String fs = System.getProperty("file.separator");
		String defaut = "level" + Integer.toString(niv) + ".mgj";
		// String ch = path + fs + defaut;
		// String ch = path + "Help" + frame.pref_langage + "/" + defaut;
		/* Modifi version 5.0
		String ch = path + "Help" + frame.pref_langage + File.separator + defaut;	// Version 4.3		
		*/
		String fs = System.getProperty("file.separator");
		String path;
		// Ajouts version 5.0
		// Obligation d'intercepter exceptions du type SecuriteException sur certains systme
		try {
			String se = System.getProperty("os.name");
			if (se.startsWith("Linux")) path = System.getProperty("user.home") + fs +".config" + fs + "/MathGraph32";
			else if (se.startsWith("Mac")) path = System.getProperty("user.home") + fs + ".MathGraph32";
			else path = System.getProperty("user.home") + fs + "MathGraph32";
		}
		catch (Exception e) {
			path = System.getProperty("user.home") + fs + "MathGraph32";
		}
		String ch = path + fs + defaut;
		// Fin ajouts version 5.0
		
		File file = new File(ch);
		FileOutputStream fos = new FileOutputStream(file);
		MathGraphDataOutputStream oups = new MathGraphDataOutputStream(fos);
		try {
			docNiveau.TexteFigure = editeurDescripteur.getText();
			docNiveau.write(oups);
			docNiveau.setDirty(false);
			oups.close();
			dialogueAvertissement(getString("sauvegardeEffectuee") +"\n" + getString("ch177"));
			// Version 4.3 : On regarde si le niveau qui a t personnalis tait le niveau actif.
			// Si oui on met  jour menus et icnes en fonction de ce niveau.
			frame.docPreferencesNiveauPersonnalise[5-niv] = docNiveau;
			if (frame.pref_NiveauPersonnalise && (frame.pref_niveau == 5 - niv)) {
				frame.docPreferencesNiveauPersonnalise[5-niv] = docNiveau;
				frame.docPreferencesNiveau[5-niv] = docNiveau;
				frame.descripteurNiveauPersonnalise[5-niv] = docNiveau.TexteFigure;
				frame.descripteurNiveau[5-niv] = docNiveau.TexteFigure;				
				MtgInternalFrame cadre = frame.cadreActif();
				frame.metAJourIconesEtMenus((cadre != null) ? cadre.document : frame.docPreferencesNiveau[frame.pref_niveau]);
			}
			dispose();
		} 
		catch (IOException e1) {
			dialogueAvertissement(getString("ch45"));
			erreurSauvegarde();
		}
		} catch (FileNotFoundException e2) {
			dialogueAvertissement(getString("ch45"));
			erreurSauvegarde();
		}
	}
	
	private void erreurSauvegarde() {
		dialogueAvertissement(getString("droitsAdministrateurs"));
		dispose();
	}

}
