/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.text.NumberFormat;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;

import mathgraph32.MtgFrame;

@SuppressWarnings("serial")
public class PolygoneRegulierDlg extends MtgDlg {
	private MtgLabel label1, label2;
	private MtgButton btnOK, btnCancel;
	public MtgFormattedTextField editeurNombrePoints;
	public int nombrePoints;

  public PolygoneRegulierDlg(MtgFrame frameInit) {
  	super(getString("PolygoneRegulierDlg1"), true, frameInit);
    initComponents();
    getRootPane().setDefaultButton(btnOK);
  }
	
  private void initComponents() {
    btnOK = new MtgButton(frame, getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed(evt);
      }
    });    
    btnCancel = new MtgButton(frame, getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed(evt);
      }
    });
    btnOK.setMaximumSize(btnCancel.getPreferredSize());  
    label1 = new MtgLabel(frame, getString("PolygoneRegulierDlg2"), JLabel.RIGHT);
    label1.setRequestFocusEnabled(false);
    label2 = new MtgLabel(frame, getString("PolygoneRegulierDlg3"), JLabel.RIGHT);
    label2.setRequestFocusEnabled(false);
    NumberFormat format;
    format = NumberFormat.getInstance();
    format.setParseIntegerOnly(true);
    format.setGroupingUsed(false);
    editeurNombrePoints = new MtgFormattedTextField(frame, format);
    editeurNombrePoints.setColumns(3);
    
    JPanel panePrincipal = new JPanel();
    JPanel paneLeft = new JPanel();
    JPanel paneRight = new JPanel();
    panePrincipal.setLayout(new FlowLayout());
    paneLeft.setLayout(new GridBagLayout());
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 0;
    constraints.insets.top = 0;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneLeft.add(label1, constraints);	
    setConstraints(constraints, 0, 1, 1, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneLeft.add(editeurNombrePoints, constraints);	
    setConstraints(constraints, 0, 2, 1, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneLeft.add(label2, constraints);
    
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
    panePrincipal.add(paneLeft);
    panePrincipal.add(paneRight);
    setContentPane(panePrincipal);
    pack();
  }
    
  private void btnOKActionPerformed(java.awt.event.ActionEvent evt) {
  	try {
  		nombrePoints = Integer.parseInt(editeurNombrePoints.getText());
  	}
		catch (NumberFormatException e) {
			dialogueAvertissement(getString("ch71"));
			editeurNombrePoints.requestFocus();
			return;
		}
 	
    if ((nombrePoints < 2 ) || nombrePoints > 100) {
    	dialogueAvertissement(getString("ch71"));
    	editeurNombrePoints.requestFocus();
    	return;
    }
    setRetourOK(true);
    dispose();
  }
  
  private void btnCancelActionPerformed(java.awt.event.ActionEvent evt) {                                          
    dispose();
  }       
  
}
