/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import mathgraph32.MtgFrame;
import objets.CInversion;

@SuppressWarnings("serial")
public class RapportInversionDlg extends MtgDlg {
	
  private EditeurRapportHom editeurRapport;
  private MtgButton btnOK, btnCancel;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctions;  
  private CInversion inv;
  private boolean modification;

  public RapportInversionDlg(MtgFrame frameInit, CInversion invInit, boolean modificationInit) {
    super(getString("rapportInversionDlg1"), true, frameInit);
    modification = modificationInit;
    setResizable(false);
    inv = invInit;
    MtgLabel label = new MtgLabel(frame, getString("rapportHomothetieDlg2"), SwingConstants.LEFT);
    editeurRapport = new EditeurRapportHom(frame, inv.rapport, inv);    
    if (modification) editeurRapport.setText(inv.rapport.chaineInfo());
    btnOK = new MtgButton(frame, getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed(evt);
      }
    });    
    btnCancel = new MtgButton(frame, getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed(evt);
      }
    });
    btnOK.setPreferredSize(btnCancel.getPreferredSize());
  	int indmax;
  	if (modification) indmax = frame.cadreActif().listePr.indexOf(inv) - 1;
  	else indmax = frame.cadreActif().listePr.size() - 1;
    paneListeValeurEtFonctions = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurRapport, true, indmax, false);
    
    JPanel pane = new JPanel();
    pane.setLayout(new GridBagLayout());
    JPanel paneBas = new JPanel(); // Pour contenir les boutons OK et Cancel
    paneBas.setLayout(new GridBagLayout());
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 40, 40, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label, constraints);
    
    setConstraints(constraints, 1, 0, 1, 1, 60, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    constraints.insets.bottom = 0;
    pane.add(editeurRapport, constraints);
    constraints.insets.bottom = 5;
    
    setConstraints(constraints, 0, 1, 2, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.EAST);
    pane.add(paneListeValeurEtFonctions, constraints);

    setConstraints(constraints, 0, 0, 1, 1, 50, 0, GridBagConstraints.NONE, GridBagConstraints.EAST);
    paneBas.add(btnOK, constraints);

    setConstraints(constraints, 1, 0, 1, 1, 50, 0, GridBagConstraints.NONE, GridBagConstraints.CENTER);
    paneBas.add(btnCancel, constraints);
    
    setConstraints(constraints, 0, 2, 2, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.CENTER);
    pane.add(paneBas, constraints);
    
    setContentPane(pane);
    
    getRootPane().setDefaultButton(btnOK);    
    pack();
  }
  private void btnOKActionPerformed(java.awt.event.ActionEvent evt) {
    if (editeurRapport.verifieValidite()) {
      setRetourOK(true);
      dispose();
    }
  }
  
  private void btnCancelActionPerformed(java.awt.event.ActionEvent evt) {                                          
    dispose();
  }         
  
}
