/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.Vector;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CCalculBase;
import objets.CListeObjets;
import objets.CSolutionEquation;
import objets.CValeurAssocieeAVariable;
import types.Int;

@SuppressWarnings("serial")
public class SolutionEquationDlg extends MtgDlg {
	private CSolutionEquation solution; // L'objet qu'on modifie
	private boolean modification; // True si on modifie un objet dja cr
	private EditeurNomCalcul editeurNom;
	private MtgButton btnOK, btnCancel;
	private MtgTextField editeurNomVariable;
  private MtgTextField editeurFormule;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctions;  
	private EditeurValeurDynamique editeurValeura;
	private EditeurValeurDynamique editeurValeurb;
	private EditeurValeurDynamique editeurValeure;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsa;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsb;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionse;
	// On utilise des copies des valeurs dynamiques en cas d'abandon aprs avoir valid
	private CValeurAssocieeAVariable valeura, valeurb, valeure;
  private MtgFocusTraversalPolicy mtgFocusTraversalPolicy;
	
  public SolutionEquationDlg(MtgFrame frameInit, CSolutionEquation solutionInit, boolean modificationInit) {
    super(getString("solutionEquationDlg1"), true, frameInit);
    solution = solutionInit;
    modification = modificationInit;
    setResizable(false);
    MtgLabel label1 = new MtgLabel(frame, getString("solutionEquationDlg2"), SwingConstants.LEFT);
    MtgLabel label2 = new MtgLabel(frame, getString("solutionEquationDlg3"), SwingConstants.LEFT);
    MtgLabel label2Bis = new MtgLabel(frame, getString("solutionEquationDlg3Bis"), SwingConstants.LEFT);
    MtgLabel label3 = new MtgLabel(frame, getString("solutionEquationDlg4"), SwingConstants.LEFT);
    MtgLabel label4 = new MtgLabel(frame, getString("solutionEquationDlg5"), SwingConstants.LEFT);
    MtgLabel label5 = new MtgLabel(frame, getString("solutionEquationDlg6"), SwingConstants.LEFT);
    MtgLabel label6 = new MtgLabel(frame, getString("solutionEquationDlg7"), SwingConstants.LEFT);
    editeurNom = new EditeurNomCalcul(frame);
    CListeObjets liste = frame.cadreActif().listePr;
    valeura = new CValeurAssocieeAVariable(solution.a);
    valeurb = new CValeurAssocieeAVariable(solution.b);
    valeure = new CValeurAssocieeAVariable(solution.incertitude);
    editeurValeura = new EditeurValeurDynamique(frame, valeura, solution);
    editeurValeurb = new EditeurValeurDynamique(frame, valeurb, solution);
    editeurValeure = new EditeurValeurDynamique(frame, valeure, solution);
    editeurNomVariable = new MtgTextField(frame,8);
    int indmax;
    if (modification) indmax = liste.indexOf(solution) - 1;
    else indmax = liste.size() - 1;
    
    editeurFormule = new MtgTextField(frame, 30);
    editeurFormule.setAutoscrolls(true);
    paneListeValeurEtFonctions = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurFormule, true, indmax, false);    
    editeurNomVariable.setText(solution.inconnue);
    if (modification) {
    	String[] var = new String[1];
    	var[0] = solution.inconnue;
      editeurFormule.setText(solution.equation.chaineCalculSansPar(var));
    	editeurNom.setText(solution.nom());
    	editeurValeura.setText(solution.a.chaineInfo());
    	editeurValeurb.setText(solution.b.chaineInfo());
    	editeurValeure.setText(solution.incertitude.chaineInfo());
    }
    
    btnOK = new MtgButton(frame, getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed();
      }
    });    
    btnCancel = new MtgButton(frame, getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed();
      }
    });
    btnOK.setPreferredSize(btnCancel.getMaximumSize());
    paneListeValeurEtFonctionsa = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurValeura, true, indmax, false);
    paneListeValeurEtFonctionsb = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurValeurb, true, indmax, false);
    paneListeValeurEtFonctionse = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurValeure, true, indmax, false);
    
    JPanel pane = new JPanel();
    pane.setLayout(new GridBagLayout());

    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 2;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 4, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label1, constraints);
    setConstraints(constraints, 0, 1, 1, 1, 25, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label2, constraints);
    setConstraints(constraints, 1, 1, 1, 1, 50, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    pane.add(editeurNom, constraints);
    setConstraints(constraints, 0, 2, 1, 1, 25, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label2Bis, constraints);
    setConstraints(constraints, 1, 2, 1, 1, 50, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    pane.add(editeurNomVariable, constraints);
    setConstraints(constraints, 3, 1, 1, 1, 25, 0, GridBagConstraints.NONE, GridBagConstraints.EAST);
    pane.add(btnOK, constraints);
    setConstraints(constraints, 0, 3, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label3, constraints);
    setConstraints(constraints, 1, 3, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    pane.add(editeurFormule, constraints);
    constraints.insets.top = 0;
    setConstraints(constraints, 1, 4, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(paneListeValeurEtFonctions, constraints);
    setConstraints(constraints, 3, 2, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.EAST);
    pane.add(btnCancel, constraints);
    setConstraints(constraints, 0, 5, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    constraints.insets.top = 5;
    pane.add(label4, constraints);
    setConstraints(constraints, 1, 5, 1, 1, 0, 10, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    pane.add(editeurValeura, constraints);
    constraints.insets.top = 0;
    setConstraints(constraints, 1, 6, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(paneListeValeurEtFonctionsa, constraints);
    constraints.insets.top = 5;
    setConstraints(constraints, 0, 7, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label5, constraints);
    setConstraints(constraints, 1, 7, 1, 1, 0, 10, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    pane.add(editeurValeurb, constraints);
    constraints.insets.top = 0;
    setConstraints(constraints, 1, 8, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(paneListeValeurEtFonctionsb, constraints);
    constraints.insets.top = 5;
    setConstraints(constraints, 0, 9, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label6, constraints);
    setConstraints(constraints, 1, 9, 1, 1, 0, 10, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    pane.add(editeurValeure, constraints);
    constraints.insets.top = 0;
    setConstraints(constraints, 1, 10, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(paneListeValeurEtFonctionse, constraints);

    setContentPane(pane);

    // On change l'ordre par dfaut des composants
    Vector<Component> order = new Vector<Component>(7);
    order.add(editeurNom);
    order.add(editeurFormule);
    order.add(editeurValeura);
    order.add(editeurValeurb);
    order.add(editeurValeure);
    order.add(btnOK);
    order.add(btnCancel);
    mtgFocusTraversalPolicy = new MtgFocusTraversalPolicy(order);
    setFocusTraversalPolicy(mtgFocusTraversalPolicy);
    getRootPane().setDefaultButton(btnOK);    
    pack();
  }

  public void btnCancelActionPerformed() {
  	dispose();
  }
  
  public void btnOKActionPerformed() {
    MtgInternalFrame cadre = frame.cadreActif();
    CListeObjets listePr = cadre.listePr;
    String chNom = editeurNom.getText();
    // On regarde d'abord si le nom propos est correct et non vide en cas de cration d'un calcul
    if (!modification) {
	    // On vrifie d'abord la validit du nom de la fonction
	    if (!listePr.validationNomCalcul(frame, chNom)) {
	      editeurNom.requestFocus();
	      return;
	    }
    }
    if (editeurValeura.verifieValidite()) {
    	if (editeurValeurb.verifieValidite()) {
      	if (editeurValeure.verifieValidite()) {
      		Int inderr = new Int(0);
      		if (verifieFormules(editeurFormule.getText(), inderr)) {
		        setRetourOK(true);
		        if (!modification)solution.donneNom(chNom);
		        // On abandonne les objets CValeurAssocieeAVariable de pointAssocie pour les remplacer par les locaaux
		        solution.a = valeura;
		        solution.b = valeurb;
		        solution.incertitude = valeure;
		        dispose();
      		}
      		else {
        		editeurFormule.setCaretPosition(inderr.getValue());
      			editeurFormule.requestFocus();
      		}
      	}
    	}
    }
  }
    
  public boolean verifieFormules(String cheq, Int indiceErreur) {
 	 // On regarde d'abord si la syntaxe est correcte comme fonction de deux variables x et y
 	 String[] parametre = new String[1];
 	 parametre[0] = editeurNomVariable.getText();
   CListeObjets listeProprietaire = solution.listeProprietaire;
 	 int indiceMax = listeProprietaire.indexOf(solution);
 	 if (indiceMax == -1) indiceMax = listeProprietaire.size() - 1;
	 boolean synt = CCalculBase.verifieSyntaxe(listeProprietaire, cheq, indiceErreur, indiceMax, parametre);
	 if (!synt) {
		 dialogueAvertissement(getString("ch5"));
		 return false;
	 }
	 // Si la syntaxe est correcte il faut vrifier qu'il n'y a qu'un seul signe d'galit
	 int nbegal = 0;
	 int par = 0;
	 for (int i = 0; i < cheq.length(); i++) {
		 char car = cheq.charAt(i);
		 if (car == '(') par++;
		 else if (car == ')') par --;
		 else if ((car == '=') && (par == 0)) nbegal++;
		 if (nbegal > 1) {
			 indiceErreur.setValue(i);
			 break;
		 }
	 }
	 if (nbegal != 1) {
		dialogueAvertissement(getString("ch5"));
	 	return false;
	 }
 	 // La syntaxe est correcte. On cre la fonction en soustrayant le secod membre du premier
 	 String chfon = new String(cheq);
		 par = 0;
		 for (int i = 0; i < chfon.length(); i++) {
			 char car = chfon.charAt(i);
			 if (car == '(') par++;
			 else if (car == ')') par --;
			 else if ((car == '=') && (par == 0)) {
				 chfon = chfon.substring(0, i) + "-(" + chfon.substring(i+1) + ")";
				 break;
			 }
		 }
		 CCalculBase fonction1 = CCalculBase.creeCalculBase(chfon, listeProprietaire, 0, chfon.length() - 1, parametre);
		 // Il faut vrifier si fonction1 dpend bien de de l'iconnue
		 if (!fonction1.dependDeVariable(0)) {
 			dialogueAvertissement(getString("ch185"));			 
			 return false;
		 }
		 // Il faut vrifier que c'est bien une quation de droite qui a t entre.
		 // Pour cela on cre les drives partielles de fonction1 par rapport  x et y et on vrifie que ces drives
		 // partielles ne dpendent ni de x ni de y
		 solution.equation = CCalculBase.creeCalculBase(cheq, listeProprietaire, 0, cheq.length() - 1, parametre);
		 solution.fonction = fonction1;
		 return true;
  }  
}
