/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.Vector;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CFoncComplexeNVar;
import objets.CListeObjets;
import objets.CSuiteRecComplexe2;
import objets.CValeurAssocieeAVariable;
import objets.CValeurAssocieeComp;
import types.NatureCalcul;

// Bote de dialogue pour crer ou modifier une suite rcurrente complexe u(n+1)=f[n,u(n)]
@SuppressWarnings("serial")
public class SuiteRecurrenteComplexe2Dlg extends MtgDlg {
	private boolean modification;
	private CSuiteRecComplexe2 suiteAssociee;
	private MtgLabel label1, label2, label3, label4, label5, label6;
	private MtgTextField editeurNom;
	private MtgTextArea info;
  private MtgButton btnOK, btnCancel;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsPremierTerme;  
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsNombreTermes;  

  private EditeurValeurComplexe editeurValeurPremierTerme;
  private EditeurNombreTermesSuiteRec editeurValeurNombreTermes;
  private ListeObjetsParNatureCalcul listeFonctions;
  private CValeurAssocieeComp valPremierTerme;
  private CValeurAssocieeAVariable valNombreTermes;
  private MtgFocusTraversalPolicy mtgFocusTraversalPolicy;

  public SuiteRecurrenteComplexe2Dlg(MtgFrame frameInit, CSuiteRecComplexe2 suiteAssocieeInit, boolean modificationInit) {
  	super(getString("suiteRecComplexe2Dlg1"), true, frameInit);
  	suiteAssociee = suiteAssocieeInit;
  	modification = modificationInit;
  	initComponents();
  	getRootPane().setDefaultButton(btnOK);      	
  }
  
  private void initComponents() {
    label1 = new MtgLabel(frame, getString("suiteRecReelleDlg2"), JLabel.RIGHT);
    label1.setRequestFocusEnabled(false);
    label2 = new MtgLabel(frame, getString("suiteRecReelleDlg3"), JLabel.RIGHT);
    label2.setRequestFocusEnabled(false);
    label3 = new MtgLabel(frame, getString("informationFonction"), JLabel.RIGHT);
    label3.setRequestFocusEnabled(false);
    label4 = new MtgLabel(frame, getString("suiteRecReelleDlg4"), JLabel.RIGHT);
    label4.setRequestFocusEnabled(false);
    label5 = new MtgLabel(frame, getString("suiteRecReelleDlg5"), JLabel.RIGHT);
    label5.setRequestFocusEnabled(false);
    label6 = new MtgLabel(frame, getString("suiteRecReelleDlg6"), JLabel.RIGHT);
    label6.setRequestFocusEnabled(false);
    editeurNom = new MtgTextField(frame,8);
    info = new MtgTextArea(frame);
    info.setEditable(false);
    info.setColumns(30);
    info.setRows(1);
    // Modifi version 4.8.0
    // JScrollPane scrollPaneInfo = new JScrollPane(info, JScrollPane.VERTICAL_SCROLLBAR_NEVER, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    JScrollPane scrollPaneInfo = new JScrollPane(info, JScrollPane.VERTICAL_SCROLLBAR_NEVER, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    
    //Quand une bote de dialogue contient deux champs associs  des valeurs dynamiques il faut
    // utiliser une copie de ces champs car sinon une des valeurs peut tre modifie quand l'utilisateur
    // quitte la bote de dialogue par escape
    CListeObjets liste = frame.cadreActif().listePr;
    int indmax;
    if (modification) {
    	indmax = liste.indexOf(suiteAssociee) - 1;
    	listeFonctions = new ListeObjetsParNatureCalcul(frame, liste, indmax, NatureCalcul.NFonctionComplexe2Var, false, 10);
      valPremierTerme = new CValeurAssocieeComp(suiteAssociee.premierTerme);
      valNombreTermes = new CValeurAssocieeAVariable(suiteAssociee.nombreTermes);
    }
    else {
    	indmax = liste.size() - 1;
    	listeFonctions = new ListeObjetsParNatureCalcul(frame, liste, indmax, NatureCalcul.NFonctionComplexe2Var, false, 10);
      valPremierTerme = new CValeurAssocieeComp(liste);
      valNombreTermes = new CValeurAssocieeAVariable(liste);
    }
    editeurValeurPremierTerme = new EditeurValeurComplexe(frame, valPremierTerme, suiteAssociee);
    editeurValeurNombreTermes = new EditeurNombreTermesSuiteRec(frame, valNombreTermes, suiteAssociee);
    listeFonctions.addListSelectionListener(new ListSelectionListener() {
    public void valueChanged(ListSelectionEvent evt) {
      	listeFonctionsSelectionChanged();
      }
    });
    if (modification)  {
    	editeurNom.setEditable(false);
    	editeurNom.setText(suiteAssociee.nom());
    	listeFonctions.setSel(suiteAssociee.fonctionAssociee);
    	editeurValeurPremierTerme.setText(suiteAssociee.premierTerme.chaineInfo());
    	editeurValeurNombreTermes.setText(suiteAssociee.nombreTermes.chaineInfo());
   }
    
    JScrollPane scrollPaneListe = new JScrollPane(listeFonctions, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    
    paneListeValeurEtFonctionsPremierTerme = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurValeurPremierTerme, false, indmax, false);
    paneListeValeurEtFonctionsNombreTermes = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurValeurNombreTermes, true, indmax, false);
    btnOK = new MtgButton(frame, getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed();
      }
    });    
    btnCancel = new MtgButton(frame, getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed();
      }
    });
    btnOK.setMaximumSize(btnCancel.getPreferredSize());
    
    JPanel panePrincipal = new JPanel();
    JPanel paneRight = new JPanel();
    panePrincipal.setLayout(new GridBagLayout());
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 30, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label1, constraints);
    
    setConstraints(constraints, 1, 0, 1, 1, 50, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(editeurNom, constraints);    
    
    setConstraints(constraints, 0, 1, 1, 1, 0, 40, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label2, constraints);

    setConstraints(constraints, 1, 1, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    panePrincipal.add(scrollPaneListe, constraints);
    
    setConstraints(constraints, 0, 2, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label3, constraints);

    setConstraints(constraints, 1, 2, 2, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(scrollPaneInfo, constraints);
    
    setConstraints(constraints, 0, 3, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label4, constraints);

    setConstraints(constraints, 1, 3, 2, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(editeurValeurPremierTerme, constraints);
    
    setConstraints(constraints, 1, 4, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(paneListeValeurEtFonctionsPremierTerme, constraints);

    setConstraints(constraints, 0, 5, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label5, constraints);

    setConstraints(constraints, 1, 5, 2, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(editeurValeurNombreTermes, constraints);
    
    setConstraints(constraints, 0, 6, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label6, constraints);

    setConstraints(constraints, 1, 6, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(paneListeValeurEtFonctionsNombreTermes, constraints);
    
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
    setConstraints(constraints, 2, 0, 1, 2, 20, 0, GridBagConstraints.NONE, GridBagConstraints.NORTH);
    panePrincipal.add(paneRight, constraints);
    
    setContentPane(panePrincipal);
    // On change l'ordre par dfaut des composants
    Vector<Component> order = new Vector<Component>(6);
    order.add(editeurNom);
    order.add(listeFonctions);
    order.add(editeurValeurPremierTerme);
    order.add(editeurValeurNombreTermes);
    order.add(btnOK);
    order.add(btnCancel);
    mtgFocusTraversalPolicy = new MtgFocusTraversalPolicy(order);
    setFocusTraversalPolicy(mtgFocusTraversalPolicy);
    listeFonctionsSelectionChanged();
    pack(); 
  }
 
  private void btnCancelActionPerformed() {                                          
    dispose();
  }       
  
  private void btnOKActionPerformed() {
    MtgInternalFrame cadre = frame.cadreActif();
    CListeObjets listePr = cadre.listePr;
    String chNom = editeurNom.getText();
    if (!modification) {
	    // On vrifie d'abord la validit du nom de la fonction
	    if (!listePr.validationNomCalcul(frame, chNom)) {
	      editeurNom.requestFocus();
	      return;
	    }
    }
    // Le nom est correct. On regarde les champs d'entre
    if (editeurValeurPremierTerme.verifieValidite()) {
    	if (editeurValeurNombreTermes.verifieValidite()) {
        // On abandonne les objets CValeurAssocieeAVariable de pointAssocie pour les remplacer par les locaaux
    		suiteAssociee.nomCalcul = chNom;
        suiteAssociee.premierTerme = valPremierTerme;
        suiteAssociee.nombreTermes = valNombreTermes;
        suiteAssociee.fonctionAssociee = (CFoncComplexeNVar) listeFonctions.elementSelectionne();
        setRetourOK(true);
        dispose();
    	}
    }
    
  	
  }
  
  public void listeFonctionsSelectionChanged() {
  	int i = listeFonctions.getSelectedIndex();
  	if (i != -1) {
  		info.setText(((CFoncComplexeNVar)listeFonctions.elementSelectionne()).infoFormule());
  		info.setCaretPosition(0);
  		pack();
  	}
  }

}
