/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import objets.CCalcul;
import objets.CListeObjets;
import objets.CTestEquivalence;
import types.NatureCalcul;

import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

@SuppressWarnings("serial")
public class TestEquivalenceDlg extends MtgDlg {
	private CTestEquivalence testAssocie;
	private boolean modification;
	private MtgLabel label1, label2, label3;
	private MtgTextField editeurNom;
	private ListeCalculsOuFonctionsParFormule listeGauche, listeDroite;
	private JScrollPane scrollPaneListeGauche, scrollPaneListeDroite;
	private MtgButton btnOK, btnCancel;
	private MtgCheckBox cbRemplacementValeurs1, cbRemplacementValeurs2, cbEquivalenceDecimalFracIrr;
	// Ajout version 4.9.8
	private EditeurValeurDynamique editeurTest;

	public TestEquivalenceDlg(MtgFrame frameInit, CTestEquivalence testAssocieInit, boolean modificationInit) {
		super(getString("testEquivalenceDlg1"), true, frameInit);
		testAssocie = testAssocieInit;
		modification = modificationInit;
    initComponents();
    getRootPane().setDefaultButton(btnOK);
    pack();
	}
	
	private void initComponents() {
  	MtgInternalFrame cadre = frame.cadreActif();
  	CListeObjets liste = cadre.listePr;
		label1 = new MtgLabel(frame, getString("creerCalculDlg1"));
		label2 = new MtgLabel(frame, getString("testEquivalenceDlg2"));
		// Ajout version 4.9.8
		label3 = new MtgLabel(frame, getString("testEquivalenceDlg4")+" ");
    editeurTest = new EditeurValeurDynamique(frame, testAssocie.test, testAssocie);
    editeurTest.setText(testAssocie.test.chaineInfo());
    //
    editeurNom = new MtgTextField(frame,8);
    cbRemplacementValeurs1 = new MtgCheckBox(frame, getString("testEquivalenceDlg3"));
    cbRemplacementValeurs2 = new MtgCheckBox(frame, getString("testEquivalenceDlg3"));
    cbEquivalenceDecimalFracIrr = new MtgCheckBox(frame, getString("testEquivalenceDlg5"));
    
    int indmax;
    if (modification) indmax = liste.indexOf(testAssocie) - 1;
    else indmax = liste.size() - 1;
    // Ajout version 4.9.8
    PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctions = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurTest, true, indmax, false);    //
    //
    listeGauche = new ListeCalculsOuFonctionsParFormule(frame, liste, indmax, false, NatureCalcul.NCalculOuFonctionParFormule);
		listeDroite = new ListeCalculsOuFonctionsParFormule(frame, liste, indmax, false, NatureCalcul.NCalculOuFonctionParFormule);
    scrollPaneListeGauche = new JScrollPane(listeGauche, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
    		JScrollPane.HORIZONTAL_SCROLLBAR_NEVER );	
    scrollPaneListeDroite = new JScrollPane(listeDroite, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
    		JScrollPane.HORIZONTAL_SCROLLBAR_NEVER );	
		listeGauche.addListSelectionListener(new ListSelectionListener() {
			
			public void valueChanged(ListSelectionEvent e) {
				listeGaucheValueChanged();
				
			}
		});
	  btnOK = new MtgButton(frame, getString("btnOK"));
	  btnOK.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnOKActionPerformed();
	    }
	  });    
	  btnCancel = new MtgButton(frame, getString("btnCancel"));
	  btnCancel.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnCancelActionPerformed();
	    }
	  });
	  btnOK.setMaximumSize(btnCancel.getPreferredSize());
		if (modification) {
      editeurNom.setText(testAssocie.nom());
      editeurNom.setEditable(false);
			listeGauche.select(testAssocie.calcul1);
			listeDroite.select(testAssocie.calcul2);
			listeGauche.ensureIndexIsVisible(listeGauche.getSelectedIndex());
			listeDroite.ensureIndexIsVisible(listeDroite.getSelectedIndex());
		}
		else {
	    listeDroite.metAJour(listeGauche.calculSelectionne().getNatureCalcul(), listeGauche.calculSelectionne());
			listeGaucheValueChanged(); // Ajout version 4.9.8
		}
		cbRemplacementValeurs1.setSelected(testAssocie.remplacementValeurs1);
		cbRemplacementValeurs2.setSelected(testAssocie.remplacementValeurs2);
		cbEquivalenceDecimalFracIrr.setSelected(testAssocie.equivalenceDecimalFracIrr);
    JPanel panePrincipal = new JPanel();
    panePrincipal.setLayout(new GridBagLayout());
    JPanel paneRight = new JPanel();
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets.bottom = 2;
		constraints.insets.top = 2;
		constraints.insets.left = 5;
		constraints.insets.right = 5;
		
		JPanel paneCenter = new JPanel();
		paneCenter.setLayout(new GridBagLayout());
		setConstraints(constraints, 0, 0, 1, 1, 50, 90, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		paneCenter.add(scrollPaneListeGauche, constraints);
		setConstraints(constraints, 1, 0, 1, 1, 50, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		paneCenter.add(scrollPaneListeDroite, constraints);
		setConstraints(constraints, 0, 1, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.CENTER);
 		paneCenter.add(cbRemplacementValeurs1, constraints);
 		setConstraints(constraints, 1, 1, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.CENTER);
 		paneCenter.add(cbRemplacementValeurs2, constraints);
 		setConstraints(constraints, 0, 2, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.CENTER);
 		paneCenter.add(cbEquivalenceDecimalFracIrr, constraints);
		
		
		setConstraints(constraints, 0, 0, 1, 1, 50, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label1, constraints);
		setConstraints(constraints, 1, 0, 1, 1, 50, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(editeurNom, constraints);
		setConstraints(constraints, 0, 1, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label2, constraints);
		setConstraints(constraints, 0, 2, 2, 1, 0, 50, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(paneCenter, constraints);	
    // Ajout version 4.9.8
    JPanel paneEditeurTest = new JPanel();
    paneEditeurTest.setLayout(new BoxLayout(paneEditeurTest, BoxLayout.X_AXIS));
    paneEditeurTest.add(label3);
    paneEditeurTest.add(editeurTest);
    constraints.insets.bottom = 0;
    setConstraints(constraints, 0, 6, 2, 1, 60, 0, GridBagConstraints.NONE, GridBagConstraints.EAST);
    panePrincipal.add(paneEditeurTest, constraints);
    constraints.insets.bottom = 5;
    setConstraints(constraints, 0, 7, 2, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.EAST);
    panePrincipal.add(paneListeValeurEtFonctions, constraints);
    //
 		setConstraints(constraints, 2, 0, 1, 2, 20, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(paneRight, constraints);
		setContentPane(panePrincipal);	  
	}
	
	private void btnOKActionPerformed() {
    MtgInternalFrame cadre = frame.cadreActif();
    CListeObjets listePr = cadre.listePr;
    String chNom = editeurNom.getText();
    if (editeurTest.verifieValidite()) {
	    if (!modification) {
		    // Modifi version 4.6
		    if (!listePr.validationNomCalcul(frame, chNom)) {
		      editeurNom.requestFocus();
		      return;
		    }
		    testAssocie.nomCalcul = chNom;
	    }
			testAssocie.calcul1 = listeGauche.calculSelectionne();
			testAssocie.calcul2 = listeDroite.calculSelectionne();
			testAssocie.remplacementValeurs1 = cbRemplacementValeurs1.isSelected();
			testAssocie.remplacementValeurs2 = cbRemplacementValeurs2.isSelected();
			testAssocie.equivalenceDecimalFracIrr = cbEquivalenceDecimalFracIrr.isSelected();
			setRetourOK(true);
			dispose();
    }
	}
	
	private void btnCancelActionPerformed() {
		dispose();
	}
	
	private void listeGaucheValueChanged() {
		CCalcul calc = listeGauche.calculSelectionne();
		listeDroite.metAJour(calc.getNatureCalcul(), calc);
		// Les deux lignes suivantes ajoutes pour version 4.7.4.1
		// car quand on changeait le choix de gauche, rien n'tait slectionn  droite
		listeDroite.setSelectedIndex(0);
		listeDroite.ensureIndexIsVisible(0);
	}
	

}
