/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import javax.swing.JDialog;
import mathgraph32.MtgFrame;

@SuppressWarnings("serial")
public class TextAreaLatexAvecCarSpeciaux extends MtgTextArea implements FocusListener, KeyListener {
	private MtgFrame frame;
	private JDialog owner; // La bote de dialogue qui contient le champ
	private int natureCarPermis; // CF NatureCarSpe donne quels sont les sets de caractres autoriss pour ce champ
	CaracteresSpeciauxLatexDlg caracteresSpeciauxDlg;
	
	public TextAreaLatexAvecCarSpeciaux(JDialog ownerInit, MtgFrame frameInit, int natureCarPermisInit) {
		super(frameInit);
		owner = ownerInit;
		frame = frameInit;
		natureCarPermis = natureCarPermisInit;
		addFocusListener(this);
		addKeyListener(this);
		caracteresSpeciauxDlg = null;
	}

	public void focusGained(FocusEvent e) {
		if (caracteresSpeciauxDlg == null) caracteresSpeciauxDlg = 
			new CaracteresSpeciauxLatexDlg(owner, MtgFrame.getString("ch84"), frame, natureCarPermis);		
		if (!caracteresSpeciauxDlg.isVisible()) {
			caracteresSpeciauxDlg.positionneParRapportDialoguePere();
			caracteresSpeciauxDlg.AssocieA(this);
			caracteresSpeciauxDlg.setVisible(true);
		}
		requestFocus();
	}

	public void focusLost(FocusEvent e) {
		if (!e.isTemporary()) {
			caracteresSpeciauxDlg.setVisible(false);
			caracteresSpeciauxDlg = null;
		}
	}
	
	private final void setCaretPositionApresAccoladeSuivante() {
		int cp = getCaretPosition();
		String ch = getText();
		int len = ch.length();
		int i;
		for (i = cp; (i < len) && (ch.charAt(i) != '{'); i++);
		if (i < len) setCaretPosition(i+1);
		// Ajout version 4.7.4
		else {
			int ind = ch.indexOf('}', cp);
			if (ind != -1) setCaretPosition(ind+1);
		}
	}

	/**
	 * Si un caractre TAB est tap, on annule son traitement (insertion par dfaut d'espace
	 */
	public void keyPressed(KeyEvent e) {
			if (e.getKeyChar() == '\t') e.consume();
	}

	public void keyReleased(KeyEvent e) {
	}

	/**
	 * Si un caractre TAB est tap, on annule son traitement (insertion par dfaut d'espace) dans keyPressed
	 * et on se positionne aprs l'accolade suivante s'il en existe une
	 */
	public void keyTyped(KeyEvent e) {
		if (e.getKeyChar() == '\t') {
			e.consume();
			setCaretPositionApresAccoladeSuivante();
		}
	}
}