/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;


import glob.*;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import objets.CListeObjets;
import objets.CVariableBornee;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

/**
 * Dialogue permettant de crer ou modifier une variable
 * @author Yves
 *
 */
@SuppressWarnings("serial")
public class VariableDlg extends MtgDlg {
	private CVariableBornee variableAModifier;
	private boolean modification; // True si on modifie un calcul dj cr
	private MtgButton btnOK;
	private MtgButton btnCancel;
  private MtgLabel label1;
  private MtgLabel label2;
  private MtgLabel label3;
  private MtgLabel label4;
  private MtgLabel label5;
  private MtgCheckBox checkDlgAssocie;
  private MtgTextField editeurNom;
  private EditeurValeurConstante editeurValeurMini;
  private EditeurValeurConstante editeurValeurMaxi;
  private EditeurValeurConstante editeurValeurPas;
  public EditeurValeurConstante editeurValeurActuelle;
  
  public VariableDlg(MtgFrame frameInit, CVariableBornee variableAModifierInit, boolean modificationInit) {
  	super(getString("variableDlg1"), true, frameInit);
  	variableAModifier = variableAModifierInit;
  	modification = modificationInit;
    initComponents();
    getRootPane().setDefaultButton(btnOK);    
  }
  
  private void initComponents() {
    btnOK = new MtgButton(frame, getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed(evt);
      }
    });    
    btnCancel = new MtgButton(frame, getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed(evt);
      }
    });
    btnOK.setMaximumSize(btnCancel.getPreferredSize());
    label1 = new MtgLabel(frame, getString("variableDlg2"), JLabel.RIGHT);
    label1.setRequestFocusEnabled(false);
    label2 = new MtgLabel(frame, getString("variableDlg3"), JLabel.RIGHT);
    label2.setRequestFocusEnabled(false);
    label3 = new MtgLabel(frame, getString("variableDlg4"), JLabel.RIGHT);
    label3.setRequestFocusEnabled(false);
    label4 = new MtgLabel(frame, getString("variableDlg5"), JLabel.RIGHT);
    label4.setRequestFocusEnabled(false);
    label5 = new MtgLabel(frame, getString("variableDlg6"), JLabel.RIGHT);
    label5.setRequestFocusEnabled(false);
    editeurNom = new MtgTextField(frame);
    editeurNom.setColumns(8);
    editeurValeurMini = new EditeurValeurConstante(frame);
    editeurValeurMaxi = new EditeurValeurConstante(frame);
    editeurValeurPas = new EditeurValeurConstante(frame);
    editeurValeurActuelle = new EditeurValeurConstante(frame);
    checkDlgAssocie = new MtgCheckBox(frame, getString("variableDlgCheck"));
    if (modification) {
    	editeurNom.setText(variableAModifier.nom());
    	editeurNom.setEditable(false);
    	editeurValeurMini.setText(variableAModifier.chaineValeurMini);
    	editeurValeurMaxi.setText(variableAModifier.chaineValeurMaxi);
    	editeurValeurPas.setText(variableAModifier.chaineValeurPas);
    	editeurValeurActuelle.setText(Kernel.chaineNombre(variableAModifier.valeurActuelle,12));
    	checkDlgAssocie.setSelected(variableAModifier.dialogueAssocie);
    }
    JPanel panePrincipal = new JPanel();
    JPanel paneLeft = new JPanel();
    JPanel paneRight = new JPanel();
    panePrincipal.setLayout(new FlowLayout());
    paneLeft.setLayout(new GridBagLayout());
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 30, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneLeft.add(label1, constraints);
    setConstraints(constraints, 1, 0, 1, 1, 70, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneLeft.add(editeurNom, constraints);  
    setConstraints(constraints, 0, 1, 1, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.EAST);
    paneLeft.add(label2, constraints);  
    setConstraints(constraints, 1, 1, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneLeft.add(editeurValeurMini, constraints);
	  setConstraints(constraints, 0, 2, 1, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.EAST);
	  paneLeft.add(label3, constraints);  
	  setConstraints(constraints, 1, 2, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  paneLeft.add(editeurValeurMaxi, constraints);
	  setConstraints(constraints, 0, 3, 1, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.EAST);
	  paneLeft.add(label4, constraints);  
	  setConstraints(constraints, 1, 3, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  paneLeft.add(editeurValeurPas, constraints);
	  setConstraints(constraints, 0, 4, 1, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.EAST);
	  paneLeft.add(label5, constraints);  
	  setConstraints(constraints, 1, 4, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  paneLeft.add(editeurValeurActuelle, constraints);
	  setConstraints(constraints, 0, 5, 1, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.EAST);
	  paneLeft.add(checkDlgAssocie, constraints);  
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
    panePrincipal.add(paneLeft);
    panePrincipal.add(paneRight);
    setContentPane(panePrincipal);
    pack();	  
	}
  
  private void btnCancelActionPerformed(java.awt.event.ActionEvent evt) {                                          
    dispose();
  }         
  
  private void btnOKActionPerformed(java.awt.event.ActionEvent evt) {
    String chNom = editeurNom.getText();
    CListeObjets liste = frame.cadreActif().listePr;
    // On regarde d'abord si le nom propos est correct et non vide en cas de cration d'un calcul
    if (!modification) {
    	// Modifi version 4.5.0.2
	    if (!liste.validationNomCalcul(frame, chNom)) {
	      editeurNom.requestFocus();
	      return;
	    }
    }
    //Le nom est correct. On vrifie maintenant le calcul
    if (!editeurValeurMini.verifieValidite()) {
    	editeurValeurMini.requestFocus();
    	return;
    }
    if (!editeurValeurMaxi.verifieValidite()) {
    	editeurValeurMaxi.requestFocus();
    	return;
    }
    if (!editeurValeurPas	.verifieValidite()) {
    	editeurValeurPas.requestFocus();
    	return;
    }
    if (!editeurValeurActuelle.verifieValidite()) {
    	editeurValeurActuelle.requestFocus();
    	return;
    }
    //On regarde si les valeurs donnes sont bien dans un ordre compatible
    double vMini = editeurValeurMini.resultat;
    double vMaxi = editeurValeurMaxi.resultat;
    double vPas = editeurValeurPas.resultat;
    double vActuelle = editeurValeurActuelle.resultat;
    if (vMini >= vMaxi ) {
    	dialogueAvertissement(getString("variableDlg8"));
    	editeurValeurMini.requestFocus();
    	return;
    }
    if (((vPas > vMaxi-vMini) || (vPas <= 0)) ) {
    	dialogueAvertissement(getString("variableDlg9"));
    	editeurValeurPas.requestFocus();
    	return;
    }
  	if ((vActuelle < vMini) || (vActuelle > vMaxi)) {
  		dialogueAvertissement(getString("variableDlg10"));
    	editeurValeurActuelle.requestFocus();
    	return;  		
  	}
  	boolean dlgAssocieModifie = (variableAModifier.dialogueAssocie != checkDlgAssocie.isSelected());
  	// boolean valeurActuelleModifiee = variableAModifier.valeurActuelle != vActuelle;
  	// Tout est correct
  	if (!modification) variableAModifier.nom = editeurNom.getText();
  	variableAModifier.chaineValeurMini = editeurValeurMini.getText();
  	variableAModifier.valeurMini = vMini;
  	variableAModifier.chaineValeurMaxi = editeurValeurMaxi.getText();
  	variableAModifier.valeurMaxi = vMaxi;
  	variableAModifier.chaineValeurPas = editeurValeurPas.getText();
  	variableAModifier.valeurPas = vPas;
  	variableAModifier.valeurActuelle = vActuelle;
  	variableAModifier.dialogueAssocie = checkDlgAssocie.isSelected();
  	MtgInternalFrame cadre = frame.cadreActif();
  	// Mmorisaton pour annuler l'action
    if (modification) cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil48"));
  	if (modification && (dlgAssocieModifie)) cadre.creePaneAffichageVariables();
  	setRetourOK(true);
  	dispose();
  	if (modification) {
  		cadre.repaintFigureAvecCalcul();
  	}
  } 
}
