/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/** Classe permettant les calculs mathmatiques globaux */

package glob;

import java.awt.*;
import java.awt.geom.*;
import java.util.Locale;
import java.util.StringTokenizer;
import java.text.*;
import javax.swing.JLabel;
import org.scilab.forge.jlatexmath.TeXConstants;
import org.scilab.forge.jlatexmath.TeXFormula;
import org.scilab.forge.jlatexmath.TeXIcon;
import objets.*;
import types.*;

public class Kernel {

  // Dfinition de certaines valeurs trs utilises
  public static final double ConvDegRad = Math.PI/180d;
  public static final double ConvRadDeg = 180d/Math.PI;
  public static final double cos30 = Math.sqrt(3)/2d;
  public static final double sin30 = 0.5;
  
    
  // Modific version 3.2.0
  public static final double distancePointPoint(double x1,double y1,double x2,double y2)
  {
    // VecteurDirecteur u = new VecteurDirecteur(x1, y1, x2, y2);
    // return u.norme();
    return Math.sqrt((x2-x1)*(x2-x1) + (y2-y1)*(y2-y1));
  }
  
  // Ajout version 3.2.0
  public static final double distancePointPointCarre(double x1,double y1,double x2,double y2)
  {
    return (x2-x1)*(x2-x1) + (y2-y1)*(y2-y1);
  }
  
  public static final double distancePointDroite(double xp,double yp,double xd,double yd,
    VecteurDirecteur u)
  {
    return Math.abs(u.y*(xd-xp)-u.x*(yd-yp))/Math.sqrt(u.x*u.x+u.y*u.y);
  }
  public static final void intersectionDroitesSecantes(double p1_x,double p1_y,
        VecteurDirecteur u1,double p2_x,double p2_y,
        VecteurDirecteur u2, Point2D.Double res)
  {
    double d,k1,k2;

    d = u1.x*u2.y-u2.x*u1.y;
    k1 = u1.y*p1_x-u1.x*p1_y;
    k2 = u2.y*p2_x-u2.x*p2_y;
    res.x = (u1.x*k2-u2.x*k1)/d;
    res.y = (u1.y*k2-u2.y*k1)/d;
  }
  
  public static final void projetteOrtho(double x1, double y1,double p1_x,double p1_y
                     ,VecteurDirecteur u,Point2D.Double res)
  {
    VecteurDirecteur v = new VecteurDirecteur();
    // orthogonal(u,v); // Modifi version 4.7.6
    u.getOrthogonal(v);
    intersectionDroitesSecantes(x1,y1,v,p1_x,p1_y,u,res);
  }

  /**
   * Renvoie true si la valeur absolue du rsultat est strictement infrieure  10^-9
   * @param r
   * @return
   */
  public static final boolean zero(double r)
  {
    return (Math.abs(r)<1e-9);
  }
  
  /**
   * Renvoie true si la valeur absolue du rsultat est strictement infrieure  10^-13
   * @param r
   * @return
   */
  public static final boolean zero13(double r)
  {
    return (Math.abs(r)<1e-13);
  }
  // Remani pour la version 1.9
  // On teste la nullit du sinus plutot que celle du dterminant
  /* Ancienne version
  static final boolean colineaires(VecteurDirecteur u, VecteurDirecteur v)
  {
    return (zero(u.x*v.y-u.y*v.x));
  }
  */
  public static final boolean colineaires(VecteurDirecteur u, VecteurDirecteur v)
  {
    double n1 = u.norme();
    double n2 = v.norme();
    // Modifi version 3.4.2.1
    // if ((n1 == 0) || (n2 == 0)) return true;
    if (zero(n1) || zero(n2)) return true;
    else return (zero((u.x*v.y-u.y*v.x)/n1/n2));
  }
  
  // Rajout pour la version 2.0 de l'applet pour vrifier si deux vecteurs sont colinaires
  // et de mme sens. Est utilis dans coincideAvec de l'objet CDemiDroite
  public static final boolean colineairesMemeSens(VecteurDirecteur u, VecteurDirecteur v)
  {
    return (colineaires(u, v) && (u.x*v.x + u.y*v.y >= 0));
  }
  

  // Test pour savoir si des coordonnes peuvent bien tre converties en int
  public static final boolean testToile(double x, double y) {
    // A cause d'un bug de la version 1.4 qui plante pour des segments de grandes dimensions,
    // je restreins testToile  short
    //return ((x<=Integer.MAX_VALUE) && (y<=Integer.MAX_VALUE) 
    //  && (x>=Integer.MIN_VALUE) && (y>=Integer.MIN_VALUE));
    return ((x<=Short.MAX_VALUE) && (y<=Short.MAX_VALUE) 
      && (x>=Short.MIN_VALUE) && (y>=Short.MIN_VALUE));
  }

  // Modifi version 3.2.4.1 pour ne plus tre cacul qu' epsilon prs
  public static final boolean appartientSegment(double x,double y,double p1x,double p1y,
                         double p2x,double p2y) {
    double k;
    double absx,absy;

    double difx = p2x-p1x;
    absx = Math.abs(difx);
    double dify = p2y-p1y;
    absy = Math.abs(dify);
    if (zero(difx) && zero(dify))
    {
      return (zero(x - p1x) && zero(y - p1y));
    }
    else
    {
      if (!(aligne(p1x,p1y,p2x,p2y,x,y)))
      return false;
      else
      {
        // Nouvelle modif
        if (absy > absx)
          k = (y-p1y)/dify;
        else
        {
          k = (x-p1x)/difx;
        }
        // Modification apporte pour la version 1.9.5 pour compter avec les erreurs d'arrondi
        return ((k>=0) && (k<=1)) || zero(k) || zero(1-k);
      }
    }
  }

  public static final boolean aligne(double xa,double ya,double xb,double yb,
              double xc,double yc) {
    double k,nu,nv;
    VecteurDirecteur u = new VecteurDirecteur();
    VecteurDirecteur v = new VecteurDirecteur();

    u.x = xb-xa;
    u.y = yb-ya;
    v.x = xc-xa;
    v.y = yc-ya;
    nu = u.norme();
    nv = v.norme();
    // Modification apporte lors de la version 1.9.5
    //if ((nu == 0) || (nv == 0))
    if (zero(nu) || zero(nv))    
      return true;
    else
    {
      k = (u.x*v.y-v.x*u.y)/nu/nv;
      return (zero(k));
    }   
  }
  
  public static final boolean egal(CPointAncetre A, CPointAncetre B)
  {
    return ((A.x == B.x) && (A.y == B.y));
  }

  
  public static void traceQuadrilaterePlein(Graphics2D pDC, double x1, double y1, double x2, double y2,
  		double x3, double y3, double x4, double y4) {
  	int[] x = new int[5];
  	int[] y = new int[5];
  	x[0] = (int) x1;
   	y[0] = (int) y1;
  	x[1] = (int) x2;
   	y[1] = (int) y2;
  	x[2] = (int) x3;
   	y[2] = (int) y3;
  	x[3] = (int) x4;
   	y[3] = (int) y4;
  	x[4] = (int) x1;
   	y[4] = (int) y1;
  	Polygon poly = new Polygon(x, y, 5);
  	pDC.fillPolygon(poly);
  }
  public static final boolean zeroAngle(double r)
  {
      return (Math.abs(r)<1e-7);
  }
  
  
  public static final boolean testAngleDroit(double angle1,double angle2)
  {
    double dif;
    dif = Math.abs(angle2 - angle1);
    return ((zeroAngle(dif - Math.PI/2)) || (zeroAngle(dif - 3*Math.PI/2)));
  }
   
  public static final void secteurSaillant(Graphics2D pDC,double xa,double ya,double xo,double yo,
            double rayon,double angle)
  {
    double xg,yg;	/// Coordonnes du sommet gauche du carr devant contenir le cercle
    int width;	/// Cot du carr contenant le cercle

    VecteurDirecteur u = new VecteurDirecteur(xo, yo, xa, ya);
    double angleDegreu = u.angleRad()/Math.PI*180;
    xg = xo - rayon;
    yg = yo - rayon;
    width = (int)Math.round(2*rayon);
    double angleArcDegre = angle/Math.PI*180;
    // Modifications par rapport au C++
    if (zero(Math.PI - angle)) angleArcDegre = 180;
    pDC.drawArc((int)Math.round(xg), (int)Math.round(yg), width, 
      width, (int)Math.round(angleDegreu), (int)Math.round(angleArcDegre));
  } 
  public static final void secteurSaillantPlein(Graphics2D pDC,double xa,double ya,double xo,double yo,
            double rayon,double angle)
  {
    double xg,yg;	/// Coordonnes du sommet gauche du carr devant contenir le cercle
    int width;	/// Cot du carr contenant le cercle

    VecteurDirecteur u = new VecteurDirecteur(xo, yo, xa, ya);
    double angleDegreu = u.angleRad()/Math.PI*180;
    xg = xo - rayon;
    yg = yo - rayon;
    // Ligne suivante modifie version 2.0
    // width = (int)Math.round(2*rayon);
    width = (int)Math.round(2*rayon) + 1;    
    double angleArcDegre = angle/Math.PI*180;
    // Modifications par rapport au C++
    if (zero(Math.PI - angle)) angleArcDegre = 180;
    pDC.fillArc((int)Math.round(xg), (int)Math.round(yg), width,
      width, (int)Math.round(angleDegreu), (int)Math.round(angleArcDegre));
  } 
  public static final void secteurDirect(Graphics2D pDC,double xa,double ya,double xo,double yo,
            double rayon,double angle)
  {
    if (angle < 0) angle = 2*Math.PI + angle;
    secteurSaillant(pDC, xa, ya, xo, yo, rayon, angle);
  }
  public static final void secteurDirectPlein(Graphics2D pDC,double xa,double ya,double xo,double yo,
            double rayon,double angle)
  {
    if (angle < 0) angle = 2*Math.PI + angle;
    secteurSaillantPlein(pDC, xa, ya, xo, yo, rayon, angle);
  }
  public static final void secteurRentrant(Graphics2D pDC,double xa,double ya,double xo,double yo,
            double rayon,double angle)
  {
    if ((angle > 0) || zero(angle - Math.PI)) { //secteur indirect?
      VecteurDirecteur u = new VecteurDirecteur(xo, yo, xa, ya);
      VecteurDirecteur v = new VecteurDirecteur();
      u.tourne(angle, v);
      double x1 = xo + v.x;
      double y1 = yo + v.y;
      secteurDirect(pDC, x1, y1, xo, yo, rayon, 2*Math.PI - angle);
    }
    else { // secteur direct
      secteurDirect(pDC, xa, ya, xo, yo, rayon, 2*Math.PI + angle);
    }
  }

  public static final void secteurRentrantPlein(Graphics2D pDC,double xa,double ya,double xo,double yo,
            double rayon,double angle)
  {
    if ((angle > 0) || zero(angle - Math.PI)) { //secteur indirect?
      VecteurDirecteur u = new VecteurDirecteur(xo, yo, xa, ya);
      VecteurDirecteur v = new VecteurDirecteur();
      u.tourne(angle, v);
      double x1 = xo + v.x;
      double y1 = yo + v.y;
      secteurDirectPlein(pDC, x1, y1, xo, yo, rayon, 2*Math.PI - angle);
    }
    else { // secteur direct
      secteurDirectPlein(pDC, xa, ya, xo, yo, rayon, 2*Math.PI + angle);
    }
  }
  /** Renvoie la chaine reprsentant le double donn avec decimales chiffres aprs la virgule
   * 
   * @param nombre
   * @param decimales
   * @return
   */
  public static final String chaineNombre(double nombre, int decimales) {
    // On regarde d'abord s'il faut ou pas passer en notation scientifique
    double abs = Math.abs(nombre);
    String retour;
    // if ((abs != 0) && (abs < 1e-12) || (abs > 1e12) || (decimales > 12)) {
    // Passage en notation scientifique seulement pour le nombre maxi de dcimales (12)
    // Erreur corrige pour la version 1.9.5 car la chaine pouvait devenir norme
    // et le tampon insuffisant
    // if ((decimales == 12) && (((abs != 0) && (abs < 1e-12)) || abs > 1e12)) {
    if (((decimales == 12) && (abs != 0) && (abs < 1e-12)) || (abs > 1e12)) {
      // Passage en notation scientifique.
      // Dans cette version compatible 1.4 on peut utiliser la classe java.text.NumberFormat
      StringBuffer chFormat = new StringBuffer(decimales + 4);
      chFormat.append('0');    
      chFormat.append('.');
      for (int i = 1; i <= decimales; i ++) {
        chFormat.append('#');
      }      
      chFormat.append('E');
      chFormat.append('0');
      DecimalFormat formatter = new DecimalFormat(chFormat.toString());
      formatter.setDecimalSeparatorAlwaysShown(false);
      retour = formatter.format(nombre);
    }
    else {
      StringBuffer chFormat = new StringBuffer(decimales + 2);
      chFormat.append('0');    
      chFormat.append('.');
      for (int i = 1; i <= decimales; i ++) {
        chFormat.append('#');
      }
      //Le deuxime paramtre pour viter l'emploi en France de la virgule
      DecimalFormat formatter = new DecimalFormat(chFormat.toString(), new DecimalFormatSymbols(Locale.ENGLISH));
      formatter.setDecimalSeparatorAlwaysShown(false);
      retour = formatter.format(nombre);
    }
    if (retour.equals("-0")) return "0";
    // else return retour;  // modifi version 4.9.2
    else {
    	if (retour.startsWith("-")) retour = "- " + retour.substring(1);
    	return retour;
    }
  }
  
  // Calcule l'intersection de deux droites et renvoie un byte du type PositionDroites
  // donnant la nature de cette intersection.
  // Si les droites sont scantes, renvoie dans point1 les coordonnes de l'intersection
  // Remani pour la version 1.9
  // n'est appel que pour des droites existant, c'est  dire ayant des vecteurs directeurs
  // non nuls
  // Calcule l'intersection de deux droites et renvoie un byte du type PositionDroites
  // donnant la nature de cette intersection.
  // Si les droites sont scantes, renvoie dans point1 les coordonnes de l'intersection

  public static final byte intersection(double p1_x,double p1_y, VecteurDirecteur u1,double p2_x,double p2_y,
      VecteurDirecteur u2, Point2D.Double pointInt)
  {
    double d,k1,k2;
    byte position;

    d = u1.x*u2.y-u2.x*u1.y;
    // Modifi pour la version 1.9 : on teste la presque nullit du sinus et
    // non du dterminant
    if (zero(d/u1.norme()/u2.norme()))
    {
      VecteurDirecteur w = new VecteurDirecteur(p1_x,p1_y,p2_x,p2_y);
      // Il faut appeler colineaires() car w peut tre nul
      if (colineaires(u1,w)) position = PositionDroites.Confondues;
      else position = PositionDroites.Paralleles;
    }
    else
    {
      position = PositionDroites.Secantes;
      k1 = u1.y*p1_x-u1.x*p1_y;
      k2 = u2.y*p2_x-u2.x*p2_y;
      pointInt.x = (u1.x*k2-u2.x*k1)/d;
      pointInt.y = (u1.y*k2-u2.y*k1)/d;
    }
    return position;
  } 
  // Calcule l'intersection d'une droite et un cercle et renvoie un byte du type PositionDroiteCercle
  // donnant la nature de cette intersection.
  // Si ils sont scants, renvoie dans point1 et dans point2 les coordonnes des points d'intersection
  public static final byte intersectionDroiteCercle(double px,double py,VecteurDirecteur u,
      double cx, double cy, double R, Point2D.Double point1, Point2D.Double point2)
  {
    double a,b,k1,k2,k3,k4,n2,distcentredroite,dis;
    double t1,t2;// paramtre
    byte pos;

    a = u.x;
    b = u.y;
    k1 = px-cx;
    k2 = py-cy;
    k3 = a*k1+b*k2;
    k4 = a*k2-b*k1;
    n2 = a*a+b*b;
    distcentredroite = Math.abs(a*k2-b*k1)/Math.sqrt(n2);
    if (zero(distcentredroite-R))
    {
       pos = PositionDroiteCercle.Tangents;
       t1 = -k3/n2;
       point1.x = px+a*t1;
       point1.y = py+b*t1;
       point2.setLocation(point1);
       return pos;
     }
     if (distcentredroite>R)
     {
       pos = PositionDroiteCercle.Vide;
       return pos;
     }
     else
     {
       pos = PositionDroiteCercle.Secants;
       dis = n2*R*R-k4*k4;
       t1 = (Math.sqrt(dis)-k3)/n2;// formules du second degr
       t2 = (-Math.sqrt(dis)-k3)/n2;
       point1.x = px+a*t1;
       point1.y = py+b*t1;
       point2.x = px+a*t2;
       point2.y = py+b*t2;
       return pos;
     }
  }
  // Idem pour l'intersection d'une droite et d'un cercle
  public final static byte intersectionCercleCercle(double c1x,double c1y,double r1,double c2x,double c2y,double r2,
    Point2D.Double point1, Point2D.Double point2)
  {
    double d,xi,yi,xh,yh,k;
    VecteurDirecteur u = new VecteurDirecteur();
    VecteurDirecteur v = new VecteurDirecteur();
    // Modifi version 2.3.5
    // if ((c1x == c2x) && (c1y == c2y))
    if (Kernel.zero(c1x - c2x) && Kernel.zero(c1y - c2y))
    {
      if (Kernel.zero(r1 - r2))  return PositionCercleCercle.Confondus; else return PositionCercleCercle.Vide;
    }
    else
    {
      xi = (c1x + c2x)/2;
      yi = (c1y + c2y)/2; // coordonnes du milieu de M1M2;
      VecteurDirecteur u0 = new VecteurDirecteur(c1x, c1y, c2x, c2y);
      d = Kernel.distancePointPoint(c1x,c1y,c2x,c2y);
      u0.vecteurColineaire(1,u); // u colinaire  M1M2 et mme sens
      k = (r1*r1-r2*r2)/2/d;
      xh = xi + k*u.x;
      yh = yi + k*u.y;
      u.getOrthogonal(v);
      byte positionDroites = intersectionDroiteCercle(xh,yh,v,c1x,c1y,r1,point1,point2);
      switch (positionDroites)
      {
        case PositionDroiteCercle.Tangents:
          return PositionCercleCercle.Tangents;
        case PositionDroiteCercle.Secants:
          return PositionCercleCercle.Secants;
        case PositionDroiteCercle.Vide:
          return PositionCercleCercle.Vide;
        default : return PositionCercleCercle.Vide; // Pour satisfaire le compilateur
      }
    }
  }
  // calcul des indices de la plus petite valeur et de la plus grande contenues dans un tableau
  // de double. c est lengthnombre d'indices du tableau. On renvoie dans point.x l'indice
  // de la valeur minimale et dans point.y celui de la valeur maximale.
  public static final void indiceMiniMaxi(byte c,double tab[], Point point)
  {
    byte imin,imax;
    double min,max;

    min = tab[0];
    max = min;
    imin = (byte) 0;
    imax = (byte) 0;
    for (byte i = 1; i <= c; i++) 
    {
      if (tab[i]<min)
      {
        imin = i;
        min = tab[i];
      }
      if (tab[i]>max)
      {
        imax = i;
        max = tab[i];
      }
    }
    point.x = imin;
    point.y = imax;
  }
  
  // Affichage d'un caractre Grec
  //void afficheCaractereGrec(Graphics2D pDC, char[] tabcar, int indcar, int col, int lig, int indiceTailleNom) {
  //  Font fonteGrecque = FonteGrecque.getFonteGrecque(indiceTailleNom);
  //  char[] tab = new char[1];
  //  pDC.setFont(fonteGrecque);
  //  pDC.drawChars(tabcar, indcar, 1, col, lig); 
  //}
  
  
  // Renvoie la hauteur d'une ligne  afficher
  /** Modifi version 4.7.5.2
  public static int hauteurLigne(Graphics2D pDC, int indiceTaille, double coef, String ligne) {
    Font font = FonteNom.getFonteNom(indiceTaille, FonteNom.Normal, coef);    
    FontMetrics fm = pDC.getFontMetrics(font);
   int hautfont;
	 	// Ligne suivante modifie version 3.2.9
		// hautfont = fm.getMaxAscent() + fm.getMaxDescent() - 3;
	 	hautfont = fm.getMaxAscent() + fm.getMaxDescent();
    // On regarde si la ligne contient un affichage Latex
    if (ligne.indexOf("$") != -1) {
      StringTokenizer st2 = new StringTokenizer(ligne, "#$",true);
      boolean latexEnCours = false;
      while (st2.hasMoreTokens()) {
        String ch = st2.nextToken();
        char premcar = ch.charAt(0);
        if ((premcar == '#') || (premcar == '$')) {
	        if ((premcar != '#')) {
			      if ((premcar == '$') && (latexEnCours)) latexEnCours = false;
			      else latexEnCours = true;
	        }
        }
        else {
	      	if (latexEnCours) {
        		TeXFormula formula =  CLatex.creeFormule(ch);
        		if (formula != null) {
	        		TeXIcon icon;
	        		try {
		        		icon = formula.createTeXIcon(TeXConstants.STYLE_DISPLAY, (float)coef*CLatex.tailleLatex[indiceTaille]);
	           	int h = icon.getIconHeight();
	           	if (h > hautfont) hautfont = h;
	        		}
	        		catch (Exception e) {
								// Si erreur on affichera le code LatTex donc on ne change pas la hauteur de la ligne
							}
        		}
	      	}
      	}	      
      }    	
    }
    return hautfont;
  }
  */
  public static int hauteurLigne(Graphics2D pDC, int indiceTaille, double coef, String ligne) {
    Font font = FonteNom.getFonteNom(indiceTaille, FonteNom.Normal, coef);    
    FontMetrics fm = pDC.getFontMetrics(font);
    int hautfont;
	 	hautfont = fm.getMaxAscent() + fm.getMaxDescent();
	 	if ((ligne.indexOf("#H(") != -1) || (ligne.indexOf("#L(") != -1)) hautfont += FonteNom.decalageLogiquePourExposantOuIndice(indiceTaille, coef); // Ajout version 5.0
    // On regarde si la ligne contient un affichage Latex
    if (ligne.indexOf("$") != -1) {
      StringTokenizer st2 = new StringTokenizer(ligne, "$",true);
      boolean latexEnCours = false;
      while (st2.hasMoreTokens()) {
        String ch = st2.nextToken();
        char premcar = ch.charAt(0);
        if (premcar == '$') {
		      if ((premcar == '$') && (latexEnCours)) latexEnCours = false;
		      else latexEnCours = true;
        }
        else {
	      	if (latexEnCours) {
        		TeXFormula formula =  CLatex.creeFormule(ch);
        		if (formula != null) {
	        		TeXIcon icon;
	        		try {
	        		// Ligne suivante modifi version 5.0
		        		icon = formula.createTeXIcon(TeXConstants.STYLE_DISPLAY, (float)coef*CLatex.tailleLatex(indiceTaille));
	           	int h = icon.getIconHeight();
	           	if (h > hautfont) hautfont = h;
	        		}
	        		catch (Exception e) {
								// Si erreur on affichera le code LatTex donc on ne change pas la hauteur de la ligne
							}
        		}
	      	}
      	}	      
      }    	
    }
    return hautfont;
  }


  // Renvoie la hauteur de la premire ligne  afficher dans le cas d'un commentaire
  // form d'une ou de plusieurs lignes. Ne doit pas comprendre GetDescent();
  /** Modifi version 4.7.5.2
  public static int ascent(Graphics2D pDC, int indiceTaille, double coef, String ligne) {
    Font font = FonteNom.getFonteNom(indiceTaille, FonteNom.Normal, coef);    
    FontMetrics fm = pDC.getFontMetrics(font);
    int hautfont;
  	// Ligne suivante modifi version 3.2.9
    // hautfont = fm.getMaxAscent() - 3;
  	hautfont = fm.getMaxAscent();
    // On regarde si la ligne contient un affichage Latex
    if (ligne.indexOf("$") != -1) {
      StringTokenizer st2 = new StringTokenizer(ligne, "#$",true);
      boolean latexEnCours = false;
      while (st2.hasMoreTokens()) {
        String ch = st2.nextToken();
        char premcar = ch.charAt(0);
        if ((premcar == '#') || (premcar == '$')) {
	        if ((premcar != '#')) {
			      if ((premcar == '$') && (latexEnCours)) latexEnCours = false;
			      else latexEnCours = true;
	        }
        }
        else {
	      	if (latexEnCours) {
        		TeXFormula formula =  CLatex.creeFormule(ch);
        		if (formula != null) {
	        		TeXIcon icon;
	        		try {
		        		icon = formula.createTeXIcon(TeXConstants.STYLE_DISPLAY, (float)coef*CLatex.tailleLatex[indiceTaille]);
		           	int h = (int)(icon.getIconHeight()*icon.getBaseLine());
		           	if (h > hautfont) hautfont = h;
	        		}
	        		catch (Exception e) {
								// Si erreur on affichera le code LatTex donc on ne change pas la hauteur de la ligne
							}
        		}
        	}
      	}	      
      }    	
    }
    return hautfont;
}
*/
  public static int ascent(Graphics2D pDC, int indiceTaille, double coef, String ligne) {
    Font font = FonteNom.getFonteNom(indiceTaille, FonteNom.Normal, coef);    
    FontMetrics fm = pDC.getFontMetrics(font);
    int hautfont;
  	// Ligne suivante modifi version 3.2.9
    // hautfont = fm.getMaxAscent() - 3;
  	hautfont = fm.getMaxAscent();
    // On regarde si la ligne contient un affichage Latex
    if (ligne.indexOf("$") != -1) {
      StringTokenizer st2 = new StringTokenizer(ligne, "$",true);
      boolean latexEnCours = false;
      while (st2.hasMoreTokens()) {
        String ch = st2.nextToken();
        char premcar = ch.charAt(0);
        if (premcar == '$') {
		      if ((premcar == '$') && (latexEnCours)) latexEnCours = false;
		      else latexEnCours = true;
        }
        else {
	      	if (latexEnCours) {
        		TeXFormula formula =  CLatex.creeFormule(ch);
        		if (formula != null) {
	        		TeXIcon icon;
	        		try {
	        			// Ligne suivante modifi version 5.0
		        		icon = formula.createTeXIcon(TeXConstants.STYLE_DISPLAY, (float)coef*CLatex.tailleLatex(indiceTaille));
		           	int h = (int)(icon.getIconHeight()*icon.getBaseLine());
		           	if (h > hautfont) hautfont = h;
	        		}
	        		catch (Exception e) {
								// Si erreur on affichera le code LatTex donc on ne change pas la hauteur de la ligne
							}
        		}
        	}
      	}	      
      }    	
    }
    return hautfont;
}

  
  // Affiche une chaine d'une seule ligne contenant ventuellemnt des caractres Grecs
  // et renvoie la colonne de fin d'affichage
  // Modifi pour la version 1.9.5 car les caractres grecs ont dj t filtrs
  public static int afficheChaineAvecCaracteresGrecs
    (Graphics2D pDC, int indiceTaille, String chaine, int col, int lig, int styleFonte,
    		double coef) {
    Font fonte = FonteNom.getFonteNom(indiceTaille, styleFonte, coef);    
    pDC.setFont(fonte);
    FontMetrics fm = pDC.getFontMetrics(fonte);
    int coldebut = col;
    pDC.drawString(chaine,col,lig);
    coldebut += fm.stringWidth(chaine);
    return coldebut;
  }
  
  // Renvoie la largeur correspondant  l'affiche de afficheChaineAvecCaracteresGrecs
  public static int largeurChaineAvecCaracteresGrecs(Graphics2D pDC, int indiceTaille, String chaine,
        int styleFonte, double coef) {
    // Modifi pour la version 1.9.5 car les caractres grecs ont dj t filtrs
    Font fonte = FonteNom.getFonteNom(indiceTaille, styleFonte, coef);    
    pDC.setFont(fonte);
    FontMetrics fm = pDC.getFontMetrics(fonte);
    return fm.stringWidth(chaine);

  }

  public static char caractereSpecial(String chaine) {
    char car = 0;
    // chaine.toLowerCase(); Modifi version 4.7.9
    chaine = chaine.toLowerCase();
    if (chaine.equals("inf")) car = (char)8734;
    else
    if (chaine.equals(">=")) car = (char)8805;
    else
    if (chaine.equals("<=")) car = (char)8804;
    else
    if (chaine.equals("int")) car = (char)8747;
    else
    if (chaine.equals("<>")) car = (char)8800;
    else
    if (chaine.equals("~")) car = (char)8776;
    else
    if (chaine.equals("->")) car = (char)8594;
    else
    if (chaine.equals("<-")) car = (char)8592;
    else
    if (chaine.equals("sqrt")) car = (char)8730;
    else
    if (chaine.equals("#")) car = '#';
    else
    if (chaine.equals("&")) car = '&';
    else
    if (chaine.equals("=>")) car = (char)8658;
    else
    if (chaine.equals("mult")) car = '';
    return car;
  }
    
  
  // Affichage d'une chane de plusieurs lignes ou d'une seule qui peut ventuellement contenir des
  // caractres Grecs.
  // A noter :  partir de la version 1.9.5, les caractres grecs et spciaux
  // ont t chargs et adpats comme des caractres normaux au chargement de la chane
  // Il n'y a donc plus  les traiter de faon spciale lors de l'affichage
  public static void afficheChaine(Graphics2D pDC, int indiceTaille, String chaine,
      int col, int lig, double coef) {
    int styleFonte = FonteNom.Normal;
    // Font fonte = FonteNom.getFonteNom(indiceTaille, styleFonte);    
    // FontMetrics fm = pDC.getFontMetrics(fonte);      
    int ligaff = lig;
    StringTokenizer st1 = new StringTokenizer(chaine, "\n");
    int descent = 0;
    char carPrecedent = ' ';
    while (st1.hasMoreTokens()) {
      String ligne = st1.nextToken();
  		int hauteurLigne = hauteurLigne(pDC, indiceTaille, coef, ligne);
  		int ascent = ascent(pDC, indiceTaille, coef, ligne);
      ligaff += ascent(pDC, indiceTaille, coef, ligne) + descent;
      descent = hauteurLigne - ascent;
      // On dcoupe la chaine en recherchant les ventuels affichages spciaux avec
      // un signe #
      StringTokenizer st2 = new StringTokenizer(ligne, "#$",true);
      boolean affspecial = false;
      boolean latexEnCours = false;
      int coldeb = col;
      while (st2.hasMoreTokens()) {
        String ch = st2.nextToken();
        char premcar = ch.charAt(0);
        if ((premcar == '#') || (premcar == '$')) {
        	if (premcar == '#') { // Modifi version 5.0
        		if (affspecial && (carPrecedent == '#')) {
        			coldeb = afficheChaineAvecCaracteresGrecs(pDC, indiceTaille, ch, coldeb + 1, ligaff, styleFonte, coef);
        			affspecial = false;
        		}
        		else affspecial = true;
        	}
        	else {
        		affspecial = false;
	        	if (latexEnCours) {
	        		latexEnCours = false;
	        		if (carPrecedent == '$') {
	        			coldeb = afficheChaineAvecCaracteresGrecs
                (pDC, indiceTaille, ch, coldeb + 1, ligaff, styleFonte, coef);	        			
	        		}
	        	}
	        	else latexEnCours = true;
        	}
        }
        else {
          if (affspecial) {
            affspecial = false;
            // Le caractere suivant donne le type d'affichage
            char carsp = ch.charAt(0);
            // Cas de changement de style de police
            // if (carsp == 'N' || carsp == 'G' || carsp == 'I') Modifi version 5.0
            if (carsp == 'N' || carsp == 'G' || carsp == 'I' || carsp == 'U') {
              switch (carsp) {
              case 'N' :
                styleFonte = FonteNom.Normal;
                break;
              case 'G' :
                // styleFonte = FonteNom.Gras; Modifi version 5.0 pour pouvoir combiner italique et gras
              	styleFonte |= FonteNom.Gras;              	
                break;
              case 'I' :
                // styleFonte = FonteNom.Italique; Modifi version 5.0 pour pouvoir combiner italique et gras
                styleFonte |= FonteNom.Italique;
              // Ajouts version 5.0
                break;
              case 'U' :
              	styleFonte |= FonteNom.Underline;
              }
              // Modification version 5.0
              /*
              coldeb = afficheChaineAvecCaracteresGrecs(pDC, indiceTaille, ch.substring(1),
                coldeb, ligaff, styleFonte, coef);
              */
              if (ch.length() != 1) coldeb = afficheChaineAvecCaracteresGrecs(pDC, indiceTaille, ch.substring(1),
                  coldeb, ligaff, styleFonte, coef);
            }
            else { // Cas d'un d'exposant ou indice
              int longch = ch.length();
              if (((carsp == 'H') || (carsp == 'L')) && (longch > 1)) {
                // On commence par rechercher la parenthse ouvrante. S'il n'y en a pas, affichage normal
                if (ch.charAt(1) != '(') {
                  coldeb += Kernel.afficheChaineAvecCaracteresGrecs(pDC, indiceTaille, ch, coldeb,
                    ligaff, styleFonte, coef);
                }
                else {
                  // On recherche la parenthse fermante correspondante
                  int indfin;
                  int indparf = ch.indexOf(')');
                  if (indparf == -1) {
                    indfin = longch - 1;
                    indparf = longch;
                  }
                  else indfin = indparf - 1;
                  switch (carsp) {
                    case 'H' :
                      {
                        String chexp =ch.substring(2, indfin + 1); 
                        // int indiceTailleExposant = FonteNom.indiceTaillePourExposantOuIndice(indiceTaille); Modifi version 5.0
                        int indiceTailleExposant = FonteNom.taillePourExposantOuIndice(indiceTaille);
                        int ligexp = ligaff - FonteNom.decalageLogiquePourExposantOuIndice(indiceTaille, coef); 
                        coldeb = afficheChaineAvecCaracteresGrecs
                          (pDC, indiceTailleExposant, chexp, coldeb + 1, ligexp, styleFonte, coef) + 1;
                        break;
                      }
                    case 'L' :
                      {
                        String chexp =ch.substring(2, indfin + 1); 
                        int indiceTailleIndice = 
                          FonteNom.taillePourExposantOuIndice(indiceTaille);
                        int ligexp = ligaff +
                          FonteNom.decalageLogiquePourExposantOuIndice(indiceTaille, coef); 
                        coldeb = afficheChaineAvecCaracteresGrecs
                          (pDC, indiceTailleIndice, chexp, coldeb + 1, ligexp, styleFonte, coef) + 1;
                      }                    
                  } ///switch
                  // On affiche normalement ce qui suit s'il y a quelquechose
                  if (indparf < longch - 1) {
                    coldeb = afficheChaineAvecCaracteresGrecs(pDC, indiceTaille,
                      ch.substring(indparf + 1, longch), coldeb, ligaff, styleFonte, coef);
                  }                
                }
              }
	            // Ligne suivante ajoute version 4.6.4
	            else coldeb = afficheChaineAvecCaracteresGrecs(pDC, indiceTaille, "#"+ ch, coldeb, ligaff, styleFonte, coef);
            }
          }
          else { // affichage normal
          	if (latexEnCours) {
            	TeXFormula formula = CLatex.creeFormule(ch);
            	if (formula != null) {
            		TeXIcon icon;
            		try {
            			// Ligne suivante modifi version 5.0
	            		icon = formula.createTeXIcon(TeXConstants.STYLE_DISPLAY, (float)coef*CLatex.tailleLatex(indiceTaille));
	            		JLabel jl = new JLabel();
	               	// jl.setForeground(pDC.getColor());
	            		icon.setForeground(pDC.getColor());
	               	coldeb++; // 1 pixel de plus
	               	icon.paintIcon(jl, pDC, coldeb, ligaff-(int)(icon.getIconHeight()*icon.getBaseLine()));
	               	coldeb += icon.getIconWidth();
            		}
            		catch (Exception e) {
            			// Si erreur on afficha le code Latex
            			coldeb = afficheChaineAvecCaracteresGrecs(pDC, indiceTaille, ch, coldeb, ligaff, styleFonte, coef);
								}
            	}
            	else
            		coldeb = afficheChaineAvecCaracteresGrecs(pDC, indiceTaille, ch, coldeb, ligaff, styleFonte, coef);
          	}
          	else coldeb = afficheChaineAvecCaracteresGrecs(pDC, indiceTaille, ch, coldeb, ligaff, styleFonte, coef);
          }
        }
        carPrecedent = ch.charAt(ch.length()-1);
      } // while (st2...
    } // while (st1
  }
  
  // Mthode renvoyant dans rectangle (width et height) les dimensions du rectangle contenant une chaine affiche
  //  l'cran en tenant compte des caractres grecs et spciaux.
  public static void dimensionsChaine(Graphics2D pDC, int indiceTaille, String chaine,
    Rectangle rectres, double coef) {
    int styleFonte = FonteNom.Normal;
    int largeur = 0;
    int hauteur = 0;
    StringTokenizer st1 = new StringTokenizer(chaine, "\n");
    while (st1.hasMoreTokens()) {
      String ligne = st1.nextToken();
      hauteur += hauteurLigne(pDC, indiceTaille, coef, ligne);
      // A revoir plus tard pour les caractres spciaux;
      // On dcoupe la chaine en recherchant les ventuels affichages spciaux avec
      // un signe #
      StringTokenizer st2 = new StringTokenizer(ligne, "#$",true);
      boolean affspecial = false;
      boolean latexEnCours = false;
      char carPrecedent = ' ';
      int coldeb = 0;
      while (st2.hasMoreTokens()) {
        String ch = st2.nextToken();
        char premcar = ch.charAt(0);
        if ((premcar == '#') || (premcar == '$')) {
        	if (premcar == '#') { // Modifi version 5.0
        		if (affspecial && (carPrecedent == '#')) {
        			coldeb += largeurChaineAvecCaracteresGrecs(pDC, indiceTaille, ch, styleFonte, coef) + 1; // +1 rajout version 5.0
        			affspecial = false;
        		}
        		else affspecial = true;
        	}
        	else {
        		affspecial = false;
	        	if (latexEnCours) {
	        		latexEnCours = false;
	        		if (carPrecedent == '$')
                coldeb += largeurChaineAvecCaracteresGrecs(pDC, indiceTaille, ch, styleFonte, coef) + 1; // +1 rajout version 5.0
	        	}
	        	else latexEnCours = true;
        	}
        }
        else {
          if (affspecial) {
            affspecial = false;
            // Le caractere suivantr donne lengthtyme d'affichage
            char carsp = ch.charAt(0);
            // if (carsp == 'N' || carsp == 'G' || carsp == 'I') { Modifi version 5.0
            if (carsp == 'N' || carsp == 'G' || carsp == 'I' || carsp == 'U') {
              switch (carsp)
              {
              case 'N' :
                styleFonte = FonteNom.Normal;
                break;
              case 'G' :
                // styleFonte = FonteNom.Gras; Modifi version 5.0
              	styleFonte |= FonteNom.Gras;
                break;
              case 'I' :
                // styleFonte = FonteNom.Italique; Modifi version 5.0
              	styleFonte |= FonteNom.Italique;
                // Ajouts version 5.0
                break;
              case 'U' :
              	styleFonte |= FonteNom.Underline;
              }
              /* Modifi version 5.0
              coldeb += largeurChaineAvecCaracteresGrecs
                (pDC, indiceTaille, ch.substring(1), styleFonte, coef);
              */
              if (ch.length() != 1) coldeb += largeurChaineAvecCaracteresGrecs
                  (pDC, indiceTaille, ch.substring(1), styleFonte, coef);
            }
            else { 
              int longch = ch.length();
              // Cas d'un affichage de racine ou de vecteur ou mesure algbrique
              if (((carsp == 'H') || (carsp == 'L')) && (longch > 1)) {            
                // On commence par rechercher la parenthse ouvrante. S'il n'y en a pas,
                // affichage normal
                if (ch.charAt(1) != '(') {
                  coldeb += largeurChaineAvecCaracteresGrecs(pDC, indiceTaille, ch, styleFonte, coef);
                }
                else {
                  // On recherche la parenthse fermante correspondante
                  int indfin;
                  int indparf = ch.indexOf(')');
                  if (indparf == -1) {
                    indfin = longch - 1;
                    indparf = longch;
                  }
                  else indfin = indparf - 1;
                  switch (carsp) {
	                  case 'H' :
	                  case 'L' :
	                      String chexp =ch.substring(2, indfin + 1); 
	                      int tailleExposant = FonteNom.taillePourExposantOuIndice(indiceTaille);
	                      coldeb += 2 + largeurChaineAvecCaracteresGrecs
	                        (pDC, tailleExposant, chexp, styleFonte, coef);
	                      break;
                  } ///switch
                  // On affiche normalement ce qui suit s'il y a quelquechose
                  if (indparf < longch - 1) {
                    coldeb += largeurChaineAvecCaracteresGrecs(pDC, indiceTaille,
                        ch.substring(indparf + 1, longch), styleFonte, coef);
                  }                
                }
              }
             else coldeb += largeurChaineAvecCaracteresGrecs
                (pDC, indiceTaille, "#" + ch.substring(0), styleFonte, coef);
            }
          }
          else {
          	if (latexEnCours) {
            	TeXFormula formula =  CLatex.creeFormule(ch);
            	if (formula != null) {
            		TeXIcon icon;
            		try {
            			// Ligne suivante modifi version 5.0
	            		icon = formula.createTeXIcon(TeXConstants.STYLE_DISPLAY, (float)coef*CLatex.tailleLatex(indiceTaille));
	            		coldeb++; // 1 pixel de plus
	               	coldeb += icon.getIconWidth();
            		}
            		catch (Exception e) {
									//Si erreur on affiche le code LaTeX
            			coldeb += largeurChaineAvecCaracteresGrecs(pDC, indiceTaille, ch, styleFonte, coef);
								}
            	}
            	else
            		coldeb += largeurChaineAvecCaracteresGrecs(pDC, indiceTaille, ch, styleFonte, coef);    		
          	}
	          else // affichage normal
	            coldeb += largeurChaineAvecCaracteresGrecs(pDC, indiceTaille, ch, styleFonte, coef);
          }
        }
        carPrecedent = ch.charAt(ch.length()-1);          
      } // while (st2...
      if (coldeb > largeur) largeur = coldeb;
    } // while (st1
    rectres.width = largeur;
    // Ligne suivnte modifi version 3.2.9
    // rectres.height = hauteur + 1;
    rectres.height = hauteur + 1;
  }
  
  // Mthode enlevant d'une chaine les retours charriot et renvoyant le rsultat
  public static final String sansCR(String chaine) {
    String res = new String();
    StringTokenizer st1 = new StringTokenizer(chaine, "\r");
    while (st1.hasMoreTokens()) {
      String str = st1.nextToken();
      res = res.concat(str);
    }
    return res;
  }
  
  // Pour la compatibilit avec la version C++ (problmes dcalage en hauteur pour les
  // commentaires ou macros conteenant une flche de vecteur ou une racine carre
  // Cette mthode recherche si la premire ligne d'une chaine contient ou pas des
  // caractre surmonts d'une flche, une barre de mesure algbrique ou une racine carr
  // et renvoie le dcalage en hauteur pour assurer la compatibilit avec l'affichage
  // ralis en C++
  public static int decalagePourCompatibilite(String chaine) {
    if (chaine.length() == 0) return 0;
    StringTokenizer st1 = new StringTokenizer(chaine, "\n");
    String ligne = st1.nextToken();
    if (ligne.indexOf("#V(") != -1) return -5;
    else {
      if (ligne.indexOf("#R(") != -1) return -3;
      else {
        if (ligne.indexOf("#A(") != -1) return -1;
        else return 0;
      }
    }
  }
  
  //  Cr pour version 1.8
  public static double mesurePrincipale(double angrad)
  {
    if (zeroAngle(Math.abs(angrad) - Math.PI)) return Math.PI;
    return angrad - 2*Math.PI*Math.floor((angrad + Math.PI)/(2*Math.PI));
  }
  
  public static int codeCarGrec(char car) {
    switch (car) {
      case 'a' :
        return 0x3b1;
      case 'b' :
        return 0x3b2;
      case 'c' :
        return 0x3c7;
      case 'd' :
        return 0x3b4;
      case 'e' :
        return 0x3b5;
      case 'f' :
        return 0x3d5;
      case 'g' :
        return 0x3b3;
      case 'h' :
        return 0x3b7;
      case 'i' :
        return 0x3b9;
      case 'j' :
        return 0x3c6;
      case 'k' :
        return 0x3ba;
      case 'l' :
        return 0x3bb;
      case 'm' :
        return 0x3bc;
      case 'n' :
        return 0x3bd;
      case 'o' :
        return 0x3bf;
      case 'p' :
        return 0x3c0;
      case 'q' :
        return 0x3b8;
      case 'r' :
        return 0x3c1;
      case 's' :
        return 0x3c3;
      case 't' :
        return 0x3c4;
      case 'u' :
        return 0x3c5;
      case 'v' :
        return 0x3d6;
      case 'w' :
        return 0x3c9;
      case 'x' :
        return 0x3be;
      case 'y' :
        return 0x3c8;
      case 'z' :
        return 0x3b6;
      case 'D' :
        return 0x394;
      case 'F' :
        return 0x3a6;
      case 'G' :
        return 0x393;
      case 'S' :
        return 0x3a3;
      case 'W' :
        return 0x3a9;
      default :
        return 0;
    }
  }
  
  // Rajout pour la version 1.9.5
  // Renvoie une chaine correspondant la la chane initiale dans laquelle les
  // caractres grecs sous la forme &caractre sont remplacs par le caractre
  // unicode correspondant
  public final static String chaineAvecCaracteresGrecsUnicode(String ch) {
    StringBuffer chret = new StringBuffer(ch.length());
    boolean carGrec = false;
    StringTokenizer st1 = new StringTokenizer(ch, "&",true);
    while (st1.hasMoreTokens()) {
      String c = st1.nextToken();
      if (c.charAt(0) == '&') {
        carGrec = true;
      }
      else {
        if (carGrec) {
          carGrec = false;
          int lon = c.length();
          char car = (char)codeCarGrec(c.charAt(0));
          chret.append(String.valueOf(car));
          if (lon > 1) chret.append(c.substring(1));
        }
        else {
          chret = chret.append(c);
        }
      }
    }
    return chret.toString();
  }
  
  // Rajout pour la version 1.9.5
  // Renvoie une chaine correspondant la la chane initiale dans laquelle les
  // caractres spciaux sous la forme #C(chaine de caractres) sont remplacs par le caractre
  // unicode correspondant
  public final static String chaineAvecCaracteresSpeciauxUnicode(String chaine) {
    StringBuffer chret = new StringBuffer(chaine.length());
    StringTokenizer st = new StringTokenizer(chaine, "#",true);
    boolean affspecial = false;
    while (st.hasMoreTokens()) {
      String ch = st.nextToken();
      if (ch.equals("#")) { 
        // Le prochain token commence par une affichage spcial entour par
        // des parenthses
        affspecial= true;
      }
      else {
        if (affspecial) {
          affspecial = false;
          int longch = ch.length();
          if (longch <= 2) chret.append('#' + ch);
          else {
            if (ch.substring(0, 2).equals("C(")) {
              // On cherche la parenthse correspondante
              int indfin;
              int indparf = ch.indexOf(')');
              if (indparf == -1) indfin = longch - 1;
              else indfin = indparf - 1;
              char car = caractereSpecial(ch.substring(2, indfin + 1));
              if (car != 0) chret.append(String.valueOf(car));
              if (indparf <= longch - 1) chret.append(ch.substring(indparf+1));
              else chret.append('#' + ch);
            }
            else chret.append('#' + ch);
          }
        }
        else chret.append(ch);
      }
    }
    return chret.toString();
  }
  
  // Ajout version 2.7
  /**
   * Fonction renvoyant true seulement si le point de coordonnes (x; y)
   * intrieur au triangle dont les sommets ont pour coordonnes (x1; y1), (x2, y2) et (x3, y3).
   * Il peut tre sur le ct M1M2 ou sur le ct M2M3
   * Servira pour dterminer la position d'un point li  l'intrieur d'un polygone.
   */
  public final static boolean estInterieurATriangle(double x, double y, double x1, double y1, double x2, double y2,
  		double x3, double y3) {
  	if (appartientSegment(x, y, x1, y1, x2, y2) || appartientSegment(x, y, x2, y2, x3, y3)) return true;
  	VecteurDirecteur u = new VecteurDirecteur(x, y, x1, y1);
  	VecteurDirecteur v = new VecteurDirecteur(x, y, x2, y2);
  	VecteurDirecteur w = new VecteurDirecteur(x, y, x3, y3);
  	double ang = u.mesureAngleVecteurs(v) + v.mesureAngleVecteurs(w) + w.mesureAngleVecteurs(u);
  	long q = Math.abs(Math.round(ang/(2*Math.PI)));
  	return q == 1;
  }
  
  // Ajout version 2.7
  /**
   * Pour un point G intrieur  un triangle dont les sommets sont (x1,y1) (x2, y2) et (x3, y3), renvoie dans alpha
   * beta et gamma les coordonnes barycentriques normalises du point de coordonnes (x,y) (la somme des coeff est 1)
   * G peut tre ventuelleent sur M1M2 ou sur M1M3
   */
  public static final void coefficientsPointInterieur(double x, double y, double x1, double y1, double x2, double y2,
  		double x3, double y3, Doub alpha, Doub beta, Doub gamma ) {
  	if (appartientSegment(x, y, x1, y1, x2, y2)) {
  		coefficientsPointSurSegment(x, y, x1, y1, x2, y2, alpha, beta);
  		gamma.setValue(0);
  		return;
  	}
  	if (appartientSegment(x, y, x1, y1, x3, y3)) {
  		coefficientsPointSurSegment(x, y, x1, y1, x3, y3, alpha, gamma);
  		beta.setValue(0);
  		return;
  		
  	}
  	VecteurDirecteur u = new VecteurDirecteur(x, y, x1, y1);
  	VecteurDirecteur v = new VecteurDirecteur(x2, y2, x3, y3);
  	//On calcule l'intersection de (AM) avec (BC)
    double d = u.x*v.y-v.x*u.y; // d ne doit pas tre nul car u et v ne sont pas colinaires
    // puisque le point est strictement intrieur au triangle
    double k1 = u.y*x1-u.x*y1;
    double k2 = v.y*x2-v.x*y2;
    double xs = (u.x*k2-v.x*k1)/d;
    double ys = (u.y*k2-v.y*k1)/d;
    // xs et ys sont les coordonnes du point d'intersection H qui est sur le segment [BC]
    double ga = u.norme();
  	VecteurDirecteur v1 = new VecteurDirecteur(xs, ys, x2, y2);
  	double hb = v1.norme();
  	VecteurDirecteur v2 = new VecteurDirecteur(xs, ys, x3, y3);
  	double hc = v2.norme();
  	VecteurDirecteur v3 = new VecteurDirecteur(xs, ys, x, y);
  	double gh = v3.norme();
  	double den1 = ga + gh;
  	double den2 = den1*(hb + hc);
  	alpha.setValue(gh/den1);
  	beta.setValue(ga*hc/den2);
  	gamma.setValue(ga*hb/den2);
  }
  /**
   * Fonction calculant les coefficients barycentriques d'un point sur un segment
   * @param x : abscisse du point qui est sur le segment
   * @param y : ordonne du point qui est sur le segment
   * @param x1 : abscisse du point1
   * @param y1 : ordonne du point1
   * @param x2 : abscisse du point2
   * @param y2 : ordonne du point1
   * @param a : coefficient de barycentre normalis relatif au point1 (valeur de retour)
   * @param b : coefficient de barycentre normalis relatif au point2 (valeur de retour)
   */
  public static final void coefficientsPointSurSegment(double x, double y, double x1, double y1,
  		double x2, double y2, Doub a, Doub b) {
  	VecteurDirecteur u = new VecteurDirecteur(x, y, x1, y1);
  	VecteurDirecteur v = new  VecteurDirecteur(x, y, x2, y2);
  	double nu = u.norme();
  	double nv = v.norme();
  	double d = nu + nv;
  	a.setValue(nv/d);
  	b.setValue(nu/d);
  }
  
  /**
   * Fonction renvoyant le PGCD de a et de b
   * Avant appel a et b doivent tre entiers et non tous les deux nuls
   * @param a
   * @param b
   * @return
   */
  public final static double pgcd(double a, double b) {
	  while (b != 0) {
	  	double r = a - b*Math.floor(a/b);
	  	a = b;
	  	b = r;
	  }
	  return a;
  }
  
  /**
   * Fonction renvoyant le PPCM de a et de b
   * Avant appel a et b doivent tre entiers et non tous les deux positifs ou nuls
   * @param a
   * @param b
   * @return
   */
  public final static double ppcm(double a, double b) {
  	if ((a == 0) || (b == 0)) return 0;
  	double q = a/pgcd(a,b);
  	return Math.floor(q*b + 0.5);
  }
  
  
  /**
   * Fonction renvoyant le coefficient du binme Cnp
   * Avant appel n et p doivent tre entiers, n > 0,
   * et p compris entre 0 et n
   * @param n
   * @param p
   * @return
   */
  public final static double ncr(double n, double p) {
  	if (p > n/2) p = n - p;
  	if (p == 0) return 1;
  	double d = n/p;
  	double p1 = p;
  	for (int k = 1; k < p1; k++) {
  		p--;
  		n--;
  		double d1 = n/p;
  		d = d*d1;
  	}
  	return Math.floor(d+0.5);
  }
  
  /**
   * Fonction renvoyant le nombre de permutations  p lments d'un ensemble  n lments
   * Avant appel n et p doivent tre entiers, n > 0,
   * et p compris entre 0 et n
   * @param n
   * @param p
   * @return
   */
  public final static double npr(double n, double p) {
  	if (p == 0) return 1;
  	double d = n;
  	for (int k = 1; k < p; k++) {
  		n--;
  		d = d*n;
  	}
  	return d;
  }
  
  // Ajout version 3.2.5
  /**
   * Fonction renvoyant true si le point est sur le cercle (ou descendant de cercle)
   * @param cercle
   * @return
   */
  public static final boolean appartientCercle(CCercle cercle, double x, double y) {
  	return Kernel.zero(Kernel.distancePointPoint(x, y, cercle.centre_x, cercle.centre_y) - cercle.rayon);
  	
  }
  
  /* Supprim version 4.7.5.2
  public static final String retireEspacesDebutEtFin(String ch) {
		int k = 0;
		int len = ch.length();
		while((k < len) && (ch.charAt(k) == ' ')) k++;
		if (k != ch.length()) ch = ch.substring(k, ch.length());
		else ch = "";
		len = ch.length();
		if (len != 0) {
			k = len - 1;
			while(ch.charAt(k) == ' ' && (k > 0)) k--;
			ch = ch.substring(0, k + 1);
		}
		return ch;
  }
  */
  
  // Modifi version 4.6 pour compatibilit LaboMep
  // private static String digit64 = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
  private static String digit64 = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+#";
  /**
   * Fonction renvoyant une chane de caractres codant en base 64 le tableau de bytes ba.
   * La dimension du tableau de bytes.
   * Si la dimension du tableau n'est pas un multiple de 3, un ou deux zros lui sont ajouts et la chane
   * de caractres se termine par un ou deux caractres =
   * @param ba
   * @return
   */
  public static final String base64Encode(byte[] ba) {
    // Il faut que la dimension du tableau de bytes soit un multiple de 3 (3 bytes seront cods sur 4 caractres)
    // A noter que un ou deux bytes mis  zro de plus seront peut-tre cods. Pour chacun on mettra un signe =  la fin
    // de la chane de caractres
    int len = ba.length;
    int r = len % 3;
    int q = (len-1) / 3;
    int q2 = q+1;
    if (r != 0) {
    	byte[] ba2 = new byte[q2*3];
    	for (int j = q*3 + r; j < q2*3; j++) ba2[j] = 0;
    	System.arraycopy(ba, 0, ba2, 0, len);
    	ba = ba2;
    }
    StringBuffer buf = new StringBuffer((q+1)*4);
    for (int i = 0; i < q2*3; i+=3) {
    	int n = ((ba[i] & 0xff) << 16) + ((ba[i+1] & 0xff) << 8) + (ba[i+2] & 0xff); // Les poids forts en premier
      char[] array = { digit64.charAt((n >> 18) & 0x3f), digit64.charAt((n >> 12) & 0x3f), digit64.charAt((n >> 6) & 0x3f), digit64.charAt(n & 0x3f)};
    	buf.append(array);
    }
	  int paddingCount = (3 - r) % 3; // nul si r = 0 // 1 si r = 2 et 2 si r = 3 c'est le nombre de =  mettre  la fin
		String ch = buf.toString();
		return ch.substring(0, (q+1)*4 - paddingCount) + "==".substring(0, paddingCount);
	}
  
  public static final byte[] base64Decode(String xmlString) {
    int paddingCount = 0;
    int len = xmlString.length();
    int indegal = xmlString.indexOf('=');
    if (indegal != -1 ) {
	    if (xmlString.charAt(len - 1) == '=') {
	    	if (xmlString.charAt(len - 2) == '=') paddingCount = 2;
	    	else paddingCount = 1;
	    }
	    xmlString = xmlString.substring(0, indegal) + "AA".substring(2-paddingCount); // On remplace les = par des An donc des zros
    }
    else paddingCount = 0;
    byte[] barray = new byte[(xmlString.length()/4+1)*3 - paddingCount];
    int der = len - paddingCount;
    int j = 0;
    for (int i = 0; i < len; i += 4) {
    	char car1 = xmlString.charAt(i);
    	char car2 = xmlString.charAt(i+1);
    	char car3 = xmlString.charAt(i+2);
    	char car4 = xmlString.charAt(i+3);
    	int n = (digit64.indexOf(car1) << 18) + (digit64.indexOf(car2) << 12) + (digit64.indexOf(car3) << 6) + digit64.indexOf(car4);
    	if (i + 3 < der) barray[j + 2] = (byte) (n & 0xff);
    	if (i + 2 < der) barray[j + 1] = (byte) ((n >> 8) & 0xff);
    	barray[j] = (byte) ((n >> 16) & 0xff);
    	j += 3;
    }
    return barray;
  }
  
  public static final String convertStringToHTML(String ch) {
  	StringBuffer buf = new StringBuffer(128);
  	for (int i = 0; i < ch.length(); i++) {
  		char car = ch.charAt(i);
	  	switch (car) {
			case '': buf.append("&Agrave;"); break;
			case '': buf.append("&Aacute;"); break;
			case '': buf.append("&Acirc;"); break;
			case '': buf.append("&Atilde;"); break;
			case '': buf.append("&Auml;"); break;
			case '': buf.append("&Aring;"); break;
			case '': buf.append("&AElig;"); break;
			case '': buf.append("&Ccedil;"); break;
			case '': buf.append("&Egrave;"); break;
			case '': buf.append("&Eacute;"); break;
			case '': buf.append("&Ecirc;"); break;
			case '': buf.append("&Euml;"); break;
			case '': buf.append("&Igrave;"); break;
			case '': buf.append("&Iacute;"); break;
			case '': buf.append("&Icirc;"); break;
			case '': buf.append("&Iuml;"); break;
			case '': buf.append("&ETH;"); break;
			case '': buf.append("&Ntilde;"); break;
			case '': buf.append("&Ograve;"); break;
			case '': buf.append("&Oacute;"); break;
			case '': buf.append("&Ocirc;"); break;
			case '': buf.append("&Otilde;"); break;
			case '': buf.append("&Ouml;"); break;
			case '': buf.append("&Ugrave;"); break;
			case '': buf.append("&Uacute;"); break;
			case '': buf.append("&Ucirc;"); break;
			case '': buf.append("&Uuml;"); break;
			case '': buf.append("&Yacute;"); break;
			case '': buf.append("&THORN;"); break;
			case '': buf.append("&szlig;"); break;
			case '': buf.append("&agrave;"); break;
			case '': buf.append("&aacute;"); break;
			case '': buf.append("&acirc;"); break;
			case '': buf.append("&atilde;"); break;
			case '': buf.append("&auml;"); break;
			case '': buf.append("&aring;"); break;
			case '': buf.append("&aelig;"); break;
			case '': buf.append("&ccedil;"); break;
			case '': buf.append("&egrave;"); break;
			case '': buf.append("&eacute;"); break;
			case '': buf.append("&ecirc;"); break;
			case '': buf.append("&euml;"); break;
			case '': buf.append("&igrave;"); break;
			case '': buf.append("&iacute;"); break;
			case '': buf.append("&icirc;"); break;
			case '': buf.append("&iuml;"); break;
			case '': buf.append("&eth;"); break;
			case '': buf.append("&ntilde;"); break;
			case '': buf.append("&ograve;"); break;
			case '': buf.append("&oacute;"); break;
			case '': buf.append("&ocirc;"); break;
			case '': buf.append("&otilde;"); break;
			case '': buf.append("&ouml;"); break;
			case '': buf.append("&ugrave;"); break;
			case '': buf.append("&uacute;"); break;
			case '': buf.append("&ucirc;"); break;
			case '': buf.append("&uuml;"); break;
			case '': buf.append("&yacute;"); break;
			case '': buf.append("&thorn;"); break;
			case '': buf.append("&yuml;"); break;
			case '': buf.append("&#339;"); break;	
			default: buf.append(car);
			}
  	}
	  return buf.toString();
  }
  
  /**
   * Ajout version 5.1.1 : Calcule de faon optimise le quotient de deux complexes
   * @param z1 : le dividende
   * @param z2 : le diviseur
   * @param zRes : le rsultat
   */
  public static final void divComp ( Complexe z1, Complexe z2, Complexe zRes) {
		double a, b, c, d, r, den;
		a = z1.x;
		b = z1.y;
		c = z2.x;
		d = z2.y;
  	if ( Math.abs (d) <= Math.abs (c) ) {
	  	r = d/c;
	  	den = c + d * r;
	  	zRes.x = (a + b * r) / den;
	  	zRes.y = (b - a * r) / den;
  	}
  	else {
	  	r = c/d;
	  	den = c * r + d;
	  	zRes.x = (a * r + b) / den;
	  	zRes.y = (b * r - a) / den;
  	}
  }
}
  