/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package objets;

// Attention :  la diffrence de la version C++, dans la version Java, chaque objet CCalculBase
// contient listePropritaire qui est un rfrence  la liste qui possde l'objet.
// Cela permet de connatre l'unit d'angle et la longueur unit dans Positionne() et
// rendValeur() et rendValeurFonction()
import java.io.*;
import java.util.ArrayList;
import java.awt.*;
import mathgraph32.MtgFrame;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import glob.Kernel;

public abstract class CCalculBase extends CObjetBase {
	
	// Ajout version 4.8 pour optimiser la fonction dependDe
	public CElementBase elementTestePourDependDe;
	public boolean dependDeElementTeste;
	// Ajout version 4.8 : Pour optimiser la fonction dependDePourReclassement
	public CElementBase elementTestePourDependDePourRec;
	public boolean dependDeElementTestePourRec;
	// Fin ajout version 4.8


	public static final int Nombre = 0;
	public static final int Addition = 1;
	public static final int Multiplication = 2;
	public static final int Puissance = 3;
	public static final int Fonction = 4;
	public static final int Valeur = 5;
	public static final int ParOuvrante = 6;
	public static final int ParFermante = 7;
	public static final int Inferieur = 8;
	public static final int Inegalite = 9;
	public static final int Blanc = 10;
	// Rajou par rapprot  la version C++ 1.8
	public static final int Carre = 11;
	public static final int Virgule = 12;
	private final static int nombreMaxParentheses = 64; // Nombre maxi de
																											// parenthses embotes.
	// Constantes ajoutes version 4.6 pour viter des appels  getClass
	protected static long natAppelFonction 								= 	1;
	protected static long natAppelFonctionInterne 				= 	1 << 1;
	protected static long natAppelFonctionInterneNVar 		= 	1 << 2;
	protected static long natAppelFonctionNVar 						= 	1 << 3;
	protected static long natConstante 										= 	1 << 4;
	protected static long natConstantei 									= 	1 << 5;
	protected static long natFonction 										= 	1 << 6;
	protected static long natFonction2Var 								= 	1 << 7;
	protected static long natFonction3Var 								= 	1 << 8;
	protected static long natIntegraleDansFormule					= 	1 << 9;
	protected static long natMoinsUnaire									= 	1 << 10;
	protected static long natOperation										= 	1 << 11;
	protected static long natPointeurFonction							= 	1 << 12;
	protected static long natPuissance										= 	1 << 13;
	protected static long natResultatValeur								= 	1 << 14;
	protected static long natResultatValeurComplexe				= 	1 << 15;
	protected static long natSommeDansFormule							= 	1 << 16;
	protected static long natProduitDansFormule						= 	1 << 17;
	protected static long natVariableFormelle							= 	1 << 18;
	
	// Ajout version 3.2.8 pour lutter contre les dpassements de capacit dans
	// les sommes indices
	// et les calculs d'intgrales
	public static final long nombreMaxiIterations = 200000;
	public static int nombreSubdivisionsPourSimpson = 200; // Pour calcul
																													// intgrale par
																													// Simpson
	protected static String parouv = "(";
	protected static String parfer = ")";
	// protected static String parouvLatex = "\\left({"; Modifi version 4.9.5
	// protected static String parferLatex = "}\\right)"; Modifi version 4.9.5
	protected static String parouvLatex = "\\left(";
	protected static String parferLatex = "\\right)";
	protected static String virg = ",";

	/** Creates new CCalculBase */
	// Modifi version 4.8
	public CCalculBase() {
		// Ajout version 4.8
		elementTestePourDependDe = null;
		elementTestePourDependDePourRec = null;
	}

	public CCalculBase(CListeObjets liste) {
		super(liste);
		// Ajout version 4.8
		elementTestePourDependDe = null;
		elementTestePourDependDePourRec = null;
	}
	
	// Ajout version 4.6
	protected abstract long nature();

	/**
	 * Ajout pour version 3.0 Fonction servant pour les fonctions  savoir quel
	 * est le nombre de variables Sera redfini pour CFonction et CFonction2Var
	 * 
	 * @return
	 */
	public int nombreVariables() {
		return 0;
	}

	/**
	 * Construction d'un calcul de type CCalculBase  partir d'une chane ch de
	 * caractres en supposant la syntaxe correcte (procdure rcursive) On cre
	 * en fait un arbre binaire de calcul ptListe est la liste propritaire
	 */
	public static final CCalculBase creeCalculBase(String ch, CListeObjets ptListe,
			int pdebut, int pfin, String[] nomVariableFormelle) {
		try {
			int ptr;
			Point info = new Point(); // Contiendra dans x l'indice du caractre
																// prcdent
			// et dans y l'oprateur trouv (correspond au ptPrecedent et
			// operateurInegalite
			// de la version C++
			// int ptPrecedent;
			// int operateurInegalite;
			char car;
			byte op;
			Int nomF = new Int();
			int longNom;
			PointeurValDyn ptVa = new PointeurValDyn();
			Int parametreTrouve = new Int(0);
			Int nombreVar = new Int(0);
	
			// On limine les blancs de dbut
			car = ch.charAt(pdebut);
			while ((car == ' ') && (pdebut < pfin)) {
				pdebut++;
				car = ch.charAt(pdebut);
			}
			// Correction version 4.9.4. Il faut aussi retirer les espaces de fin car sinon quand on value une criture dcimale avec un espace
			// en fin (ou plusieurs) il y avait erreur
			car = ch.charAt(pfin);
			while ((car == ' ') && (pdebut < pfin)) {
				pfin--;
				car = ch.charAt(pfin);
			}
			// Ajout version 4.9.5 : On recherche d'abord les oprateurs logiques qui ont la priorit la plus faible
			// On recherche d'abord les oprateurs d'ingalits
			ptr = premierOperateurLogique(ch, pdebut, pfin, info);
			if (ptr != -1) {
				return new COperation(ptListe, creeCalculBase(ch, ptListe, pdebut,
						info.x, nomVariableFormelle), creeCalculBase(ch, ptListe, ptr + 1,
						pfin, nomVariableFormelle), info.y);
			}
			ptr = premiereInegalite(ch, pdebut, pfin, info);
			if (ptr != -1) {
				return new COperation(ptListe, creeCalculBase(ch, ptListe, pdebut,
						info.x, nomVariableFormelle), creeCalculBase(ch, ptListe, ptr + 1,
						pfin, nomVariableFormelle), info.y);
			} else {
				// On recherche ensuite les operateurs d'addition ou soustraction
				ptr = premiereSomme(ch, pdebut, pfin);
				if (ptr != -1) {
					// On regarde d'abord si la chaine commence par un signe - ou
					// un signe +
					car = ch.charAt(ptr);
					if (car == '+')
						op = Operateur.Plus;
					else
						op = Operateur.Moin;
					if (ptr == pdebut) {
						car = ch.charAt(ptr);
						if (car == '+') {
							return creeCalculBase(ch, ptListe, pdebut + 1, pfin,
									nomVariableFormelle);
						} else {
							// Modification version 4.6
							// return new COperation(ptListe, new CConstante(ptListe, 0),
							// creeCalculBase(ch,ptListe,pdebut+1,pfin,nomVariableFormelle),op);
							return new CMoinsUnaire(ptListe, creeCalculBase(ch, ptListe,
									pdebut + 1, pfin, nomVariableFormelle));
						}
					} else {
						return new COperation(ptListe, creeCalculBase(ch, ptListe, pdebut,
								ptr - 1, nomVariableFormelle), creeCalculBase(ch, ptListe,
								ptr + 1, pfin, nomVariableFormelle), op);
					}
				} // if (ptr != NULL)
				else {
					ptr = premierProduit(ch, pdebut, pfin);
					if (ptr != -1) {
						car = ch.charAt(ptr);
						if (car == '*')
							op = Operateur.Mult;
						else
							op = Operateur.Divi;
						return new COperation(ptListe, creeCalculBase(ch, ptListe, pdebut,
								ptr - 1, nomVariableFormelle), creeCalculBase(ch, ptListe,
								ptr + 1, pfin, nomVariableFormelle), op);
					} else
					// Le calcul ne contient ni addition, ni diffrence, ni produit
					// ni quotient}
					// Attention : par rapport  la version C++ 1.8, autorisation
					// du symbole  d'lvation au carr.
					{
						ptr = premierePuissance(ch, pdebut, pfin);
						if (ptr != -1) {
							if (ch.charAt(ptr) == '') {
								return new CPuissance(ptListe, creeCalculBase(ch, ptListe,
										pdebut, ptr - 1, nomVariableFormelle), new CConstante(
										ptListe, 2));
							} else
								return new CPuissance(ptListe, creeCalculBase(ch, ptListe,
										pdebut, ptr - 1, nomVariableFormelle), creeCalculBase(ch,
										ptListe, ptr + 1, pfin, nomVariableFormelle));
						} else {
							// On regarde si la chaine commence par le nom d'une fonction
							// ou d'une valeur dj dfinie
							longNom = ptListe.commenceParNomFonctionOuValeurOuParametre(
									ch.substring(pdebut), nomVariableFormelle, ptVa, nomF,
									parametreTrouve, nombreVar);
							if (longNom > 0) {
								if (parametreTrouve.getValue() != -1)
									return new CVariableFormelle(ptListe,
											parametreTrouve.getValue());
								else {
									if (ptVa.getPointeur() == null) {
										// Modification version 3.0. Il faut regarder si la fonction
										// est une fonction de plusieurs paramtres ou non
										// Ajout version 4.7.4.1
										if (parentheseFermante(ch, pdebut + longNom) == -1) throw new ErreurCalculException();
										
										if (nombreVar.getValue() == 1)
											return new CFonction(ptListe, nomF.getValue(),
													creeCalculBase(ch, ptListe, pdebut + longNom, pfin,
															nomVariableFormelle));
										else {
											if (nombreVar.getValue() == 2) {
												int indicePremiereVirg = premiereVirgule(ch, pdebut
														+ longNom + 1);
												CCalculBase operande1 = creeCalculBase(ch, ptListe,
														pdebut + longNom + 1, indicePremiereVirg - 1,
														nomVariableFormelle);
												int indiceParF = parentheseFermanteApresVirgule(ch,
														indicePremiereVirg + 1);
												// A revoir plus tard si on cre des fonctions de trois
												// variables ou plus
												CCalculBase operande2 = creeCalculBase(ch, ptListe,
														indicePremiereVirg + 1, indiceParF - 1,
														nomVariableFormelle);
												return new CFonction2Var(ptListe, nomF.getValue(),
														operande1, operande2);
											} else {
												if (nombreVar.getValue() == 3) { // 3 variables
													CCalculBase operande1;
													int indicePremiereVirg = premiereVirgule(ch, pdebut
															+ longNom + 1);
													operande1 = creeCalculBase(ch, ptListe, pdebut
															+ longNom + 1, indicePremiereVirg - 1,
															nomVariableFormelle);
													int indiceDeuxiemeVirg = premiereVirgule(ch,
															indicePremiereVirg + 1);
													CCalculBase operande2 = creeCalculBase(ch, ptListe,
															indicePremiereVirg + 1, indiceDeuxiemeVirg - 1,
															nomVariableFormelle);
													int indiceParF = parentheseFermanteApresVirgule(ch,
															indiceDeuxiemeVirg + 1);
													// A revoir plus tard si on cre des fonctions de trois
													// variables ou plus
													CCalculBase operande3 = creeCalculBase(ch, ptListe,
															indiceDeuxiemeVirg + 1, indiceParF - 1,
															nomVariableFormelle);
													return new CFonction3Var(ptListe, nomF.getValue(),
															operande1, operande2, operande3);
												} else {
													if (nombreVar.getValue() == 4) {
														// Seule fonction  4 variables : Cas d'une intgrale
														int indicePremiereVirg = premiereVirgule(ch, pdebut
																+ longNom + 1);
														int indiceDeuxiemeVirg = premiereVirgule(ch,
																indicePremiereVirg + 1);
														int indiceTroisiemeVirg = premiereVirgule(ch,
																indiceDeuxiemeVirg + 1);
														int indiceParF = parentheseFermanteApresVirgule(ch,
																indiceTroisiemeVirg + 1);
														String nomVariableSommation = ch.substring(
																indicePremiereVirg + 1, indiceDeuxiemeVirg);
														/* nomVariableSommation = Kernel
																.retireEspacesDebutEtFin(nomVariableSommation); */ // Modifi version 4.7.5.2
														nomVariableSommation = nomVariableSommation.trim();														int n = 0;
														if (nomVariableFormelle != null) n = nomVariableFormelle.length;
														String[] nomVariableFormelle2 = new String[n + 1];
														// Bug corrig version 4.6
														/*
														for (int j = 0; j < n; j++)
															nomVariableFormelle2[j] = new String(
																	nomVariableFormelle[j]);
														nomVariableFormelle2[n] = new String(
																nomVariableSommation);
														*/
														for (int j = 0; j < n; j++) nomVariableFormelle2[j] = new String(nomVariableFormelle[j]);
														// nomVariableFormelle2[n+1] = new String(nomVariableSommation);
														nomVariableFormelle2[n] = new String(nomVariableSommation);
														/*
														return new CIntegraleDansFormule(
																ptListe,
																creeCalculBase(ch, ptListe, pdebut + longNom + 1,
																		indicePremiereVirg - 1, nomVariableFormelle2),
																creeCalculBase(ch, ptListe,
																		indiceDeuxiemeVirg + 1,
																		indiceTroisiemeVirg - 1, nomVariableFormelle2),
																creeCalculBase(ch, ptListe,
																		indiceTroisiemeVirg + 1, indiceParF - 1,
																		nomVariableFormelle2), nomVariableSommation,
																false);
														 */
														return new CIntegraleDansFormule(
																ptListe,
																creeCalculBase(ch, ptListe, pdebut + longNom + 1,
																		indicePremiereVirg - 1, nomVariableFormelle2),
																creeCalculBase(ch, ptListe,
																		indiceDeuxiemeVirg + 1,
																		indiceTroisiemeVirg - 1, nomVariableFormelle),
																creeCalculBase(ch, ptListe,
																		indiceTroisiemeVirg + 1, indiceParF - 1,
																		nomVariableFormelle), nomVariableSommation,
																false);
														
													} else {
														// fonctions  5 variables : la somme indice et
														// produit indic
														int indicePremiereVirg = premiereVirgule(ch, pdebut
																+ longNom + 1);
														int indiceDeuxiemeVirg = premiereVirgule(ch,
																indicePremiereVirg + 1);
														int indiceTroisiemeVirg = premiereVirgule(ch,
																indiceDeuxiemeVirg + 1);
														int indiceQuatriemeVirg = premiereVirgule(ch,
																indiceTroisiemeVirg + 1);
														int indiceParF = parentheseFermanteApresVirgule(ch,
																indiceQuatriemeVirg + 1);
														String nomVariableSommation = ch.substring(
																indicePremiereVirg + 1, indiceDeuxiemeVirg);
														/*nomVariableSommation = Kernel
																.retireEspacesDebutEtFin(nomVariableSommation);*/ // Modifi version 4.7.5.2
														nomVariableSommation = nomVariableSommation.trim();
														int n = 0;
														if (nomVariableFormelle != null)
															n = nomVariableFormelle.length;
														String[] nomVariableFormelle2 = new String[n + 1];
														for (int j = 0; j < n; j++)
															nomVariableFormelle2[j] = new String(
																	nomVariableFormelle[j]);
														nomVariableFormelle2[n] = new String(
																nomVariableSommation);
														if (nomF.getValue() == OperateurFonction5Var.somme)
															return new CSommeDansFormule(ptListe,
																	creeCalculBase(ch, ptListe, pdebut + longNom
																			+ 1, indicePremiereVirg - 1,
																			nomVariableFormelle2), creeCalculBase(ch,
																			ptListe, indiceDeuxiemeVirg + 1,
																			indiceTroisiemeVirg - 1,
																			nomVariableFormelle2), creeCalculBase(ch,
																			ptListe, indiceTroisiemeVirg + 1,
																			indiceQuatriemeVirg - 1,
																			nomVariableFormelle2), creeCalculBase(ch,
																			ptListe, indiceQuatriemeVirg + 1,
																			indiceParF - 1, nomVariableFormelle2),
																	nomVariableSommation);
														else
															return new CProduitDansFormule(ptListe,
																	creeCalculBase(ch, ptListe, pdebut + longNom
																			+ 1, indicePremiereVirg - 1,
																			nomVariableFormelle2), creeCalculBase(ch,
																			ptListe, indiceDeuxiemeVirg + 1,
																			indiceTroisiemeVirg - 1,
																			nomVariableFormelle2), creeCalculBase(ch,
																			ptListe, indiceTroisiemeVirg + 1,
																			indiceQuatriemeVirg - 1,
																			nomVariableFormelle2), creeCalculBase(ch,
																			ptListe, indiceQuatriemeVirg + 1,
																			indiceParF - 1, nomVariableFormelle2),
																	nomVariableSommation);
	
													}
												}
											}
										}
									} else {
										if (ptVa.getPointeur().estFonctionOuSuite()) {
											// Ajout version 4.7.4.1
											if (parentheseFermante(ch, pdebut + longNom) == -1) throw new ErreurCalculException();
											CCalculAncetre calc = (CCalculAncetre) ptVa.getPointeur();
											int nbvar = calc.nombreVariables();
											if (nbvar == 1)
												return new CAppelFonction(ptListe,
														(CCalculAncetre) (ptVa.getPointeur()),
														creeCalculBase(ch, ptListe, pdebut + longNom, pfin,
																nomVariableFormelle));
											else {
												CCalculBase[] tabcal = new CCalculBase[nbvar];
												int deb = pdebut + longNom;
												int fin = 0;
												for (int i = 0; i < nbvar - 1; i++) {
													fin = premiereVirgule(ch, deb + 1);
													tabcal[i] = creeCalculBase(ch, ptListe, deb + 1,
															fin - 1, nomVariableFormelle);
													deb = fin;
												}
												deb++;
												fin = parentheseFermanteApresVirgule(ch, deb);
												tabcal[nbvar - 1] = creeCalculBase(ch, ptListe, deb,
														fin - 1, nomVariableFormelle);
												return new CAppelFonctionNVar(ptListe, nbvar,
														(CFoncNVar) (ptVa.getPointeur()), tabcal);
											}
										} else {
											return new CResultatValeur(ptListe, ptVa.getPointeur());
										}
									}
								}
							} else {
								// Dans le cas o l'expression commence par une parenthse
								// ouvrante, elle finit par une fermante et il suffit
								// d'valuer ce que la parenthse contient
								car = ch.charAt(pdebut);
								// Si le premier caractre est une parenthse ouvrante
								// il faut faire attention car le dernier peut tre
								// un espace.
								// Cela a donn un bug dans la version 16 bits
								if (car == '(') {
									if (parentheseFermante(ch, pdebut) == -1) throw new ErreurCalculException();
									else return creeCalculBase(ch, ptListe, pdebut + 1,
											parentheseFermante(ch, pdebut) - 1, nomVariableFormelle);
								}
								else
									// Le rsultat ne peut tre que numrique}
									{
										/* Abandonn version 4.9.2
										String chaineNombre = new String();
										int ptch = pdebut;
										// car = ch.charAt(ptch); modifi par rapport au C++
										while ((FonteNom.chiffre(car) || (car == '.'))
												&& (ptch <= pfin)) {
											car = ch.charAt(ptch);
											chaineNombre = chaineNombre + car;
											ptch++;
										}
										return new CConstante(ptListe, Double.parseDouble(chaineNombre));
										*/
										String chaineNombre = ch.substring(pdebut, pfin+1);
										if (FonteNom.estDecimalCorrect(chaineNombre)) return new CConstante(ptListe, Double.parseDouble(chaineNombre));
										else return new CCalculBaseNull(chaineNombre);
								}
							}
						}
					}
				}
			}
		}
		catch (Exception e) {
			// Amlioration version 4.7.4.1 : Mme si l'expression est incorrecte, on remplace les * de multiplication par des \times
			// Abandonn version 4.9.4
			return new CCalculBaseNull(ch.substring(pdebut,pfin+1));
		}
	}

	/**
	 * Construction d'un calcul de type CCalculBase  partir d'une chane ch de
	 * caractres en supposant la syntaxe correcte (procdure rcursive) On cre
	 * en fait un arbre binaire de calcul ptListe est la liste propritaire
	 */
	public static final CCalculBase creeCalculBaseComplexe(String ch,
			CListeObjets ptListe, int pdebut, int pfin, String[] nomVariableFormelle) {
		try {
			int ptr;
			Point info = new Point(); // Contiendra dans x l'indice du caractre
																// prcdent
			// et dans y l'oprateur trouv (correspond au ptPrecedent et
			// operateurInegalite
			// de la version C++
			// int ptPrecedent;
			// int operateurInegalite;
			char car;
			byte op;
			Int nomF = new Int();
			int longNom;
			PointeurValDyn ptVa = new PointeurValDyn();
			Int parametreTrouve = new Int(0);
			Int nombreVar = new Int(0);
			Bool complexeiTrouve = new Bool(false);
	
			// On limine les blancs de dbut
			car = ch.charAt(pdebut);
			while ((car == ' ') && (pdebut < pfin)) {
				pdebut++;
				car = ch.charAt(pdebut);
			}
			// Correction version 4.9.4. Il faut aussi retirer les espaces de fin car sinon quand on value une criture dcimale avec un espace
			// en fin (ou plusieurs) il y avait erreur
			car = ch.charAt(pfin);
			while ((car == ' ') && (pdebut < pfin)) {
				pfin--;
				car = ch.charAt(pfin);
			}	
	
			// Ajout version 4.9.5 : On recherche d'abord les oprateurs logiques qui ont la priorit la plus faible
			// On recherche d'abord les oprateurs d'ingalits
			ptr = premierOperateurLogique(ch, pdebut, pfin, info);
			if (ptr != -1) {
				return new COperation(ptListe, creeCalculBase(ch, ptListe, pdebut,
						info.x, nomVariableFormelle), creeCalculBase(ch, ptListe, ptr + 1,
						pfin, nomVariableFormelle), info.y);
			}
			// On recherche d'abord les oprateurs d'ingalits
			ptr = premiereInegalite(ch, pdebut, pfin, info);
			if (ptr != -1) {
				return new COperation(ptListe, creeCalculBaseComplexe(ch, ptListe,
						pdebut, info.x, nomVariableFormelle), creeCalculBaseComplexe(ch,
						ptListe, ptr + 1, pfin, nomVariableFormelle), info.y);
			} else {
				// On recherche ensuite les operateurs d'addition ou soustraction
				ptr = premiereSomme(ch, pdebut, pfin);
				if (ptr != -1) {
					// On regarde d'abord si la chaine commence par un signe - ou
					// un signe +
					car = ch.charAt(ptr);
					if (car == '+')
						op = Operateur.Plus;
					else
						op = Operateur.Moin;
					if (ptr == pdebut) {
						car = ch.charAt(ptr);
						if (car == '+') {
							return creeCalculBaseComplexe(ch, ptListe, pdebut + 1, pfin,
									nomVariableFormelle);
						} else {
							// Modifi version 4.6
							// return new COperation(ptListe, new CConstante(ptListe, 0),
							// creeCalculBaseComplexe(ch,ptListe,pdebut+1,pfin,nomVariableFormelle),op);
							return new CMoinsUnaire(ptListe, creeCalculBaseComplexe(ch,
									ptListe, pdebut + 1, pfin, nomVariableFormelle));
						}
					} else {
						return new COperation(ptListe, creeCalculBaseComplexe(ch, ptListe,
								pdebut, ptr - 1, nomVariableFormelle), creeCalculBaseComplexe(ch,
								ptListe, ptr + 1, pfin, nomVariableFormelle), op);
					}
				} // if (ptr != NULL)
				else {
					ptr = premierProduit(ch, pdebut, pfin);
					if (ptr != -1) {
						car = ch.charAt(ptr);
						if (car == '*')
							op = Operateur.Mult;
						else
							op = Operateur.Divi;
						return new COperation(ptListe, creeCalculBaseComplexe(ch, ptListe,
								pdebut, ptr - 1, nomVariableFormelle), creeCalculBaseComplexe(ch,
								ptListe, ptr + 1, pfin, nomVariableFormelle), op);
					} else
					// Le calcul ne contient ni addition, ni diffrence, ni produit
					// ni quotient}
					// Attention : par rapport  la version C++ 1.8, autorisation
					// du symbole  d'lvation au carr.
					{
						ptr = premierePuissance(ch, pdebut, pfin);
						if (ptr != -1) {
							if (ch.charAt(ptr) == '') {
								return new CPuissance(ptListe, creeCalculBaseComplexe(ch,
										ptListe, pdebut, ptr - 1, nomVariableFormelle),
										new CConstante(ptListe, 2));
							} else
								return new CPuissance(ptListe, creeCalculBaseComplexe(ch,
										ptListe, pdebut, ptr - 1, nomVariableFormelle),
										creeCalculBaseComplexe(ch, ptListe, ptr + 1, pfin,
												nomVariableFormelle));
						} else {
							// On regarde si la chaine commence par le nom d'une fonction
							// ou d'une valeur dj dfinie
							longNom = ptListe
									.commenceParNomFonctionOuValeurOuParametreComplexe(
											ch.substring(pdebut), nomVariableFormelle, ptVa, nomF,
											parametreTrouve, complexeiTrouve, nombreVar);
							if (longNom > 0) {
								if (complexeiTrouve.getValue()) {
									return new CConstantei(ptListe);
								} else {
									if (parametreTrouve.getValue() != -1) {
										return new CVariableFormelle(ptListe,
												parametreTrouve.getValue());
									} else {
										if (ptVa.getPointeur() == null) {
											// Modification version 3.0. Il faut regarder si la fonction
											// est une fonction de plusieurs paramtres ou non
											// Ajout version 4.7.4.1
											if (parentheseFermante(ch, pdebut + longNom) == -1) throw new ErreurCalculException();
											if (nombreVar.getValue() == 1)
												return new CFonction(ptListe, nomF.getValue(),
														creeCalculBaseComplexe(ch, ptListe, pdebut + longNom,
																pfin, nomVariableFormelle));
											else {
												if (nombreVar.getValue() == 2) {
													int indicePremiereVirg = premiereVirgule(ch, pdebut
															+ longNom + 1);
													CCalculBase operande1 = creeCalculBaseComplexe(ch,
															ptListe, pdebut + longNom + 1,
															indicePremiereVirg - 1, nomVariableFormelle);
													int indiceParF = parentheseFermanteApresVirgule(ch,
															indicePremiereVirg + 1);
													// A revoir plus tard si on cre des fonctions de trois
													// variables ou plus
													CCalculBase operande2 = creeCalculBaseComplexe(ch,
															ptListe, indicePremiereVirg + 1, indiceParF - 1,
															nomVariableFormelle);
													return new CFonction2Var(ptListe, nomF.getValue(),
															operande1, operande2);
												} else {
													if (nombreVar.getValue() == 3) { // 3 variables
														CCalculBase operande1;
														int indicePremiereVirg = premiereVirgule(ch, pdebut
																+ longNom + 1);
														// Traitement de syntaxe particulier pour un calcul
														// d'intgrale de fonction, le premier argument devant
														// tre le nom
														// d'une fonction prdfinie
														operande1 = creeCalculBaseComplexe(ch, ptListe,
																pdebut + longNom + 1, indicePremiereVirg - 1,
																nomVariableFormelle);
														int indiceDeuxiemeVirg = premiereVirgule(ch,
																indicePremiereVirg + 1);
														CCalculBase operande2 = creeCalculBaseComplexe(ch,
																ptListe, indicePremiereVirg + 1,
																indiceDeuxiemeVirg - 1, nomVariableFormelle);
														int indiceParF = parentheseFermanteApresVirgule(ch,
																indiceDeuxiemeVirg + 1);
														// A revoir plus tard si on cre des fonctions de
														// trois variables ou plus
														CCalculBase operande3 = creeCalculBaseComplexe(ch,
																ptListe, indiceDeuxiemeVirg + 1, indiceParF - 1,
																nomVariableFormelle);
														return new CFonction3Var(ptListe, nomF.getValue(),
																operande1, operande2, operande3);
													}
													// Pas de calcul d'intgrale dans un calcul complexe
													else {
														if (nombreVar.getValue() == 4) { // Calcul d'intgrale
															// Si les bornes ne sont pas relles, c'est leur
															// partie relle qui sera utilise
															int indicePremiereVirg = premiereVirgule(ch, pdebut
																	+ longNom + 1);
															int indiceDeuxiemeVirg = premiereVirgule(ch,
																	indicePremiereVirg + 1);
															int indiceTroisiemeVirg = premiereVirgule(ch,
																	indiceDeuxiemeVirg + 1);
															int indiceParF = parentheseFermanteApresVirgule(ch,
																	indiceTroisiemeVirg + 1);
															String nomVariableSommation = ch.substring(
																	indicePremiereVirg + 1, indiceDeuxiemeVirg);
															/* nomVariableSommation = Kernel
																	.retireEspacesDebutEtFin(nomVariableSommation); */ // Modifi version 4.7.5.2
															nomVariableSommation = nomVariableSommation.trim();
															// Un seul paramtre autoris pour un calcul
															// d'intgrale dans un calcul complexe : la variable
															// formelle
															String[] nomVariableFormelle3 = new String[1];
															nomVariableFormelle3[0] = new String(
																	nomVariableSommation);
															// Bug corrig version 4.6
															/*
															return new CIntegraleDansFormule(ptListe,
																	creeCalculBase(ch, ptListe, pdebut + longNom
																			+ 1, indicePremiereVirg - 1,
																			nomVariableFormelle3),
																	creeCalculBaseComplexe(ch, ptListe,
																			indiceDeuxiemeVirg + 1,
																			indiceTroisiemeVirg - 1,
																			nomVariableFormelle3),
																	creeCalculBaseComplexe(ch, ptListe,
																			indiceTroisiemeVirg + 1, indiceParF - 1,
																			nomVariableFormelle3),
																	nomVariableSommation, true);
															*/
															return new CIntegraleDansFormule(ptListe,
																	creeCalculBase(ch, ptListe, pdebut + longNom
																			+ 1, indicePremiereVirg - 1,
																			nomVariableFormelle3),
																	creeCalculBaseComplexe(ch, ptListe,
																			indiceDeuxiemeVirg + 1,
																			indiceTroisiemeVirg - 1,
																			null),
																	creeCalculBaseComplexe(ch, ptListe,
																			indiceTroisiemeVirg + 1, indiceParF - 1,
																			null),
																	nomVariableSommation, true);
														} else {
															// Fonction  5 variables : la somme indice et le
															// produit indic
															int indicePremiereVirg = premiereVirgule(ch, pdebut
																	+ longNom + 1);
															int indiceDeuxiemeVirg = premiereVirgule(ch,
																	indicePremiereVirg + 1);
															int indiceTroisiemeVirg = premiereVirgule(ch,
																	indiceDeuxiemeVirg + 1);
															int indiceQuatriemeVirg = premiereVirgule(ch,
																	indiceTroisiemeVirg + 1);
															int indiceParF = parentheseFermanteApresVirgule(ch,
																	indiceQuatriemeVirg + 1);
															String nomVariableSommation = ch.substring(
																	indicePremiereVirg + 1, indiceDeuxiemeVirg);
															/* nomVariableSommation = Kernel
																	.retireEspacesDebutEtFin(nomVariableSommation); */
															nomVariableSommation = nomVariableSommation.trim();
															int n = 0;
															if (nomVariableFormelle != null)
																n = nomVariableFormelle.length;
															String[] nomVariableFormelle2 = new String[n + 1];
															for (int j = 0; j < n; j++)
																nomVariableFormelle2[j] = new String(
																		nomVariableFormelle[j]);
															nomVariableFormelle2[n] = new String(
																	nomVariableSommation);
															if (nomF.getValue() == OperateurFonction5Var.sommeC)
																return new CSommeDansFormule(ptListe,
																		creeCalculBaseComplexe(ch, ptListe, pdebut
																				+ longNom + 1, indicePremiereVirg - 1,
																				nomVariableFormelle2),
																		creeCalculBaseComplexe(ch, ptListe,
																				indiceDeuxiemeVirg + 1,
																				indiceTroisiemeVirg - 1,
																				nomVariableFormelle2),
																		creeCalculBaseComplexe(ch, ptListe,
																				indiceTroisiemeVirg + 1,
																				indiceQuatriemeVirg - 1,
																				nomVariableFormelle2),
																		creeCalculBaseComplexe(ch, ptListe,
																				indiceQuatriemeVirg + 1, indiceParF - 1,
																				nomVariableFormelle2),
																		nomVariableSommation);
															else
																return new CProduitDansFormule(ptListe,
																		creeCalculBaseComplexe(ch, ptListe, pdebut
																				+ longNom + 1, indicePremiereVirg - 1,
																				nomVariableFormelle2),
																		creeCalculBaseComplexe(ch, ptListe,
																				indiceDeuxiemeVirg + 1,
																				indiceTroisiemeVirg - 1,
																				nomVariableFormelle2),
																		creeCalculBaseComplexe(ch, ptListe,
																				indiceTroisiemeVirg + 1,
																				indiceQuatriemeVirg - 1,
																				nomVariableFormelle2),
																		creeCalculBaseComplexe(ch, ptListe,
																				indiceQuatriemeVirg + 1, indiceParF - 1,
																				nomVariableFormelle2),
																		nomVariableSommation);
														}
													}
												}
											}
										} else {
											if (ptVa.getPointeur().estFonctionOuSuite()) {
												CCalculAncetre calc = (CCalculAncetre) ptVa.getPointeur();
												int nbvar = calc.nombreVariables();
												if (nbvar == 1)
													return new CAppelFonctionComplexe(ptListe,
															(CCalculAncetre) (ptVa.getPointeur()),
															creeCalculBaseComplexe(ch, ptListe, pdebut
																	+ longNom, pfin, nomVariableFormelle));
												else {
													CCalculBase[] tabcal = new CCalculBase[nbvar];
													int deb = pdebut + longNom;
													int fin = 0;
													for (int i = 0; i < nbvar - 1; i++) {
														fin = premiereVirgule(ch, deb + 1);
														tabcal[i] = creeCalculBaseComplexe(ch, ptListe,
																deb + 1, fin - 1, nomVariableFormelle);
														deb = fin;
													}
													deb++;
													fin = parentheseFermanteApresVirgule(ch, deb);
													tabcal[nbvar - 1] = creeCalculBaseComplexe(ch, ptListe,
															deb, fin - 1, nomVariableFormelle);
													// Modifi version 4.9.7 car on peut appeler une fonction relle de plusieurs variables
													// dans un calcul ou une fonction complexe
													/*
													return new CAppelFonctionComplexeNVar(ptListe, nbvar,
															(CFoncComplexeNVar) (ptVa.getPointeur()), tabcal);
													*/
													return new CAppelFonctionComplexeNVar(ptListe, nbvar,
															(CFoncNVar) (ptVa.getPointeur()), tabcal);
												}
											} else {
												return new CResultatValeurComplexe(ptListe,
														ptVa.getPointeur());
											}
										}
									}
								}
							} else {
								// Dans le cas o l'expression commence par une parenthse
								// ouvrante, elle finit par une fermante et il suffit
								// d'valuer ce que la parenthse contient
								car = ch.charAt(pdebut);
								// Si le premier caractre est une parenthse ouvrante
								// il faut faire attention car le dernier peut tre
								// un espace.
								// Cela a donn un bug dans la version 16 bits
								if (car == '(')
									if (parentheseFermante(ch, pdebut) == -1) throw new ErreurCalculException();
									else return creeCalculBaseComplexe(ch, ptListe, pdebut + 1,
											parentheseFermante(ch, pdebut) - 1, nomVariableFormelle);
								else
								// Le rsultat ne peut tre que numrique}
								{
									/* Abandonn version 4.9.2
									String chaineNombre = new String();
									int ptch = pdebut;
									// car = ch.charAt(ptch); modifi par rapport au C++
									while ((FonteNom.chiffre(car) || (car == '.'))
											&& (ptch <= pfin)) {
										car = ch.charAt(ptch);
										chaineNombre = chaineNombre + car;
										ptch++;
									}
									return new CConstante(ptListe, Double.parseDouble(chaineNombre));
									*/
									String chaineNombre = ch.substring(pdebut, pfin+1);
									if (FonteNom.estDecimalCorrect(chaineNombre)) return new CConstante(ptListe, Double.parseDouble(chaineNombre));
									else return new CCalculBaseNull(chaineNombre);
								}
							}
						}
					}
				}
			}
		}
		catch (Exception e) {
			return new CCalculBaseNull(ch.substring(pdebut,pfin+1));
		}
	}

	public static final boolean autorisationSyntaxe(int item, int itemSuivant) {
		switch (item) {
		case Nombre:
			switch (itemSuivant) {
			case Addition:
			case Multiplication:
			case Puissance:
			case Carre:
			case ParFermante:
			case Inegalite:
			case Blanc:
			case Virgule:
				return true;
			default:
				return false;
			}
		case Addition:
		case Multiplication:
			switch (itemSuivant) {
			case Nombre:
			case ParOuvrante:
			case Valeur:
			case Fonction:
			case Blanc:
				return true;
			default:
				return false;
			}
		case Puissance:
			switch (itemSuivant) {
			case Nombre:
			case Valeur:
			case ParOuvrante:
			case Fonction:
			case Blanc:
				return true;
			default:
				return false;
			}
		case Fonction:
			switch (itemSuivant) {
			case ParOuvrante:
			case Blanc:
				return true;
			default:
				return false;
			}
		case Valeur:
		case Carre:
			switch (itemSuivant) {
			case Addition:
			case Multiplication:
			case Puissance:
			case Carre:
			case ParFermante:
			case Inegalite:
			case Blanc:
			case Virgule:
				return true;
			default:
				return false;
			}
		case ParOuvrante:
			switch (itemSuivant) {
			case Addition:
			case ParOuvrante:
			case Nombre:
			case Valeur:
			case Fonction:
			case Blanc:
				return true;
			default:
				return false;
			}
		case ParFermante:
			switch (itemSuivant) {
			case ParFermante:
			case Addition:
			case Multiplication:
			case Puissance:
			case Carre:
			case Inegalite:
			case Blanc:
			case Virgule:
				return true;
			default:
				return false;
			}
		case Inegalite:
			switch (itemSuivant) {
			case Addition:
			case Nombre:
			case ParOuvrante:
			case Valeur:
			case Fonction:
			case Blanc:
				return true;
			default:
				return false;
			}
		case Virgule:
			switch (itemSuivant) {
			case Addition:
			case Nombre:
			case ParOuvrante:
			case Valeur:
			case Fonction:
			case Blanc:
				return true;
			default:
				return false;
			}
		default:
			return false; // / normalement inutile
		} // switch Item
	}

	abstract CCalculBase getClone(CListeObjets listeSource,
			CListeObjets listeCible);

	// Modification version 3.5.1 : existe prend un paramtre boolean infoRandom
	// C'est d  l'optimisation de la fonction de 3 variables si(test,
	// valeurSiVrai, valeurSiFaux) qui ncessite de connatre la valeur
	// du premier argument test pour savoir si le rsultat existe ou non. Si test
	// renvoie 1, existe renvoie valeurSiVrai.existe(inforRandom)
	// et sinon ValeurSiFaux.existe(infoRandom)
	public boolean existe(boolean infoRandom) {
		return true;
	}
	// Ajout version version 4.9.5.4.
	// Pour une fonction relle ou complexe de une ou plusieurs variables, quand on teste son existence, on appellait
	// existe. Or CFonction3Var.existe, dans le cas d'une fonctio si provoque un appel de calcul pour un calcul normal qu'il faut viter quand
	// une fonction utilise un si et qu'on teste son existence
	// A revoir : Le paramtre random n'est pas utilis.
	public boolean existePourFonc(boolean infoRandom) {
		return existe(infoRandom);
	}
	
	

	// / resultat sera redfinie pour tous les descendants pouvant ne pas exister
	public double resultat(boolean infoRandom) throws ErreurCalculException {
		return 0;
	}

	// Ajout pour la version 2.0 calculs sur les complexes.
	// Le rsultat est renvoy dans z
	public void resultatComplexe(boolean infoRandom, Complexe zRes)
			throws ErreurCalculException {
		// / Sera redfinie pour tous les descendants pouvant ne pas exister
		zRes.x = resultat(infoRandom);// Par dfaut on suppose le calcul rel
		zRes.y = 0;
	}

	public double resultatFonction(boolean infoRandom, double valeurparametre)
			throws ErreurCalculException {
		return resultat(infoRandom);
	}

	/**
	 * Ajout pour la version Java 3.0 / Appel de fonction avec plusieurs
	 * paramtres
	 */
	public double resultatFonction(boolean infoRandom, double[] valeurparametre)
			throws ErreurCalculException {
		return resultat(infoRandom);
	}

	// / Sera redfinie pour tous les descendants pouvant ne pas exister
	void resultatFonctionComplexe(boolean infoRandom, Complexe valeurParametre,
			Complexe zRes) throws ErreurCalculException {
		resultatComplexe(infoRandom, zRes);
	}

	/**
	 * Ajout pour la version Java 3.0 / Appel de fonction complexe avec plusieurs
	 * paramtres
	 */
	void resultatFonctionComplexe(boolean infoRandom, Complexe[] valeurParametre,
			Complexe zRes) throws ErreurCalculException {
		resultatComplexe(infoRandom, zRes);
	}

	public boolean dependDe(CElementBase p) {
		elementTestePourDependDe = p;
		return false;
	}
	
	// Ajout version 4.8
	public void initialisePourDependance() {
		elementTestePourDependDe = null;
		elementTestePourDependDePourRec = null; // Ajout version 4.8
	}

	// Ajout version 4.8
	public boolean memoriseDependance(boolean resultat) {
		dependDeElementTeste = resultat;
		return resultat;
	}

	// Ajout version 4.8
	public boolean memoriseDependancePouRec(boolean resultat) {
		dependDeElementTestePourRec = resultat;
		return resultat;
	}


	public boolean dependDePourReclassement(CElementBase p) {
		elementTestePourDependDePourRec = p; // Rajout version 4.8
		return false;
	}

	/**
	 * La fonction suivante renverra le mme rsultat que DependDe sauf dans le
	 * cas o le calcul fait rferrence  la fonction Rand de gnration d'un
	 * nombre alatoire
	 * 
	 */
	public boolean dependDePourBoucle(CElementBase p) {
		return false;
	}

	/**
	 * Renvoie 0 si pChaine ne commence pas par le nom d'une fonction prdfinie
	 * relle. Si on renvoie la longueur de la chaine reprsentant cette fonction
	 * pour la syntaxe (par exemple 3) pour Sin et renvoie dans Resultat.x
	 * l'indice de la fonction trouve La comparaison se fait sans tenir compte de
	 * la casse Modifi version 3.0 pour tenir compte des fonctions de 2 variables
	 * Modifi version 4.7.7 en rajoutant une parenthse pour le test car il peut par exemple
	 * y avoir une mesure de longuer ch alors que ch est un nom de fonction prdfinie.
	 */
	public static final int commenceParNomFonctionReellePredefinie(
			String pChaine, Int resultat, Int nombreVariables) {
		int longueurNom = 0;
		int i = 0;
		for (i = 0; i < OperateurFonction.nomsFonctions.length; i++) {
			if (pChaine.toLowerCase().startsWith(
					OperateurFonction.nomsFonctions[i].toLowerCase()+"(")) {
				if (OperateurFonction.nomsFonctions[i].length() > longueurNom) {
					nombreVariables.setValue(1);
					resultat.setValue(i);
					longueurNom = OperateurFonction.nomsFonctions[i].length();
				}
			}
		}
		i = 0;
		for (i = 0; i < OperateurFonction2Var.nomsFonctions2Var.length; i++) {
			if (pChaine.toLowerCase().startsWith(
					OperateurFonction2Var.nomsFonctions2Var[i].toLowerCase()+"(")) {
				if (OperateurFonction2Var.nomsFonctions2Var[i].length() > longueurNom) {
					nombreVariables.setValue(2);
					resultat.setValue(i);
					longueurNom = OperateurFonction2Var.nomsFonctions2Var[i].length();
				}
			}
		}
		i = 0;
		for (i = 0; i < OperateurFonction3Var.nomsFonctions3Var.length; i++) {
			if (pChaine.toLowerCase().startsWith(
					OperateurFonction3Var.nomsFonctions3Var[i].toLowerCase()+"(")) {
				if (OperateurFonction3Var.nomsFonctions3Var[i].length() > longueurNom) {
					nombreVariables.setValue(3);
					resultat.setValue(i);
					longueurNom = OperateurFonction3Var.nomsFonctions3Var[i].length();
				}
			}
		}
		for (i = 0; i < OperateurFonction4Var.nomsFonctions4Var.length; i++) {
			if (pChaine.toLowerCase().startsWith(
					OperateurFonction4Var.nomsFonctions4Var[i].toLowerCase()+"(")) {
				if (OperateurFonction4Var.nomsFonctions4Var[i].length() > longueurNom) {
					nombreVariables.setValue(4);
					resultat.setValue(i);
					longueurNom = OperateurFonction4Var.nomsFonctions4Var[i].length();
				}
			}
		}

		i = 0;
		for (i = 0; i < OperateurFonction5Var.nomsFonctions5Var.length; i++) {
			if (pChaine.toLowerCase().startsWith(
					OperateurFonction5Var.nomsFonctions5Var[i].toLowerCase()+"(")) {
				if (OperateurFonction5Var.nomsFonctions5Var[i].length() > longueurNom) {
					nombreVariables.setValue(5);
					resultat.setValue(i);
					longueurNom = OperateurFonction5Var.nomsFonctions5Var[i].length();
				}
			}
		}
		return longueurNom;
	}

	/**
	 * Renvoie 0 si pChaine ne commence pas par le nom d'une fonction prdfinie
	 * complexe. Si on renvoie la longueur de la chaine reprsentant cette
	 * fonction pour la syntaxe (par exemple 3) pour Sin et renvoie dans
	 * Resultat.x l'indice de la fonction trouve La comparaison se fait sans
	 * tenir compte de la casse
	 * Modifi version 4.7.7 en rajoutant une parenthse pour le test pour viter les confusions avec des mesures de longueur
	 */
	public static final int commenceParNomFonctionComplexePredefinie(
			String pChaine, Int resultat, Int nombreVariables) {
		int longueurNom = 0;
		int i = 0;
		for (i = 0; i < OperateurFonction.nomsFonctionsComplexes.length; i++) {
			if (pChaine.toLowerCase().startsWith(
					OperateurFonction.nomsFonctionsComplexes[i].toLowerCase()+"(")) {
				if (OperateurFonction.nomsFonctionsComplexes[i].length() > longueurNom) {
					nombreVariables.setValue(1);
					resultat.setValue(i + OperateurFonction.indPremiereFoncComplexe);
					longueurNom = OperateurFonction.nomsFonctionsComplexes[i].length();
				}
			}
		}
		;
		i = 0;
		for (i = 0; i < OperateurFonction2Var.nomsFonctionsComplexes2Var.length; i++) {
			if (pChaine.toLowerCase().startsWith(
					OperateurFonction2Var.nomsFonctionsComplexes2Var[i].toLowerCase()+"(")) {
				if (OperateurFonction2Var.nomsFonctionsComplexes2Var[i].length() > longueurNom) {
					nombreVariables.setValue(2);
					resultat.setValue(i
							+ OperateurFonction2Var.indicePremiereFonctionComplexe);
					longueurNom = OperateurFonction2Var.nomsFonctionsComplexes2Var[i]
							.length();
				}
			}
		}
		i = 0;
		for (i = 0; i < OperateurFonction3Var.nomsFonctionsComplexes3Var.length; i++) {
			if (pChaine.toLowerCase().startsWith(
					OperateurFonction3Var.nomsFonctionsComplexes3Var[i].toLowerCase()+"(")) {
				if (OperateurFonction3Var.nomsFonctionsComplexes3Var[i].length() > longueurNom) {
					nombreVariables.setValue(3);
					resultat.setValue(i
							+ OperateurFonction3Var.indicePremiereFonctionComplexe);
					longueurNom = OperateurFonction3Var.nomsFonctionsComplexes3Var[i]
							.length();
				}
			}
		}
		i = 0;
		for (i = 0; i < OperateurFonction4Var.nomsFonctionsComplexes4Var.length; i++) {
			if (pChaine.toLowerCase().startsWith(
					OperateurFonction4Var.nomsFonctionsComplexes4Var[i].toLowerCase()+"(")) {
				if (OperateurFonction4Var.nomsFonctionsComplexes4Var[i].length() > longueurNom) {
					nombreVariables.setValue(4);
					resultat.setValue(i
							+ OperateurFonction4Var.indicePremiereFonctionComplexe);
					longueurNom = OperateurFonction4Var.nomsFonctionsComplexes4Var[i]
							.length();
				}
			}
		}
		i = 0;
		for (i = 0; i < OperateurFonction5Var.nomsFonctionsComplexes5Var.length; i++) {
			if (pChaine.toLowerCase().startsWith(
					OperateurFonction5Var.nomsFonctionsComplexes5Var[i].toLowerCase()+"(")) {
				if (OperateurFonction5Var.nomsFonctionsComplexes5Var[i].length() > longueurNom) {
					nombreVariables.setValue(5);
					resultat.setValue(i
							+ OperateurFonction5Var.indicePremiereFonctionComplexe);
					longueurNom = OperateurFonction5Var.nomsFonctionsComplexes5Var[i]
							.length();
				}
			}
		}
		return longueurNom;
	}

	/**
	 * Fonction renvoyant true si pChaine commence par le nom d'une fonction
	 * prdfinie relle ou complexe
	 */
	public static boolean commenceParNomFonctionPredefinie(String pChaine) {
		Int res = new Int();
		Int nvar = new Int();
		return ((commenceParNomFonctionReellePredefinie(pChaine, res, nvar) != 0) || (commenceParNomFonctionComplexePredefinie(
				pChaine, res, nvar) != 0));
	}

	/**
	 * Fonction renvoyant true si chaine est le nom d'une fonction relle
	 * prdfinie
	 * 
	 * @param chaine
	 * @return
	 */
	public static final boolean egalNomFonctionReellePredefinie(String chaine) {
		int i = 0;
		while (i < OperateurFonction.nomsFonctions.length) {
			if (chaine.toLowerCase().equals(
					(OperateurFonction.nomsFonctions[i].toLowerCase())))
				return true;
			i++;
		}
		; // while
		i = 0;
		while (i < OperateurFonction2Var.nomsFonctions2Var.length) {
			if (chaine.toLowerCase().equals(
					(OperateurFonction2Var.nomsFonctions2Var[i].toLowerCase())))
				return true;
			i++;
		}
		; // while
		i = 0;
		while (i < OperateurFonction3Var.nomsFonctions3Var.length) {
			if (chaine.toLowerCase().equals(
					(OperateurFonction3Var.nomsFonctions3Var[i].toLowerCase())))
				return true;
			i++;
		}
		; // while
		i = 0;
		while (i < OperateurFonction4Var.nomsFonctions4Var.length) {
			if (chaine.toLowerCase().equals(
					(OperateurFonction4Var.nomsFonctions4Var[i].toLowerCase())))
				return true;
			i++;
		}
		; // while
		i = 0;
		while (i < OperateurFonction5Var.nomsFonctions5Var.length) {
			if (chaine.toLowerCase().equals(
					(OperateurFonction5Var.nomsFonctions5Var[i].toLowerCase())))
				return true;
			i++;
		}
		; // while
		return false;
	}

	/**
	 * Fonction renvoyant true si chaine est le nom d'une fonction complexe
	 * prdfinie
	 * 
	 * @param chaine
	 * @return
	 */
	public static final boolean egalNomFonctionComplexePredefinie(String chaine) {
		int i = 0;
		while (i < OperateurFonction.nomsFonctionsComplexes.length) {
			if (chaine.toLowerCase().equals(
					(OperateurFonction.nomsFonctionsComplexes[i].toLowerCase())))
				return true;
			i++;
		}
		; // while
		i = 0;
		while (i < OperateurFonction2Var.nomsFonctions2Var.length) {
			if (chaine.toLowerCase().equals(
					(OperateurFonction2Var.nomsFonctionsComplexes2Var[i].toLowerCase())))
				return true;
			i++;
		}
		; // while
		i = 0;
		while (i < OperateurFonction3Var.nomsFonctionsComplexes3Var.length) {
			if (chaine.toLowerCase().equals(
					(OperateurFonction3Var.nomsFonctionsComplexes3Var[i].toLowerCase())))
				return true;
			i++;
		}
		; // while
		i = 0;
		while (i < OperateurFonction4Var.nomsFonctionsComplexes4Var.length) {
			if (chaine.toLowerCase().equals(
					(OperateurFonction3Var.nomsFonctionsComplexes3Var[i].toLowerCase())))
				return true;
			i++;
		}
		; // while
		while (i < OperateurFonction5Var.nomsFonctionsComplexes5Var.length) {
			if (chaine.toLowerCase().equals(
					(OperateurFonction5Var.nomsFonctionsComplexes5Var[i].toLowerCase())))
				return true;
			i++;
		}
		; // while
		return false;
	}

	/**
	 * Fonction renvoyant true si chaine est le nom d'une fonction relle ou
	 * complexe prdfinie
	 * 
	 * @param chaine
	 * @return
	 */
	public static final boolean egalNomFonctionReelleOuComplexePredefinie(
			String chaine) {
		return egalNomFonctionReellePredefinie(chaine)
				|| egalNomFonctionComplexePredefinie(chaine);
	}

	// Fonction renvoyant, dans une chaine de caractres ch dont le dbut
	// a pour indice pdeb et la fin pfin l'indice du premier
	// signe + ou - rencont dans une expression algbrique  partir de la fin
	// Renvoie -1 si n'en trouve pas
	public static final int premiereSomme(String ch, int pdeb, int pfin) {
		int ptch;
		char car;
		int resultat;

		ptch = pfin;
		do {
			car = ch.charAt(ptch);
			resultat = ptch;
			if (car == ')')
				ptch = parentheseOuvrante(ch, pdeb, ptch);
			ptch--;
		} while ((ptch >= pdeb) && (car != '+') && (car != '-'));
		if ((car == '+') || (car == '-'))
			return resultat;
		else
			return -1;
	}

	// Fonction renvoyant, dans une chaine de caractres ch dont le dbut
	// a pour indice pdeb et la fin pfin l'indice du premier
	// signe * ou / rencont dans une expression algbrique  partir de la fin
	// Renvoie -1 si n'en trouve pas
	public static final int premierProduit(String ch, int pdeb, int pfin) {
		int ptch;
		char car;
		int resultat;

		ptch = pfin;
		do {
			car = ch.charAt(ptch);
			resultat = ptch;
			if (car == ')')
				ptch = parentheseOuvrante(ch, pdeb, ptch);
			ptch--;
		} while ((ptch >= pdeb) && (car != '*') && (car != '/'));
		if ((car == '*') || (car == '/'))
			return resultat;
		else
			return -1;
	}

	// Fonction renvoyant l'indice dans la chaine ch de la premire ingalit
	// rencontree et -1 s'il n'y en a pas.
	// Renvoie dans retour.x l'indice du caractre prcdent l'ingalit
	// et dans retour.y l'oprateur associ  cette ingalit (dfini dans
	// COperateur)
	public static final int premiereInegalite(String ch, int pdeb, int pfin,
			Point retour) {
		int ptch;
		char car, carPrecedent;
		int resultat;

		ptch = pfin;
		do {
			car = ch.charAt(ptch);
			resultat = ptch;
			if (car == ')')
				ptch = parentheseOuvrante(ch, pdeb, ptch);
			ptch = ptch - 1;
		} while ((ptch >= pdeb) && (car != '=') && (car != '>') && (car != '<'));
		switch (car) {
		case '<':
			retour.x = resultat - 1;
			retour.y = Operateur.Inf;
			return resultat;
		case '>':
			retour.x = resultat - 1;
			retour.y = Operateur.Sup;
			if (ptch > pdeb) {
				ptch = resultat - 1;
				carPrecedent = ch.charAt(ptch);
				if (carPrecedent == '<') {
					retour.x = ptch - 1;
					retour.y = Operateur.Diff;
				}
			}
			return resultat;
		case '=':
			if (ptch >= pdeb) {
				ptch = resultat - 1;
				carPrecedent = ch.charAt(ptch);
				switch (carPrecedent) {
				case '<':
					retour.x = ptch - 1;
					retour.y = Operateur.InfOuEgal;
					return resultat;
				case '>':
					retour.x = ptch - 1;
					retour.y = Operateur.SupOuEgal;
					return resultat;
				default:
					retour.x = ptch;
					retour.y = Operateur.Egalite;
					return resultat;
				} // switch CarPrecedent
			}			
		default:
			return -1;
		} // switch car
	}

	// Fonction renvoyant l'indice dans la chaine ch du premier oprateur logique
	// rencontr et -1 s'il n'y en a pas.
	// Renvoie dans retour.x l'indice du caractre prcdent l'ingalit
	// et dans retour.y l'oprateur associ  cette ingalit (dfini dans
	// COperateur)
	public static final int premierOperateurLogique(String ch, int pdeb, int pfin,
			Point retour) {
		int ptch;
		char car;
		int resultat;

		ptch = pfin;
		do {
			car = ch.charAt(ptch);
			resultat = ptch;
			if (car == ')')
				ptch = parentheseOuvrante(ch, pdeb, ptch);
			ptch = ptch - 1;
		} while ((ptch >= pdeb) && (car != '&') && (car != '|'));
		switch (car) {
		case '&':
			retour.x = resultat - 1;
			retour.y = Operateur.And;
			return resultat;
		case '|':
			retour.x = resultat - 1;
			retour.y = Operateur.Or;
			return resultat;
			
		default:
			return -1;
		} // switch car
	}

	// Fonction renvoyant, dans une chaine de caractres ch dont le dbut
	// a pour indice pdeb et la fin pfin l'indice du premier
	// signe ^ rencont dans une expression algbrique  partir de la fin
	// Renvoie -1 si n'en trouve pas
	// Ajout d'une fonctionnali par rapport  la version C++ 1.8 : dtecte aussi
	// les carrs 
	public final static int premierePuissance(String ch, int pdeb, int pfin) {
		int ptch;
		char car;
		int resultat;

		ptch = pfin;
		do {
			car = ch.charAt(ptch);
			resultat = ptch;
			if (car == ')')
				ptch = parentheseOuvrante(ch, pdeb, ptch);
			ptch--;
		} while ((ptch >= pdeb) && (car != '^') && (car != ''));
		if ((car == '^') || (car == ''))
			return resultat;
		else
			return -1;
	}

	// Fonction appele dans le cas ou pRecherche est l'indice dans chaine d'une
	// parenthse fermante et renvoyant l'indice de la parenthse
	// ouvrante correspondante. Elle ne doit tre appele que
	// si pdebut<=pRecherche et si pRecherche pointe sur une
	// parenthse fermante et si la syntaxe est correcte au point de vue
	// parenthses
	public static final int parentheseOuvrante(String chaine, int pdebut,
			int pRecherche) {
		int p;
		char ch;
		int somme;

		somme = 0;
		p = pRecherche;
		do {
			ch = chaine.charAt(p);
			if (ch == ')')
				somme--;
			else {
				if (ch == '(')
					somme++;
			}
			p = p - 1;
		} while ((somme != 0) && (p >= pdebut));
		return p + 1;
	}

	// Ajout pour version 2 du CNED
	// Fonction appele dans le cas ou pDebut est l'indice dans chaine d'une
	// parenthse ouvrante et renvoyant l'indice de la parenthse
	// fermante correspondante. Elle ne doit tre appele que
	// si on est sur que la parenthse fermante existe bien
	/* Modifi version 4.7.4.1
	public static final int parentheseFermante(String chaine, int pdebut) {
		int p;
		char ch;
		int somme;

		somme = 0;
		p = pdebut;
		do {
			ch = chaine.charAt(p);
			if (ch == '(')
				somme++;
			else {
				if (ch == ')')
					somme--;
			}
			p++;
		} while (somme != 0);
		return p - 1;
	}
	*/
	// Version 4.7.4.1 : Comme on peut appeler cette fonction avec une erreur de syntaxe
	// Il peut ne pas y avoir de parenthse fermante correspondant
	public static final int parentheseFermante(String chaine, int pdebut) {
		int p;
		char ch;
		int somme;

		somme = 1;
		p = pdebut + 1;
		while (p < chaine.length()) {
			ch = chaine.charAt(p);
			if (ch == '(')
				somme++;
			else {
				if (ch == ')')
					somme--;
			}
			if (somme == 0) break;
			p++;
		}
		if (somme == 0) return p;
		else return - 1; // On renvoie -1 si pas trouv
	}

	/**
	 * Fonction renvoyant l'indice de la premire virgule suivant une parenthse
	 * ouvrante Avant appel pDebut pointe sur le caractre suivant la parenthse
	 * ouvrante. La chane doit tre correcte sur le plan syntaxique acant appel
	 * 
	 * @param chaine
	 * @param pDebut
	 * @return
	 */
	public static final int premiereVirgule(String chaine, int pDebut) {
		char car;
		int somme = 1;
		int i;
		for (i = pDebut; i < chaine.length(); i++) {
			car = chaine.charAt(i);
			if (car == '(')
				somme++;
			else if (car == ')')
				somme--;
			else if ((car == ',') && (somme == 1))
				return i;
		}
		return i; // Pour satisfaire le compilateur mais on ne doit pas arriver ici
	}

	/**
	 * Fonction renvoyant l'indice de la parenthse fermante correspondant  une
	 * virgue Avant appel pDebut pointe sur le caractre suivant la virgule. La
	 * chane doit tre correcte sur le plan syntaxique acant appel
	 * 
	 * @param chaine
	 * @param pDebut
	 * @return
	 */
	public static final int parentheseFermanteApresVirgule(String chaine,
			int pDebut) {
		char car;
		int somme = 1;
		int i;
		for (i = pDebut; i < chaine.length(); i++) {
			car = chaine.charAt(i);
			if (car == '(')
				somme++;
			else if (car == ')') {
				somme--;
				if (somme == 0)
					return i;
			}
		}
		return i; // Pour satisfaire le compilateur mais on ne doit pas arriver ici
	}

	/**
	 * Procedure vrifiant la syntaxe de l'expression relle pointe par Ch en
	 * tenant compte du fait qu'elle ne doit pas utiliser de valeur dont l'indice
	 * dans PtListe est strictement suprieur  IndiceMaxiDansListe la diffrence
	 * par rapport  la version C++ est que cette fonction ne renvoie pas de
	 * renseignement sur le message d'erreur  afficher
	 */
	public final static boolean verifieSyntaxe(CListeObjets ptListe, String ch,
			Int indiceErreur, int indiceMaxiDansListe, String[] parametreFormel) {
		int i;
		int indiceDansListe;
		int pdebut;
		char car, carSuivant;
		boolean erreur;
		int longueurCh;
		int itemPrecedent = 0;
		int itemSuivant = 0;
		boolean pointDecimalRencontre;
		int sommeParentheses;
		int longNom;
		Int nomF = new Int();
		PointeurValDyn ptVa = new PointeurValDyn();
		boolean queDesBlancs;
		Int parametreTrouve = new Int(0);
		Int nombreVariables = new Int(0);
		int nivPar = 0; // Niveau de parenthses
		boolean[] appelFonction = new boolean[nombreMaxParentheses + 1]; // 64
																																			// niveaux
																																			// de
																																			// parenthses
																																			// imbriqus
																																			// maxi
		for (int j = 0; j <= nombreMaxParentheses; j++)
			appelFonction[j] = false;

		erreur = false;
		longueurCh = ch.length();
		// On vrifie d'abord qu'il n'y a pas d'erreur de parenthse
		if (longueurCh == 0) {
			// IndiceMessageErreur = IDE_NOUVEAUCALCUL4;
			indiceErreur.setValue(0);
			// MessageBeep(0);
			return false;
		}
		i = 0;
		sommeParentheses = 0;
		while ((i < longueurCh) && (sommeParentheses >= 0)) {
			car = ch.charAt(i);
			switch (car) {
			case '(':
				sommeParentheses++;
				break;
			case ')':
				sommeParentheses--;
			} // switch
			i++;
		}
		if (sommeParentheses != 0) {
			indiceErreur.setValue(i);
			// IndiceMessageErreur = IDE_SYNTAXE2;
			return false;
		}
		// Ajout version 4.5.2
		if (ch.equals(".")) {
			indiceErreur.setValue(0);
			return false;
		}
		// Fin ajout
		i = 0;
		// Tout se passe au dbut comme si l'expression tait prcde
		// d'une parenthse ouvrante
		itemPrecedent = ParOuvrante;
		while ((i < longueurCh) && !erreur) {
			car = ch.charAt(i);
			pointDecimalRencontre = false;
			if (FonteNom.chiffre(car) || (car == '.')) {
				while ((FonteNom.chiffre(car) || (car == '.')) && (i < longueurCh)
						&& !erreur) {
					if (car == '.') {
						if (pointDecimalRencontre) {
							erreur = true;
							indiceErreur.setValue(i - 1);
							// IndiceMessageErreur = IDE_SYNTAXE1;
						} else
							pointDecimalRencontre = true;
					}
					i++;
					if (i < longueurCh)
						car = ch.charAt(i);
				} // while
				if (!erreur)
					itemSuivant = Nombre;
			} else // cas o le caractre en cours n'est pas un nombre
			// le caractre en cours n'est pas un chiffre ou un point dcimal}
			{
				pdebut = i;
				// On regarde si cela commence par le nom d'une fonction ou
				// une valeur dj dfinie
				longNom = ptListe.commenceParNomFonctionOuValeurOuParametre(
						ch.substring(pdebut), parametreFormel, ptVa, nomF, parametreTrouve,
						nombreVariables);
				if (longNom > 0) {
					if (parametreTrouve.getValue() != -1) {
						itemSuivant = Valeur;
						i = i + longNom;
					} else {
						appelFonction[nivPar + 1] = true;
						if (ptVa.getPointeur() == null) {
							if (((i + longNom) < ch.length())
									&& ch.charAt(i + longNom) != '(') {
								erreur = true;
								indiceErreur.setValue(i + longNom);
							} else {
								itemSuivant = Fonction;
								// Ajout version 4.5.2 pour corriger un bug
								if (!autorisationSyntaxe(itemPrecedent, itemSuivant)) {
									erreur = true;
									indiceErreur.setValue(i);
									break;
								}
								// Fin ajout
								// Il faut regarder si la fonction est de plusieurs variables
								int nvirg = nombreVirgules(ch, i + longNom + 1);
								if (nvirg != nombreVariables.getValue() - 1) {
									if (nvirg <= nombreVariables.getValue() - 1)
										indiceErreur.setValue(indiceCaractereVirgule(ch, i
												+ longNom + 1, nvirg));
									else
										indiceErreur.setValue(indiceCaractereVirgule(ch, i
												+ longNom + 1, nombreVariables.getValue() - 1) - 1);
									erreur = true;
								} else {
									// Si le nombre de variables est suprieur ou gal  4, il
									// s'agit soit d'un calcul d'intgrale
									// soit d'une somme indice pour laquelle la syntaxe est la
									// mme
									if (nombreVariables.getValue() >= 4) {

										nivPar++;
										int indicePremVirg = premiereVirgule(ch, i + longNom + 1);
										String chformuleASommer = ch.substring(i + longNom + 1,
												indicePremVirg);
										int indiceDeuxiemeVirg = premiereVirgule(ch,
												indicePremVirg + 1);
										String nomVariableSommation = ch.substring(
												indicePremVirg + 1, indiceDeuxiemeVirg);
										// Il faut retirer les espaces de dbut et de fin
										// On vrifie si le nom de variable formelle pour la somme
										// est valable
										/* nomVariableSommation = Kernel
												.retireEspacesDebutEtFin(nomVariableSommation); */
										nomVariableSommation = nomVariableSommation.trim();
										if (!ptListe
												.validationNomCalculSansMessage(nomVariableSommation)
												|| ptListe.egalNomValeurOuFonctionOuMesure(
														nomVariableSommation, -1)) {
											erreur = true;
											indiceErreur.setValue(indiceDeuxiemeVirg);
										} else {
											// On vrifie si la syntaxe du calcul  sommer est
											// correct.
											// Pour cela on appelle  nouveau vrifieSyntaxe
											Int indiceErreur2 = new Int();
											int n = 0;
											if (parametreFormel != null) n = parametreFormel.length;
											String[] parametreFormel2 = new String[n + 1];
											for (int j = 0; j < n; j++)
												parametreFormel2[j] = new String(parametreFormel[j]);
											parametreFormel2[n] = new String(nomVariableSommation);
											if (!verifieSyntaxe(ptListe, chformuleASommer,
													indiceErreur2, indiceMaxiDansListe, parametreFormel2)) {
												erreur = true;
												indiceErreur.setValue(i + longNom + 1
														+ indiceErreur2.getValue());
											} else {
												i = indiceDeuxiemeVirg + 1;
												itemPrecedent = Valeur;
												itemSuivant = Virgule;
											}
										}
									} else
										i += longNom;
								}
							}
						} else {
							CValeurDynamique vd = ptVa.getPointeur();
							indiceDansListe = ptListe.indexOf(vd);
							if (indiceDansListe > indiceMaxiDansListe) {
								erreur = true;
								indiceErreur.setValue(i);
							} else {
								i += longNom;
								if (vd.estFonctionOuSuite()) {
									if ((i < ch.length()) && ch.charAt(i) != '(') {
										erreur = true;
										indiceErreur.setValue(i + longNom);
									} else {
										itemSuivant = Fonction;
										CCalculAncetre calc = (CCalculAncetre) vd;
										int nbvar = calc.nombreVariables();
										if (nbvar >= 2) {
											int nvirg = nombreVirgules(ch, i + 1);
											if (nvirg != nombreVariables.getValue() - 1) {
												if (nvirg <= nombreVariables.getValue() - 1)
													indiceErreur.setValue(indiceCaractereVirgule(ch,
															i + 1, nvirg));
												else
													indiceErreur.setValue(indiceCaractereVirgule(ch,
															i + 1, nombreVariables.getValue() - 1) - 1);
												erreur = true;
											}
										}
										// Bug corrig version 4.6 : Il ne faut pas qu'un appel de fonction d'une variable comprenne des virgules
										else {
											if (nombreVirgules(ch, i + 1) > 0) {
												indiceErreur.setValue(ch.indexOf(',', i+1));
												erreur = true;
											}
										}
									}
								} else
									itemSuivant = Valeur;
							}
						}
					}
				} else {
					switch (car) {
					case '+':
					case '-':
						itemSuivant = Addition;
						i++;
						break;
					case '*':
					case '/':
						itemSuivant = Multiplication;
						i++;
						break;
					case '(':
						if (nivPar >= nombreMaxParentheses - 1) {
							erreur = true;
							indiceErreur.setValue(i);
						} else {
							itemSuivant = ParOuvrante;
							i++;
							nivPar++;
						}
						break;
					case ')':
						appelFonction[nivPar] = false;
						nivPar--;
						itemSuivant = ParFermante;
						i++;
						break;
					case '^':
						itemSuivant = Puissance;
						i++;
						break;
					// Ajout par rapport  la version C++ 1.8, autorisation du carr 
					case '':
						itemSuivant = Carre;
						i++;
						break;
					// ni nombre ni oprateur}
					case '<':
						itemSuivant = Inegalite;
						i++;
						if (i <= longueurCh - 2) {
							carSuivant = ch.charAt(i);
							if ((carSuivant == '=') || (carSuivant == '>'))
								i++;
						}
						break;
					case '>':
						itemSuivant = Inegalite;
						i++;
						if (i <= longueurCh - 2) {
							carSuivant = ch.charAt(i);
							if (carSuivant == '=')
								i++;
						}
						break;
					// Ajout version 4.9.5
					case '&':
					case '|':
					// Fin ajout
					case '=':
						itemSuivant = Inegalite;
						i++;
						break;
					case ' ':
						itemSuivant = Blanc;
						i++;
						break;
					case ',':
						if (!appelFonction[nivPar]) {
							erreur = true;
							indiceErreur.setValue(i);
						} else {
							itemSuivant = Virgule;
							i++;
						}
						break;
					default: // switch Car
						erreur = true;
						// IndiceMessageErreur = IDE_SYNTAXE1;
						indiceErreur.setValue(i - 1);
					} // switch Car
				}// else du if LongNom > 0
			} // else du if Chiffre(Car) || (Car == '.'))
			if (!erreur) {
				if (autorisationSyntaxe(itemPrecedent, itemSuivant)) {
					if (itemSuivant != Blanc)
						itemPrecedent = itemSuivant;
				} else {
					erreur = true;
					// IndiceMessageErreur = IDE_SYNTAXE1;
					indiceErreur.setValue(i - 1);
				}
			}
		} // while
		// Il faut aussi regarder si la fin de l'expression est correcte
		itemSuivant = ParFermante;
		if (!erreur) {
			if (!autorisationSyntaxe(itemPrecedent, itemSuivant)) {
				// MessageBeep(0);
				erreur = true;
				indiceErreur.setValue(longueurCh);
				// IndiceMessageErreur = IDE_SYNTAXE1;
			}
		}
		if (indiceErreur.getValue() < 0) indiceErreur.setValue(0);
		// Modification version 4.6
		/*
		else if (indiceErreur.getValue() >= longueurCh)
			indiceErreur.setValue(longueurCh - 1);
		*/
		// Modification version 4.7.9
		else if (indiceErreur.getValue() > longueurCh) indiceErreur.setValue(longueurCh);
		// On regarde si la chane ne contient que des blancs
		i = 0;
		queDesBlancs = true;
		while ((i <= longueurCh - 1) && queDesBlancs) {
			car = ch.charAt(i);
			if (car != ' ')
				queDesBlancs = false;
			else
				i++;
		}
		if (queDesBlancs) {
			erreur = true;
			indiceErreur.setValue(longueurCh);
			// IndiceMessageErreur = IDE_SYNTAXE1;
		}
		return !erreur;
	}

	/**
	 * Fonction appele dans VerifieSyntaxe pour un appel de fonctions  plusieurs
	 * variables. Renvoie le nombre de virgules trouves pour un appel de
	 * paramtre Renvoie - 1 si la syntaxe n'est pas correcte
	 * 
	 * @param ch
	 * @param inddeb
	 *          : l'indice de dbut de recherche dans ch aprs la parenthse
	 *          ouvrante
	 * @return
	 */
	public final static int nombreVirgules(String ch, int inddeb) {
		int i = inddeb;
		int nbvirg = 0;
		if (i >= ch.length())
			return -1;
		int parentheses = 1;
		while ((i < ch.length()) && (parentheses >= 1)) {
			char car = ch.charAt(i);
			if (car == '(')
				parentheses++;
			if (car == ')')
				parentheses--;
			if ((car == ',') && (parentheses == 1))
				nbvirg++;
			i++;
		}
		return nbvirg;
	}

	/**
	 * Fonction renvoyant l'indice dans la chane ch du caractre suivant la
	 * virgule n i
	 * 
	 * @param ch
	 * @param inddeb
	 * @param virg
	 * @return
	 */
	public static final int indiceCaractereVirgule(String ch, int inddeb, int virg) {
		int i = inddeb;
		int nbvirg = 0;
		if ((virg == 0) || (i > ch.length()))
			return ch.length() - 1;
		int parentheses = 1;
		while ((i < ch.length()) && (parentheses >= 1) && (nbvirg <= virg)) {
			char car = ch.charAt(i);
			if (car == '(')
				parentheses++;
			if (car == ')')
				parentheses--;
			if ((car == ',') && (parentheses == 1))
				nbvirg++;
			i++;
		}
		return i;
	}

	/**
	 * Procedure vrifiant la syntaxe de l'expression relle pointe par Ch en
	 * tenant compte du fait qu'elle ne doit pas utiliser de valeur dont l'indice
	 * dans PtListe est strictement suprieur  IndiceMaxiDansListe la diffrence
	 * par rapport  la version C++ est que cette fonction ne renvoie pas de
	 * renseignement sur le message d'erreur  afficher
	 */
	public final static boolean verifieSyntaxeComplexe(CListeObjets ptListe,
			String ch, Int indiceErreur, int indiceMaxiDansListe,
			String parametreFormel[]) {
		int i;
		int indiceDansListe;
		int pdebut;
		char car, carSuivant;
		boolean erreur;
		int longueurCh;
		int itemPrecedent = 0;
		int itemSuivant = 0;
		boolean pointDecimalRencontre;
		int sommeParentheses;
		int longNom;
		Int nomF = new Int();
		PointeurValDyn ptVa = new PointeurValDyn();
		boolean queDesBlancs;
		Bool complexei = new Bool(false);
		Int parametreTrouve = new Int(0);
		Int nombreVariables = new Int(0);
		int nivPar = 0; // Niveau de parenthses
		boolean[] appelFonction = new boolean[nombreMaxParentheses + 1]; // 64
																																			// niveaux
																																			// de
																																			// parenthses
																																			// imbriqus
																																			// maxi
		for (int j = 0; j <= nombreMaxParentheses; j++)
			appelFonction[j] = false;

		erreur = false;
		longueurCh = ch.length();
		// On vrifie d'abord qu'il n'y a pas d'erreur de parenthse
		if (longueurCh == 0) {
			indiceErreur.setValue(0);
			return false;
		}
		i = 0;
		sommeParentheses = 0;
		while ((i < longueurCh) && (sommeParentheses >= 0)) {
			car = ch.charAt(i);
			switch (car) {
			case '(':
				sommeParentheses++;
				break;
			case ')':
				sommeParentheses--;
			} // switch
			i++;
		}
		if (sommeParentheses != 0) {
			indiceErreur.setValue(i);
			// IndiceMessageErreur = IDE_SYNTAXE2;
			return false;
		}
		// Ajout version 4.5.2
		if (ch.equals(".")) {
			indiceErreur.setValue(0);
			return false;
		}
		// Fin ajout
		i = 0;
		// Tout se passe au dbut comme si l'expression tait prcde
		// d'une parenthse ouvrante
		itemPrecedent = ParOuvrante;
		while ((i < longueurCh) && !erreur) {
			car = ch.charAt(i);
			pointDecimalRencontre = false;
			if (FonteNom.chiffre(car) || (car == '.')) {
				while ((FonteNom.chiffre(car) || (car == '.')) && (i < longueurCh)
						&& !erreur) {
					if (car == '.') {
						if (pointDecimalRencontre) {
							erreur = true;
							indiceErreur.setValue(i - 1);
							// IndiceMessageErreur = IDE_SYNTAXE1;
						} else
							pointDecimalRencontre = true;
					}
					i++;
					if (i < longueurCh)
						car = ch.charAt(i);
				} // while
				if (!erreur)
					itemSuivant = Nombre;
			} else // cas o le caractre en cours n'est pas un nombre
			// le caractre en cours n'est pas un chiffre ou un point dcimal}
			{
				pdebut = i;
				// On regarde si cela commence par le nom d'une fonction ou
				// une valeur dj dfinie
				longNom = ptListe.commenceParNomFonctionOuValeurOuParametreComplexe(
						ch.substring(pdebut), parametreFormel, ptVa, nomF, parametreTrouve,
						complexei, nombreVariables);
				if (longNom > 0) {
					if (complexei.getValue()) {
						itemSuivant = Valeur;
						i = i + longNom;
					} else {
						if (parametreTrouve.getValue() != -1) {
							itemSuivant = Valeur;
							i = i + longNom;
						} else {
							appelFonction[nivPar + 1] = true;
							if (ptVa.getPointeur() == null) {
								if (((i + longNom) < ch.length())
										&& ch.charAt(i + longNom) != '(') {
									erreur = true;
									indiceErreur.setValue(i + longNom);
								} else {
									itemSuivant = Fonction;
									// Ajout version 4.5.2 pour corriger un bug
									if (!autorisationSyntaxe(itemPrecedent, itemSuivant)) {
										erreur = true;
										indiceErreur.setValue(i);
										break;
									}
									//
									int nvirg = nombreVirgules(ch, i + longNom + 1);
									if (nvirg != nombreVariables.getValue() - 1) {
										if (nvirg <= nombreVariables.getValue() - 1)
											indiceErreur.setValue(indiceCaractereVirgule(ch, i
													+ longNom + 1, nvirg));
										else
											indiceErreur.setValue(indiceCaractereVirgule(ch, i
													+ longNom + 1, nombreVariables.getValue() - 1) - 1);
										erreur = true;
									} else {
										// Seule fonction complexe  4 variables ou plus : le calcul
										// de somme indice
										if (nombreVariables.getValue() >= 4) {
											nivPar++;
											int indicePremVirg = premiereVirgule(ch, i + longNom + 1);
											String chformuleASommer = ch.substring(i + longNom + 1,
													indicePremVirg);
											int indiceDeuxiemeVirg = premiereVirgule(ch,
													indicePremVirg + 1);
											String nomVariableSommation = ch.substring(
													indicePremVirg + 1, indiceDeuxiemeVirg);
											// Il faut retirer les espaces de dbut et de fin
											// On vrifie si le nom de variable formelle pour la somme
											// est valable
											/* nomVariableSommation = Kernel
													.retireEspacesDebutEtFin(nomVariableSommation);*/
											nomVariableSommation = nomVariableSommation.trim();
											if (!ptListe
													.validationNomCalculSansMessage(nomVariableSommation)
													|| ptListe.egalNomValeurOuFonctionOuMesure(
															nomVariableSommation, -1)) {
												erreur = true;
												indiceErreur.setValue(indiceDeuxiemeVirg);
											} else {
												// On vrifie si la syntaxe du calcul  sommer est
												// correct.
												// Pour cela on appelle  nouveau vrifieSyntaxe
												Int indiceErreur2 = new Int();
												int n = 0;
												if (parametreFormel != null)
													n = parametreFormel.length;
												String[] parametreFormel2 = new String[n + 1];
												for (int j = 0; j < n; j++)
													parametreFormel2[j] = new String(parametreFormel[j]);
												parametreFormel2[n] = new String(nomVariableSommation);
												boolean b;
												if ((nombreVariables.getValue() == 4)
														&& (nomF.getValue() == OperateurFonction4Var.integraleC)) {
													// Cas d'un calcul d'intgrale
													// Ne doit utiliser que fonctions ou valeurs relles
													// On ne passe donc comme variable formelle que
													// nomVariableSommation
													String[] parametreFormel3 = new String[1];
													parametreFormel3[0] = nomVariableSommation;
													b = verifieSyntaxe(ptListe, chformuleASommer,
															indiceErreur2, indiceMaxiDansListe,
															parametreFormel3);
												} else
													b = verifieSyntaxeComplexe(ptListe, chformuleASommer,
															indiceErreur2, indiceMaxiDansListe,
															parametreFormel2);
												if (!b) {
													erreur = true;
													indiceErreur.setValue(i + longNom + 1
															+ indiceErreur2.getValue());
												} else {
													i = indiceDeuxiemeVirg + 1;
													itemPrecedent = Valeur;
													itemSuivant = Virgule;
												}
											}
										} else
											i += longNom;
									}
								}
							} else {
								CValeurDynamique vd = ptVa.getPointeur();
								indiceDansListe = ptListe.indexOf(vd);
								if (indiceDansListe > indiceMaxiDansListe) {
									erreur = true;
									indiceErreur.setValue(i);
								} else {
									i += longNom;
									if (vd.estFonctionOuSuite()) {
										if (vd.estFonctionOuSuite()) {
											if ((i < ch.length()) && ch.charAt(i) != '(') {
												erreur = true;
												indiceErreur.setValue(i + longNom);
											} else {
												itemSuivant = Fonction;
												CCalculAncetre calc = (CCalculAncetre) vd;
												int nbvar = calc.nombreVariables();
												if (nbvar >= 2) {
													int nvirg = nombreVirgules(ch, i + 1);
													if (nvirg != nombreVariables.getValue() - 1) {
														if (nvirg <= nombreVariables.getValue() - 1)
															indiceErreur.setValue(indiceCaractereVirgule(ch,
																	i + 1, nvirg));
														else
															indiceErreur.setValue(indiceCaractereVirgule(ch,
																	i + 1, nombreVariables.getValue() - 1) - 1);
														erreur = true;
													}
												}
												// Bug corrig version 4.6 : Il ne faut pas qu'un appel de fonction d'une variable comprenne des virgules
												else {
													if (nombreVirgules(ch, i + 1) > 0) {
														indiceErreur.setValue(ch.indexOf(',', i+1));
														erreur = true;
													}
												}
											}
										}
									} else
										itemSuivant = Valeur;
								}
							}
						}
					}
				} else {
					switch (car) {
					case '+':
					case '-':
						itemSuivant = Addition;
						i++;
						break;
					case '*':
					case '/':
						itemSuivant = Multiplication;
						i++;
						break;
					case '(':
						if (nivPar >= nombreMaxParentheses - 1) {
							erreur = true;
							indiceErreur.setValue(i);
						} else {
							itemSuivant = ParOuvrante;
							i++;
							nivPar++;
						}
						break;
					case ')':
						appelFonction[nivPar] = false;
						nivPar--;
						itemSuivant = ParFermante;
						i++;
						break;
					case '^':
						itemSuivant = Puissance;
						i++;
						break;
					// Ajout par rapport  la version C++ 1.8, autorisation du carr 
					case '':
						itemSuivant = Carre;
						i++;
						break;
					// ni nombre ni oprateur}
					case '<':
						itemSuivant = Inegalite;
						i++;
						if (i <= longueurCh - 2) {
							carSuivant = ch.charAt(i);
							if ((carSuivant == '=') || (carSuivant == '>'))
								i++;
						}
						break;
					case '>':
						itemSuivant = Inegalite;
						i++;
						if (i <= longueurCh - 2) {
							carSuivant = ch.charAt(i);
							if (carSuivant == '=')
								i++;
						}
						break;
						// Ajout version 4.9.5
					case '&':
					case '|':
					// Fin ajout
					case '=':
						itemSuivant = Inegalite;
						i++;
						break;
					case ' ':
						itemSuivant = Blanc;
						i++;
						break;
					case ',':
						if (!appelFonction[nivPar]) {
							erreur = true;
							indiceErreur.setValue(i);
						} else {
							itemSuivant = Virgule;
							i++;
						}
						break;
					default: // switch Car
						erreur = true;
						indiceErreur.setValue(i - 1);
					} // switch Car
				}// else du if LongNom > 0
			} // else du if Chiffre(Car) || (Car == '.'))
			if (!erreur) {
				if (autorisationSyntaxe(itemPrecedent, itemSuivant)) {
					if (itemSuivant != Blanc)
						itemPrecedent = itemSuivant;
				} else {
					erreur = true;
					// IndiceMessageErreur = IDE_SYNTAXE1;
					indiceErreur.setValue(i - 1);
				}
			}
		} // while
		// Il faut aussi regarder si la fin de l'expression est correcte
		itemSuivant = ParFermante;
		if (!erreur) {
			if (!autorisationSyntaxe(itemPrecedent, itemSuivant)) {
				// MessageBeep(0);
				erreur = true;
				indiceErreur.setValue(longueurCh);
				// IndiceMessageErreur = IDE_SYNTAXE1;
			}
		}
		if (indiceErreur.getValue() < 0)
			indiceErreur.setValue(0);
		// Modification version 4.6
		/*
		else if (indiceErreur.getValue() >= longueurCh)
			indiceErreur.setValue(longueurCh - 1);
			*/
		// Modification version 4.7.9
		else if (indiceErreur.getValue() > longueurCh) indiceErreur.setValue(longueurCh);
		// On regarde si la chane ne contient que des blancs
		i = 0;
		queDesBlancs = true;
		while ((i <= longueurCh - 1) && queDesBlancs) {
			car = ch.charAt(i);
			if (car != ' ')
				queDesBlancs = false;
			else
				i++;
		}
		if (queDesBlancs) {
			// MessageBeep(0);
			erreur = true;
			indiceErreur.setValue(longueurCh);
			// IndiceMessageErreur = IDE_SYNTAXE1;
		}
		// if (Erreur) MessageBeep(0);
		return !erreur;
	}

	// Ajout pour la version 1.9.5
	// Procdure rcursive servant  savoir si un calcul dpend de la variable
	// formelle
	public abstract boolean dependDeVariable(int ind);

	// Ajout pour la version 1.9.5
	// Sert  crer un clone d'un objet CCalcul
	// Utilis dans l'objet CDerivee
	public abstract CCalculBase getCopie();

	// Ajout pour la version 1.9.5.
	// Procdure rcursive crant l'objet CCalculBase reprsentant l'arbre de la
	// drive formelle d'un autre
	// Sera utilis au chargement d'un objet du type CDerivee
	// Nouveaut de la version 1.5. Cration d'un calcul qui reprsente la drive
	// formelle
	// d'un calcul. Cette procdure est rcursive
	/**
	 * Procdure rcursive crant l'objet CCalculBase reprsentant l'arbre de la
	 * drive d'une fonction Sera utilis au chargement d'un objet du type
	 * CDerivee Modifi pour version 3.0 : un paramtre est ajout spcifiant
	 * l'indice de la variable ce qui permet de calculer les drives partielles
	 * d'une fonction de plusieurs variables
	 */
	public static CCalculBase creeDerivee(CCalculBase calc, int indiceVar) {
		CListeObjets li = calc.listeProprietaire;
		boolean op1dep, op2dep; // Dplac ici version 5.1.2
		long natcalc = calc.nature();
		if (natcalc == natConstante) return new CConstante(li, 0);
		else if (natcalc == natVariableFormelle) {
			// Si l'indice de la variable formelle est le bon on renvoie 1 sinon 0
			CVariableFormelle var = (CVariableFormelle) calc;
			if (var.indvar == indiceVar)
				return new CConstante(li, 1);
			else
				return new CConstante(li, 0);
		} else if (natcalc == natResultatValeur) return new CConstante(li, 0);
		else if (natcalc == natAppelFonction) {
			CAppelFonction appfon = (CAppelFonction) calc;
			// Ici il y a un problme car l'appel de fonction peut tre un appel
			//  une suite rcurrente. Cet appel est licite s'il ne fait pas appel
			//  la variable formelle.
			// Il faudra le vrifier avant d'autoriser le calcul d'une drive.
			// Ici on suppose que c'est la cas
			// Modifi version 4.6
			// if (appfon.fonctionAssociee.getClass() == CSuiteRec.class) {
			if (appfon.fonctionAssociee.getNatureCalcul() == NatureCalcul.NSuiteRecurrenteReelle) {
				// L'appel de suite sans paramtre est considr comme une constante
				return new CConstante(li, 0);
			} else // Cas d'une vraie fonction
			{
				CFonc f = (CFonc) (appfon.fonctionAssociee);
				// On regarde si l'appel de fonction est seulement de la forme
				// f(paramtre)
				if (appfon.operande.nature() == natVariableFormelle) {
					CVariableFormelle var = (CVariableFormelle) appfon.operande;
					if (var.indvar == indiceVar)
						return creeDerivee(f.calcul, indiceVar);
					else {
						CCalculBase uder = creeDerivee(appfon.operande, indiceVar);
						CCalculBase uCopie = appfon.operande.getCopie();
						CCalculBase fcalcder = creeDerivee(f.calcul, indiceVar);
						CAppelFonctionInterne appelfder = new CAppelFonctionInterne(
								fcalcder, uCopie);
						return new COperation(li, uder, appelfder, Operateur.Mult);
					}
				} else { // Dans ce cas il faut driver comme une compose du type fou
					CCalculBase uder = creeDerivee(appfon.operande, indiceVar);
					CCalculBase uCopie = appfon.operande.getCopie();
					CCalculBase fcalcder = creeDerivee(f.calcul, indiceVar);
					CAppelFonctionInterne appelfder = new CAppelFonctionInterne(fcalcder,
							uCopie);
					return new COperation(li, uder, appelfder, Operateur.Mult);
				}
			}
		} else if (natcalc == natOperation) {
			COperation op = (COperation) calc;
			switch (op.operateur) {
			case Operateur.Plus:
				/* Modification version 5.1.1 pour optiliser le calcul
				return new COperation(li, creeDerivee(op.operande1, indiceVar),
						creeDerivee(op.operande2, indiceVar), Operateur.Plus);
						*/
				op1dep = op.operande1.dependDeVariable(indiceVar);
				op2dep = op.operande2.dependDeVariable(indiceVar);
				if (!op1dep) {
					if (!op2dep) return new CConstante(li, 0);
					else return creeDerivee(op.operande2, indiceVar);
				}
				if (!op2dep)
					return creeDerivee(op.operande1, indiceVar);
				else return new COperation(li, creeDerivee(op.operande1, indiceVar),
						creeDerivee(op.operande2, indiceVar), Operateur.Plus);
			case Operateur.Moin:
				/* Modification version 5.1.1 pour optiliser le calcul
				return new COperation(li, creeDerivee(op.operande1, indiceVar),
						creeDerivee(op.operande2, indiceVar), Operateur.Moin);
				*/
				op1dep = op.operande1.dependDeVariable(indiceVar);
				op2dep = op.operande2.dependDeVariable(indiceVar);
				if (!op1dep) {
					if (!op2dep) return new CConstante(li, 0);
					else return new CMoinsUnaire(li,creeDerivee(op.operande2, indiceVar));
				}
				if (!op2dep) return creeDerivee(op.operande1, indiceVar);
			 else return new COperation(li, creeDerivee(op.operande1, indiceVar),
						creeDerivee(op.operande2, indiceVar), Operateur.Moin);
			case Operateur.Mult: {
				// On regarde d'abord si les facteurs du produit dpendent de la
				// variable
				op1dep = op.operande1.dependDeVariable(indiceVar);
				op2dep = op.operande2.dependDeVariable(indiceVar);
				// Optimis version 5.1.2
				// if (!op1dep && !op2dep)
				// 	return new CConstante(li, 0);
				// if (!op1dep && op2dep)
				//	return new COperation(li, op.operande1.getCopie(), creeDerivee(
				//			op.operande2, indiceVar), Operateur.Mult);
				// if (op1dep && !op2dep) 
				// 	return new COperation(li, creeDerivee(op.operande1, indiceVar),
				//			op.operande2.getCopie(), Operateur.Mult);
				//
				if (!op1dep) {
					if (!op2dep) return new CConstante(li, 0);
					else  return new COperation(li, op.operande1.getCopie(), creeDerivee(op.operande2, indiceVar), Operateur.Mult);
				}
				if (!op2dep) 
					return new COperation(li, creeDerivee(op.operande1, indiceVar),
							op.operande2.getCopie(), Operateur.Mult);				
				// Formule u'v + uv'
				else
					return new COperation(li,
							new COperation(li, creeDerivee(op.operande1, indiceVar),
									op.operande2.getCopie(), Operateur.Mult), new COperation(li,
									creeDerivee(op.operande2, indiceVar),
									op.operande1.getCopie(), Operateur.Mult), Operateur.Plus);
			}
			case Operateur.Divi: // Formule (u'v-uv')/v
				// On regarde d'abord si le dnominateur dpend de la variable
				if (!op.operande2.dependDeVariable(indiceVar))
					return new COperation(li, creeDerivee(op.operande1, indiceVar),
							op.operande2.getCopie(), Operateur.Divi);
				else
					return new COperation(li, new COperation(li, new COperation(li,
							creeDerivee(op.operande1, indiceVar), op.operande2.getCopie(),
							Operateur.Mult), new COperation(li, creeDerivee(op.operande2,
							indiceVar), op.operande1.getCopie(), Operateur.Mult),
							Operateur.Moin), new CPuissance(li, op.operande2.getCopie(),
							new CConstante(li, 2)), Operateur.Divi);
			case Operateur.Inf:
			case Operateur.Sup:
			case Operateur.InfOuEgal:
			case Operateur.SupOuEgal:
			case Operateur.Egalite:
			case Operateur.Diff:
				// Ajout version 4.9.5
			case Operateur.And:
			case Operateur.Or:
				return new CConstante(li, 0);
			}
		} else if (natcalc == natFonction) {
			CFonction f = (CFonction) calc;
			switch (f.operateurFonction) {
			case OperateurFonction.Abso:
				// La fonction valeur absolue n'tant pas drivable en 0
				// On va renvoyer comme formule
				// 1/((operande>0)-(Operande<0))*(Operande)'
				return new COperation(li, new COperation(li, new CConstante(li, 1),
						new COperation(li, new COperation(li, f.operande.getCopie(),
								new CConstante(li, 0), Operateur.Sup), new COperation(li,
								f.operande.getCopie(), new CConstante(li, 0), Operateur.Inf),
								Operateur.Moin), Operateur.Divi), creeDerivee(f.operande,
						indiceVar), Operateur.Mult);
			case OperateurFonction.Enti: // la drive existe et est nulle sauf en
																		// tout entier
				// On renvoie donc comme formule 0/(operande<>int(operande))
				return new COperation(li, new CConstante(li, 0), new COperation(li,
						f.operande.getCopie(), new CFonction(li, OperateurFonction.Enti,
								f.operande.getCopie()), Operateur.Diff), Operateur.Divi);
			case OperateurFonction.Raci:
			case OperateurFonction.Raci2: // Formule u'/(2Racine(u))
				return new COperation(
						li,
						creeDerivee(f.operande, indiceVar),
						new COperation(li, new CConstante(li, 2), new CFonction(li,
								OperateurFonction.Raci, f.operande.getCopie()), Operateur.Mult),
						Operateur.Divi);
			case OperateurFonction.Sinu: // La drive ne sera correcte que si on est
																		// en mode radian
				// On applique la formule u'*cos(u)
				if (li.uniteAngle == UniteAngle.Radian)
					return new COperation(li, creeDerivee(f.operande, indiceVar),
							new CFonction(li, OperateurFonction.Cosi, f.operande.getCopie()),
							Operateur.Mult);
				else
					return new COperation(li, new CConstante(li, Kernel.ConvDegRad),
							new COperation(li, creeDerivee(f.operande, indiceVar),
									new CFonction(li, OperateurFonction.Cosi,
											f.operande.getCopie()), Operateur.Mult), Operateur.Mult);

			case OperateurFonction.Cosi:// La drive ne sera correcte que si on est
																	// en mode radian
				// On applique la formule -u'*sin(u)
				if (li.uniteAngle == UniteAngle.Radian)
					return new COperation(li, new COperation(li, creeDerivee(f.operande,
							indiceVar), new CFonction(li, OperateurFonction.Sinu,
							f.operande.getCopie()), Operateur.Mult), new CConstante(li, -1),
							Operateur.Mult);
				else
					return new COperation(li, new CConstante(li, Kernel.ConvDegRad),
							new COperation(li, new COperation(li, creeDerivee(f.operande,
									indiceVar), new CFonction(li, OperateurFonction.Sinu,
									f.operande.getCopie()), Operateur.Mult), new CConstante(li,
									-1), Operateur.Mult), Operateur.Mult);

			case OperateurFonction.Tang: // On applique la formule u'/cos(u)
				if (li.uniteAngle == UniteAngle.Radian)
					return new COperation(li, creeDerivee(f.operande, indiceVar),
							new CPuissance(li, new CFonction(li, OperateurFonction.Cosi,
									f.operande.getCopie()), new CConstante(li, 2)),
							Operateur.Divi);
				else
					return new COperation(li, new CConstante(li, Kernel.ConvDegRad),
							new COperation(li, creeDerivee(f.operande, indiceVar),
									new CPuissance(li, new CFonction(li, OperateurFonction.Cosi,
											f.operande.getCopie()), new CConstante(li, 2)),
									Operateur.Divi), Operateur.Mult);
				// Le nombre de fonctions est limit pour la version collge
			case OperateurFonction.Logn: // Formule u'/(u*(u>0))
				return new COperation(li, creeDerivee(f.operande, indiceVar),
						new COperation(li, f.operande.getCopie(), new COperation(li,
								f.operande.getCopie(), new CConstante(li, 0), Operateur.Sup),
								Operateur.Mult), Operateur.Divi);
			case OperateurFonction.Expo: // Formule u'*exp(u)
				return new COperation(li, creeDerivee(f.operande, indiceVar),
						new CFonction(li, OperateurFonction.Expo, f.operande.getCopie()),
						Operateur.Mult);
			case OperateurFonction.ATan: // Formule u'/(1+u)
				if (li.uniteAngle == UniteAngle.Radian)
					return new COperation(
							li,
							creeDerivee(f.operande, indiceVar),
							new COperation(li, new CConstante(li, 1), new CPuissance(li,
									f.operande.getCopie(), new CConstante(li, 2)), Operateur.Plus),
							Operateur.Divi);
				else
					return new COperation(li, new CConstante(li, Kernel.ConvRadDeg),
							new COperation(li, creeDerivee(f.operande, indiceVar),
									new COperation(li, new CConstante(li, 1), new CPuissance(li,
											f.operande.getCopie(), new CConstante(li, 2)),
											Operateur.Plus), Operateur.Divi), Operateur.Mult);
			case OperateurFonction.ACos: // Formule -u'/(rac(1-u))
				if (li.uniteAngle == UniteAngle.Radian)
					return new COperation(li, new COperation(li, creeDerivee(f.operande,
							indiceVar),
							new CFonction(li, OperateurFonction.Raci, new COperation(li,
									new CConstante(li, 1), new CPuissance(li, f.operande
											.getCopie(), new CConstante(li, 2)), Operateur.Moin)),
							Operateur.Divi), new CConstante(li, -1), Operateur.Mult);
				else
					return new COperation(li, new COperation(li, creeDerivee(f.operande,
							indiceVar),
							new CFonction(li, OperateurFonction.Raci, new COperation(li,
									new CConstante(li, 1), new CPuissance(li, f.operande
											.getCopie(), new CConstante(li, 2)), Operateur.Moin)),
							Operateur.Divi), new CConstante(li, -Kernel.ConvRadDeg),
							Operateur.Mult);
			case OperateurFonction.ASin: // Formule u'/(rac(1-u))
				if (li.uniteAngle == UniteAngle.Radian)
					return new COperation(li, creeDerivee(f.operande, indiceVar),
							new CFonction(li, OperateurFonction.Raci, new COperation(li,
									new CConstante(li, 1), new CPuissance(li,
											f.operande.getCopie(), new CConstante(li, 2)),
									Operateur.Moin)), Operateur.Divi);
				else
					return new COperation(li, new CConstante(li, Kernel.ConvRadDeg),
							new COperation(li, creeDerivee(f.operande, indiceVar),
									new CFonction(li, OperateurFonction.Raci, new COperation(li,
											new CConstante(li, 1), new CPuissance(li,
													f.operande.getCopie(), new CConstante(li, 2)),
											Operateur.Moin)), Operateur.Divi), Operateur.Mult);
			case OperateurFonction.Cosh: // Formule u'*Sinh(u)
				return new COperation(li, creeDerivee(f.operande, indiceVar),
						new CFonction(li, OperateurFonction.Sinh, f.operande.getCopie()),
						Operateur.Mult);
			case OperateurFonction.Sinh: // Formule u'*Cosh(u)
				return new COperation(li, creeDerivee(f.operande, indiceVar),
						new CFonction(li, OperateurFonction.Cosh, f.operande.getCopie()),
						Operateur.Mult);
			case OperateurFonction.Tanh: // Formule u'/Cosh(u)
				return new COperation(li, creeDerivee(f.operande, indiceVar),
						new CPuissance(li, new CFonction(li, OperateurFonction.Cosh,
								f.operande.getCopie()), new CConstante(li, 2)), Operateur.Divi);
			case OperateurFonction.ArgCh: // Formule u'/Rac(u-1)
				return new COperation(li, creeDerivee(f.operande, indiceVar),
						new CFonction(li, OperateurFonction.Raci,
								new COperation(li, new CPuissance(li, f.operande.getCopie(),
										new CConstante(li, 2)), new CConstante(li, 1),
										Operateur.Moin)), Operateur.Divi);
			case OperateurFonction.ArgSh: // Formule u'/(Rac(1+u)
				return new COperation(li, creeDerivee(f.operande, indiceVar),
						new CFonction(li, OperateurFonction.Raci,
								new COperation(li, new CPuissance(li, f.operande.getCopie(),
										new CConstante(li, 2)), new CConstante(li, 1),
										Operateur.Plus)), Operateur.Divi);
			case OperateurFonction.ArgTh: // Formule (Abs(u)<1)*u'/(1-u)
				return new COperation(li, new COperation(li, new CFonction(li,
						OperateurFonction.Abso, f.operande.getCopie()), new CConstante(li,
						1), Operateur.Inf), new COperation(li, creeDerivee(f.operande,
						indiceVar), new COperation(li, new CConstante(li, 1),
						new CPuissance(li, f.operande.getCopie(), new CConstante(li, 2)),
						Operateur.Moin), Operateur.Divi), Operateur.Mult);
			case OperateurFonction.Rand:
				return new CConstante(li, 0);
			// Rajout version 4.7.2
			case OperateurFonction.Left:
				return creeDerivee(f.operande.membreGauche(), indiceVar);
			case OperateurFonction.Right:
				return creeDerivee(f.operande.membreDroit(), indiceVar);
			}
		} else if (natcalc == natPuissance) {
			CPuissance pui = (CPuissance) calc;
			// Si l'exposant ne dpend pas de la variable formelle on emploie la
			// formule
			// (u^n)' = nu'u^(n-1)
			// Optimisation version 4.6. Si l'operande et l'exposant sont constants drive nulle
			/*
			if (!pui.exposant.dependDeVariable(indiceVar))
				return new COperation(
						li,
						new COperation(li, pui.exposant.getCopie(), creeDerivee(
								pui.operande, indiceVar), Operateur.Mult),
						new CPuissance(li, pui.operande.getCopie(), new COperation(li,
								pui.exposant.getCopie(), new CConstante(li, 1), Operateur.Moin)),
						Operateur.Mult);
			*/
			if (!pui.exposant.dependDeVariable(indiceVar)) {
				if (!pui.operande.dependDeVariable(indiceVar))
					return new CConstante(li, 0);
				/* Modifiction version 5.1.2 pour tenir compte du cas o l'exposant est une constante
				else return new COperation(
						li,
						new COperation(li, pui.exposant.getCopie(), creeDerivee(
								pui.operande, indiceVar), Operateur.Mult),
						new CPuissance(li, pui.operande.getCopie(), new COperation(li,
								pui.exposant.getCopie(), new CConstante(li, 1), Operateur.Moin)),
						Operateur.Mult);
			}
			*/
				else { 
					if (pui.exposant.nature() == natConstante) {
						CConstante co = (CConstante) pui.exposant;
						if (co.valeur == 0) return new CConstante(li, 0);
						else if (co.valeur == 1) return creeDerivee(	pui.operande, indiceVar);
						else return new COperation(li,
								new COperation(li, pui.exposant.getCopie(), creeDerivee(
										pui.operande, indiceVar), Operateur.Mult),
									new CPuissance(li, pui.operande.getCopie(), new CConstante(li, co.valeur-1)),
									Operateur.Mult);
					}
					else return new COperation(
						li,
						new COperation(li, pui.exposant.getCopie(), creeDerivee(
								pui.operande, indiceVar), Operateur.Mult),
						new CPuissance(li, pui.operande.getCopie(), new COperation(li,
								pui.exposant.getCopie(), new CConstante(li, 1), Operateur.Moin)),
						Operateur.Mult);
				}
			}

			// Sinon on considre u^v comme exp(v*lnu) de drive
			// (v'*lnu+v*u'/u)*u^v
			else
				return new COperation(
						li,
						new COperation(li, new COperation(li, creeDerivee(pui.exposant,
								indiceVar), new CFonction(li, OperateurFonction.Logn,
								pui.operande.getCopie()), Operateur.Mult), new COperation(li,
								new COperation(li, creeDerivee(pui.operande, indiceVar),
										pui.operande.getCopie(), Operateur.Divi), pui.exposant
										.getCopie(), Operateur.Mult), Operateur.Plus),
						new CPuissance(li, pui.operande.getCopie(), pui.exposant.getCopie()),
						Operateur.Mult);
		}
		// Ajout version 4.6 : Nouvel oprateur moins unaire
		else if (natcalc == natMoinsUnaire) {
			CMoinsUnaire mun = (CMoinsUnaire) calc;
			return new CMoinsUnaire(li, creeDerivee(mun.operande, indiceVar));
		} 
		// Ajout version 3.0 pour le cas d'un appel  une fonction utilisateur de
		// plusieurs variables
		// Pour une fonction du type f(t) = g[u(t),v(t)] on utilise la formule
		// f'(t) = u'(t) g'u(u(t),v(t)) + v'(t) g'v(u(t),v(t))
		else if (natcalc == natAppelFonctionNVar) {
			// Version 3.0 : seulement des fonctions jutilisateur de 2 ou 3 variables
			// peuvent tre cres.
			CAppelFonctionNVar appfon = (CAppelFonctionNVar) calc;
			CFoncNVar f = (CFoncNVar) (appfon.fonctionAssociee);
			int nvar = f.nbVar;
			if (nvar == 2) {
				CCalculBase uder = creeDerivee(appfon.operande[0], indiceVar); // Calcul
																																				// de
																																				// u'(t)
				CCalculBase vder = creeDerivee(appfon.operande[1], indiceVar); // Calcul
																																				// de
																																				// v'(t)
				CCalculBase uCopie = appfon.operande[0].getCopie();
				CCalculBase vCopie = appfon.operande[1].getCopie();
				CCalculBase[] tabcal = new CCalculBase[2];
				tabcal[0] = uCopie;
				tabcal[1] = vCopie;
				CCalculBase fcalcderu = creeDerivee(f.calcul, 0); // Drive partielle
																													// par rapport  la
																													// premire variable u
				CCalculBase fcalcderv = creeDerivee(f.calcul, 1); // Drive partielle
																													// par rapport  la
																													// deuxime variable v
				CAppelFonctionInterneNVar appelfderu = new CAppelFonctionInterneNVar(
						fcalcderu, tabcal);
				CAppelFonctionInterneNVar appelfderv = new CAppelFonctionInterneNVar(
						fcalcderv, tabcal);
				/* Optimisation version 5.1.4
				COperation prod1 = new COperation(li, uder, appelfderu, Operateur.Mult);
				COperation prod2 = new COperation(li, vder, appelfderv, Operateur.Mult);
				COperation sum1 = new COperation(li, prod1, prod2, Operateur.Plus);
				*/
				CCalculBase prod1, prod2;
				if (uder.nature() == CCalculBase.natConstante) {
					if (((CConstante)uder).valeur == 0) prod1 = new CConstante(li, 0);
					else prod1 = new COperation(li, uder, appelfderu, Operateur.Mult);
				}
				else prod1 = new COperation(li, uder, appelfderu, Operateur.Mult);
				if (vder.nature() == CCalculBase.natConstante) {
					if (((CConstante)vder).valeur == 0) prod2 = new CConstante(li, 0);
					else prod2 = new COperation(li, vder, appelfderv, Operateur.Mult);
				}
				else prod2 = new COperation(li, vder, appelfderv, Operateur.Mult);
				COperation sum1 = new COperation(li, prod1, prod2, Operateur.Plus);
				return sum1;
			} else {
				CCalculBase uder = creeDerivee(appfon.operande[0], indiceVar); // Calcul
																																				// de
																																				// u'(t)
				CCalculBase vder = creeDerivee(appfon.operande[1], indiceVar); // Calcul
																																				// de
																																				// v'(t)
				CCalculBase wder = creeDerivee(appfon.operande[2], indiceVar); // Calcul
																																				// de
																																				// v'(t)
				CCalculBase uCopie = appfon.operande[0].getCopie();
				CCalculBase vCopie = appfon.operande[1].getCopie();
				CCalculBase wCopie = appfon.operande[2].getCopie();
				CCalculBase[] tabcal = new CCalculBase[3];
				tabcal[0] = uCopie;
				tabcal[1] = vCopie;
				tabcal[2] = wCopie;
				CCalculBase fcalcderu = creeDerivee(f.calcul, 0); // Drive partielle
																													// par rapport  la
																													// premire variable u
				CCalculBase fcalcderv = creeDerivee(f.calcul, 1); // Drive partielle
																													// par rapport  la
																													// deuxime variable v
				CCalculBase fcalcderw = creeDerivee(f.calcul, 2); // Drive partielle
																													// par rapport  la
																													// troisime variable
																													// w
				CAppelFonctionInterneNVar appelfderu = new CAppelFonctionInterneNVar(
						fcalcderu, tabcal);
				CAppelFonctionInterneNVar appelfderv = new CAppelFonctionInterneNVar(
						fcalcderv, tabcal);
				CAppelFonctionInterneNVar appelfderw = new CAppelFonctionInterneNVar(
						fcalcderw, tabcal);
				/* Optimisation version 5.1.4
				COperation prod1 = new COperation(li, uder, appelfderu, Operateur.Mult);
				COperation prod2 = new COperation(li, vder, appelfderv, Operateur.Mult);
				COperation sum1 = new COperation(li, prod1, prod2, Operateur.Plus);
				COperation prod3 = new COperation(li, wder, appelfderw, Operateur.Mult);
				*/
				CCalculBase prod1, prod2, prod3;
				if (uder.nature() == CCalculBase.natConstante) {
					if (((CConstante)uder).valeur == 0) prod1 = new CConstante(li, 0);
					else prod1 = new COperation(li, uder, appelfderu, Operateur.Mult);
				}
				else prod1 = new COperation(li, uder, appelfderu, Operateur.Mult);
				if (vder.nature() == CCalculBase.natConstante) {
					if (((CConstante)vder).valeur == 0) prod2 = new CConstante(li, 0);
					else prod2 = new COperation(li, vder, appelfderv, Operateur.Mult);
				}
				else prod2 = new COperation(li, vder, appelfderv, Operateur.Mult);
				COperation sum1 = new COperation(li, prod1, prod2, Operateur.Plus);
				if (wder.nature() == CCalculBase.natConstante) {
					if (((CConstante)wder).valeur == 0) prod3 = new CConstante(li, 0);
					else prod3 = new COperation(li, wder, appelfderw, Operateur.Mult);
				}
				else prod3 = new COperation(li, wder, appelfderw, Operateur.Mult);
				return new COperation(li, sum1, prod3, Operateur.Plus);
			}
		} else if (natcalc == natAppelFonctionInterneNVar) {
			// Version 3.0 : seulement des fonctions jutilisateur de 2 ou 3 variables
			// peuvent tre cres.
			CAppelFonctionInterneNVar appfon = (CAppelFonctionInterneNVar) calc;
			CCalculBase f = (CCalculBase) appfon.fonctionInterne;
			int nvar = appfon.operande.length;
			if (nvar == 2) {
				CCalculBase uder = creeDerivee(appfon.operande[0], indiceVar); // Calcul
																																				// de
																																				// u'(t)
				CCalculBase vder = creeDerivee(appfon.operande[1], indiceVar); // Calcul
																																				// de
																																				// v'(t)
				CCalculBase uCopie = appfon.operande[0].getCopie();
				CCalculBase vCopie = appfon.operande[1].getCopie();
				CCalculBase[] tabcal = new CCalculBase[2];
				tabcal[0] = uCopie;
				tabcal[1] = vCopie;
				CCalculBase fcalcderu = creeDerivee(f, 0); // Drive partielle par
																										// rapport  la premire
																										// variable u
				CCalculBase fcalcderv = creeDerivee(f, 1); // Drive partielle par
																										// rapport  la deuxime
																										// variable v
				CAppelFonctionInterneNVar appelfderu = new CAppelFonctionInterneNVar(
						fcalcderu, tabcal);
				CAppelFonctionInterneNVar appelfderv = new CAppelFonctionInterneNVar(
						fcalcderv, tabcal);
				COperation prod1 = new COperation(li, uder, appelfderu, Operateur.Mult);
				COperation prod2 = new COperation(li, vder, appelfderv, Operateur.Mult);
				COperation sum1 = new COperation(li, prod1, prod2, Operateur.Plus);
				return sum1;
			} else {
				CCalculBase uder = creeDerivee(appfon.operande[0], indiceVar); // Calcul
																																				// de
																																				// u'(t)
				CCalculBase vder = creeDerivee(appfon.operande[1], indiceVar); // Calcul
																																				// de
																																				// v'(t)
				CCalculBase wder = creeDerivee(appfon.operande[2], indiceVar); // Calcul
																																				// de
																																				// v'(t)
				CCalculBase uCopie = appfon.operande[0].getCopie();
				CCalculBase vCopie = appfon.operande[1].getCopie();
				CCalculBase wCopie = appfon.operande[2].getCopie();
				CCalculBase[] tabcal = new CCalculBase[3];
				tabcal[0] = uCopie;
				tabcal[1] = vCopie;
				tabcal[2] = wCopie;
				CCalculBase fcalcderu = creeDerivee(f, 0); // Drive partielle par
																										// rapport  la premire
																										// variable u
				CCalculBase fcalcderv = creeDerivee(f, 1); // Drive partielle par
																										// rapport  la deuxime
																										// variable v
				CCalculBase fcalcderw = creeDerivee(f, 2); // Drive partielle par
																										// rapport  la troisime
																										// variable w
				CAppelFonctionInterneNVar appelfderu = new CAppelFonctionInterneNVar(
						fcalcderu, tabcal);
				CAppelFonctionInterneNVar appelfderv = new CAppelFonctionInterneNVar(
						fcalcderv, tabcal);
				CAppelFonctionInterneNVar appelfderw = new CAppelFonctionInterneNVar(
						fcalcderw, tabcal);
				COperation prod1 = new COperation(li, uder, appelfderu, Operateur.Mult);
				COperation prod2 = new COperation(li, vder, appelfderv, Operateur.Mult);
				COperation sum1 = new COperation(li, prod1, prod2, Operateur.Plus);
				COperation prod3 = new COperation(li, wder, appelfderw, Operateur.Mult);
				return new COperation(li, sum1, prod3, Operateur.Plus);
			}
		} else if (natcalc == natFonction3Var) {
			CFonction3Var f = (CFonction3Var) calc;
			switch (f.operateurFonction) {
			// La fonction valeur absolue n'tant pas drivable en 0
			// On va renvoyer comme formule (operande1=1)*(Operande2)' +
			// (operande1!=1)*(Operande3)'
			case OperateurFonction3Var.si:
				return new COperation(li, new COperation(li, new COperation(li,
						f.operande1.getCopie(), new CConstante(li, 1), Operateur.Egalite),
						creeDerivee(f.operande2, indiceVar), Operateur.Mult),
						new COperation(li, new COperation(li, f.operande1.getCopie(),
								new CConstante(li, 1), Operateur.Diff), creeDerivee(
								f.operande3, indiceVar), Operateur.Mult), Operateur.Plus);
			default:
				return new CConstante(li, 0);
			}
		} else if (natcalc == natSommeDansFormule) {
			CSommeDansFormule s = (CSommeDansFormule) calc;
			return new CSommeDansFormule(li, creeDerivee(s.calculASommer, indiceVar),
					s.indiceDebut.getCopie(), s.indiceFin.getCopie(), s.pas.getCopie(),
					s.nomVariable);
		} else
		// Si on a autoris un produit indic c'est qu'il ne dpendait pas de la
		// variable de drivation
		// donc la drive est null
		if (natcalc == natProduitDansFormule)
			// new CConstante(li, 0);
			return new CConstante(li, 0); // Corrig version 4.7.5.2
		// Pour les autres types, on considre le calcul constant (fonction
		// prdifinie de deux variables comme max() par exemple
		return new CConstante(li, 0); // Pour satisfaire le compilateur
	}

	// Ajout pour la version 2.0
	// Procdure levant un double  la puissance n quand n est un entier compris
	// entre 1 et 255
	public static double PuisExpEnt(double c1, int exposant) {
		double tampon = c1;
		double res = 1;
		// Version n'utilisant que des dcalages de bits pour optimisation
		int b = 128;
		int indfin = 7;
		while ((b & exposant) == 0) {
			indfin--;
			b = b >> 1;
		}
		// Attention aux erreurs d'arrondi. Il faut prendre l'entier le plus proche
		// int indfin = (int) floor(log(exposant)/log(2)+0.5); // plus lent.
		// Abandonn
		for (int i = 0; i <= indfin; i++) {
			if ((exposant % 2) != 0) {
				res = res * tampon;
			}
			exposant >>= 1;
			tampon = tampon * tampon;
		}
		return res;
	}

	// Fonctions rajoutes pour la version 2.1.1 pour rendre le calcul sur les
	// complexes plus
	// prcis en particulier le calcul de puissances.
	// Sans utiliser cette mthode pour calculer par exemple (1+i)^256 on trouve
	// un rsultat aberrant
	public static double sinus(double x) {
		return Math.sin(x - Math.floor(x / (2 * Math.PI)) * 2 * Math.PI);
	}

	public static double cosinus(double x) {
		return Math.cos(x - Math.floor(x / (2 * Math.PI)) * 2 * Math.PI);
	}

	public static double tangente(double x) {
		return Math.tan(x - Math.floor(x / (2 * Math.PI)) * 2 * Math.PI);
	}

	/**
	 * Sert  reconstituer une formule  partir de son arbre binaire conjointement
	 *  chaineCalculSansPar A redfinir pour les descendants
	 */
	public String chaineCalcul(String[] varFor) {
		return "";
	}

	/**
	 * Sert  reconstituer une formule  partir de son arbre binaire conjointement
	 *  chaineCalcul A redfinir pour les descendants
	 */
	public String chaineCalculSansPar(String[] varFor) {
		return chaineCalcul(varFor);
	}

	/**
	 * Sert  reconstituer une formule LaTeX  partir de son arbre binaire
	 * conjointement  chaineLatexSansPar A redfinir pour les descendants
	 */
	public String chaineLatex(String[] varFor) {
		return chaineCalcul(varFor);
	}

	/**
	 * Sert  reconstituer une formule  partir de son arbre binaire conjointement
	 *  chaineCalcul A redfinir pour les descendants
	 */
	public String chaineLatexSansPar(String[] varFor) {
		return chaineLatex(varFor);
	}

	/**
	 * Rajout pour la version 3.0 : renvoie true si le calcul est constant sans
	 * faire appel  la fonction Rand() A redfinir pour tous les calculs
	 * renvoyant une constante
	 * 
	 * @return
	 */
	public boolean estConstant() {
		return false;
	}
	/**
	 * Rajout version 4.7.9 : Renvoie true si la calcul est une constante nulle et 0 sinon
	 */
	public boolean estConstante0() {
		return false;
	}

	/**
	 * Calcul d'une intgrale par la mthode de Simpson
	 * 
	 * @param f
	 *          : un CCalculBase reprsentant une fonction de n variables, la
	 *          dernire tant la variable d'intgration
	 * @param var
	 *          : lles paramtres passs  la fonction (le dernier sera en fait
	 *          affect dans la fonction)
	 * @param a
	 *          : la borne infrieure
	 * @param b
	 *          : la borne suprieure
	 * @param infoRandom
	 *          : true si les tirages alatoires par Random sont permis
	 * @param existe
	 *          : false au retour si erreur de calcul ou dpassement de capacit
	 * @return : la valeur approche de l'intgrale
	 */
	public static double integrale(CCalculBase f, double[] var, double a,
			double b, boolean infoRandom) throws ErreurCalculException {
		double s1, s2, s;
		double t1, t2, pas;
		int n = var.length;
		int k;
		if (a == b)
			return 0;
		int nb = nombreSubdivisionsPourSimpson; // Le nombre de subdivisions
		pas = (b - a) / (double) nb;
		s = f.resultatFonction(infoRandom, a) + f.resultatFonction(infoRandom, b);
		t1 = a;
		t2 = a - pas / 2d;
		s1 = 0;
		s2 = 0;
		for (k = 1; k < nb; k++) {
			t1 = t1 + pas;
			var[n - 1] = t1;
			s1 = s1 + f.resultatFonction(infoRandom, var);
		}
		for (k = 1; k <= nb; k++) {
			t2 = t2 + pas;
			var[n - 1] = t2;
			s2 = s2 + f.resultatFonction(infoRandom, var);
		}
		return (b - a) * (s + 2d * s1 + 4d * s2) / (6d * nb);
	}

	/**
	 * Sert  savoir si un objet est dfini uniquement par des objets figurant
	 * dans une liste A redfinir pour les descendants
	 */
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
		return true;
	}

	@Override
	public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list)
			throws IOException {
		listeProprietaire = list;
	}

	/**
	 * Fonction ajoute pour la version 3.2?8 Renvoie false seulement si le calcul
	 * utilise un objet CProduitDansFormule pour lequel le calcul de drive
	 * formelle est impossible
	 * 
	 * @return
	 */
	public boolean deriveePossible(int indiceVariable) {
		return true;
	}

	public static final String getString(String key) {
		return MtgFrame.getString(key);
	}

	/**
	 * Nouveau version 4.6 : Renvoie un calcul normalis quivalent Si
	 * rempval est true les appels  des calculs ou variables rels
	 * sont remplacs par leur valeur numrique
	 * Si sousdivnorm est true les soustractions sont remplaces par des additions d'opposs
	 * et les divisions pas des multiplications par l'inverse
	 * Modifi version 5.1.3 : Ajout d'un troisime parmtre rempDecParFrac qui, si il est true, provoque le remplacement des constantes dcimales
	 * non entires par des fractions irrductibles.
	 * @return
	 */
	public final CCalculBase calculNormalise(boolean rempval, boolean sousdivnorm, boolean rempDecParFrac) {
		CListeObjets liste = listeProprietaire;
		long nat = nature();
		if (nat == natOperation) {
			COperation op = (COperation) this;
			CCalculBase calcOperande1 = op.operande1.calculNormalise(rempval, sousdivnorm, rempDecParFrac);
			CCalculBase calcOperande2 = op.operande2.calculNormalise(rempval, sousdivnorm, rempDecParFrac);
			long natop1 = calcOperande1.nature();
			long natop2 = calcOperande2.nature();
			if (op.operateur == Operateur.Moin) {
				// On remplace les a - 0 par a
				if (rempval) {
					if ((natop2 == natConstante)) {
						CConstante resval = (CConstante) calcOperande2;
						if (resval.valeur == 0) return calcOperande1;
					}
					// On remplace les 0 - a par - a ou par abs(a) si a est un moins unaire
					if ((natop1 == natConstante)) {
						CConstante resval = (CConstante) calcOperande1;
						if (resval.valeur == 0) {
							if (calcOperande2.nature() == natMoinsUnaire) return ((CMoinsUnaire)calcOperande2).operande;
							return new CMoinsUnaire(liste, calcOperande2);		
						}
					}					
					// On remplace a - (-b) par a + b
					if (natop2 == natMoinsUnaire) {
						CMoinsUnaire mun = (CMoinsUnaire) calcOperande2;
						// return new COperation(liste, calcOperande1, mun.operande, Operateur.Plus); // Modifi version 4.7.2.5
						return new COperation(liste, calcOperande1, mun.operande, Operateur.Plus).calculNormalise(rempval, sousdivnorm, rempDecParFrac);
					}
				}
				// On remplace les a - b par a + (-b)
				if (sousdivnorm) return new COperation(liste, calcOperande1, new CMoinsUnaire(liste, calcOperande2), Operateur.Plus);
				else return new COperation(liste, calcOperande1, calcOperande2, Operateur.Moin);
			}
			if (op.operateur == Operateur.Divi) {
				// Si on est en mode remplacement de valeurs et que le dnominateur est un rsultat de valeur 1 on simplifie
				if (rempval) {
					if (natop2 == natConstante) {
						CConstante resval2 = (CConstante) calcOperande2;
						if (resval2.valeur == 1) {
							// Pour 1/1 on renvoie 1
							if (natop1 == natConstante) {
								CConstante resval1 = (CConstante) calcOperande1;
								if (resval1.valeur == 1) {
									return new CConstante(liste, 1);
								}
							}
							return calcOperande1;
						}
					}
					// Si le numrateur est une constante qui vaut 1 on garde le quotient
					if (natop1 == natConstante) {
						CConstante resval = (CConstante) calcOperande1;
						double r = resval.valeur;
						if (r == 1) {
							if (calcOperande2.nature() == natMoinsUnaire) {
								CMoinsUnaire mun2 = (CMoinsUnaire) calcOperande2;
								return new CMoinsUnaire(liste, new COperation(liste,
								new CConstante(liste,  1), mun2.operande, Operateur.Divi).calculNormalise(rempval, sousdivnorm, rempDecParFrac));
							}
							else return new COperation(liste, new CConstante(liste,  1), calcOperande2, Operateur.Divi);
						}
						// S'il vaut 0 on renvoie un CResultatValeur contenant 0
						if (r == 0) {
							return new CConstante(liste, 0);
						}
					}
					// Si le numrateur ou le dnominateur sont des moins unaires on simplifie
					if (natop1 == natMoinsUnaire) {
						CMoinsUnaire mun1 = (CMoinsUnaire) calcOperande1;
						if (natop2 == natMoinsUnaire) {
							CMoinsUnaire mun2 = (CMoinsUnaire) calcOperande2;
							if (sousdivnorm) return new COperation(liste, mun1.operande, new COperation(liste, new CConstante(liste, 1),
									mun2.operande, Operateur.Divi), Operateur.Mult).calculNormalise(rempval, sousdivnorm, rempDecParFrac);
							else return new COperation(liste, mun1.operande, mun2.operande, Operateur.Divi).calculNormalise(rempval, sousdivnorm, rempDecParFrac);
						}
						if (sousdivnorm) return new CMoinsUnaire(liste, new COperation(liste, mun1.operande, new COperation(liste, new CConstante(liste, 1),
								calcOperande2, Operateur.Divi), Operateur.Mult)).calculNormalise(rempval, sousdivnorm, rempDecParFrac);
						else return new CMoinsUnaire(liste, new COperation(liste, mun1.operande, calcOperande2, Operateur.Divi).calculNormalise(rempval, sousdivnorm, rempDecParFrac));
					}
					if (natop2 == natMoinsUnaire) {
						CMoinsUnaire mun2 = (CMoinsUnaire) calcOperande2;
						if (sousdivnorm) return new CMoinsUnaire(liste, new COperation(liste, calcOperande1, new COperation(liste, new CConstante(liste, 1),
								mun2.operande, Operateur.Divi), Operateur.Mult)).calculNormalise(rempval, sousdivnorm, rempDecParFrac);
						else return new CMoinsUnaire(liste, new COperation(liste, calcOperande1, mun2.operande, Operateur.Divi).calculNormalise(rempval, sousdivnorm, rempDecParFrac));						
					}
				}
				// Ajout version 4.7.2.5
				// On remplace (-a)/b par - (a/b)
				// Si le numrateur ou le dnominateur sont des moins unaires on simplifie
				if (natop1 == natMoinsUnaire) {
					CMoinsUnaire mun1 = (CMoinsUnaire) calcOperande1;
					long natnum = mun1.operande.nature();
					// Si le numrateur est une constante -1 on renvoie -(1/denominateur)
					if (natnum == natConstante) {
						CConstante c = (CConstante) mun1.operande;
						if (c.valeur == 1)
							return new CMoinsUnaire(liste, new COperation(liste, new CConstante(liste, 1), calcOperande2, Operateur.Divi));
					}
					if (sousdivnorm) return new CMoinsUnaire(liste, new COperation(liste, mun1.operande, new COperation(liste, new CConstante(liste, 1),
							calcOperande2, Operateur.Divi), Operateur.Mult)).calculNormalise(rempval, sousdivnorm, rempDecParFrac);
					else return new CMoinsUnaire(liste, 
							new COperation(liste, mun1.operande, calcOperande2, Operateur.Divi)).calculNormalise(rempval, sousdivnorm, rempDecParFrac);
				}
				// Fin ajout version 4.7.2.5
				// On remplace a / b par a * (1/b) sauf si a est une constante  1
				if (natop1 == natConstante) {
					CConstante cons = (CConstante) calcOperande1;
					if (cons.valeur == 1) return new COperation(liste, cons, calcOperande2, Operateur.Divi);
				}
				// Si l'oprande de gauche est un produit dont un des facteurs est une constante 1
				// On le remplace par le facteur de gauche
				if (natop1 == natOperation) {
					COperation op1 = (COperation) calcOperande1;
					if (op1.operateur == Operateur.Mult) {
						if (op1.operande1.nature() == natConstante) {
							if( ((CConstante)op1.operande1).valeur == 1) return
									/** Un appel de trop modifi version 4.7.5.2
									new COperation(liste, op1.operande2, calcOperande2,
											Operateur.Divi).calculNormalise(rempval,sousdivnorm).calculNormalise(rempval, sousdivnorm);
											*/
									new COperation(liste, op1.operande2, calcOperande2, Operateur.Divi).calculNormalise(rempval,sousdivnorm, rempDecParFrac);
						}
						if (op1.operande2.nature() == natConstante) {
							if( ((CConstante)op1.operande2).valeur == 1) return 
									/** Un appel de trop modifi version 4.7.5.2
									new COperation(liste, op1.operande1, calcOperande2,
											Operateur.Divi).calculNormalise(rempval,sousdivnorm).calculNormalise(rempval, sousdivnorm);
											*/
									new COperation(liste, op1.operande1, calcOperande2, Operateur.Divi).calculNormalise(rempval,sousdivnorm, rempDecParFrac) ;
					}
					}
				}
				if (sousdivnorm) return new COperation(liste, calcOperande1,new COperation(liste, new CConstante(liste, 1), calcOperande2,
								Operateur.Divi), Operateur.Mult);
				else return new COperation(liste, calcOperande1, calcOperande2, Operateur.Divi);
			}
			if (op.operateur == Operateur.Plus) {
				if (rempval) {
					if (natop1 == natConstante) {
						CConstante resval1 = (CConstante) calcOperande1;
						if (resval1.valeur == 0) return calcOperande2;
					}
					if (natop2 == natConstante) {
						CConstante resval2 = (CConstante) calcOperande2;
						if (resval2.valeur == 0) return calcOperande1;
					}
				}	
				if (sousdivnorm) return new COperation(liste, calcOperande1, calcOperande2, Operateur.Plus);
				else {
					// On remplace a + (-b) par a - b
					if (natop2 == natMoinsUnaire) {
						CMoinsUnaire mun2 = (CMoinsUnaire) calcOperande2;
						// Correction version 5.3
						// return new COperation(liste, calcOperande1, mun2.operande, Operateur.Moin);
						return new COperation(liste, calcOperande1, mun2.operande, Operateur.Moin).calculNormalise(rempval, sousdivnorm, rempDecParFrac);
					}
				}
			}
			if (op.operateur == Operateur.Mult) {
				if (rempval) {
					// Il  faut faire "remonter" les moins unaires le plus haut possible
					if (natop1 == natMoinsUnaire) {
						CMoinsUnaire mun1 = (CMoinsUnaire) calcOperande1;
						if (natop2 == natMoinsUnaire) {
							CMoinsUnaire mun2 = (CMoinsUnaire) calcOperande2;
							return new COperation(liste, mun1.operande, mun2.operande, Operateur.Mult).calculNormalise(rempval, sousdivnorm, rempDecParFrac);
						}
						else return new CMoinsUnaire(liste, 
								new COperation(liste, mun1.operande, calcOperande2, Operateur.Mult).calculNormalise(rempval, sousdivnorm, rempDecParFrac));
					}
					else {
						if (natop2 == natMoinsUnaire) {
						CMoinsUnaire mun2 = (CMoinsUnaire) calcOperande2;
						return new CMoinsUnaire(liste, new COperation(liste, calcOperande1, mun2.operande, Operateur.Mult).
								calculNormalise(rempval, sousdivnorm, rempDecParFrac));
						}
					}
					if (natop1 == natConstante) {
						CConstante resval1 = (CConstante) calcOperande1;
						double r = resval1.valeur;
						if (r == 1) return calcOperande2;
						if (r == 0) {
							return new CConstante(liste, 0);
						}
					}
						
					if (natop2 == natConstante) {
						CConstante resval2 = (CConstante) calcOperande2;
						double r = resval2.valeur;
						if (r == 1) return calcOperande1;
						if (r == 0) {
							return new CConstante(liste, 0);
						}
					}
					return new COperation(liste, calcOperande1, calcOperande2, Operateur.Mult);
				}
			}
			// Autres cas . Rien   simplifier et on simplifie seulement les oprandes.
			return new COperation(liste, calcOperande1, calcOperande2, op.operateur);
		}
		if (nat == natResultatValeur) {
			CResultatValeur rv = (CResultatValeur) this;
			if (rempval)
				return new CConstante(liste, rv.valeurAssociee.rendValeur());
			else
				return this;
		}
		if (nat == natResultatValeurComplexe) {
			CResultatValeurComplexe rv = (CResultatValeurComplexe) this;
			if (rempval) {
				Complexe z = new Complexe();
				rv.resultatComplexe(false, z);
				if (z.y == 0) return new CConstante(liste, rv.valeurAssociee.rendValeur());
				else return new COperation(liste, new CConstante(liste, z.x), 
						new COperation(liste, new CConstante(liste, z.y), new CConstantei(liste), Operateur.Mult), Operateur.Plus);
			}
			else
				return this;
		}
		if (nat == natPuissance) {
			CPuissance pui = (CPuissance) this;
			// Amlioration version 4.7.9 : Remplace a^1 par a
			CCalculBase calcexp = pui.exposant.calculNormalise(rempval, sousdivnorm, rempDecParFrac);
			long natexp = calcexp.nature();
			if (natexp == natConstante) {
				double res = ((CConstante)calcexp).valeur;
				if (res == 1) return pui.operande.calculNormalise(rempval, sousdivnorm, rempDecParFrac); 
			}
			return new CPuissance(liste, pui.operande.calculNormalise(rempval, sousdivnorm, rempDecParFrac),
					pui.exposant.calculNormalise(rempval, sousdivnorm, rempDecParFrac));
		}
		if (nat == natFonction) {
			CFonction fon = (CFonction) this;
			// Ajout version 4.7.8.1
			switch (fon.operateurFonction) {
				case OperateurFonction.Left :
				case OperateurFonction.LeftC : 
					return fon.operande.membreGauche().calculNormalise(rempval, sousdivnorm, rempDecParFrac);
				case OperateurFonction.Right :
				case OperateurFonction.RightC : 
					return fon.operande.membreDroit().calculNormalise(rempval, sousdivnorm, rempDecParFrac);
				default : return new CFonction(liste, fon.operateurFonction, fon.operande.calculNormalise(rempval, sousdivnorm, rempDecParFrac));
			}
			// return new CFonction(liste, fon.operateurFonction,
			//		fon.operande.calculNormalise(rempval, sousdivnorm));
		}
		if (nat == natAppelFonction) {
			CAppelFonction apfon = (CAppelFonction) this;
			return new CAppelFonction(liste, apfon.fonctionAssociee,
					apfon.operande.calculNormalise(rempval, sousdivnorm, rempDecParFrac));
		}
		if (nat == natFonction2Var) {
			CFonction2Var fon = (CFonction2Var) this;
			CCalculBase[] op = new CCalculBase[2];
			op[0] = fon.operande1.calculNormalise(rempval, sousdivnorm, rempDecParFrac);
			op[1] = fon.operande2.calculNormalise(rempval, sousdivnorm, rempDecParFrac);
			return new CFonction2Var(liste, fon.operateurFonction, op[0], op[1]);
		}
		if (nat == natFonction3Var) {
			CFonction3Var fon = (CFonction3Var) this;
			CCalculBase[] op = new CCalculBase[3];
			op[0] = fon.operande1.calculNormalise(rempval, sousdivnorm, rempDecParFrac);
			op[1] = fon.operande2.calculNormalise(rempval, sousdivnorm, rempDecParFrac);
			op[2] = fon.operande3.calculNormalise(rempval, sousdivnorm, rempDecParFrac);
			return new CFonction3Var(liste, fon.operateurFonction, op[0], op[1],
					op[2]);  // Corrig version 4.9.5.1
		}
		if (nat == natAppelFonctionNVar) {
			CAppelFonctionNVar apfon = (CAppelFonctionNVar) this;
			CCalculBase[] op = new CCalculBase[apfon.nbVar];
			for (int i = 0; i < apfon.nbVar; i++)
				op[i] = apfon.operande[i].calculNormalise(rempval, sousdivnorm, rempDecParFrac);
			return new CAppelFonctionNVar(liste, apfon.nbVar, apfon.fonctionAssociee,
					op);
		}
		if (nat == natMoinsUnaire) {
			CMoinsUnaire mun = (CMoinsUnaire) this;
			// On remplace -(-b) par b
			CCalculBase calcOperande = mun.operande.calculNormalise(rempval, sousdivnorm, rempDecParFrac);
			if (rempval) {
				long natop1 = calcOperande.nature();
				if (natop1 == natMoinsUnaire) {
					CMoinsUnaire mun2 = (CMoinsUnaire) calcOperande;
					return mun2.operande.calculNormalise(rempval, sousdivnorm, rempDecParFrac);
				}
				// Ajout version 4.7.9
				if (natop1 == natConstante) {
					CConstante resval = (CConstante) calcOperande;
					double r = resval.valeur;
					if (r == 0) return new CConstante(liste, 0);
				}
			}
			return new CMoinsUnaire(liste,  calcOperande);
		}
		// Ajout version 5.1.3 pour accepter les fractions irrductibles  la place des dcimaux et vice-versa
		if (rempDecParFrac && nat == natConstante) {
			CConstante co  = (CConstante) this;
			double va = co.valeur;
			if (va == Math.floor(va)) return this;
			double n = Math.floor(va*Math.pow(10, 9)+0.5);
			double d = 1000000000;
			if (!Kernel.zero13(va-n/Math.pow(10, 9))) return this;
			double g = Kernel.pgcd(n, 1000000000);
			double num = n/g;
			double den  = d/g;
			if (sousdivnorm) {
				if (num == 1) return new COperation(liste,new CConstante(liste,1),new CConstante(liste,d/g),Operateur.Divi);
				else return new COperation(liste,new CConstante(liste, num),new COperation(liste,new CConstante(liste,1),new CConstante(liste,den),Operateur.Divi),Operateur.Mult);
			}
			else return new COperation(liste,new CConstante(liste, num), new CConstante(liste,den), Operateur.Divi);
		}
		//
		// Dans les autres cas on renvoie le mme calcul
		return this;
	}

	/**
	 * Ajout version 4.6 Fonction ajoutant  liste tous les operandes d'addition
	 * issus d'un mme noeud d'addition
	 * 
	 * @param liste
	 */
	public final void ajouteOperateursPlus(ArrayList<CCalculBase> liste) {
		long nat = nature();
		if (nat == natOperation) {
			COperation op = (COperation) this;
			if (op.operateur == Operateur.Plus) {
				op.operande1.ajouteOperateursPlus(liste);
				op.operande2.ajouteOperateursPlus(liste);
			} else
				liste.add(this);
		} else
			liste.add(this);
	}

	/**
	 * Ajout version 4.6 Fonction ajoutant  liste tous les operandes de
	 * multiplication issus d'un mme noeud de multiplication
	 * 
	 * @param liste
	 */
	public final void ajouteOperateursMult(ArrayList<CCalculBase> liste) {
		long nat = nature();
		if (nat == natOperation) {
			COperation op = (COperation) this;
			if (op.operateur == Operateur.Mult) {
				op.operande1.ajouteOperateursMult(liste);
				op.operande2.ajouteOperateursMult(liste);
			} else
				liste.add(this);
		} else
			liste.add(this);
	}

	/**
	 * Fonction renvayant true si this est quivalent  calc. Avant appel, this et
	 * calc doivent avoir t normaliss de faon que this et calc ne contiennent
	 * que des additions et divisions.
	 * 
	 * @param calc
	 * @return
	 */
	public final boolean estEquivalentA(CCalculBase calc) {
		long nat1 = nature();
		long nat2 = calc.nature();
		if (nat1 != nat2)
			return false;
		if (nat1 == natConstante) {
			CConstante const1 = (CConstante) this;
			CConstante const2 = (CConstante) calc;
			return const1.valeur == const2.valeur;
		}
		if (nat1 == natMoinsUnaire) {
			CMoinsUnaire mun1 = (CMoinsUnaire) this;
			CMoinsUnaire mun2 = (CMoinsUnaire) calc;
			return mun1.operande.estEquivalentA(mun2.operande);
		}
		if (nat1 == natFonction) {
			CFonction fon1 = (CFonction) this;
			CFonction fon2 = (CFonction) calc;
			// Gros bug corrig version 4.7.8.1
			// return fon1.operande.estEquivalentA(fon2.operande);
			return OperateurFonction.estEquivalent(fon1.operateurFonction, fon2.operateurFonction) && fon1.operande.estEquivalentA(fon2.operande);
		}
		if (nat1 == natAppelFonction) {
			CAppelFonction apfon1 = (CAppelFonction) this;
			CAppelFonction apfon2 = (CAppelFonction) calc;
			// Bug corrig version 4.7.8.1
			// return apfon1.operande.estEquivalentA(apfon2.operande);
			return (apfon1.fonctionAssociee ==  apfon2.fonctionAssociee) && apfon1.operande.estEquivalentA(apfon2.operande);
		}
		if (nat1 == natFonction2Var) {
			CFonction2Var fon1 = (CFonction2Var) this;
			CFonction2Var fon2 = (CFonction2Var) calc;
			// Gros bug corrig version 4.7.8.1
			// return fon1.operande1.estEquivalentA(fon2.operande1)
			//		&& fon1.operande2.estEquivalentA(fon2.operande2);
			return (fon1.operateurFonction == fon2.operateurFonction) && fon1.operande1.estEquivalentA(fon2.operande1)
					&& fon1.operande2.estEquivalentA(fon2.operande2);
		}
		if (nat1 == natFonction3Var) {
			CFonction3Var fon1 = (CFonction3Var) this;
			CFonction3Var fon2 = (CFonction3Var) calc;
			// Gros bug corrig version 4.7.8.1
			// return fon1.operande1.estEquivalentA(fon2.operande1)
			//		&& fon1.operande2.estEquivalentA(fon2.operande2)
			//		// && fon1.estEquivalentA(fon2.operande3); // Corrig version 4.7.5.2
			//		&& fon1.operande3.estEquivalentA(fon2.operande3);
			return (fon1.operateurFonction == fon2.operateurFonction) && fon1.operande1.estEquivalentA(fon2.operande1)
					&& fon1.operande2.estEquivalentA(fon2.operande2)
					&& fon1.operande3.estEquivalentA(fon2.operande3);
		}
		if (nat1 == natAppelFonctionNVar) {
			CAppelFonctionNVar apfon1 = (CAppelFonctionNVar) this;
			CAppelFonctionNVar apfon2 = (CAppelFonctionNVar) calc;
			// Corrig version 4.7.8.1
			// if (apfon1.nbVar != apfon2.nbVar)
			// 	return false;
			if (apfon1.fonctionAssociee != apfon2.fonctionAssociee) return false;
			CCalculBase[] op1 = new CCalculBase[apfon1.nbVar];
			for (int i = 0; i < apfon1.nbVar; i++)
				op1[i] = apfon1.operande[i];
			CCalculBase[] op2 = new CCalculBase[apfon2.nbVar];
			for (int i = 0; i < apfon2.nbVar; i++)
				op2[i] = apfon2.operande[i];
			boolean res = true;
			for (int i = 0; i < apfon1.nbVar; i++)
				res = res && op1[i].estEquivalentA(op2[i]);
			return res;
		}
		if (nat1 == natVariableFormelle) {
			CVariableFormelle for1 = (CVariableFormelle) this;
			CVariableFormelle for2 = (CVariableFormelle) calc;
			return for1.indvar == for2.indvar;
		}
		if (nat1 == natOperation) {
			COperation op1 = (COperation) this;
			COperation op2 = (COperation) calc;
			long natop1 = op1.operateur;
			long natop2 = op2.operateur;
			if (natop1 != natop2) {
				if (natop1 == Operateur.InfOuEgal) {
					// a <= b quivalent  b>= a)
					if (natop2 == Operateur.SupOuEgal)
						return (op1.operande1.estEquivalentA(op2.operande2) && op1.operande2.estEquivalentA(op2.operande1));
					else return false; // Rajout version 4.7.2
				}
				else {
					if (natop1 == Operateur.Inf) {
					// a < b quivalent  b > a)
						if (natop2 == Operateur.Sup)
							return (op1.operande1.estEquivalentA(op2.operande2) && op1.operande2.estEquivalentA(op2.operande1));
						else return false; // Rajout version 4.7.2
					}
					else {
						if (natop1 == Operateur.SupOuEgal) {
							// a >= b quivalent  b <= a)
							if (natop2 == Operateur.InfOuEgal)
								return (op1.operande1.estEquivalentA(op2.operande2) && op1.operande2.estEquivalentA(op2.operande1));
							else return false; // Rajout version 4.7.2
						}
						else {
							if (natop1 == Operateur.Sup) {
								// a > b quivalent  b < a)
								if (natop2 == Operateur.Inf)
									return (op1.operande1.estEquivalentA(op2.operande2) && op1.operande2.estEquivalentA(op2.operande1));
								else return false; // Rajout version 4.7.2
							}
							else return false;
						}
					}
				}
			}
			// Les deux oprateurs sont les mmes
			if (natop1 == Operateur.Plus) {
				// Il faut vrifier que tous les oprandes sous-jascents sont
				// quivalents
				ArrayList<CCalculBase> list1 = new ArrayList<CCalculBase>();
				ArrayList<CCalculBase> list2 = new ArrayList<CCalculBase>();
				op1.ajouteOperateursPlus(list1);
				op2.ajouteOperateursPlus(list2);
				if (list1.size() != list2.size())
					return false;
				for (int i = 0; i < list1.size(); i++) {
					CCalculBase c1 = list1.get(i);
					boolean trouve = false;
					for (int j = 0; j < list2.size(); j++) {
						CCalculBase c2 = list2.get(j);
						if (c1.estEquivalentA(c2)) {
							trouve = true;
							list2.remove(j);
							break;
						}
					}
					if (!trouve)
						return false;
					else {
						list1.remove(i);
						i--; // Car le terme d'indice i a t retir
					}
				}
				return true;
			}
			if (natop1 == Operateur.Mult) {
				// Il faut vrifier que tous les oprandes sous-jascents sont
				// quivalents
				ArrayList<CCalculBase> list1 = new ArrayList<CCalculBase>();
				ArrayList<CCalculBase> list2 = new ArrayList<CCalculBase>();
				op1.ajouteOperateursMult(list1);
				op2.ajouteOperateursMult(list2);
				if (list1.size() != list2.size())
					return false;
				for (int i = 0; i < list1.size(); i++) {
					CCalculBase c1 = list1.get(i);
					boolean trouve = false;
					for (int j = 0; j < list2.size(); j++) {
						CCalculBase c2 = list2.get(j);
						if (c1.estEquivalentA(c2)) {
							trouve = true;
							list2.remove(j);
							break;
						}
					}
					if (!trouve)
						return false;
					else {
						list1.remove(i);
						i--; // Car le terme d'indice i a t retir
					}
				}
				return true;
			}
			// a = b quivalent  b = a et idem pour le ou et le et logique
			// if (natop1 == Operateur.Egalite) { // Modifi version 4.9.5
			if ((natop1 == Operateur.Egalite) || (natop1 == Operateur.And) || (natop1 == Operateur.Or)) {
				return (op1.operande1.estEquivalentA(op2.operande1) && op1.operande2.estEquivalentA(op2.operande2))
						|| (op1.operande1.estEquivalentA(op2.operande2) && op1.operande2.estEquivalentA(op2.operande1));
			}
			// Dans les autres cas (division comprise, pour tre quivalent les
			// oprandes respectifs
			// doivent tre quivalents
			return (op1.operande1.estEquivalentA(op2.operande1))
					&& (op1.operande2.estEquivalentA(op2.operande2));
		}
		if (nat1 == natPuissance) {
			CPuissance pui1 = (CPuissance) this;
			CPuissance pui2 = (CPuissance) calc;
			return (pui1.operande.estEquivalentA(pui2.operande) && pui1.exposant.estEquivalentA(pui2.exposant));
		}
		if (nat1 == natConstantei) return true;
		if (nat1 == natResultatValeur) {
			CResultatValeur resval1 = (CResultatValeur) this;
			CResultatValeur resval2 = (CResultatValeur) calc;
			return resval1.valeurAssociee == resval2.valeurAssociee;
		}
		if (nat1 == natResultatValeurComplexe) {
			CResultatValeurComplexe resval1 = (CResultatValeurComplexe) this;
			CResultatValeurComplexe resval2 = (CResultatValeurComplexe) calc;
			return resval1.valeurAssociee == resval2.valeurAssociee;
		}
		return false;
	}
	
	/**
	 * Ajout version 4.7.9. Renvoie true si la calcul est factoris par calc.
	 * Avant appel, this et calc doivent tre rendus normaliss pour pouvoir reconnatre les rductions au mme dnominateur
	 */
	// Modifi version 4.9.3. Si l'oprateur est un moins unaire il faut tester la factorisation par son argument
	public final boolean estFactorisePar(CCalculBase calc) {
		long nat1 = nature();
		if (nat1 == natMoinsUnaire) {
			CMoinsUnaire mun= (CMoinsUnaire) this;
			return mun.operande.estFactorisePar(calc);
		}
		else {
			if (nat1 == natOperation) {
				COperation op1 = (COperation) this;
				long natop1 = op1.operateur;
				if ((natop1 == Operateur.InfOuEgal) || (natop1 == Operateur.Inf) || (natop1 == Operateur.SupOuEgal) || (natop1 == Operateur.Sup)
						|| (natop1 == Operateur.Egalite) || (natop1 == Operateur.Diff)) {
					return ((op1.operande1.estFactorisePar(calc) && op1.operande2.estConstante0()) || (op1.operande2.estFactorisePar(calc) && op1.operande1.estConstante0()));
				}
				if (natop1 == Operateur.Mult) {
					ArrayList<CCalculBase> list1 = new ArrayList<CCalculBase>();
					ArrayList<CCalculBase> list2 = new ArrayList<CCalculBase>();
					op1.ajouteOperateursMult(list1);
					long nat2 = calc.nature();
					if (nat2 != natOperation) list2.add(calc);
					else {
						COperation op2 = (COperation) calc;
						long natop2 = op2.operateur;
						if (natop2 == Operateur.Mult) op2.ajouteOperateursMult(list2); else list2.add(calc);
					}
					if (list1.size() < list2.size()) return false;
					for (int i = 0; i < list2.size(); i++) {
						CCalculBase c2 = list2.get(i);
						boolean trouve = false;
						for (int j = 0; j < list1.size(); j++) {
							CCalculBase c1 = list1.get(j);
							if (c1.estEquivalentA(c2)) {
								trouve = true;
								list1.remove(j);
								break;
							}
						}
						if (!trouve)
							return false;
						else {
							list2.remove(i);
							i--; // Car le terme d'indice i a t retir
						}
					}
					return true;
				}
			}
			// return false; // Modifi version 4.9.6 pour qu'une expression soit considre comme factorise par elle-mme
			return estEquivalentA(calc);
		}
	}
	
	// Ajout version 4.6
	public CCalculBase calculAvecValeursRemplacees() {
		return this;
	}
	
	// Ajout version 4.6
	/**
	 * Fonction renvoyant dans le cas d'un test le membre de gauche du test et sinon this
	 * A redfinir pour COperation, CResultatValeur et CResultatValeurComplexe
	 * @return
	 */
	public CCalculBase membreGauche() {
		// return null; Modifi version 4.7.2
		return this;	
	}
	
	// Ajout version 4.6
	/**
	 * Fonction renvoyant dans le cas d'un test le membre de droite du teste et sinon this
	 * A redfinir pour COperation, CResultatValeur et CResultatValeurComplexe
	 * @return
	 */
	public CCalculBase membreDroit() {
		// return null; Modifi version 4.7.2
		return this;
	}
	
}