/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 *
 * @author  Yves
 * @version 
 * Dans cette version comprenant des constructions cette classe n'est pas abstract
 * car dans les outils utilisant la cration visuelle d'objets (rapporteur) des
 * cercles de ce type sont instancis. Mais aucun objet de ce type n'est jamais
 * cr dans la liste des objets enregistrs dans une figure.
 * C'est la raison pour laquelle getClone est quand mme dfini mais renvoie null
 */
package objets;


import glob.*;

import java.awt.*;
import java.awt.geom.Point2D.Double;
import types.*;


// Modification pour la version 1.9.5
// A la suite du rajout des lieux d'objets cette classe n'est plus abstract

public class CCercle extends CElementLigne {
		// Ajout version 4.9.9 pour gnrer un nom
		public static int indice = 0;
		//
		public double centre_x;
		public double centre_y;
		public double rayon;
		// Coordonnes du carr de trac
		public double xCarre1,yCarre1,xCarre2,yCarre2;
		// Ajout version 3.2.0. Sera true quand le cercle ne peut pas tre trac
		public boolean horsFenetre;

  /** Creates new CCercle */
  public CCercle() {
  }
  
  /** Constructeur pour objets crs lors de la cration visuelle d'objets
   *  Le centre et le rayon resteront fixes
   */

  public CCercle(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, StyleTrait styleInit,
    double centre_xInit, double centre_yInit, double rayonInit) {
     super (liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit);
     centre_x = centre_xInit;
     centre_y = centre_yInit;
     rayon = rayonInit;
  }

  // Constructeur pour clonage
  public CCercle(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, StyleTrait styleInit) {
    super (liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit);      
  }
  
  @Override
  public String complementInfo() {
    return getString("chinfo43");
  }
  // Ajout version 4.9.9
  // Ajout version 4.9.9
  public String genereNom() {
  	indice++;
  	return getString("rcerc") + indice;
  }

  @Override
  public void setClone(CElementBase ptel) {
    super.setClone(ptel);
    centre_x = ((CCercle)ptel).centre_x;
    centre_y = ((CCercle)ptel).centre_y;
    rayon = ((CCercle)ptel).rayon;
    xCarre1 = ((CCercle)ptel).xCarre1;
    yCarre1 = ((CCercle)ptel).yCarre1;
    xCarre2 = ((CCercle)ptel).xCarre2;
    yCarre2 = ((CCercle)ptel).yCarre2;
    horsFenetre = ((CCercle)ptel).horsFenetre; // Rajout version 4.8.0.2 indisensable
  }

  // Ici getClone est cre pour viter une erreur de compilation mais ne sert pas
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible){
    return null;
  }

  // Devra tre redfini pour les descendants qui ne sont pas des cercles.
  // Permet de savoir si un point est ou pas sur l'objet
  public boolean surArc(double xt,double yt) {
    return true; // true pour un vrai cercle
  }

  // Ajout version 3.2.5
  /**
   * Fonction renvoyant la position relative de l'objet cercle (ou un descendant) avec un autre objet
   * du type point ou droite ou cercle.
   * L'entier retourn doit tre dfini par la classe PositionRelative
   */
  @Override
  public int positionRelative(CElementGraphique elg, Bool bperpendiculaires){
  	if (elg.estDeNature(NatureObjet.NToutPoint)) {
  		bperpendiculaires.setValue(false);
  		CPointAncetre pt = (CPointAncetre) elg;
  		if (Kernel.appartientCercle(this, pt.x, pt.y) && surArc(pt.x, pt.y)) return PositionRelative.appartient;
  		else return PositionRelative.nappartient;
  	}
  	else {
  		if (elg.estDeNature(NatureObjet.NToutCercle)) {
  			bperpendiculaires.setValue(false);
	  		CCercle cercle = (CCercle) elg;
		  	Double point1 = new Double();
				Double point2 = new Double();
				byte position = Kernel.intersectionCercleCercle(centre_x, centre_y, rayon, cercle.centre_x, cercle.centre_y, cercle.rayon, point1, point2);
				switch (position) {
					case PositionCercleCercle.Vide : return PositionRelative.vide;
					case PositionCercleCercle.Confondus :
						if ((getNature() == NatureObjet.NCercle) && (cercle.getNature() == NatureObjet.NCercle)) return PositionRelative.confondus;
						else return PositionRelative.memeSupportCercle;
					case PositionCercleCercle.Tangents:
						if (surArc(point1.x, point1.y) && cercle.surArc(point1.x, point1.y)) return PositionRelative.tangents;
						else return PositionRelative.vide;
					case PositionCercleCercle.Secants :
						boolean b1 = surArc(point1.x, point1.y) && cercle.surArc(point1.x, point1.y);
						boolean b2 = surArc(point2.x, point2.y) && cercle.surArc(point2.x, point2.y);
						if (b1 && b2) return PositionRelative.secants2Point;
						if (b1 || b2) return PositionRelative.secants1Point;
						return PositionRelative.vide;
					default : return PositionRelative.vide; // Pour satisfairele compilateur
				}
	  	}
  		else {
  			CDroiteAncetre dte = (CDroiteAncetre) elg;
  			return dte.positionRelative(this, bperpendiculaires);
  		}
  	}
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    // devra tre appele par tous les descendants de cercle.positionne
    // pour vrifier que le rayon est positif
    existe = (rayon > 0);
    if (! existe) return;
    /* Modifi version 5.2.0.2 car algo incorrect
    double rayon2 = rayon*rayon;
    // Modification version 3.2.0 pour optimsation temps de trac
    // On met horsEcran  true
    // On calcule les distances du centre aux quatre cois de la fentre
    d1 = Kernel.distancePointPointCarre(centre_x, centre_y, 0, 0);
    d2 = Kernel.distancePointPointCarre(centre_x, centre_y, 0, dimfen.longueurOrdonnees);
    d3 = Kernel.distancePointPointCarre(centre_x, centre_y, dimfen.longueurAbscisses, 0);
    d4 = Kernel.distancePointPointCarre(centre_x, centre_y, dimfen.longueurAbscisses, dimfen.longueurOrdonnees);
  	if ((rayon2 >= d1) && (rayon2 >= d2) && (rayon2 >= d3) && (rayon2 >= d4)) {
  		horsFenetre = true;
  		return;
  	}
  	if (!dimfen.dansFenetre(centre_x, centre_y)) {
    	// On calcule les distances du centre aux quatre cts de la fentre.
    	// Si le rayon est infrieur  la plus petite des quatre, le cercle n'est pas visible
    	d1 = Math.abs(centre_y);
    	d2 = Math.abs(centre_y - dimfen.longueurOrdonnees);
    	d3 = Math.abs(centre_x);
    	d4 = Math.abs(centre_x - dimfen.longueurAbscisses);
    	if ((rayon <= d1) && (rayon <= d2) && (rayon <= d3) && (rayon <= d4)) {
    		horsFenetre = true;
    		return;
    	}
    }
    */
    horsFenetre = !dimfen.testCercleDansFen(this.centre_x, this.centre_y, this.rayon);
    if (horsFenetre) return;
    xCarre1 = centre_x - rayon;
    yCarre1 = centre_y - rayon;
    xCarre2 = centre_x + rayon;
    yCarre2 = centre_y + rayon;
    horsFenetre = false;
  }
  
  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
    if (horsFenetre) return;
    StyleTrait styleTrait;
    if (masque) styleTrait = StyleTrait.traitPointille;
    else styleTrait = style;
    if ((Kernel.testToile(xCarre1,yCarre1)) && (Kernel.testToile(xCarre2,yCarre2)))
    {
      pDC.setStroke(Pinceau.CreePinceau(styleTrait, listeProprietaire));
      pDC.setColor(couleur);
      pDC.drawOval((int)Math.round(xCarre1),(int)Math.round(yCarre1),
          (int)Math.round(xCarre2 - xCarre1),(int)Math.round(yCarre2 - yCarre1));
    }
  }   

  public double abscisseMaximale() {
    // Modification de la version 1.9.5
    // return 2*Math.PI-1e-12;
    return 2*Math.PI;
  }

  public double abscisseMinimale() {
    return 0;
  }

  public double abscisseCurviligne(double ang)
  {
    return ang;
  }

  // Mthode dfinie pour la version 1.8
  public int natureArc() {
    return NatureArc.NatureArcCercle;
  }
    
  // Java : inutile de redfinir ChargeDepuisFlux    

  @Override
  public double distancePoint(double xp, double yp, boolean masquage) {
  	 if (!(existe) || (masquage && masque)) return -1;
  	 else return distancePointPourSurface(xp,yp,masquage);
  }
  
  public double distancePointPourSurface(double xp,double yp, boolean masquage) {
		if (!existe) return -1;
		else {
			double distance = Kernel.distancePointPoint(centre_x,centre_y,xp,yp);
			return Math.abs(distance - rayon);
    }      
  }
  
  // Modifi version 3.2.4.
  /**
   * Fonction renvoyant true seulement si le cercle est visible dans la fentre dimfen
   * @param dimfen
   */
  @Override
  public boolean dansFenetre(DimensionFenetre dimfen) {
  	/*
  	double dist[] = new double[4];
  	Point2D.Double int1 = new Point2D.Double();
  	Point2D.Double int2 = new Point2D.Double();
  	VecteurDirecteur u = new VecteurDirecteur();
  	double sommetsx[] = new double[5];
  	double sommetsy[] = new double[5];
		sommetsx[0] = 0;
    sommetsy[0] = 0;
    sommetsx[1] = dimfen.longueurAbscisses;
    sommetsy[1] = 0;
    sommetsx[2] = dimfen.longueurAbscisses;
    sommetsy[2] = dimfen.longueurOrdonnees;
    sommetsx[3] = 0;
    sommetsy[3] = dimfen.longueurOrdonnees;
    // Pour fermer la boucle des sommets, un cinquime
    sommetsx[4] = 0;
    sommetsy[4] = 0;
    if (dimfen.dansFenetre(centre_x,centre_y))
		{
    	for (int i = 0; i <= 3; i++) dist[i] = Kernel.distancePointPoint(centre_x,centre_y,sommetsx[i],sommetsy[i]);
			double maxDist = dist[0];
			for (int j = 1; j<=3; j++) {
				if (dist[j] > maxDist) maxDist = dist[j];
			}
  		if (rayon <= maxDist) return true; else return false;
    }
    else
    {
    	boolean resultat = false;
    	for (int i = 0; i <= 3; i++)
      {
      	u.x = sommetsx[i+1] - sommetsx[i];
        u.y = sommetsy[i+1] - sommetsy[i];
      	int position = Kernel.intersectionDroiteCercle(sommetsx[i],sommetsy[i],u,
        	centre_x,centre_y,rayon,int1,int2);
        switch (position)
				{
        	case PositionDroiteCercle.Secants:
					{
						resultat = (resultat || Kernel.appartientSegment(int1.x,int1.y,sommetsx[i],
          		sommetsy[i],sommetsx[i+1],sommetsy[i+1]));
						resultat = (resultat || Kernel.appartientSegment(int2.x,int2.y,sommetsx[i],
          		sommetsy[i],sommetsx[i+1],sommetsy[i+1]));
						break;
					}
          case PositionDroiteCercle.Tangents:
					{ 
						resultat = (resultat || Kernel.appartientSegment(int1.x,int1.y,
								sommetsx[i],sommetsy[i],sommetsx[i+1],sommetsy[i+1]));
					}
        }// switch
		} // for
      return resultat;
    }
    */
  	return !horsFenetre;
  }
  

  //Ajout pour version 2 du CNED
  @Override
  public long getNature() {
    return NatureObjet.NCercle;
  }
  
  public void setRayon(double ray) {
  	rayon = ray;
  }
  
  @Override
  public boolean coincideAvec(CElementBase p) {
    if (p.getNature() != NatureObjet.NCercle) return false;
    CCercle cerc = (CCercle) p;
    return Kernel.zero(this.centre_x-cerc.centre_x) && Kernel.zero(this.centre_y-cerc.centre_y)
      && Kernel.zero(this.rayon-cerc.rayon);
  } 
  /**
   * Fonction rajoute pour la version 2.1 et servant  savoir si un point
   * est par dfinition sur une droite. Renverra true par exemple
   * si la droite est dfinie par deux points et le point est un de ces deux points
   * ou si le point est li  la droite
   * A t rajout pour amliorer l'outil d'intersection
   * A redfinir pour les descendants quand ncessaire
   * */
  public boolean contientParDefinition(CPointAncetre po)
  {
    return false;
  }

  @Override
  public String chaineDesignation() {
    return getString("designationCercle");
  }
  
  /**
   * Fonction renvoyant true si le point de coordonnes (x; y) est intrieur au polygne
   * c'est  dire s'il est sur un des cts ou si la somme des mesures principales d'angle orient
   * est un multiple impair de 2*pi.
   */
  public boolean pointInterieur(double x, double y) {
  	return (Kernel.distancePointPoint(x, y, centre_x, centre_y) <= rayon);
  }
  
	/**
	 * Ajout version 4.8.1
	 */
	@Override
	public String tikz(DimensionFenetre dimf, boolean nomaff, double coefmult, boolean bu) {
		double x1 = listeProprietaire.tikzXcoord(centre_x, dimf, bu);
		double y1 = listeProprietaire.tikzYcoord(centre_y, dimf, bu);
		double ray = listeProprietaire.tikzLongueur(rayon, dimf, bu);
		return "\\draw " + tikzLineStyle(dimf, coefmult) + "(" + Kernel.chaineNombre(x1, 3) +  "," + Kernel.chaineNombre(y1, 3) +") circle " +
			"(" + Kernel.chaineNombre(ray, 3) + ");";
	}

}
