/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/**
 * Ajout version 4.8. Utile pour les constructions itratives et rcursives
 */
package objets;

import java.awt.Color;
import java.io.IOException;

import mathgraph32.MtgFrame;
import types.DimensionFenetre;
import types.NatureObjet;
import types.StyleTrait;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CCercleClone extends CCercle {
	public CCercle cercle; // Le cercle clon

  /** Constructeur */
  public CCercleClone() {
  }
  
  public CCercleClone(CListeObjets liste, CImplementationProto impProtoInit,boolean estElementFinalInit,
      Color couleurInit, boolean masqueInit, StyleTrait styleInit, CCercle cercleInit) {
  	super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit);
      cercle = cercleInit;
  }
  
  @Override  
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(cercle);
    int ind2 = listeSource.indexOf(impProto);
    CElementBase ptelb = new CCercleClone(listeCible,(CImplementationProto)(listeCible.get(ind2,CImplementationProto.class)),
    	estElementFinal, couleur, masque, style, (CCercle)(listeCible.get(ind1, CCercle.class)));
    return ptelb;
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || cercle.dependDe(p));
  }

  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || cercle.dependDePourReclassement(p));
  }

  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || cercle.dependDePourBoucle(p));
  }

	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return listeOb.contains(cercle) || cercle.estDefiniParObjetDans(listeOb);
	}    

  @Override  
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	// Il faut mmoriser si le'objet est masqu ou non car setClone va le rinitialiser  la valeur du point copi
  	boolean b = masque;
  	setClone(cercle);
  	masque = b;
  }
  
 	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
		return nom() + " : " + getString("natCercleClone") +  " " + getString("ch31") + " " + cercle.nom();
	}
  
  @Override
  public long getNature() {
    return NatureObjet.NCercleClone;
  }

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(cercle);
  }    

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    cercle = (CCercle) (list.get(ind1,CCercle.class));
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(cercle);
    oups.writeInt(ind1);
  }

}
