/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;
// Java : Important : en C++ CDemiDroite hritait de CDroite mais appelait la mthode de srialization
// de CElement ligne car il est inutile de mmoriser l'origine et le vecteur
// c'est impossible en java. Une demi-droite ne pouvant pas tre nomme, inutile s'avoir
// un champ abscisseNom
// Mais cela pose un problme car dans la mthode de serialization de CDemiDroiteOA qui hrite de
// CdemiDroite, c'est la mthode de srialization de CDroite qui tait appele, ce qui est illogique.
// Dans chargeDepuisFlux de CDemiDroiteOA, il faut donc dpiler les donnes inutils
import java.io.*;
import java.awt.Color;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

import mathgraph32.MtgFrame;


public class CDemiDroiteOA extends CDemiDroite {
  CPointAncetre O;
  CPointAncetre A;
  /** Creates new CDemiDroiteOA */
  public CDemiDroiteOA() {
    super(); // Pour que le vecteur directeur soir cr
  }

  // Constructeur pour clonage
  public CDemiDroiteOA(CListeObjets liste, CImplementationProto impProtoInit,boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, StyleTrait styleInit, CPointAncetre OInit, CPointAncetre AInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit);
    O = OInit;
    A = AInit;
  }

  /** Constructeur plus court pour objets visuels */
  public CDemiDroiteOA(CListeObjets liste, CImplementationProto impProtoInit,boolean estElementFinalInit,
    Color couleurInit, StyleTrait styleInit, CPointAncetre OInit, CPointAncetre AInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, false, styleInit);
    O = OInit;
    A = AInit;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(O);
    int ind2 = listeSource.indexOf(A);
    int ind3 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CDemiDroiteOA(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, masque, style,
      (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
      (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)));
    return ptelb;
  }
  
  // Inutil de redfinir setClone();

 	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
  	return nom() + " : " + getString("natDemiDroite") + " " + "[" + O.nom() + A.nom() + ")";
	}

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(O);
    liste.add(A);
  } 
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
		existe = (O.existe) && (A.existe);
		if (!existe) return;
		point_x = O.x;
		point_y = O.y;
		vecteur.x = A.x - O.x;
		vecteur.y = A.y - O.y;
		super.positionne(infoRandom, dimfen);
  }

  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
  	return memoriseDependance(super.dependDe(p) || O.dependDe(p) || A.dependDe(p));
  }

  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || O.dependDePourReclassement(p) || A.dependDePourReclassement(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
  	return ((p == this) || O.dependDePourBoucle(p) || A.dependDePourBoucle(p));
  }
  
/** Rajout pour la version 2.0 : permet de connatre si l'lment point par pt
   * est confondu avec l'lment en question ou trs proche.
   * Pour cette version applet, ne sera redfini que pour les objets que l'utilisateur
   * pourra effectivement crer en ligne.
  **/
  @Override
  public boolean confonduAvec(CElementBase p) {
    CDemiDroiteOA pt;
    if (p.getClass() == this.getClass()) {
      pt = (CDemiDroiteOA) p;
      return ((O == pt.O) && (A == pt.A));
    }
    else return false;
  }
  
  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (O == ancienPoint) O = nouveauPoint;
    if (A == ancienPoint) A = nouveauPoint;
  }

  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		(listeOb.contains(O) || O.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(A) || A.estDefiniParObjetDans(listeOb));
	}
    
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {

    super.chargeDepuisFlux(inps, list);
    // A cause de l'erreur de conception en C++, il faut dpiler les donnes inutiles
    // srializes !
    @SuppressWarnings("unused")
		double inutile = inps.readCDouble();
    int ind1 = inps.readCInt();
    int ind2 = inps.readCInt();
    O = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    A = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    O = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    A = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(O);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(A);
    oups.writeInt(ind2);
  }
  /**
   * Fonction rajoute pour la version 2.1 et servant  savoir si un point
   * est par dfinition sur une droite. Renverra true par exemple
   * si la droite est dfinie par deux points et le point est un de ces deux points
   * ou si le point est li  la droite
   * A t rajout pour amliorer l'outil d'intersection
   * A redfinir pour les descendants quand ncessaire
   * */
  @Override
  public boolean contientParDefinition(CPointAncetre po)
  {
    return (A == po) || (O == po);
  }
  
  @Override
  public boolean estParalleleParDefinition(CDroiteAncetre dte){
  	return (dte == this) || (O.appartientParDefinition(dte) && A.appartientParDefinition(dte))
  		|| dte.formentDroiteParallele(O, A);
  }
  
}
