/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.Color;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

import mathgraph32.MtgFrame;

public class CDroiteAB extends CDroite {

  CPointAncetre A;
	CPointAncetre B;

  /** Creates new CDroiteAB */
  public CDroiteAB() {
    super(); // Pour que lengthvecteur soit cr
  }

  // Constructeur pour clonage
  public CDroiteAB(CListeObjets liste, CImplementationProto impProtoInit,boolean estElementFinalInit,
    Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit, boolean masqueInit, String nomInit,
    int indiceTailleNomInit, StyleTrait styleInit, double abscisseNomInit, CPointAncetre AInit,
      CPointAncetre BInit) {
      super(liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
        dec_xInit, dec_yInit, masqueInit, nomInit, indiceTailleNomInit, styleInit, abscisseNomInit);
      A = AInit;
      B = BInit;
  } 
  /** Constructeur plus court pour objets visuels */
  public CDroiteAB(CListeObjets liste, CImplementationProto impProtoInit,boolean estElementFinalInit,
    Color couleurInit, StyleTrait styleInit,  CPointAncetre AInit, CPointAncetre BInit, int indiceTailleNomInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, false, styleInit, indiceTailleNomInit);
    A = AInit;
    B = BInit;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(A);
    int ind2 = listeSource.indexOf(B);
    int ind3 = listeSource.indexOf(impProto);
    CElementBase ptelb = new CDroiteAB(listeCible,(CImplementationProto)(listeCible.get(ind3,CImplementationProto.class)),
    	estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom, indiceTailleNom, style, abscisseNom,
      (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),(CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)));
    return ptelb;
  }
  // Inutile de redfinir setClone
  
 	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  // Ajout version 4.9.9
  @Override
  public String nom() {
  	return "(" + A.nom() + B.nom() + ")";
  }
  @Override
	public String info() {
  	return nom() + " : " + getString("menuCreerDroite") + " " + getString("chinfo196") + " " +
  			A.nom() + " " + getString("chinfo101") + " " + B.nom();
	}

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(A);
    liste.add(B);
  } 
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
  	return memoriseDependance(super.dependDe(p) || A.dependDe(p) || B.dependDe(p));
  }
  
  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || A.dependDePourReclassement(p) || B.dependDePourReclassement(p));
  }

  public boolean dependDePourBoucle(CElementBase p) {
  	return ((p == this) || A.dependDePourBoucle(p) || B.dependDePourBoucle(p));
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
		existe = (A.existe) && (B.existe);
		if (!(existe)) return;
		point_x = A.x;
		point_y = A.y;
		vecteur.x = B.x - A.x;
		vecteur.y = B.y - A.y;
		super.positionne(infoRandom, dimfen);
  }  

  /** Rajout pour la version 2.0 : permet de connatre si l'lment point par pt
   * est confondu avec l'lment en question ou trs proche.
   * Pour cette version applet, ne sera redfini que pour les objets que l'utilisateur
   * pourra effectivement crer en ligne.
  **/
  public boolean confonduAvec(CElementBase p) {
    CDroiteAB pt;
    if (p.getClass() == this.getClass()) {
      pt = (CDroiteAB) p;
      return (((A==pt.A) && (B==pt.B)) || ((B==pt.A) && (A==pt.B)));
    }
    else return false;
  }
  
  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (A == ancienPoint) A = nouveauPoint;
    if (B == ancienPoint) B = nouveauPoint;
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		(listeOb.contains(A) || A.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(B) || B.estDefiniParObjetDans(listeOb));
	}
  
  

  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
     super.chargeDepuisFlux(inps, list);
     int ind1 = inps.readCInt();
     int ind2 = inps.readCInt();
     A = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
     B = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    A = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    B = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(A);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(B);
    oups.writeInt(ind2);
  }
  /**
   * Fonction rajoute pour la version 2.1 et servant  savoir si un point
   * est par dfinition sur une droite. Renverra true par exemple
   * si la droite est dfinie par deux points et le point est un de ces deux points
   * ou si le point est li  la droite
   * A t rajout pour amliorer l'outil d'intersection
   * A redfinir pour les descendants quand ncessaire
   * */
  @Override
  public boolean contientParDefinition(CPointAncetre po)
  {
    return (A == po) || (B == po);
  }
  
  @Override
  public boolean estParalleleParDefinition(CDroiteAncetre dte){
  	return (dte == this) || (A.appartientParDefinition(dte) && B.appartientParDefinition(dte))
  	|| dte.formentDroiteParallele(A, B);
  }
  
}

