/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package objets;

import java.io.*;
import java.util.ListIterator;
import types.*;
import dialogs.MtgDlg;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

/**
 * Elment anctre de tous les objets graphiques et non graphiques
 * 
 * @author Yves
 * 
 */
public abstract class CElementBase extends CObjetBase {
	public boolean existe;
	public String nom = ""; // Ajout version 4.9.9
	// Membres rajouts pour la version 2.5
	public CImplementationProto impProto; // null si n'est pas un objet d'une
	// implmentation de prototype
	// Trs important : lors du chargement d'un prototype, comme on ne peut pas
	// faire pointer par exemple
	// un pointeur vers un CRepere vers un lement gnrique on est oblig de
	// crer un nouveau repre
	// Mais lors de l'implmentatiion du prototype, dans indexOf ne nouvel objet
	// doit donner indexOf
	// de l'lment gnrique.
	// Voir CListeObjets.get et CPrototype.indexOf pour comprendre
	public CElementGenerique elementGeneriqueAssocie;
	public boolean estElementFinal; // true si est lement final d'une implmentation de prototype
	// Ajouts pour version 3.4.2 pour optimiser la fonction dependDe(CElementBase p)
	// cette fonction mmorisera p dans elementTestePourDependDe
	// et le rsultat dans dependDeElementTeste
	// Cette amlioration a t introduite pour optimiser la recherche des lments
	// dpendants pour les lieux de points et lieux d'objets
	// Une fonction initialisePourDependance remettra elementTeste  null;
	public CElementBase elementTestePourDependDe;
	public boolean dependDeElementTeste;
	// Ajout version 4.8 : Pour optimiser la fonction dependDePourReclassement
	public CElementBase elementTestePourDependDePourRec;
	public boolean dependDeElementTestePourRec;
	// Ajout version 4.8.0 : A la site des contructions fractales, il s'est avr des ralentissements pour les figures
	// avec un grand nombre d'objets. 'tait du  la lenteur de indexOf JArrayList. On rajoute donc un membre index
	//  chaque lment qui sera son indice dans la liste  laquelle il appartient.
	public int index;


	/** Creates new CElementBase */
	public CElementBase() {
		elementGeneriqueAssocie = null;
		// Modifi version 4.8
		elementTestePourDependDe = null;
		elementTestePourDependDePourRec = null;
		index = -1; // Indispensable pour version 4.8 pour que indexOf renvoie -1 quand l'lment n'a pas encore t rajout  une liste
	}

	public CElementBase(CListeObjets liste, CImplementationProto impProtoInit,
			boolean estElementFinalInit) {
		super(liste);
		elementGeneriqueAssocie = null;
		impProto = impProtoInit;
		estElementFinal = estElementFinalInit;
		// Modifi version 4.8
		elementTestePourDependDe = null;
		elementTestePourDependDePourRec = null; // Ajout version 4.8
		index = -1; // Indispensable
	}
	
	/**
	 * Contrairement  la version C++, la nature des objets des lments non
	 * graphiques sera NAucunObjet Contrairement  la version C++ (la fonction
	 * n'existait pas en C++) renvoie uniquement la nature graphique de l'objet
	 */
	public long getNature() {
		return NatureObjet.NAucunObjet;
	}

	/**
	 * Fonction renvoyant true si la nature graphique de l'objet est d'un des
	 * types spcifis dans nat (obtenu par l'oprateur |  partir de plusieurs
	 * long spcifiant une nature graphique
	 * 
	 * @param nat
	 * @return
	 */
	public final boolean estDeNature(long nat) {
		return (getNature() & nat) != 0;
	}

	/**
	 * Fonction renvoyant true si la nature numrique de l'objet est d'un des
	 * types spcifis dans nat (obtenu par l'oprateur |  partir de plusieurs
	 * long spcifiant une nature graphique
	 * 
	 * @param nat
	 * @return
	 */
	public final boolean estDeNatureCalcul(long nat) {
		return (getNatureCalcul() & nat) != 0;
	}

	/**
	 * Ajout pour la version java. Pour une valeur numrique renverra la chane la
	 * reprsentant et pour un repre la chane le reprsentant (par exemple
	 * "(O,I,J)")
	 * 
	 * @return
	 */
	// Modifi version 4.9.9
	public String nom() {
		// return "";
		return nom;
	}

	/**
	 * Fonction n'existant pas dans la version C++. Renvoie la nature de calcul de
	 * l'objet. Si l'objet n'est pas de type calcul renvoie NAucunCalcul qui est
	 * nul. A redfinir pour tous les objets de type calcul
	 * 
	 * @return
	 */
	public long getNatureCalcul() {
		return NatureCalcul.NAucunCalcul;
	}

	/**
	 * Fonction n'existant pas en C++ et charge de mettre  jour l'objet quand un
	 * lment dont il dpend a t modifi. Devra tre appel par les lieux de
	 * points et lieux d'objets pour remettre  jour les liste qu'ils utilisent de
	 * faon interne
	 */
	public void metAJour() {
	}
	// Ajout version 4.9.8.2
	/**
	 * Fonction mettant  jour certains objets lorsqu'un objet a t rajout  la figure.
	 * Les objets  mettre  jour sont les macros d'affectation de valeur  une variable et de modification de variable
	 */
	public void metAJourPourCreation() {
	}
	
	/**
	 * Fonction renvoyant la chane  afficher dans la bote de dialogue
	 * d'historique pour dcrire l'objet. A redfinir pour les descendants
	 * 
	 * @return
	 */
	// Modifi version 4.9.9
	public String infoPourHistorique() {
		return info();
	}
	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane ourte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
	public String info() {
		return infoPourHistorique();
	}

	/**
	 * Fonction ajoutant  list les lments qui ont gnr l'objet. A redfinir
	 * pour les descendants. Par exemple pour une droite passant par deux points,
	 * ajoutera  la liste un pointeur sur ces deux points.
	 * 
	 * @param frame
	 *          ce paramtre sert quabd on rahoute des lmnts provisoires comme
	 *          pour un barycentre
	 * @param liste
	 */
	public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
		return;
	}

	/**
	 * La mthode suivante n'existait pas dans la version C++ et sert  crer un
	 * clone de tout objet dans une autre liste
	 * 
	 */
	public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
		return null;
	}

	/**
	 * Procdure servant  cloner l'tat d'un objet identique au lieu de le
	 * positionner Utilis pour les lieux d'objets
	 */
	public void setClone(CElementBase ptel) {
		existe = ptel.existe;
	}

	// Ajout version 4.8.1
	public void setCloneTikz(CElementBase ptel) {
		setClone(ptel);
	}

	/**
	 * Calcule l'lment pour le positionner dans la figure
	 * 
	 * @param infoRandom
	 * @param dimfen
	 */
	public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
		existe = true;
	}
	/**
	 * Ajout version 4.9.3 : Sera edfini pour les test d'quivalences ou les test de factorisation
	 * Normalement ces calculs ne sont positionns qu'une  fois quand on change leur formule
	 * Dans les produites et somme indics ce sera cette fonction qui sera utilise
	 * Permet de crer des exercices de factorisation sophistiqus.
	 * @param infoRandom
	 * @param dimfen
	 */
	public void positionneFull(boolean infoRandom, DimensionFenetre dimfen) {
		positionne(infoRandom, dimfen);
	}
	
	
	// Ajout version 4.8.1
	public void positionneTikz (boolean infoRandom, DimensionFenetre dimfen) {
		positionne(infoRandom, dimfen);
	}
	
	public void initialisePourDependance() {
		elementTestePourDependDe = null;
		elementTestePourDependDePourRec = null; // Ajout version 4.8
	}
	
	public boolean memoriseDependance(boolean resultat) {
		dependDeElementTeste = resultat;
		return resultat;
	}

	public boolean dependDe(CElementBase p) {
		// Ajout version 3.4.2
		elementTestePourDependDe = p;
		boolean res = false;
		// Modifi pour la version 2.5 pour qu'un objet final ou intermdiaire
		// d'une implmentation de prototype dpende de cette implmentation
		if (impProto != null) {
			if (impProto == p)
				res = true;
		}
		res = res || (p == this);
		return res;
	}

	public boolean dependDeAuMoinsUn(CListeObjets liste) {
		boolean res = false;
		ListIterator<CElementBase> li = liste.listIterator();
		// Optimis pour la version 3.4.0.1
		// while (li.hasNext()) {
		while (li.hasNext() && !res) {		
			CElementBase el = li.next();
			res = res || dependDe(el);
		}
		return res;
	}

	/**
	 * Pour le reclassement d'objets, un lment dpendant d'un objet final d'une
	 * implmentation de prototype doit tre considr comme dpendant de tous les
	 * objets sources de cette implmentation A redfinir chaque fois que
	 * dependDe() est redfinie
	 * Modifi version 4.8
	 */
	public boolean dependDePourReclassement(CElementBase p) {
		boolean res = false;
		// Ajout version 3.4.2
		elementTestePourDependDePourRec = p;

		// Modifi pour la version 2.5 pour qu'un objet final ou intermdiaire
		// d'une implmentation de prototype dpende des objets sources de cette
		// implmentation
		if (impProto != null)
			res = impProto.dependDe(p);
		res = res || (p == this);
		return res;
	}
	
	// Ajout version 4.8
	public boolean memoriseDependancePourRec(boolean resultat) {
		dependDeElementTestePourRec = resultat;
		return resultat;
	}


	/**
	 * Ajout version 4.8
	 * Renverra true si l'objet dpend du prototype proto de la figure.
	 * Sera redfini pour CMacroConstructionIterative et CMacroConstructionRecursive et CMacroSuiteMacros
	 * @param proto
	 * @return
	 */
	public boolean dependDeProto(CPrototype proto) {
		return false;
	}
	/**
	 * Fonction renvoyant la mme chose que dependDePourReclassement sauf dans le
	 * cas o l'lment p est un lment final de construction auquel cas on
	 * revoie l'quivalent mais pour le bloc d'lments finaux de la construction
	 * auquel appartient p. Est utilis dans le reclassement d'un objet en fin de
	 * liste des objets crs
	 * 
	 * @param p
	 * @return
	 */
	public boolean dependDeBlocPourReclassement(CElementBase p) {
		if (p.impProto == null)
			return dependDePourReclassement(p);
		else {
			boolean b = false;
			CImplementationProto imp = p.impProto;
			int ind = listeProprietaire.indexOf(imp);
			// Correction d'un bug version 4.5.2
			// for (int i = 1; i < imp.nbFinaux + imp.nbIntermediaires; i++) {
			int nb = imp.nbFinaux + imp.nbIntermediaires;
			for (int i = 1; (i <= nb) && !b; i++) {
				CElementBase elb = listeProprietaire.get(ind + i);
				if (!elb.estElementIntermediaire())
					b = b || dependDePourReclassement(elb);
			}
			return b;
		}
	}

	/**
	 * Fonction renvoyant le mme rsultat que dependDeBlocPourReclassement si
	 * l'objet n'est pas un objet final et sinon renvoyant true si le bloc
	 * constitu par l'implmentation de prototype dont l'objet est un objet final
	 * contient au moins un lment pour lequel dependDeBlocPourReclassement
	 * renvoie true; Sert dans CListeObjet.decaleDependants
	 * 
	 * @param p
	 * @return
	 */
	public boolean appartientABlocDependantPourReclassement(CElementBase p) {
		if (!estElementFinal)
			return dependDeBlocPourReclassement(p);
		else {
			boolean res = false;
			int ind = listeProprietaire.indexOf(impProto);
			int nb = impProto.nbFinaux + impProto.nbIntermediaires;
			// Optimis version 4.5.2
			// 
			for (int i = 1; (i <= nb) && !res; i++) {
				CElementBase el = listeProprietaire.get(ind + i);
				// Bug corrig version 4.2.2
				// res = res || el.dependDePourReclassement(p);
				res = res || el.dependDeBlocPourReclassement(p);
			}
			return res;
		}
	}

	public boolean dependDePourBoucle(CElementBase p) {
		return (p == this);
	}

	public boolean dependDePourCapture(CElementBase p) {
		return dependDe(p);
	}

	/**
	 * Rajout pour la version 2.0 : permet de connatre si l'lment point par
	 * pt est confondu avec l'lment en question ou trs proche. Pour cette
	 * version applet, ne sera redfini que pour les objets que l'utilisateur
	 * pourra effectivement crer en ligne.
	 **/
	public boolean confonduAvec(CElementBase p) {
		// return false;
		// Modifi version 5.1.4
		return this == p;
	}

	/**
	 * Rajout pour la version 2.0 : permet de connatre si l'lment point par
	 * pt est graphiquement (presque) confondu avec l'lment en question ou trs
	 * proche. Pour cette version applet, ne sera redfini que pour les objets que
	 * l'utilisateur pourra effectivement crer en ligne de faon  tester la
	 * validit de la construction.
	 **/
	public boolean coincideAvec(CElementBase p) {
		return false;
	}

	/**
	 * Fonction replaant un pointeur sur un point par un autre point Utilise
	 * entre autres dans le reclassement d'objets. A redfinir pour chacun des
	 * descendants maintenant un pointeur sur un point.
	 **/
	public void remplacePoint(CPointAncetre ancienPoint,
			CPointAncetre nouveauPoint) {
	}

	/**
	 * fonction n'existant pas en C++ et renvoyant un pointeur vers l'antcdent
	 * direct de l'objet. Si l'objet est un objet qui ne ncessite pas lors de sa
	 * cration la cration d'objets supplmentaires, renvoie this et sinon envoi
	 * un pointeur sur le premier de ces objets supplmentaires crs. Par
	 * exemple, pour un point li  un bipoint (intersection de deux cercles ou
	 * d'un cercle et une droite) renverra un pointeur sur le bipoint cr avant
	 * ce CPointLieBipoint. Pour une objet image, renverra un pointeur vers la
	 * transformation utilise etc ...
	 */
	public CElementBase antecedentDirect() {
		return this;
	}

	/*
	 * Fonction rajoute pour la version 2.5
	 */
	// Ajout pour la version 2.5
	public boolean estElementIntermediaire() {
		return !estElementFinal && (impProto != null);
	}

	/**
	 * Fonction qui renverra true seulement pour les mesures utilisant point car
	 * alors ce point ne doit pas tre renomm avec un nom vide
	 * 
	 * @return
	 */
	public boolean nomIndispensable(CElementGraphique el) {
		return false;
	}

	/**
	 * Fonction qui renverra true si la classe de l'objet possde un menu pour le
	 * modifier Dans le cas o cette fonction renvoie true, la classe devra
	 * dfinir une fonction dialogueModification() qui renverra une dialogue
	 * modifiant l'objet
	 * Attention   partir de la version 3.5, les seuls objets finaux de construction qu'on peut modifier sont
	 * CCommentaire et les CLatex, les lieux de points et lieux d'objets.
	 * 
	 * @return
	 */
	public boolean modifiableParMenu() {
		return false;
	}

	/**
	 * A redfinir pour les descendants dont la fonction modifiableParMenu
	 * renverra true Renvoie un pointeur sur une bote de dialogue permettant de
	 * modifier l'objet
	 * 
	 * @param frame
	 * @return
	 */
	public MtgDlg dialogueModification(MtgFrame frame) {
		return null;
	}

	/**
	 * A redfinir pour les descendants dont la fonction modifiableParMeenu
	 * renverra true Renvoie une chane indiquant quelle a t la modification
	 * pour le gestionnaire d'annulation refaire
	 * 
	 * @return
	 */
	public String informationModification() {
		return "";
	}

	/**
	 * Fonction devant tre redfinie pour tous les objets images par une
	 * transformation Sera utilise lorsqu'on modifie une image par une
	 * transformation pour que cette image soit associe  une autre
	 * transformation lrsque d'autres objets taient des images par cette mme
	 * transformation
	 */
	public void associeATransformation(CTransformation trans) {
	}

	/**
	 * A redfinir pour tous les descendants faiasant partie des objets pouvant
	 * figurer dans une liste Sert pour les constructions  savoir si un objet est
	 * dfini uniquement par des objets figurant dans une liste
	 */
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
		return false; // Par exemple CBipoint ne redfinit pas cette fonction
	}

	// Ajout version 4.6.2
	/**
	 * Adaptation de certains objets pour passer en haute rsolution avant
	 * impression notamment
	 * Multiplie les coordonnes de tous les objets par ceof
	 * @param coef
	 */
	public void adaptePourHauteResolution(double coef) {
	}
	
	/**
	 * Fonction renvoyant true si l'objet dpend des positions gnres pas les
	 * dplacements d'un point li A redfinir pour les lieux de points et lieux
	 * d'objets et macros utilisant les dplacements d'un point li
	 * 
	 * @param pointlie
	 * @return
	 */
	public boolean estGenereParPointLie(CPointLie pointlie) {
		return false;
	}

	public static long natureGraphiquePourImplementationProto(long nat) {
		if ((nat & (NatureObjet.NToutPoint - NatureObjet.NPointLie)) != 0)
			return NatureObjet.NToutPoint;
		else
			return nat;
	}

	public static long natureObjetNonGraphiquePourImplementationProto(long nat) {
		if (nat == NatureCalcul.NCalculReel)
			return NatureCalcul.NTouteValeurReelle;
		if (nat == NatureCalcul.NCalculComplexe)
			return NatureCalcul.NTouteValeurComplexe;
		// Dans les autres cas mme nature attendue
		return nat;
	}

	public static final String chaineNatureGraphiquePourImplementationProto(
			long nat) {
		if ((nat & NatureObjet.NToutPointSaufPointLie) != 0)
			return getString("point");
		if (nat == NatureObjet.NDroite)
			return getString("droite");
		if (nat == NatureObjet.NDemiDroite)
			return getString("demidroite");
		if (nat == NatureObjet.NCercle)
			return getString("cercle");
		if (nat == NatureObjet.NSegment)
			return getString("segment");
		if (nat == NatureObjet.NArc)
			return getString("arccercle");
		if (nat == NatureObjet.NPointLie)
			return getString("pointlie");
		if (nat == NatureObjet.NPolygone)
			return getString("polygone");
		if (nat == NatureObjet.NLigneBrisee)
			return getString("lignebrisee");
		if (nat == NatureObjet.NLieu)
			return getString("lieuPoints");
		else
			return ""; // Pour le compilateur
	} 

	public static final String chaineNatureCalculPourImplementationProto(long nat) {
		// Version 3.1.5 ; Si un objet source est une variable, on ne peut lui
		// affecter qu'une variable
		if (nat == NatureCalcul.NVariable)
			return getString("variable");
		if ((nat & (NatureCalcul.NCalculReel | NatureCalcul.NTouteValeurReelle)) != 0)
			return getString("valeurnumeriquereelle");
		if ((nat & (NatureCalcul.NCalculComplexe | NatureCalcul.NTouteValeurComplexe)) != 0)
			return getString("valeurnumeriquecomplexe");
		if (nat == NatureCalcul.NFonction)
			return getString("fonctionreelle");
		if (nat == NatureCalcul.NFonction2Var)
			return getString("fonctionreelle2Var");
		if (nat == NatureCalcul.NFonction3Var)
			return getString("fonctionreelle3Var");
		if (nat == NatureCalcul.NFonctionComplexe)
			return getString("fonctioncomplexe");
		if (nat == NatureCalcul.NFonctionComplexe2Var)
			return getString("fonctioncomplexe2Var");
		if (nat == NatureCalcul.NFonctionComplexe3Var)
			return getString("fonctioncomplexe3Var");
		if (nat == NatureCalcul.NSuiteRecurrenteReelle)
			return getString("suiterecurrentereelle");
		if (nat == NatureCalcul.NSuiteRecurrenteComplexe)
			return getString("suiterecurrentecomplexe");
		// Ajout version 4.7.5
		if (nat == NatureCalcul.NSuiteRecurrenteComplexe2)
			return getString("suiterecurrentecomplexe2");
		//
		if (nat == NatureCalcul.NRepere)
			return getString("repere");
		else
			return ""; // Pour le compilateur
	}

	/**
	 * Fonction rajoute  la version java et renvoyant true si l'objet ncessite
	 * une longueur unit pour fonctionner A redfinir pour ce type d'objet :
	 * mesure de longueur, d'aire, cercle par rayon
	 * 
	 * @return
	 */
	public boolean utiliseLongueurUnite() {
		return false;
	}

	@Override
	public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list)
			throws IOException {
		super.chargeDepuisFlux(inps, list);
		if (listeProprietaire.numeroVersion <= 3) {
			estElementFinal = false;
			impProto = null;
		} else {
			estElementFinal = inps.readBoolean();
			int ind1 = inps.readCInt();
			impProto = (CImplementationProto) (list.get(ind1));
		}
	}

	@Override
	public void read(MathGraphDataInputStream inps, CListeObjets list)
			throws IOException {
		super.read(inps, list);
		estElementFinal = inps.readBoolean();
		int ind1 = inps.readInt();
		impProto = (CImplementationProto) (list.get(ind1));
	}

	@Override
	public void write(MathGraphDataOutputStream oups, CListeObjets list)
			throws IOException {
		super.write(oups, list);
		oups.writeBoolean(estElementFinal);
		int ind1;
		if (impProto == null)
			ind1 = -1;
		else
			ind1 = list.indexOf(impProto);
		oups.writeInt(ind1);
	}
	
  public static final String getString(String key) {
  	return MtgFrame.getString(key);
  }
  
  /*
   * Ajout version 4.5
   * Renvoie true si un lment est capturable  la souris.
   * Sera redfini pour les leiux d'objets gnrs par un CCommentaire ou un CLatex de faon  ce que
   * l'outil de capture permette de faire glisser un tel liue eb faisant glisser le CCoemntaire ou
   * le cLatex qui l'a gnr. 
   */
  public boolean estCapturableSouris() {
  	return true;
  }
  
  // Ajout version 4.5.2
  /**
   * Fonction retirant le composant CComponent instanci par l'objet
   * Redfini pour CEditeurFormule
   * @param cadre
   */
  public void retireComposant(MtgInternalFrame cadre) {
  	
  }
  
  // Ajout version 4.5.2
  /**
   * Si l'lement maintient un composant (CEDiteurFormule) le montre
   */
  public void montreComposant() {
  	
  }
  
  // Ajout version 4.5.2
  /**
   * Si l'lement maintient un composant (CEDiteurFormule) le cache
   */
  public void cacheComposant() {
  	
  }
  // Ajout version 4.9.9
  /**
   * Gnre un nom pour l'historique et le reclassement
   * @return
   */
  public String genereNom() {
  	return "";
  }
  
  // Ajout version 4.9.9
  /**
   * Fonction renvoyant true si l'lment a  tre visible quand on fait l'historique des objets crs.
   * Sera redfini pour CBipoint et CTransformation
   * @return
   */
  public boolean estVisibledansHistorique() {
  	return true;
  }
	
}
