/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

// Classe de transformation inversion. Contrairement aux autres
// transformations, elle ne pourra tre utilise que pour des images
// de points et pas de droites, cercles ou autres objets
// Comme elle est dfinie par son centre et son rapport, elle drive de
// la classe CHomothtie
import java.awt.geom.*;
import mathgraph32.MtgFrame;
import dialogs.MtgDlg;
import dialogs.RapportInversionDlg;
import types.*;

public class CInversion extends CHomothetie {

  /** Creates new CInversion */
  public CInversion() {
  }
  
  // Constructeur pour clonage
  public CInversion(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    CPointAncetre centreInit, CValeurAssocieeAVariable rapportInit) {
    super(liste, impProtoInit, estElementFinalInit, centreInit, rapportInit);
  }

  @Override
	public int natureTransformation() {
		return inversion;
	}
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(centre);
    int ind2 = listeSource.indexOf(impProto);        
    CValeurAssocieeAVariable rapportClone = (CValeurAssocieeAVariable) rapport.getClone(listeSource, listeCible);
    CElementBase ptelb = new CInversion(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)), rapportClone);
    return ptelb;
  }
    
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    boolean resultat = super.dependDe(p);
    if (listeProprietaire.pointeurLongueurUnite == null)
      return memoriseDependance(resultat);
    else return memoriseDependance(resultat || (listeProprietaire.pointeurLongueurUnite.dependDe(p)));
  }

  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
  	boolean resultat = super.dependDePourReclassement(p);
    if (listeProprietaire.pointeurLongueurUnite == null)
      return memoriseDependancePourRec(resultat);
    else return memoriseDependancePourRec(resultat || (listeProprietaire.pointeurLongueurUnite.dependDePourReclassement(p)));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    boolean resultat = super.dependDe(p);
    if (listeProprietaire.pointeurLongueurUnite == null)
      return resultat;
    else return resultat || (listeProprietaire.pointeurLongueurUnite.dependDePourBoucle(p));
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    CLongueur pLongUnite;
    double longueurUnite;

    super.positionne(infoRandom, dimfen);
    // A noter : valeurRapport a t calcul dans Homothetie.positionne()
    pLongUnite = listeProprietaire.pointeurLongueurUnite;
    existe = existe && pLongUnite.existe;
    if (!existe) return;
    longueurUnite = pLongUnite.rendLongueur();
    if (longueurUnite == 0)
    {
      existe = false;
    }
  }

  @Override
  public void transformePoint(double x, double y, Point2D.Double pointr) {
  // Le point image du point d'affixe z est le point d'affixe z'
  // dfini par z' - w = k(z - w)/|z - w|, mais le module de
  // z - w doit tre calcul relativement  la longueur unit
  // (w dsigant l'affixe du centre)
    CLongueur pLongUnite;
    double longueurUnite;

    pLongUnite = listeProprietaire.pointeurLongueurUnite;
    longueurUnite = pLongUnite.rendLongueur();

    VecteurDirecteur u = new VecteurDirecteur(centre.x, centre.y, x, y);;
    double longueur = u.norme()/longueurUnite;
    double d = longueur*longueur;
    pointr.x = centre.x + valeurRapport*u.x/d;
    pointr.y = centre.y + valeurRapport*u.y/d;
  }

  @Override
  public  boolean pointAUneImage(CPointAncetre point) {
  	return (!point.egal(centre));
  }
  // Ajout version 4.7.7 pour corriger oubli
  @Override
  public MtgDlg dialogueModificationImage(MtgFrame frame) {
  	return new RapportInversionDlg(frame, this, true);
  }
  
	// Ajout version 4.9.9.
	/**
	 * Fonction renvoyant complment d'info pour liste totale d'objets
	 * @return
	 */
  @Override
	String complementInfo() {
    return " " + getString("chinfo24") + " " + getString("chinfo202") + " " + centre.nom() + " " + getString("chinfo28")+ " " + rapport.chaineInfo();
	}
  
    
}
