/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.*;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

import mathgraph32.MtgFrame;

public class CLongueur extends CValeurDynamique {

	CPointAncetre point1;
	CPointAncetre point2;
	double longueur;
  
  /** Creates new CLongueur */
  public CLongueur() {
  }

  // Constructeur pour clonage
  public CLongueur(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      CPointAncetre point1Init, CPointAncetre point2Init) {
    // Inutile d'appeler le constructeur anctre vide
    super(liste, impProtoInit, estElementFinalInit);
    point1 = point1Init;
    point2 = point2Init;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(point1);
    int ind2 = listeSource.indexOf(point2);
    int ind3 = listeSource.indexOf(impProto);        
    CElementBase ptelb =  new CLongueur(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
      (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)));
    return ptelb;
  }
 
  
  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NMesureLongueur;
  }
  
  @Override
  public String infoPourHistorique() {
    String ch = getString("chinfo17") + " " + point1.nom()+ point2.nom();
    if (this == listeProprietaire.pointeurLongueurUnite) ch = ch + "\n" + getString("ch34");
    if (existe) ch = ch + "\n" + getString("chinfo97") + " " + glob.Kernel.chaineNombre(longueur, 16);
    else ch = ch + "\n" + getString("ch18");
    return ch;
  }
	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
    String ch = point1.nom()+ point2.nom();
    if (this == listeProprietaire.pointeurLongueurUnite) ch = ch + " (" + getString("ch34")+")";
    return ch;
	}

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(point1);
    liste.add(point2);
  } 
  
  @Override
  public void setClone(CElementBase ptel) {
    longueur = ((CLongueur) ptel).longueur;
  }

  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    boolean resultat = super.dependDe(p)|| point1.dependDe(p) || point2.dependDe(p);
    if ((listeProprietaire.pointeurLongueurUnite == this)
      || listeProprietaire.pointeurLongueurUnite == null)
      return memoriseDependance(resultat);
    else return memoriseDependance(resultat ||(listeProprietaire.pointeurLongueurUnite.dependDe(p)));
  }

  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    boolean resultat = super.dependDePourReclassement(p)|| point1.dependDePourReclassement(p)
    	|| point2.dependDePourReclassement(p);
    if ((listeProprietaire.pointeurLongueurUnite == this)
      || listeProprietaire.pointeurLongueurUnite == null)
      return memoriseDependancePourRec(resultat);
    else return memoriseDependancePourRec(resultat ||(listeProprietaire.pointeurLongueurUnite.dependDePourReclassement(p)));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    boolean resultat = (p == this)|| point1.dependDePourBoucle(p) || point2.dependDePourBoucle(p);
    if ((listeProprietaire.pointeurLongueurUnite == this)
      || listeProprietaire.pointeurLongueurUnite == null)
      return resultat;
    else return resultat ||(listeProprietaire.pointeurLongueurUnite.dependDePourBoucle(p));
  }
  
  @Override
  public boolean nomIndispensable(CElementGraphique el) {
    return ((el == point1) || (el == point2));
  }
  
  public double rendLongueur() {
    VecteurDirecteur u = new VecteurDirecteur(point1, point2);
    return u.norme();
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    CLongueur pLongUnite;
    double longueurUnite;
    pLongUnite = listeProprietaire.pointeurLongueurUnite;
    if (this == pLongUnite) {
      existe = point1.existe && point2.existe && (rendLongueur() != 0) ;
      longueur = 1;
      return;
    }
		existe = point1.existe && point2.existe && pLongUnite.existe;
    if (!existe) return;
    longueurUnite = pLongUnite.rendLongueur();
    if (longueurUnite == 0) {
      existe = false;
      return;
    }
    VecteurDirecteur u = new VecteurDirecteur(point1, point2);
    longueur = u.norme()/longueurUnite;    
  }

  @Override
	public
  double rendValeur() {
    return longueur;
  }

  // Ajout pour la version 2 du CNED
  @Override
  public boolean chaineCommenceParNom(String pChaine,Point longueurNom){
    // Pour une longueur AB par exemple, le nom BA est quivalent
    if (super.chaineCommenceParNom(pChaine,longueurNom)){
      return true;
    }
    else {
      String chaineNom = point2.nom+point1.nom;
      longueurNom.x = chaineNom.length();
      int longueurChaine2 = pChaine.length();
      if (longueurChaine2 < longueurNom.x) return false;
      else {
        return (chaineNom.equals(pChaine.substring(0, longueurNom.x)));
      }
    }
  }
  /**
   * Ajout version 4.8.0.3
   * Renvoie true si le nom de la valeur commence par la chane 
   * @param pChaine
   * @param longueurNom
   * @return
   */
  public boolean nomCommencePar(String st){
  	if (super.nomCommencePar(st)) return true;
    String ch = point2.nom+point1.nom;
    return ch.startsWith(st);
  }

  /**
   * Renverra true si la chaine ch est gale (sans tenir compte de la casse)
   * soit au nom de la valeur dans le cas d'un calcul ou une fonction soit 
   * la chane reprsentant la mesure dans le cas d'une mesure
   * Modifi version 3.5.0 : on diffrencie majuscules et minuscules
   * @param ch
   * @return
   */
  @Override
  public boolean chaineEgaleANom(String ch) {
    if (super.chaineEgaleANom(ch)) return true;
    String ch2 = point2.rendNom() + point1.rendNom();
    return (ch.equals(ch2));
  }
  
  @Override
  public String nom() {
    return point1.nom + point2.nom;
  }
  /** Rajout pour la version 2.0 : permet de connatre si l'lment point par pt
   * est confondu avec l'lment en question ou trs proche.
   * Pour cette version applet, ne sera redfini que pour les objets que l'utilisateur
   * pourra effectivement crer en ligne.
  **/
  @Override
  public boolean confonduAvec(CElementBase p) {
    CLongueur pt;
    if (p.getClass() == this.getClass()) {
      pt = (CLongueur) p;
      return (((point1==pt.point1) && (point2==pt.point2))
      || ((point2==pt.point1) && (point1==pt.point2)));
    }
    else return false;
  }
  
  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (point1 == ancienPoint) point1 = nouveauPoint;
    if (point2 == ancienPoint) point2 = nouveauPoint;
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		(listeOb.contains(point1) || point1.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(point2) || point2.estDefiniParObjetDans(listeOb));
	}  
  
  
  @Override
  public boolean utiliseLongueurUnite() {
  	return true;
  }
  
  /**
   * Grande amlioration par rapport au C++. C'est la longueur unit qui informe sa liste propritaire
   * qu'elle est la longueur unit de la liste, car c'est tout simplement la premire longueur renconte
   */
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    int ind2 = inps.readCInt();
    point1 = (CPointAncetre) (list.get(ind1, CPointAncetre.class));
    point2 = (CPointAncetre) (list.get(ind2, CPointAncetre.class)); 
    if (list.pointeurLongueurUnite == null) list.pointeurLongueurUnite = this;
  }
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    point1 = (CPointAncetre) (list.get(ind1, CPointAncetre.class));
    point2 = (CPointAncetre) (list.get(ind2, CPointAncetre.class));  
    if (list.pointeurLongueurUnite == null) list.pointeurLongueurUnite = this;
  }  
  /* Enregistrement dans un flux java */
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(point1);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(point2);
    oups.writeInt(ind2);
  }
}
