/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.IOException;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;
import types.*;

//Version 5.0. L'objet descend maintenant de CCalculAncetre
// public class CMesureAngleOriente extends CMesureAngleGeometrique {
public class CMesureAngleOriente extends CCalculAncetre {
	// Ajout version 5.0
	CPointAncetre point1;
	CPointAncetre point2;
	CPointAncetre point3;
	private final static String debutNom = "ang";
	//
	double mesure;
  /** Creates new CMesureAngleOriente */
  public CMesureAngleOriente() {
  }

  /**
   * Version 5.0. Lors de la cration, l'utilisateur affecte un nom  la mesure d'o un changement de version
   */
  @Override
  public int numeroVersion() {
  	return 2;
  }  

  // Constructeur pour clonage
  /* Modifi version 5.0. Un paramtre de plus pour le nom
  public CMesureAngleOriente(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      CPointAncetre point1Init, CPointAncetre point2Init, CPointAncetre point3Init) {
    super(liste, impProtoInit, estElementFinalInit, point1Init, point2Init, point3Init);
  }
  */
  public CMesureAngleOriente(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      CPointAncetre point1Init, CPointAncetre point2Init, CPointAncetre point3Init, String nomCalculInit) {
    super(liste, impProtoInit, estElementFinalInit, nomCalculInit);
    point1 = point1Init;
    point2 = point2Init;
    point3 = point3Init;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(point1);
    int ind2 = listeSource.indexOf(point2);   
    int ind3 = listeSource.indexOf(point3);  
    int ind4 = listeSource.indexOf(impProto);        
    // Si le nom est vide, c'est que la liste source est un CPrototype. On gnre alors un nom
    String nomCalc = (nomCalcul.isEmpty()) ? listeCible.genereNomPourCalcul(debutNom, true) : nomCalcul;
    CElementBase ptelb = new CMesureAngleOriente(listeCible, (CImplementationProto)(listeCible.get(ind4, CImplementationProto.class)),
      estElementFinal, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
      (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)),
      (CPointAncetre)(listeCible.get(ind3, CPointAncetre.class)), nomCalc);
    return ptelb;    
  }
  
  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NMesureAngleOriente;
  }
  ///////////////////////////////////////////////////////////////////
  // Ajouts version 5.0 car ne drive plus de CMesureAngleGeometrique
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || point1.dependDe(p) || point2.dependDe(p) || point3.dependDe(p));
  }

  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || point1.dependDePourReclassement(p)
    	|| point2.dependDePourReclassement(p) || point3.dependDePourReclassement(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || point1.dependDePourBoucle(p) || point2.dependDePourBoucle(p)
      || point3.dependDePourBoucle(p);
  }
///////////////////////////////////////////////////////////////////
  
  @Override
  public String infoPourHistorique() {
    String ch = info();
    if (existe) ch = ch + "\n" + getString("chinfo97") + " " + Kernel.chaineNombre(mesure, 16);
    else ch = ch + "\n" + getString("ch18");
    return ch;        
  }
 	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
  // Modifi vesion 5.0
	public String info() {
    return nom() + " : " + getString("chinfo30") + " " + "(" + point2.nom + point1.nom + "," + point2.nom + point3.nom + ')';
	}
  
  // Ajout version 5.0
  @Override
  public double rendValeur() {
    return mesure;
  }
  //
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    double prosca,deter,tangente,resul;
    
    existe = point1.existe && point2.existe && point3.existe
			&& !(point2.egal(point1)) && !(point2.egal(point3));
    if (!existe) return;
    // L'orientation sur l'cran est l'inverse de celle du repre
		// choisi.Il faut donc permuter les rles de U et V
    VecteurDirecteur v = new VecteurDirecteur(point2,point1);
    VecteurDirecteur u = new VecteurDirecteur(point2,point3);
    prosca = u.x*v.x + u.y*v.y;
    deter = u.x*v.y - u.y*v.x;
    if (prosca == 0)
		{
			if (deter > 0)  resul = Math.PI/2;
			else resul = - Math.PI/2;
		}
    else
    {
      tangente = deter/prosca;
      if (prosca > 0) resul = Math.atan(tangente);
      else
      {
        if (deter >= 0) resul = Math.atan(tangente) + Math.PI;
        else resul = Math.atan(tangente) - Math.PI;
      }
    }
    // Nouveau pour version 1.3.6 : on regarde si l'angle est trs proche de - pi. Si oui on renvoie pi
    if (Math.abs(resul + Math.PI)< 1e-13)resul = Math.PI;
    if (listeProprietaire.uniteAngle == UniteAngle.Radian) mesure = resul;
    // Modifi version 5.1.1
    // else mesure = resul * Kernel.ConvRadDeg;
    else mesure = Math.toDegrees(resul);
	}
  /* Supprim version 5.0. La mesure d'abscisse a maintenant un nom
  @Override
  public String nom() {
    return "(" + point2.nom + point1.nom + "," + point2.nom + point3.nom + ')';
  }
  */
  /** Rajout pour la version 2.0 : permet de connatre si l'lment point par pt
   * est confondu avec l'lment en question ou trs proche.
   * Pour cette version applet, ne sera redfini que pour les objets que l'utilisateur
   * pourra effectivement crer en ligne.
  **/
  public boolean confonduAvec(CElementBase p) {
    CMesureAngleOriente pt;
    if (p.getClass() == this.getClass()) {
      pt = (CMesureAngleOriente) p;
      return ((point2 == pt.point2) && 
			(point1 == pt.point1) && (point3 == pt.point3));
    }
    else return false;
  }
 
  // Ajout version 5.0.5 car n'est plus hrit de l'anctre
  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (point1 == ancienPoint) point1 = nouveauPoint;
    if (point2 == ancienPoint) point2 = nouveauPoint;
    if (point3 == ancienPoint) point3 = nouveauPoint;
  }

  // Ajout version 5.0 car n'est plus hrit
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		(listeOb.contains(point1) || point1.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(point2) || point2.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(point3) || point3.estDefiniParObjetDans(listeOb));
	}  

  // Ajout version 5.0
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
     super.chargeDepuisFlux(inps, list);
     int ind1 = inps.readCInt();
     int ind2 = inps.readCInt();
     int ind3 = inps.readCInt();
     point1 = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
     point2 = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
     point3 = (CPointAncetre) (list.get(ind3,CPointAncetre.class));
   	 if (listeProprietaire.getClass() == CPrototype.class) nomCalcul = "";
   	 else nomCalcul = listeProprietaire.genereNomPourCalcul("ang", true);
  }

  // Ajouts version 5.0
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    if (nVersion>=2) super.read(inps, list);
    else {
    	super.readSansNom(inps, list);
    	// Attention : Si la liste propritaire est un CPrototype, on ne peut pas lui demander  ce
    	// niveau de gnrer un nom.
    	// Ce sera fait dans getclone() en testant si le nom est vide.
    	if (listeProprietaire.getClass() == CPrototype.class) nomCalcul = "";
    	else nomCalcul = listeProprietaire.genereNomPourCalcul("ang", true);
    }
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    int ind3 = inps.readInt();
    point1 = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    point2 = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
    point3 = (CPointAncetre) (list.get(ind3,CPointAncetre.class));
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(point1);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(point2);
    oups.writeInt(ind2);
    int ind3 = list.indexOf(point3);
    oups.writeInt(ind3);
  }  
}
