/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package objets;

import java.io.IOException;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import types.Complexe;
import types.ErreurCalculException;
import types.Operateur;

public class CMoinsUnaire extends CCalculBase {
  CCalculBase operande;
  Complexe c1 = new Complexe();
  
	public CMoinsUnaire() {
	}
	
  public CMoinsUnaire(CListeObjets liste, CCalculBase operandeInit) {
    super(liste);
    operande = operandeInit;
	}

  @Override
  protected long nature() {
  	return natMoinsUnaire;
  }  
  
	public CCalculBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
	  CCalculBase cloneOperande = operande.getClone(listeSource, listeCible); 
	  CCalculBase ptcalc = new CMoinsUnaire(listeCible, cloneOperande);
	  return ptcalc;
	}     

  // Ajout version 4.8
  @Override
  public void initialisePourDependance() {
  	super.initialisePourDependance();
  	operande.initialisePourDependance();
  }

	// Modifi version 4.8
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
  	// elementTestePourDependDe = p;
    return memoriseDependance(super.dependDe(p) || operande.dependDe(p));
  }

  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
  	// elementTestePourDependDePourRec = p;
    return memoriseDependancePouRec(super.dependDePourReclassement(p) || operande.dependDePourReclassement(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
  	return operande.dependDePourBoucle(p);
  }

  @Override
	public boolean existe(boolean infoRandom) {
  	return operande.existe(infoRandom);
  }

  @Override
  public double resultat(boolean infoRandom) throws ErreurCalculException {
     return - operande.resultat(infoRandom);
  }

  @Override
  public double resultatFonction(boolean infoRandom, double valeurParametre) throws ErreurCalculException {
     return - operande.resultatFonction(infoRandom, valeurParametre);
  }
  
  @Override
  public double resultatFonction(boolean infoRandom, double[] valeurParametre) throws ErreurCalculException {
     return - operande.resultatFonction(infoRandom, valeurParametre);
  }

  @Override
  public void resultatComplexe(boolean infoRandom, Complexe zRes)  throws ErreurCalculException {
    operande.resultatComplexe(infoRandom, c1);
    zRes.x = -c1.x;
    zRes.y = -c1.y;
  }
  
  @Override
  void  resultatFonctionComplexe(boolean infoRandom, Complexe valeurParametre, Complexe zRes)
  throws ErreurCalculException
  {
    operande.resultatFonctionComplexe(infoRandom, valeurParametre, c1);
    zRes.x = -c1.x;
    zRes.y = -c1.y;
  }

  @Override
  void  resultatFonctionComplexe(boolean infoRandom, Complexe[] valeurParametre, Complexe zRes)
  throws ErreurCalculException
  {
    operande.resultatFonctionComplexe(infoRandom, valeurParametre, c1);
    zRes.x = -c1.x;
    zRes.y = -c1.y;
  }

  @Override
  public boolean  dependDeVariable(int ind) {
    return operande.dependDeVariable(ind);
  }
  
  @Override
  public CCalculBase getCopie() {
   	return new CMoinsUnaire(listeProprietaire, operande.getCopie());
  }

  @Override
	public String chaineCalcul(String[] varFor) {
  	String ch;
		if (operande.nature() == natOperation) {
			int op1 = ((COperation) operande).operateur;
			if ((op1 == Operateur.Plus) || (op1 == Operateur.Moin) || Operateur.estTest(op1))
				ch = operande.chaineCalcul(varFor);
			else ch = operande.chaineCalculSansPar(varFor); 
		}
		else ch = operande.chaineCalcul(varFor);
		return parouv + "-" + ch + parfer;
  }
  
  @Override
	public String chaineCalculSansPar(String[] varFor) {
  	String ch;
		if (operande.nature() == natOperation) {
			int op1 = ((COperation) operande).operateur;
			if ((op1 == Operateur.Plus) || (op1 == Operateur.Moin) || Operateur.estTest(op1))
				ch = operande.chaineCalcul(varFor);
			else ch = operande.chaineCalculSansPar(varFor); 
		}
		else ch = operande.chaineCalculSansPar(varFor);
		return "-" + ch;
  }
  
	public String chaineLatex(String[] varFor) {
  	String ch;
		if (operande.nature() == natOperation) {
			int op1 = ((COperation) operande).operateur;
			if ((op1 == Operateur.Mult) || (op1 == Operateur.Divi) || Operateur.estTest(op1))
				ch = operande.chaineLatexSansPar(varFor);
			else ch = operande.chaineLatex(varFor); 
		}
		else ch = operande.chaineLatex(varFor);
		return parouvLatex + "-" + ch + parferLatex;
	}  

	public String chaineLatexSansPar(String[] varFor) {
  	String ch;
		if (operande.nature() == natOperation) {
			int op1 = ((COperation) operande).operateur;
			if ((op1 == Operateur.Mult) || (op1 == Operateur.Divi) || Operateur.estTest(op1))
				ch = operande.chaineLatexSansPar(varFor);
			else ch = operande.chaineLatex(varFor); 
		}
		else ch = operande.chaineLatexSansPar(varFor);
		return "-" + ch;
	}  
	
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return operande.estDefiniParObjetDans(listeOb);
	}	   
    
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    operande = (CCalculBase) (inps.readObject(list));
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeObject(operande);
  }
  
  @Override
	public boolean estConstant() {
		return operande.estConstant();
	}
  
  @Override
  public boolean deriveePossible(int indiceVariable) {
  	return operande.deriveePossible(indiceVariable);
  }

  // Ajout version 4.6
  @Override
  public CCalculBase calculAvecValeursRemplacees() {
  	return new CMoinsUnaire(listeProprietaire, operande.calculAvecValeursRemplacees()) ;
  }
  
  // Ajout version 4.9.6 pour que le membre gauche de -(a/b) soit considr comme (-a)/b idem pour -(ab)
  @Override
	public CCalculBase membreGauche() {
  	if (operande.nature() == natOperation) {
  		COperation op = (COperation)operande;
  		if ((op.operateur == Operateur.Mult) || (op.operateur == Operateur.Divi)) return new CMoinsUnaire(listeProprietaire,op.operande1);
  	}
  	return super.membreGauche();
  }
  // Ajout version 4.9.6 pour que le membre droit de -(a/b) soit considr comme (-a)/b idem pour -(a*b)
  @Override
	public CCalculBase membreDroit() {
  	if (operande.nature() == natOperation) {
  		COperation op = (COperation)operande;
  		if ((op.operateur == Operateur.Mult) || (op.operateur == Operateur.Divi)) return op.operande2;
  	}
  	return super.membreDroit();
  }
  
}
