/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

// Attention java :dans la version C++ CPointLiePoint drivait de CPointLie mais n'appelait

import java.awt.Color;
import java.awt.geom.Point2D;
import java.io.IOException;
import mathgraph32.MtgFrame;
import types.DimensionFenetre;
import types.Doub;
import types.NatureObjet;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

// pas sa srialization. C'est impossible en java donc il drive en java de CPointAncetre

public class CPointLiePoint extends CPointAncetre {
	CPointAncetre pointAssocie;

  /** Creates new CPointLiePoint */
  public CPointLiePoint() {
  }
  
  // Pour satisfaire lengthcompilateur
  public boolean testDeplacement(DimensionFenetre dimfen, double xtest, double ytest,
        Point2D.Double pointr, Doub abscr) {
    return true;
  }
  

  // Constructeur pour clonage
  public CPointLiePoint(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
        Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit, boolean masqueInit, String nomInit, int indiceTailleNomInit,
        int motifInit, CPointAncetre pointAssocieInit, boolean marquePourTraceInit) {
     super (liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
       dec_xInit, dec_yInit, masqueInit, nomInit, indiceTailleNomInit, motifInit, marquePourTraceInit);
     pointAssocie = pointAssocieInit;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(pointAssocie);
    int ind2 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CPointLiePoint(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom, indiceTailleNom,
      motif, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)), marquePourTrace);
    return ptelb;
  }
  
 	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
  	return nom() + " : " + getString("chinfo2") + " " + pointAssocie.nom();
	}
  
  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(pointAssocie);
  }
  

  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || pointAssocie.dependDe(p));
  }

  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || pointAssocie.dependDePourReclassement(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || pointAssocie.dependDePourBoucle(p));
  }


  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = pointAssocie.existe;
    if (existe)  {
    	super.placeEn(pointAssocie.x, pointAssocie.y);
    	super.positionne(infoRandom, dimfen); // Ajout version 4.6.4
    }
  }

  CElementBase lieA() {
    return pointAssocie;
  }
  
  @Override
  public long getNature() {
    return NatureObjet.NPointLiePoint;
  }
  
  @Override
  public boolean confonduAvec(CElementBase p) {
    CPointLiePoint pt;
    if (p.getClass() == this.getClass()) {
      pt = (CPointLiePoint) p;
      return (pointAssocie == pt.pointAssocie);
    }
    else return false;
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return listeOb.contains(pointAssocie) || pointAssocie.estDefiniParObjetDans(listeOb);
	}  	  
  
  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (pointAssocie == ancienPoint) pointAssocie = nouveauPoint;
  }
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    pointAssocie = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
  }
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    pointAssocie = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(pointAssocie);
    oups.writeInt(ind1);
  }
}
