/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.IOException;


import types.UniteAngle;
import entreesSorties.MathGraphDataInputStream;
import glob.Kernel;

public class CValeurAngle extends CValeurAssocieeAVariableAncetre {
  

  /** Creates new CValeurAngle */
  public CValeurAngle() {
  }
  
  /** Constructeur  partir d'un autre objet du mme type */
  public CValeurAngle(CValeurAngle valInit) {
  	super(valInit.listeProprietaire);
    calcul = valInit.calcul.getClone(listeProprietaire, listeProprietaire);
  }
  
  
  // Constructeur pour clonage
  public CValeurAngle(CListeObjets liste, CCalculBase calculInit, double valeurInit) {
    super (liste, calculInit);
    valeur = valeurInit;
  }
  
  /**
   * Ajout pour la version 3.0.
   * Cre une valeur d'angle dont la valeur est valeurInit en radians
   * chaineCalcul doit tre cohrente avec valeurInit
   * @param list
   * @param valeurInit
   * @param chaineCalcul
   */
  public CValeurAngle(CListeObjets list, double valeurInit, String chaineCalcul) {
  	super(list);
  	valeur = valeurInit;
  	calcul = CCalculBase.creeCalculBase(chaineCalcul, list, 0, chaineCalcul.length() - 1, null);
  	estConstant = true;
  	existe = true;
  }
  
  /**
   * Constructeur pour les objest visuels
   * @param list
   * @param valeurInit
   */
  public CValeurAngle(CListeObjets list, double valeurInit) {
  	super(list);
  	valeur = valeurInit;
  	calcul = new CConstante(list, valeurInit);
  	estConstant = true;
  	existe = true;
  }
  
  
  /**
   * Ajour pour la version 3.0
   * Cration  partor d'une valeur dynamique existante.
   * Utilis pour les objets visuels
   * @param list
   * @param valdyn
   */
  public CValeurAngle(CListeObjets list, CValeurDynamique valdyn) {
  	super(list);
  	calcul = new CResultatValeur(list, valdyn);
  	estConstant = false;
  }
  
  // Constructeur pour clonage
  CValeurAngle getClone(CListeObjets listeSource, CListeObjets listeCible) {
    CCalculBase calculClone = calcul.getClone(listeSource, listeCible);
    return new CValeurAngle(listeCible, calculClone, valeur);
  }
    
  public double rendValeurRadian() {
  	double val = valeur;
  	// Modifi version 5.1.1
    // if (listeProprietaire.uniteAngle == UniteAngle.Degre) val = val*Kernel.ConvDegRad;
  	if (listeProprietaire.uniteAngle == UniteAngle.Degre) val = Math.toRadians(val);
  	val = Kernel.mesurePrincipale(val);
  	return val;
  }
  
  /**
   * Il faut redfinir chargeDepuisFlux car les angles taient en C++ traits d'une faon diffrente
   */
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
  	listeProprietaire = list;
  	valeur = inps.readCDouble();
  	int ind1 = inps.readCInt();
  	if (ind1 == -1) {
  		// Si dans la version C++ la valeur tait constante, on doit crer un CCalculBase renvoyant cette constante
  		// Si l'unit d'angle tait le radian pas de changement
  		// Trs important : Si on ne charge un prototype, on ne peut pas appelr CreeCalculBase car cela peut faire appel
  		//  desnomes d'lments gnriques. par exempe un CMesureX fera appel aux noms des trois points dfinissant le
  		// Or ces noms sont  null dans le cas d'un lment gnrique.
  		if (list.getClass() == CPrototype.class) {
  			if (list.uniteAngle == UniteAngle.Radian) {
  				calcul = new CConstante(list, valeur);
  			}
  			else {
  				// Modifi version 5.1.1
  				// calcul = new CConstante(list, valeur*Kernel.ConvRadDeg);
  				calcul = new CConstante(list, Math.toDegrees(valeur));
  				valeur *= Kernel.ConvRadDeg;
  			}
  		}
  		else {
  			if (list.uniteAngle == UniteAngle.Radian) {
  				String ch = "";
  				// Si l'angle tait un angle remarquable choisi en degr on  cre un calcul de fraction de pi
  				if (Kernel.zero(valeur-Math.PI/3)) ch = "pi/3";
  				else if(Kernel.zero(valeur+Math.PI/3)) ch = "-pi/3";
  				else if(Kernel.zero(valeur-Math.PI/6)) ch = "pi/6";
  				else if(Kernel.zero(valeur+Math.PI/6)) ch = "-pi/6";
  				else if(Kernel.zero(valeur-Math.PI/4)) ch = "pi/4";
  				else if(Kernel.zero(valeur+Math.PI/4)) ch = "-pi/4";
  				else if(Kernel.zero(valeur-Math.PI/2)) ch = "pi/2";
  				else if(Kernel.zero(valeur+Math.PI/2)) ch = "-pi/2";
  				else if(Kernel.zero(valeur-Math.PI)) ch = "pi";
  				else if(Kernel.zero(valeur+Math.PI)) ch = "-pi";
  				if (ch.length() != 0) {
  					calcul = CCalculBase.creeCalculBase(ch, list, 0, ch.length() - 1, null);								
  				}
  				else calcul = new CConstante(list, valeur);
  			}
  			else {
  				// Sinon comme il faut convertir l'angle en degrs
  				// Modifi version 5.1.1
  				// calcul = new CConstante(list, valeur*Kernel.ConvRadDeg);
  				calcul = new CConstante(list, Math.toDegrees(valeur));
  				valeur *= Kernel.ConvRadDeg;
  			}
				estConstant = true;
				existe = true;  			
  		}
  	}
  	else {
  		// On cre un calcul renvoyant la valeur dynamique associe en C++
  		CValeurDynamique valeurDynamiqueAssociee = (CValeurDynamique) (list.get(ind1,CValeurDynamique.class));
  		calcul = new CResultatValeur(list, valeurDynamiqueAssociee);
  		estConstant = false;
  	}
  }
  
}
