/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import glob.Kernel;

import java.awt.Point;

import mathgraph32.MtgFrame;
import objets.CAffichageLieAPoint;
import objets.CArcDeCercle;
import objets.CCercleOA;
import objets.CDroiteDirectionFixe;
import objets.CElementGraphique;
import objets.CHomothetie;
import objets.CListeObjets;
import objets.CMathGrafDoc;
import objets.CMesureAngleGeometrique;
import objets.CMesureAngleOriente;
import objets.CPointAncetre;
import objets.CPointImage;
import objets.CPointLieCercle;
import objets.CPointLieDroite;
import objets.CSegment;
import objets.CValeurAffichee;
import objets.CValeurAngle;
import types.Couleur;
import types.MotifPoint;
import types.StyleEncadrement;
import types.StyleTrait;
import types.UniteAngle;

public class OutilArcPetitParRapporteur extends OutilPar2pts {
	CMesureAngleOriente ptMesureAngle;	// Pointeur
	// sur la mesure d'angle contenue dans la liste des objets visuels.
	// La valeur de cette mesure arrondie au degr prs sera utilise comme
	// angle de la rotation.
	CCercleOA ptCercle; // Pointeur sur le cercle cr dans la liste visuelle
	// auquel sera li un point.
	CPointLieCercle ptPointLie; // Le point li
  
  public OutilArcPetitParRapporteur(MtgFrame frameInit) {
    super(frameInit);
  }
	/**
	 * Ajout versions 5.0.
	 * Renverra true si on peut crer un point quand on clique sur un endroit vide de la figure.
	 * Devra tre redfini pour les outils pour lesquels ce n'est pas possible comme les outils de mesure.
	 * @return
	 */
	protected boolean creationPointPossible() {
		return true;
	}

  @Override
  public void select() {
    super.select();
    frame.indication(getString("arcPetitParPointFinal1"));
  }
  
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    if (point1 == null)
    {
      frame.indication(getString("arcPetitParPointFinal2"));
    	// Ajout version 5.0
    	if (elg == null) {
    		point1 = frame.ajoutePoint(pointClique.getX(), pointClique.getY(), false);
    	}
    	//
    	else point1 = (CPointAncetre) elg;
    	/* Supprim verion 5.0
      excluDeDesignation(elg);
      ajouteClignotementDe(elg);
      */
    	// Ajout version 5.0
      excluDeDesignation(point1);
      ajouteClignotementDe(point1);
      //
      resetClignotement();
      ajouteObjetsVisuels(); // On rajoute un rapporteur
      frame.listeObjetsVisuels.positionne(false, frame.cadreActif().dimf);
    }
    else
    {
      if (point2 == null)
      {
        frame.indication(getString("rapporteur3"));
      	// Ajout version 5.0
      	if (elg == null) {
      		point2 = frame.ajoutePoint(pointClique.getX(), pointClique.getY(), false);
      	}
      	//
      	else point2 = (CPointAncetre) elg;
      	/* Supprim version 5.0
        excluDeDesignation(elg);
        ajouteClignotementDe(elg);
        */
      	// Ajout version 5.0
        excluDeDesignation(point2);
        ajouteClignotementDe(point2);
        //
        ajouteObjetsVisuels();
        frame.positionneListeVisuelle();      
        resetClignotement();
      }
      // C'est l'outil de capture qui par un clic gauche une fois
      // l'angle visuellement choisi qui provoquera l'appel de CreeObjet
    }    
  }
  
  @Override
  public void creeObjet() {
    CListeObjets liste = frame.cadreActif().listePr;

    double valeurAngle = ptMesureAngle.rendValeur();
    // On arrondit au degr prs en degrs
    if (valeurAngle != Math.floor(valeurAngle)) valeurAngle = Math.floor(valeurAngle + 0.5);
    if (valeurAngle == - 180) valeurAngle = 180;
    CValeurAngle ptValeurAngle = new CValeurAngle(liste,  valeurAngle);
    CArcDeCercle ptArc = new CArcDeCercle(frame.cadreActif().listePr, null, false, 
        frame.couleurActive(), false, frame.styleTraitActif(), point1, point2, null, ptValeurAngle);
    // A la diffrence de la version C++ on ne regarde pas si existe dj une
    // rotation quivalente
    if (frame.ajouteElementAvecVerification(ptArc))
      frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil28"));
    else {
      frame.detruitDernierElement();
      frame.cadreActif().repaintFigureAvecCalcul();
    }
    annuleClignotement();
    select();
  }
  
  @Override
  public void ajouteObjetsVisuels() {
    CPointAncetre ptp = null;
    CListeObjets listeVisuelle = frame.listeObjetsVisuels;
    if (point2 == null)
    {
      ptp = ajouteRapporteur(point1); // Le pointeur renvoy dans ptp est
      // un pointeur sur un point d'intersection d'une demi-droite avec un cercle.
      ptCercle = new CCercleOA(listeVisuelle, null, false, Couleur.TraduitCouleur(1),
        false,StyleTrait.traitFin,point1,ptp);
      ptCercle.cache();
      frame.ajouteObjetVisuel(ptCercle);

    }
    else
    {
      // Si le point dont on veut crer l'image a t dsign, le cercle prcdent
      // devient fixe en remplaant dans la liste visuelle le point souris
      // par Point2
      listeVisuelle.remplacePoint(frame.cadreActif().paneFigure.pointSouris,point2);
      // Pour pouvoir initialiser l'abscisse du point li, il faut mesurer
      // un angle orient  partir d'une horizontale
      CDroiteDirectionFixe ptdh = new CDroiteDirectionFixe(listeVisuelle,
        Couleur.TraduitCouleur(1), StyleTrait.traitFin, point1, true, 2);
      ptdh.cache();
      frame.ajouteObjetVisuel(ptdh);
      // On lui lie un point d'abscisse 1
      CPointLieDroite ptld = new CPointLieDroite(listeVisuelle, null, false, Couleur.TraduitCouleur(1), false, 0d, 0d, false, "", 13,
        MotifPoint.PetitRond, true, 1, ptdh, false);       
      ptld.cache();
      frame.ajouteObjetVisuel(ptld);
      // On mesure l'angle
      CMesureAngleOriente ptmao = new CMesureAngleOriente(listeVisuelle, null, false,
        ptld, point1, point2, ""); // Modifi version 5.0
      frame.ajouteObjetVisuel(ptmao);
      frame.positionneListeVisuelle();
      // On rajoute un point li au cercle point par ptCercle
      // Son abscisse est son angle polaire en radians
      double ang = ptmao.rendValeur()*Kernel.ConvDegRad;
      if (ang < 0) ang = ang + 2*Math.PI;
      ptPointLie = new CPointLieCercle(listeVisuelle, null, false, Couleur.TraduitCouleur(1), false, 0d, 0, false, "", 13,
        MotifPoint.PetitRond, true, ang, ptCercle, false);
      frame.ajouteObjetVisuel(ptPointLie);
      // On cre la mesure de l'angle orient point2, point1, pointSouris
      ptMesureAngle = new CMesureAngleOriente(listeVisuelle, null, false, point2, point1, ptPointLie, ""); // Modifi version 5.0
      frame.ajouteObjetVisuel(ptMesureAngle);
      // On cre aussi la mesure de l'angle gomtrique pour affichage sur le rapporteur
      CMesureAngleGeometrique ptmesag = new CMesureAngleGeometrique(listeVisuelle, null, false,
        point2,point1,ptPointLie);
      frame.ajouteObjetVisuel(ptmesag);
      // Changement par rapport  la version C++.
      // Il faudra veiller  ce que le rapporteur ne soit accessible qu'en mode degr,
      // ce qui est logique.
      // On cre une nouveau point cart du rapporteur
      // Pour cela on cre une homothtie
      CHomothetie ptHom2 = new CHomothetie(listeVisuelle, null, false, point1, 1.25);
      frame.ajouteObjetVisuel(ptHom2);
      CPointImage ptimh = new CPointImage(listeVisuelle, null, false, Couleur.TraduitCouleur(1),MotifPoint.Rond, 13, ptPointLie,ptHom2);
      ptimh.cache();
      frame.ajouteObjetVisuel(ptimh);
      // On cre l'image provisoire du point
      // D'abord une rotation
      CValeurAngle valang = new CValeurAngle(listeVisuelle, ptMesureAngle);
      CMathGrafDoc doc = frame.cadreActif().document;
      // On affiche la valeur lie au point prcdent avce effacement de ce qu'il y a dessous
      CValeurAffichee ptaff = new CValeurAffichee(listeVisuelle, null, false,
        Couleur.TraduitCouleur(2),0d, 0d, 0d, 0d, false, ptimh,
        MtgFrame.screenIsHighResolution ? 26 : 13, StyleEncadrement.Sans, true, doc.couleurFond, ptmesag, "", "", 0,
				CAffichageLieAPoint.alignementHorizontalCentre, CAffichageLieAPoint.alignementVerticalCentre);
      frame.ajouteObjetVisuel(ptaff);
      // L'arc de cercle
      CArcDeCercle ptArc = new CArcDeCercle(frame.cadreActif().listePr, null, false, 
          frame.couleurActive(), false, frame.styleTraitActif(), point1, point2, null, valang);
      frame.ajouteObjetVisuel(ptArc);
      // On cre deux segments pointills (on ne sait pas si on est  l'intrieur
      // ou l'extrieur du rapporteur
      CSegment ptseg1 = new CSegment(listeVisuelle, null, false, point1, ptPointLie); // En pointills par dfaut
      frame.ajouteObjetVisuel(ptseg1);
      // On ne trace plus le deuxime segment pour ne pas cacher l'angle
      // CSegment ptseg2 = new CSegment(listeVisuelle, null, false, point1, ptimrot); // Idem
      // frame.ajouteObjetVisuel(ptseg2);
      // On passe la main  l'outil de dsignation ad hoc
      frame.outilPointageActif = frame.outilPointageRapporteur;
      frame.outilPointageActif.reset();
      frame.outilPointageRapporteur.associePointLie(ptPointLie);
    }
  }
  
  @Override
  public boolean activationValide() {
    return ((frame.cadreActif().listePr.uniteAngle != UniteAngle.Radian)
    		&& frame.pref_AideVisuelle);
  }
  
	@Override
	public String messageActivationNonValide() {
		String ch = "";
		if (!frame.pref_AideVisuelle) ch = getString("ch182");
		if (frame.cadreActif().listePr.uniteAngle == UniteAngle.Radian) ch = ch + "\n" + getString("messageDegreIndispensable");
		return ch;
	}	
	}