/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Point;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CAffichageLieAPoint;
import objets.CEditeurFormule;
import objets.CElementGraphique;
import objets.CListeObjets;
import objets.CPointAncetre;
import types.NatureObjet;

public class OutilCreationLiaisonAffichage extends Outil {
	private CAffichageLieAPoint aff;
	private CPointAncetre point;
  
	public OutilCreationLiaisonAffichage(MtgFrame frameInit) {
		super(frameInit, true);
	}

  @Override
	public void select() {
		super.select();
		aff = null;
		point = null;
		CListeObjets liste = frame.cadreActif().listePr;  
		frame.listeClignotante.ajouteAffichagesNonLiesAPointVisibles(liste);
		frame.outilPointageActif = frame.outilPointageObjetClignotant;
		frame.outilPointageActif.designationDe(NatureObjet.NAffichageLieAPoint);
		frame.indication(getString("creationLiaisonAffichage1"));
		resetClignotement();		
	}
	
  @Override
	public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    if (aff == null) {
      aff = (CAffichageLieAPoint) elg;
      annuleClignotement();
      frame.listeClignotante.add(aff);
      resetClignotement();
      frame.outilPointageActif = frame.outilPointageCre;
      frame.outilPointageActif.designationDe(NatureObjet.NToutPoint);
      frame.outilPointageActif.reset();
  		frame.indication(getString("creationLiaisonAffichage2"));
    }
    else {
    	if (point == null) {
        point = (CPointAncetre)elg;
    		annuleClignotement();
    		creeObjet();
    	}
    }
	}
	
  // Modifi version 4.8
  @Override
  public void creeObjet() {
		MtgInternalFrame cadre = frame.cadreActif();
    CListeObjets liste = cadre.listePr; 
  	aff.pointLie = point;
		int indicePoint = liste.indexOf(point);
		int indiceAff;
		// Si l'objet auquel on veut lier l'affichage est un lment final de construction
		// il faut reclasser aprs le dernier objet final de la construction
		if (aff.estElementFinal) indiceAff = liste.indexOf(aff.impProto.dernierObjetFinal());
		else indiceAff = liste.indexOf(aff);
		
		if (indiceAff < indicePoint){
			liste.decaleDependants(indiceAff, indicePoint);
		}
    // Ajout version 4.8.0 : IL faut remettre  jour les index de chaque lment de la liste (CElementBase.index)
    liste.updateIndexes();
    //
    cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil97"));
    // Version 5.0 : Correction d'un bug qui faisait que quand on liait un diteur de formule  un point
    // l'diteur lui-mme ne bougeait pas
    cadre.repaintFigureAvecCalcul();
    if (aff.getNature() == NatureObjet.NEditeurFormule) ((CEditeurFormule)aff).dejaAffiche = false;
    frame.activeOutilCapture();		
    cadre.videListesPourConstructions();  // On remet  jour une construction ventuellement commence
    // liste.initialiseDependances(); // Ajout version 4.8 Supprim version 4.9.2. Fait par GestionnaireEdition.sauvegardeFigureEnCours()
  }
  
  @Override
  public boolean activationValide() {
  	CListeObjets liste = frame.cadreActif().listePr;
    return liste.nombreAffichagesLiesAPointVisibles(false) != 0;
  }
}
