/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package outils;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Point;

import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CAffichageLieAPoint;
import objets.CCommentaire;
import objets.CElementBase;
import objets.CElementGenerique;
import objets.CElementGraphique;
import objets.CListeObjets;
import objets.CMacroConstructionRecursive;
import objets.CMathGrafDoc;
import objets.CPrototype;
import objets.CSousListeObjets;
import types.NatureCalcul;
import types.StyleEncadrement;
import types.StyleTrait;
import dialogs.ChoixObjetsNonGraphiquesPourImplementationProtoDlg;
import dialogs.MacroConstructionRecursiveDlg;

public class OutilMacroConstructionRecursive extends Outil {
	int nombreObjetsCalculADetruire; // il faut que, dans le cas o une bote de dialogue
	// de choix d'objets sources graphiques a t utilise et des calculs y ont t crs, qu'ils soient dtruits dans deselect()
	// lorsque la construction ne va pas  son terme.
	int indiceElementSourceGraphique;
	CMacroConstructionRecursive macro;
	private boolean dialogueRempli;
	private CPrototype proto;
	CSousListeObjets sousListe;
	
  public OutilMacroConstructionRecursive(MtgFrame frameInit) {
    super(frameInit, true);
  }
  
  @Override
	public Cursor curseurUtilise() {
		if (dialogueRempli) return MtgFrame.crossHairCursor;
		else return MtgFrame.cornerCursor;
	}
  
	@Override
	public void deselect() {
		super.deselect();
		if (nombreObjetsCalculADetruire != 0) 
			frame.detruitNDerniersElements(nombreObjetsCalculADetruire);
	}

  @Override
  public void select() {
    super.select();
    nombreObjetsCalculADetruire = 0;
    frame.outilPointageActif = frame.outilPointageClic;
    frame.indication(getString("macro1"));
    dialogueRempli = false;
  }
  
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    int x = pointClique.x;
    int y = pointClique.y;
    MtgInternalFrame cadre = frame.cadreActif();
    CListeObjets liste = cadre.listePr;
    // Avant l'affichage du dialogue on cre un message qui sera ajout  la liste des objets clignotants
    // pour matrialiser l'emplacement de l'affichage
    if (!dialogueRempli) {
    	sousListe = new CSousListeObjets();
    	/* Modifi version 5.0
	    CCommentaire comClignotant = new CCommentaire(liste, null, false, frame.couleurActive(), x, y, 0d, 0d, false, null, 2,
	    		StyleEncadrement.Sans, false, frame.cadreActif().couleurFond, getString("ch80"),
					CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut);
			*/
	    CCommentaire comClignotant = new CCommentaire(liste, null, false, frame.couleurActive(), x, y, 0d, 0d, false, null, (int)(MtgFrame.coefMultForSystemFonts*13),
	    		StyleEncadrement.Sans, false, frame.cadreActif().couleurFond, getString("ch80"),
					CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut);
	    comClignotant.positionne(false, frame.cadreActif().dimf);
	    ajouteClignotementDe(comClignotant);
	    resetClignotement();
	    /* Modifi version 5.0
	    macro = new CMacroConstructionRecursive(liste, null, false, frame.couleurActive(), 
	    		x, y, 0d, 0d, false, null, 2, StyleEncadrement.Effet3D, false, cadre.couleurFondAffichage, "", "", null,
	    		CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut, 1, null, 1, 0, 0, false, false, Color.black,
	    	  0, StyleTrait.traitFin, Color.black, false);
	    */
	    macro = new CMacroConstructionRecursive(liste, null, false, frame.couleurActive(), 
	    		x, y, 0d, 0d, false, null, MtgFrame.screenIsHighResolution ? 26 : 13, StyleEncadrement.Effet3D, false, cadre.couleurFondAffichage, "", "", null,
	    		CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut, 1, null, 1, 0, 0, false, false, Color.black,
	    	  0, StyleTrait.traitFin, Color.black, false);
	    frame.indication("");
	    MacroConstructionRecursiveDlg dlg = new MacroConstructionRecursiveDlg(frame, macro, false);
			dlg.setLocationRelativeTo(frame);
	    dlg.setVisible(true);
	    if (!dlg.retourOK()) {
	    	frame.activeOutilCapture();
	    	return;
	    }
	    dialogueRempli = true;
	    annuleClignotement();
	    proto = macro.tableProto.get(0); // Tous les prototypes fonctionnent de la mme faon. On prend le premier
	    // Il peut arriver que la construction n'utilise aucun objet source
	    if (proto.nbObjSources == 0) {
	    	creeObjet();
	    	frame.activeOutilCapture();
	    	return;
	    }
	    if (proto.natureSources != CPrototype.ConstructionEltsGraphiquesSeuls) {
	    	// Choix des objets sources de type calcul
	    	// Modification version 3.5 : Si la macro ne possde qu'un seul objet source et que c'est un repre
	    	// et qu'il n'y a qu'un repre dans la figure (qui ne soit pas objet final, on l'affecte automatiquement)
	    	if ((proto.nombreSourcesTypeCalcul() == 1) && (proto.nombreObjetsSourcesCalcul(NatureCalcul.NRepere) == 1)
	    			&& liste.nombreObjetsParNatureCalculNonIntermediaires(NatureCalcul.NRepere) == 1) {
	    		CElementGenerique gen = (CElementGenerique)proto.get(0);
	    		gen.elementAssocie = liste.premierRepere();	    		
	    	}
	    	else {
		    	// ChoixObjetsNonGraphiquesPourImplementationProtoDlg dlg2 = new ChoixObjetsNonGraphiquesPourImplementationProtoDlg(frame, proto); // Modifi version 5.1
	    		ChoixObjetsNonGraphiquesPourImplementationProtoDlg dlg2 = new ChoixObjetsNonGraphiquesPourImplementationProtoDlg(frame, proto, false);
		  		dlg2.setLocationRelativeTo(frame);
		    	dlg2.setVisible(true);
		    	if (dlg2.retourOK()) {
		    		nombreObjetsCalculADetruire = dlg2.nombreCalculsCrees;
		    	}
		    	else {
		      	frame.activeOutilCapture();
		      	return;    		
		    	}
		    }
	    }
	    if (proto.natureSources == CPrototype.ConstructionEltsCalculsSeuls) {
	    	creeObjet();
	    	return;
	    }
	    else {
	  		frame.outilPointageActif = frame.outilPointageCre;
	  		indiceElementSourceGraphique = proto.nombreSourcesTypeCalcul();
	  		CElementGenerique gen = (CElementGenerique)proto.get(indiceElementSourceGraphique);
	  		long nat = gen.natureGraphique;
	  		frame.outilPointageActif.designationDe(CElementBase.natureGraphiquePourImplementationProto(nat));
	  		frame.indication(proto.chaineIndicationImplementationProto(indiceElementSourceGraphique + 1, nat)
	  				+ ' ' + getString("ch158"));
	  		resetClignotement();
	    }
    }
    else {
	    // Traitement des objets graphiques
			CElementGenerique gen = (CElementGenerique)proto.get(indiceElementSourceGraphique);
			gen.elementAssocie = elg;
			ajouteClignotementDe(elg);
			excluDeDesignation(elg);
			if (indiceElementSourceGraphique == proto.nbObjSources - 1) {
				creeObjet();
				return;
			}
			indiceElementSourceGraphique++;
  		CElementGenerique gen2 = (CElementGenerique)proto.get(indiceElementSourceGraphique);
  		long nat = gen2.natureGraphique;
  		frame.outilPointageActif.designationDe(CElementBase.natureGraphiquePourImplementationProto(nat));
  		frame.indication(proto.chaineIndicationImplementationProto(indiceElementSourceGraphique + 1, nat)
  				+ ' ' + getString("ch158"));
			
    }
  }
  
  @Override
  public void creeObjet() {
  	// On ajoute  la liste de la macro les objets choisis comme objets sources
  	for (int i = 0; i < proto.nbObjSources; i++) sousListe.add(((CElementGenerique)proto.get(i)).elementAssocie);
  	macro.listeAssociee = sousListe;
  	annuleClignotement();
		if (frame.ajouteElementAvecVerification(macro)) {
		  frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil160"));
		  frame.cadreActif().repaintFigureAvecCalcul();
		}
		frame.activeOutilCapture();
  }

  @Override
  public boolean activationValide() {
  	CMathGrafDoc doc = frame.cadreActif().document;
    if (doc.tablePrototypes.size() == 0) return false;
    return (doc.nombrePrototypesPourIterOuRecur() != 0);
  }
  
	@Override
	public String messageActivationNonValide() {
		CMathGrafDoc doc = frame.cadreActif().document;
		String ch = getString("messagePasDeConstruction");
		if (doc.tablePrototypes.size() == 0) return ch;
		else return ch + " " + getString("utilisable");
	}	

}
