/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.event.MouseEvent;

import objets.CElementGraphique;
import objets.CPointBase;
import types.InfoProx;
import types.NatureObjet;
import dialogs.ChoixTypeDlg;
import dialogs.PremierDernierDlg;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class OutilPointageCreationCurseur extends OutilPointageCre {

  /** Creates a new instance of OutilPointageCre */
  public OutilPointageCreationCurseur(MtgFrame frameInit) {
    super(frameInit);
  }
	
  /**
   * Si l'utilisateur ne clique pas sur un point, et si nommerPointSiAjoute renvoie true, un point sera cr
   * et ce point se verra affect un nom si les noms automatiques ont t choisis dans les prfrences
   * Sera redfini pour OutilPointageCreationSegmentParongueur
   * @return
   */
  protected boolean nommerPointSiAjoute() {
  	return false;
  }
  public void mousePressed(java.awt.event.MouseEvent mouseEvent, MtgInternalFrame cadre) {
  	if (mouseEvent.getButton() == MouseEvent.BUTTON1) {
	    InfoProx info = frame.infoProx;
	    if (info.nombreObjetsProches != 0) {
	      CElementGraphique elg;
	      if (info.nombreTypesProches == 1) {
	        int i;
	        for (i =0; i < NatureObjet.nombreTypesDesignables; i++) {
	          if (info.infoParType[i].nombreVoisins > 0) break;
	        }
	        // i contient l'indice du type d'objets proches
	        if (info.infoParType[i].nombreVoisins >= 2) {
	          // On ouvre un dialogue pour demander de choisir entre le premier et le dernier objet
	          PremierDernierDlg dlg = new PremierDernierDlg(frame);
	      		dlg.setLocationRelativeTo(frame);
	          dlg.setVisible(true);
	          if (!dlg.retourOK()) return;
	          // On regarde quelle a t la rponse de l'utilisateur
	          if (dlg.getPremierChoisi()) elg = info.infoParType[i].premierVoisin;
	          else elg = info.infoParType[i].dernierVoisin;
	        }
	        else {
	          elg = info.infoParType[i].premierVoisin;
	        }
	      }
	      else {
	        // Il faut d'abord ouvrir un dialogue demandant le type de l'objet
	        // que l'utilisateur souhaite dsigner
	        ChoixTypeDlg dlgct = new ChoixTypeDlg(frame, info.typesProches);
	    		dlgct.setLocationRelativeTo(frame);
	        dlgct.setVisible(true);
	        if (!dlgct.retourOK()) return;    
	        int indType = dlgct.indiceTypeChoisi;
	        // Il faut regarder si dans le type choisi un seul objet est proche ou plusieurs
	        if (info.infoParType[indType].nombreVoisins == 1) {
	          elg = info.infoParType[indType].premierVoisin;
	        }
	        else { // Plusieurs objets du type choixi sont proches du curseur
	          PremierDernierDlg dlg = new PremierDernierDlg(frame);
	      		dlg.setLocationRelativeTo(frame);
	          dlg.setVisible(true);
	          if (!dlg.getRetourOK()) return;
	          // On regarde quelle a t la rponse de l'utilisateur
	          if (dlg.getPremierChoisi()) elg = info.infoParType[indType].premierVoisin;
	          else elg = info.infoParType[indType].dernierVoisin;        
	        }  
	      }
	      frame.outilActif.traiteObjetDesigne(elg, mouseEvent.getPoint());
	      return;
	    }
	  	else {
	  		// Pour la cration d'un curseur, si  l'utilisateur n'a pas cliqu sur un point dj cr on cre un nouveau point libre
	      int x = mouseEvent.getX();
	      int y = mouseEvent.getY();
	      // Modification version 4.7.0.1
	      /*
	      CPointBase pt = new CPointBase(frame.cadreActif().listePr, null, false,
	          frame.couleurActive(), false, 0, 0, false, frame.genereNomPourPoint(), frame.pref_indiceTaillePoliceNom,
	          frame.stylePointActif(), true, x, y, false);
	      */
	      CPointBase pt = new CPointBase(frame.cadreActif().listePr, null, false,
	          frame.couleurActive(), false, 0, 0, false, nommerPointSiAjoute() ? frame.genereNomPourPoint() : "", frame.pref_indiceTaillePoliceNom,
	          frame.stylePointActif(), true, x, y, false);
	      frame.ajouteElement(pt);
	      pt.positionne(false, cadre.dimf);
	      frame.outilActif.traiteObjetDesigne(pt, mouseEvent.getPoint());
	  	}
	  }
  }  
}
