/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import glob.Kernel;

import java.awt.*;


import objets.*;
import types.*;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;


/**
 *
 * @author Yves
 */
public class OutilRapporteur extends OutilPar2pts {

	CMesureAngleOriente ptMesureAngle;	// Pointeur
	// sur la mesure d'angle contenue dans la liste des objets visuels.
	// La valeur de cette mesure arrondie au degr prs sera utilise comme
	// angle de la rotation.
	CCercleOA ptCercle; // Pointeur sur le cercle cr dans la liste visuelle
	// auquel sera li un point.
	CPointLieCercle ptPointLie; // Le point li
  
  /** Creates a new instance of OutilRapporteur */
  public OutilRapporteur(MtgFrame frameInit) {
    super(frameInit);
  }
  

  @Override
  public void select() {
    super.select();
    frame.indication(getString("rapporteur1"));
  }
   
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    if (point1 == null)
    {
      frame.indication(getString("rapporteur2"));
      point1 = (CPointAncetre) elg;
      excluDeDesignation(elg);
      ajouteClignotementDe(elg);
      resetClignotement();
      ajouteObjetsVisuels(); // On rajoute un rapporteur
      frame.listeObjetsVisuels.positionne(false, frame.cadreActif().dimf);
      // DecalePointeurSouris(UneVue);
    }
    else
    {
      if (point2 == null)
      {
        frame.indication(getString("rapporteur3"));
        point2 = (CPointAncetre) elg;
        excluDeDesignation(elg);
        ajouteClignotementDe(elg);
        ajouteObjetsVisuels();
        frame.positionneListeVisuelle();      
        resetClignotement();
      }
      // C'est l'outil de capture qui par un clic gauche une fois
      // l'angle visuellement choisi qui provoquera l'appel de CreeObjet
    }    
  }
  
  @Override
  // Modifi version 5.0
  public void creeObjet() {
    CListeObjets liste = frame.cadreActif().listePr;

    double valeurAngle = ptMesureAngle.rendValeur();
    // On arrondit au degr prs en degrs
    // Modifi version 5.1.1
    // if (liste.uniteAngle == UniteAngle.Radian) valeurAngle = valeurAngle*CValeurAngle.convRadDeg;
    if (liste.uniteAngle == UniteAngle.Radian) valeurAngle = Math.toDegrees(valeurAngle);
    if (valeurAngle != Math.floor(valeurAngle)) valeurAngle = Math.floor(valeurAngle + 0.5);
    if (valeurAngle == - 180) valeurAngle = 180;
    // valeurAngle = valeurAngle * CValeurAngle.convDegRad;
    CValeurAngle ptValeurAngle = new CValeurAngle(liste,  valeurAngle);
    CRotation ptr = new CRotation(liste, null, false, point1,  ptValeurAngle);    
    frame.ajouteElement(ptr);    
    // A la diffrence de la version C++ on ne regarde pas si existe dj une
    // rotation quivalente
    CPointImage ptp = new CPointImage(liste, null, false, frame.couleurActive(),
      frame.stylePointActif, frame.pref_indiceTaillePoliceNom, point2,ptr);
		ptp.donneNom(liste.genereNomPourPoint(false));
    if (frame.ajouteElementAvecVerification(ptp)) {
    	MtgInternalFrame cadre = frame.cadreActif();
    	// Modification version 4.0 : On ajoute une marque d'angle et une mesure affcihe ainsi que deux segments s'ils ne sont pas dj crs
  		CSegment seg1 = new CSegment(liste, null, false, frame.couleurActive(), false,
  				frame.styleTraitActif(), point1, point2);
  		if (!liste.existeDeja(seg1)) liste.add(seg1);
  		CSegment seg2 = new CSegment(liste, null, false, frame.couleurActive(), false,
  				frame.styleTraitActif(), point1, ptp);
  		liste.add(seg2);
  		// Version 5.0 : Comme on va afficher une mesure les rois points dooivent tre nomms
  		if (point1.nom.isEmpty()) point1.donneNom(liste.genereNomPourPoint(false));
  		if (point2.nom.isEmpty()) point2.donneNom(liste.genereNomPourPoint(false));
  		//
  		CPrototype proto = frame.chargeConstructionDansDossierMtgFrame("MesureAngle.mgc");
  		((CElementGenerique)proto.get(0)).elementAssocie = point2;
  		((CElementGenerique)proto.get(1)).elementAssocie = point1;
  		((CElementGenerique)proto.get(2)).elementAssocie = ptp;
  		CImplementationProto impProto = new CImplementationProto(liste, proto);
  		impProto.implemente(liste, proto, cadre, 0); // Modifi version 4.8
  		impProto.nomProto = getString("mesureAngle");  		
  		CValeurAffichee affval = (CValeurAffichee) liste.get(liste.size() - 1);
  		// affval.enTete = point1.rendNom() + point2.rendNom() + " = ";
  		affval.couleurFondAffichage = cadre.couleurFond;
  		affval.donneCouleur(frame.couleurActive());
  		// affval.indiceTaillePolice = 3; // Ajout version 4.6.3 Modifi version 5.0
  		// Modifi version 5.0 pour tenir compte des crans haute dfinition
  		affval.indiceTaillePolice = (int) (MtgFrame.coefMultForFonts(cadre.document.sizeWindowx, cadre.document.sizeWindowy)*17);
  		boolean b = MtgFrame.screenIsHighResolution;
  		CMarqueAngleGeometrique arc = (CMarqueAngleGeometrique)liste.get(liste.size() - 2);
  		if (b) arc.rayon *= 2 ;
  		CPointLieDroite ptl = (CPointLieDroite) liste.get(liste.size() - 4);
  		if (b) ptl.abscisse *= 2;
  		arc.donneCouleur(frame.couleurActive());
  		arc.donneStyle(frame.styleTraitActif());
  		arc.donneStyleMarque(frame.styleMarqueAngleActif());
      cadre.repaintFigureAvecCalcul();
    	// Fin modif 5.0
      frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil13"));
    }
    else {
      // Si l'image existe dj on la dtruit ainsi que la rotation
      frame.detruitDernierElement();
      frame.cadreActif().repaintFigureAvecCalcul();
    }
    annuleClignotement();
    select();
  }
  
  /** Mthode ajoutant les objets visuels  la liste listeObjetsVisuels de l'applet
   * au fur et  mesure que l'on dsigne les obejts  la souris
   * Doit tre redfini pour chaque outil utilisant une aide visuelle  la cration
   */
  @Override
  public void ajouteObjetsVisuels() {
    CPointAncetre ptp = null;
    CListeObjets listeVisuelle = frame.listeObjetsVisuels;
    if (point2 == null)
    {
      ptp = ajouteRapporteur(point1); // Le pointeur renvoy dans ptp est
      // un pointeur sur un point d'intersection d'une demi-droite avec un cercle.
      ptCercle = new CCercleOA(listeVisuelle, null, false, Couleur.TraduitCouleur(1),
        false,StyleTrait.traitFin,point1,ptp);
      ptCercle.cache();
      frame.ajouteObjetVisuel(ptCercle);

    }
    else
    {
      // Si le point dont on veut crer l'image a t dsign, le cercle prcdent
      // devient fixe en remplaant dans la liste visuelle le point souris
      // par Point2
      listeVisuelle.remplacePoint(frame.cadreActif().paneFigure.pointSouris,point2);
      // Pour pouvoir initialiser l'abscisse du point li, il faut mesurer
      // un angle orient  partir d'une horizontale
      CDroiteDirectionFixe ptdh = new CDroiteDirectionFixe(listeVisuelle,
        Couleur.TraduitCouleur(1), StyleTrait.traitFin, point1, true, 13);
      ptdh.cache();
      frame.ajouteObjetVisuel(ptdh);
      // On lui lie un point d'abscisse 1
      CPointLieDroite ptld = new CPointLieDroite(listeVisuelle, null, false,
        Couleur.TraduitCouleur(1), false, 0d, 0d, false, "", 13, MotifPoint.PetitRond, true, 1, ptdh, false);       
      ptld.cache();
      frame.ajouteObjetVisuel(ptld);
      // On mesure l'angle
      CMesureAngleOriente ptmao = new CMesureAngleOriente(listeVisuelle, null, false,
        ptld, point1, point2, ""); // Modifi version 5.0
      frame.ajouteObjetVisuel(ptmao);
      frame.positionneListeVisuelle();
      // On rajoute un point li au cercle point par ptCercle
      // Son abscisse est son angle polaire en radians
      double ang = ptmao.rendValeur()*Kernel.ConvDegRad;
      if (ang < 0) ang = ang + 2*Math.PI;
      ptPointLie = new CPointLieCercle(listeVisuelle, null, false,
        Couleur.TraduitCouleur(1), false, 0d, 0, false, "", 13, MotifPoint.PetitRond, true, ang, ptCercle, false);
      frame.ajouteObjetVisuel(ptPointLie);
      // On cre la mesure de l'angle orient point2, point1, pointSouris
      ptMesureAngle = new CMesureAngleOriente(listeVisuelle, null, false, point2,point1,ptPointLie, ""); // Modifi version 5.0
      frame.ajouteObjetVisuel(ptMesureAngle);
      // On cre aussi la mesure de l'angle gomtrique pour affichage sur le rapporteur
      CMesureAngleGeometrique ptmesag = new CMesureAngleGeometrique(listeVisuelle, null, false,
        point2,point1,ptPointLie);
      frame.ajouteObjetVisuel(ptmesag);
      // Changement par rapport  la version C++.
      // Il faudra veiller  ce que le rapporteur ne soit accessible qu'en mode degr,
      // ce qui est logique.
      // On cre une nouveau point cart du rapporteur
      // Pour cela on cre une homothtie
      CHomothetie ptHom2 = new CHomothetie(listeVisuelle, null, false, point1, 1.25);
      frame.ajouteObjetVisuel(ptHom2);
      CPointImage ptimh = new CPointImage(listeVisuelle, null, false,
        Couleur.TraduitCouleur(1),MotifPoint.Rond, 13, ptPointLie,ptHom2);
      ptimh.cache();
      frame.ajouteObjetVisuel(ptimh);
      // On cre l'image provisoire du point
      // D'abord une rotation
      CValeurAngle valang = new CValeurAngle(listeVisuelle, ptMesureAngle);
      CRotation ptrot = new CRotation(listeVisuelle, null, false, point1, valang);
      frame.ajouteObjetVisuel(ptrot);
      CMathGrafDoc doc = frame.cadreActif().document;
      // On affiche la valeur lie au point prcdent avce effacement de ce qu'il y a dessous
      CValeurAffichee ptaff = new CValeurAffichee(listeVisuelle, null, false,
        Couleur.TraduitCouleur(2),0d, 0d, 0d, 0d, false, ptimh,
        MtgFrame.screenIsHighResolution ? 26 : 13, StyleEncadrement.Sans, true, doc.couleurFond, ptmesag, "", "", 0,
				CAffichageLieAPoint.alignementHorizontalCentre, CAffichageLieAPoint.alignementVerticalCentre);
      frame.ajouteObjetVisuel(ptaff);
      // L'image
      CPointImage ptimrot = new CPointImage(listeVisuelle, null, false,
        frame.couleurActive(),frame.stylePointActif(), 2, point2, ptrot);
      frame.ajouteObjetVisuel(ptimrot);
      // On cre deux segments pointills (on ne sait pas si on est  l'intrieur
      // ou l'extrieur du rapporteur
      CSegment ptseg1 = new CSegment(listeVisuelle, null, false, point1, ptPointLie); // En pointills par dfaut
      frame.ajouteObjetVisuel(ptseg1);
      // On ne trace plus le deuxime segment pour ne pas cacher l'angle
      // CSegment ptseg2 = new CSegment(listeVisuelle, null, false, point1, ptimrot); // Idem
      // frame.ajouteObjetVisuel(ptseg2);
      // On passe la main  l'outil de dsignation ad hoc
      frame.outilPointageActif = frame.outilPointageRapporteur;
      frame.outilPointageActif.reset();
      frame.outilPointageRapporteur.associePointLie(ptPointLie);
    }
  }
  
  @Override
  public boolean activationValide() {
    return ((frame.cadreActif().listePr.uniteAngle != UniteAngle.Radian)
    		&& frame.pref_AideVisuelle);
  }
  
	@Override
	public String messageActivationNonValide() {
		String ch = "";
		if (!frame.pref_AideVisuelle) ch = getString("ch182");
		if (frame.cadreActif().listePr.uniteAngle == UniteAngle.Radian) ch = ch + "\n" + getString("messageDegreIndispensable");
		return ch;
	}	
  
}