/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package outils;

import java.awt.Point;
import mathgraph32.MtgFrame;
import objets.CElementGraphique;
import types.Bool;
import types.NatureObjet;
import types.PositionRelative;

public class OutilRelationEntreObjets extends Outil {
  CElementGraphique elt1, elt2;
  
  /** Creates a new instance of OutilInt */
  public OutilRelationEntreObjets(MtgFrame frameInit) {
    super(frameInit, true);
  }
  
  /** Appel lorsque l'outil est slectionn */
  @Override
  public void select() {
    super.select();
    elt1 = null;
    elt2 = null;
    frame.outilPointageCre.designationDe(NatureObjet.NTouteDroite | NatureObjet.NToutCercle |NatureObjet.NToutPoint);
    frame.outilPointageActif = frame.outilPointageCre;
    frame.outilPointageActif.reset();
    frame.indication(getString("relation1"));
  }
  
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    if (elt1 == null) {
    	elt1 = elg;
      if ((elt1.getNature() & NatureObjet.NToutPoint) != 0) {
        frame.outilPointageActif.designationDe(NatureObjet.NTouteDroite | NatureObjet.NToutCercle);
      	frame.indication(getString("intersection1"));
      }
      excluDeDesignation(elg);
      ajouteClignotementDe(elg);
      resetClignotement();
    }
    else {
      if (elt2 == null) {
        elt2 = elg;
        ajouteClignotementDe(elg);        
        creeObjet();
        annuleClignotement();
        frame.activeOutilCapture();
      }
    }
  }
 
  @Override
  public void creeObjet() {
  	String ch;
    if ((elt2.getNature() & NatureObjet.NToutPoint) != 0) {
    	CElementGraphique elt3 = elt2;
    	elt2 = elt1;
    	elt1 = elt3;
    }
    Bool b = new Bool(false);
    switch (elt2.positionRelative(elt1, b)) {
	    case PositionRelative.appartient :
	    	ch = getString("positionAppartient");
	    	break;
	    case PositionRelative.nappartient :
	    	ch = getString("positionNAppartient");
	    	break;
	    case PositionRelative.paralleles :
	    	ch = getString("positionParallelles");
	    	break;
	    case PositionRelative.confondus :
	    	ch = getString("positionConfondus");
	    	break;
	    case PositionRelative.secants :
	    	ch = getString("positionSecants");
	    	break;
	    case PositionRelative.secants1Point :
	    	ch = getString("positionSecants1Point");
	    	break;
	    case PositionRelative.secants2Point :
	    	ch = getString("positionSecants2Point");
	    	break;
	    case PositionRelative.tangents :
	    	ch = getString("positionTangents");
	    	break;
	    case PositionRelative.vide :
	    	ch = getString("positionVide");
	    	break;
	    case PositionRelative.memeSupportDroite :
	    	ch = getString("positionMemeSupportDroite");
	    	break;
	    case PositionRelative.memeSupportCercle :
	    	ch = getString("positionMemeSupportCercle");
	    	break;
	    default :
	    	ch = getString("positionVide"); // Pour satisfaire le compilateur
    }
    if (b.getValue()) ch = ch + "\n" + getString("positionOrthogonaux");
    frame.cadreActif().videDesignation();
    frame.dialogueInformation(ch);
  }

}
