/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.*;

import objets.*;
import types.*;
import mathgraph32.MtgFigurePanel;
import mathgraph32.MtgFrame;

/**
 *
 * @author Yves
 */
public class OutilSymAxiale extends Outil {
  private CDroiteAncetre droite;
  private CElementGraphique objet;
  private int nombreImagesCrees;
  private CSymetrieAxiale ptSym;
  
  /** Creates a new instance of OutilTranslation */ 
  public OutilSymAxiale(MtgFrame frameInit) {
    super(frameInit, true);
  }

  @Override
  public void select() {
    super.select();
    droite = null;
    objet = null;
    ptSym = null;
    nombreImagesCrees = 0;
    frame.outilPointageCre.designationDe(NatureObjet.NTouteDroite);
    frame.outilPointageActif = frame.outilPointageCre;
    frame.indication(getString("symAxiale1"));
  }

  /**
   * Il est ncessaire de regarder quand l'outil est dselectionn si on a vraiment cr des images
   * par une transformation. Sinon il faut supprimer le dernier objet qui est une homothtie
   */
  @Override
  public void deselect() {
    CListeObjets liste = frame.cadreActif().listePr;
    if (ptSym == liste.get(liste.size() - 1)){
    	frame.detruitDernierElement();
    }
    super.deselect();
  }
  
  @Override
  public void creeObjet() {
    CListeObjets liste = frame.cadreActif().listePr;
    if (nombreImagesCrees == 0) {
    	ptSym = new CSymetrieAxiale(liste, null, false, droite);
    	CTransformation sym = liste.transformationConfondueAvec(ptSym);
    	if (sym != null) {
    		ptSym = (CSymetrieAxiale) sym;
     	}
    	else frame.ajouteElement(ptSym);
    }
    Int nbImagesCrees = new Int();
    CElementGraphique elg = imageParTransformation(liste,  objet, ptSym, nbImagesCrees);
    if (frame.ajouteElementsImageAvecVerification(elg, nbImagesCrees.getValue(), ptSym)) {
    	nombreImagesCrees++;
    	if (nombreImagesCrees != 0) {
    		frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil17"));
    	}
  		frame.cadreActif().repaintFigureAvecCalcul();    	
    }
  }  
  /** Mthode ajoutant les objets visuels  la liste listeObjetsVisuels de l'applet
   * au fur et  mesure que l'on dsigne les obejts  la souris
   * Doit tre redfini pour chaque outil utilisant une aide visuelle  la cration
   */
  @Override
  public void ajouteObjetsVisuels() {
    CListeObjets liste = frame.listeObjetsVisuels;
    MtgFigurePanel pane = frame.cadreActif().paneFigure;
    CSymetrieAxiale sym = new CSymetrieAxiale(liste, null, false, droite);
    frame.ajouteObjetVisuel(sym);
    CPointImage ptim = new CPointImage(liste, null, false, frame.couleurActive(),
       frame.stylePointActif(), 13, pane.pointSouris, sym);
    frame.ajouteObjetVisuel(ptim);
    CMilieu ptm = new CMilieu(liste, null, false, Couleur.TraduitCouleur(1), 
       MotifPoint.PetitRond, pane.pointSouris, ptim);
    frame.ajouteObjetVisuel(ptm);
    CSegment seg1 = new CSegment(liste, null, false, Couleur.TraduitCouleur(1),
       false, StyleTrait.traitPointille, pane.pointSouris,
      ptm);
    frame.ajouteObjetVisuel(seg1);
    CSegment seg2 = new CSegment(liste, null, false, Couleur.TraduitCouleur(1), false,
       StyleTrait.traitPointille, ptm, ptim);
    frame.ajouteObjetVisuel(seg2);
    CMarqueSegment mar1 = new CMarqueSegment(liste, null, false, Couleur.TraduitCouleur(1),
       false, StyleTrait.traitEpais,
      MotifMarqueSegment.MarqueSimple, seg1);
    frame.ajouteObjetVisuel(mar1);
    CMarqueSegment mar2 = new CMarqueSegment(liste, null, false, Couleur.TraduitCouleur(1),
       false, StyleTrait.traitEpais,
      MotifMarqueSegment.MarqueSimple, seg2);
    frame.ajouteObjetVisuel(mar2);
    CDroitePerpendiculaire perp = new CDroitePerpendiculaire(liste, null, false, Color.black, true, 0, 0, true, "", 13, StyleTrait.traitFin, 0.9,pane.pointSouris, droite);
    frame.ajouteObjetVisuel(perp);
    CMarqueAngleDroit mar = new CMarqueAngleDroit(liste, null, false, perp, MtgFrame.screenIsHighResolution ? 24 : 12);
    frame.ajouteObjetVisuel(mar);
  }

  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    if (droite == null) {
      frame.indication(getString("symAxiale2"));
      droite = (CDroiteAncetre) elg;
      excluDeDesignation(elg);
      ajouteClignotementDe(elg);
      ajouteObjetsVisuels();
      frame.outilPointageActif.designationDe(NatureObjet.NToutPoint | NatureObjet.NTouteDroiteSaufVecteur |
                NatureObjet.NCercle | NatureObjet.NArc | NatureObjet.NSegment | NatureObjet.NPolygone
                | NatureObjet.NLigneBrisee);
      resetClignotement();
    }
    else {
    	// Cration d'images multiples possibles
      objet = elg;
      creeObjet();
    }
  }  
}