/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CCalcul;
import objets.CDroiteAncetre;
import objets.CElementGenerique;
import objets.CImplementationProto;
import objets.CListeObjets;
import objets.CPrototype;
import objets.CResultatValeur;
import objets.CValeurAssocieeAVariable;
import objets.CValeurDynamique;
import types.NatureCalcul;
import dialogs.TangenteDlg;

public class OutilTangente extends Outil {
	TangenteDlg dlg;
	
	public OutilTangente(MtgFrame frameInit) {
		super(frameInit, false);
	}
	
  @Override
	public void select() {
		super.select();
  	dlg = new TangenteDlg(frame);
		dlg.setLocationRelativeTo(frame);
  	dlg.setVisible(true);
  	if (dlg.retourOK()) creeObjet();
  	else frame.activeOutilCapture();
	}

  @Override
  public void creeObjet() {
  	CValeurDynamique valeurAbs;
  	MtgInternalFrame cadre = frame.cadreActif();
  	CListeObjets liste= cadre.listePr;
  	frame.cadreActif().videDesignation();
  	// On regarde d'abord si pour le champ d'dition de l'abscisse l'utilisateur a fait rfrence  une valeur
  	// existante ou entr un calcul
  	CValeurAssocieeAVariable valAbs =dlg.valAbs;
  	if (valAbs.calcul.getClass() != CResultatValeur.class) {
  		String nomMin = liste.genereNomPourCalcul("abs", true);
  		CCalcul calculMin = new CCalcul(liste, null, false, nomMin, valAbs.calcul.chaineCalculSansPar(null), valAbs.calcul);
  		frame.ajouteElement(calculMin);
  		valeurAbs = calculMin;
  	}
  	else {
  		valeurAbs = ((CResultatValeur)valAbs.calcul).valeurAssociee;
  	}
  	CPrototype proto = frame.chargeConstructionDansDossierMtgFrame("Tangente.mgc");
  	//On implmente maintenant ce prototype dans la figure
		((CElementGenerique)proto.get(0)).elementAssocie = dlg.listeFonctions.fonctionSelectionnee();
		((CElementGenerique)proto.get(1)).elementAssocie = valeurAbs;
		((CElementGenerique)proto.get(2)).elementAssocie = dlg.listeReperes.repereChoisi();
		CImplementationProto impProto = new CImplementationProto(liste, proto);
		impProto.implemente(liste, proto, cadre, 0); // Modifi version 4.8
		// On donne aux calculs finaux les noms choisis dans la bote de dialogue
		((CValeurDynamique)liste.get(liste.size() - 2)).donneNom(dlg.editeurNomCoeff.getText());
		// On modifie le nom de l'implmentation (pour internationalisation)
		impProto.nomProto = getString("menuCreerTangente");
		// On modifie le dernier objet implment qui est un affichage de valeur li  un point
		CDroiteAncetre droite = (CDroiteAncetre) liste.get(liste.size() - 1);
		droite.donneStyle(frame.styleTraitActif());
		droite.donneCouleur(frame.couleurActive());
    cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil126"));
    cadre.repaintFigureAvecCalcul();
    frame.activeOutilCapture();
    // Ajout version 4.9.9
    cadre.nameEditor.associeA(droite);
    cadre.nameEditor.setPositionOnInternaleFrame();
    
  }	
  	
  @Override
	public boolean activationValide() {
		CListeObjets liste = frame.cadreActif().listePr;
		return (liste.nombreObjetsParNatureCalculNonIntermediaires(NatureCalcul.NRepere)!=0)
						&& (liste.nombreFonctionsDerivables()!=0);
	}
	
	@Override
	public String messageActivationNonValide() {
		CListeObjets liste = frame.cadreActif().listePr;
		String ch = "";
		if (liste.nombreObjetsParNatureCalculNonIntermediaires(NatureCalcul.NRepere)==0) ch = ch + getString("messagePasDeRepere") + "\n";
		if (liste.nombreFonctionsDerivables() == 0) ch = ch + getString("messagePasDeFonctionDerivable");
		return ch;
	}	  	
}
