/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package types;

import java.io.IOException;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

/**
 * Renvoie les styles de traits via ses mthodes de classe
 * Modifi verion 5.0.
 * Est maintenant un objet qui contient deux membres
 * style : Qui contient un indice donnant le style de trait
 * epaisseur : qui donne l'paisseur du trait
 * @author  Yves
 * @version 
 */

public class StyleTrait {
	public int style;
	public int epaisseur;
	
	// Les constructeurs ajouts pour la version 5.0
	public StyleTrait() {
	}
	public StyleTrait(int styleInit, int epaisseurInit) {
		style = styleInit;
		epaisseur = epaisseurInit;
	}
	
	public boolean equals(StyleTrait s) {
		return (style == s.style) && (epaisseur == s.epaisseur);
	}
	
	public void read(MathGraphDataInputStream inps) throws IOException {
		style = inps.readByte();
		epaisseur = inps.readByte();
	}
	public void write(MathGraphDataOutputStream oups) throws IOException {
		oups.writeByte(style);
		oups.writeByte(epaisseur);
	}
	// Chargement pour avant la version 5.0
	// Modifi version 5.0.4 avec ajout d'undeuxime paramtre car les macros de construction itratives et rcursives avaient enregistr
	// le style de ligne sous forme d'un int et pas d'un byte
	public void readOld(MathGraphDataInputStream inps, boolean bbyte) throws IOException {
		int oldstyle =bbyte ?  inps.readByte() : inps.readInt();
		switch(oldstyle) {
			case straitFin : 
				style = styleTraitContinu;
				epaisseur = 1;
				break;
			case straitPointille :
				style = styleTraitPointille;
				epaisseur = 1;
				break;
			case straitEpais :
				style = styleTraitContinu;
				epaisseur = 2;
				break;
			case straitTrait :
				style = styleTraitTrait;
				epaisseur = 1;
				break;
			case straitPointPoint :
				style = styleTraitPointPoint;
				epaisseur = 1;
				break;
			case straitTresEpais :
				style = styleTraitContinu;
				epaisseur = 3;
				break;
			case straitPointilleEpais :
				style = styleTraitPointille;
				epaisseur = 2;
				break;
			default :
				style = styleTraitContinu;
				epaisseur = 1;
		}
	}
	// Ajouts version 5.0
	public final static int styleTraitContinu = 0;
	public final static int styleTraitPointille = 1;
	public final static int styleTraitTrait = 2;
	public final static int styleTraitPointPoint = 3;
	public final static int styleTraitPointPointPoint = 4;
	public final static int styleTraitTraitLongTrait = 5;
	
	public static final StyleTrait traitFin = new StyleTrait(styleTraitContinu, 1);
	public static final StyleTrait traitPointille = new StyleTrait(styleTraitPointille, 1);
	public static final StyleTrait traitEpais = new StyleTrait(styleTraitContinu, 2);
	//

	// Constantes suivantes renommes mais gardes pour compatibilit avec versions prcdentes.
  public final static int straitFin = 0;
  public final static int straitPointille = 1;
  public final static int straitEpais = 2;
  public final static int straitTrait = 3;
  public final static int straitPointPoint = 4;
  public final static int straitTresEpais = 5;
  public final static int straitPointilleEpais = 6; // Nouveau style pour la version java
  public final static int straitMini = straitFin;
  public final static int straitMaxi = straitTresEpais;
}
